C ICAM (Idealized Coupleable Atmospheric Model)
C icamsrc $Revision: 1.8 $ created $Date: 1996/05/02 21:31:03 $
C
C Author: R. Saravanan, Climate and Global Dynamics Division
C                       National Center for Atmospheric Research
C                       Boulder, CO 80307-3000, U.S.A.
C                       E-mail: svn@ncar.ucar.edu
C                       URL: http://www.cgd.ucar.edu:/gds/svn/svn.html
C
C astate.F: Fortran source module
C Contains routines for reading and writing the state of the atmospheric
C model
C
 
#include "mmain.h"
 
C Routines:
C ATMSAV: Save state of atmospheric model to a file (called by ATMFIN)
C        (All the remaining routines called by ATMINI)
C GETTIM: Read model timing information from save file
C GETPAR: Read model parameters from save file
C GETMSK: Read model "masks" from save file
C GETPGQ: Read prognostic model variables from save file
C INPAR:  Initialize model parameters from parameter file
C
 
      SUBROUTINE ATMSAV( IUNIT )
C Write state of atmospheric model to unit IUNIT ("create save file")
C (in ASCII form)
C
C Input:  (mstate.h, Parts: 1-2; mparams.h, Parts: 2-3)
C Output: (model state is written to unit IUNIT)
C
#include "mgrid.h"
#include "mparams.h"
#include "mstate.h"
C
      INTEGER IUNIT
C
      INTEGER IB, IPGQ, J, K, L, M, N
C
C     Include file "mstate.h", Part 1: Time-dependent information
C
      WRITE(IUNIT,AFMT) INIPRG
      WRITE(IUNIT,AFMT) INITIM
      WRITE(IUNIT,EFMT) TIM
      WRITE(IUNIT,IFMT) IPART, ISTEP
C
C     Include file "mparams.h", Part 2: Model parameters
C
      WRITE(IUNIT,CFMT) 'C', 'Model parameters'
      WRITE(IUNIT,AFMT) RTITLE
      WRITE(IUNIT,AFMT) SEAFIL
      WRITE(IUNIT,AFMT) OROFIL
      WRITE(IUNIT,AFMT) SSTFIL
      WRITE(IUNIT,AFMT) FADFIL
      WRITE(IUNIT,AFMT) ICEFIL
C
      WRITE(IUNIT,IFMT) NSPDAY, NDPYR, NSYNC
      WRITE(IUNIT,IFMT) MMAX, NMAX, NLEV, NTRC
C
      WRITE(IUNIT,LFMT) COUFLG, SSNFLG, ICEFLG
C
      WRITE(IUNIT,EFMT) PLID, PTROP
      WRITE(IUNIT,EFMT) EQPFAC, PPFAC, YPHASE
      WRITE(IUNIT,EFMT) IMPLCT, REFFAC, D8DAMP, IPRNDL
      WRITE(IUNIT,EFMT) VISC0,  DMPBOT, DZBOT,  DMPTOP, DPTOP
      WRITE(IUNIT,EFMT) LWFRAC, DLWAIR, DLWH2O, DSWAIR, DSWO3
      WRITE(IUNIT,EFMT) DLWCO2, MULCO2, MULTIM
      WRITE(IUNIT,EFMT) ALBSEA, ALBLAN, ALBICE
      WRITE(IUNIT,EFMT) LANHCP, SATURF, BUKDEP, MIXDEP
C
      WRITE(IUNIT,IFMT) NBASIN, NBCOUP, IRUNOF
      DO 100 IB=1,NBASIN
        WRITE(IUNIT,EFMT) LONBLO(IB), LONBHI(IB),
     1                    LATBLO(IB), LATBHI(IB),
     2                    ROFAC(IB)
  100   CONTINUE
C
C     Include file "mparams.h", Part 3: Model "masks"
C
C     Save sea mask
      WRITE(IUNIT,CFMT) 'C', 'Sea-mask'
      DO 200 IB=1,NBASIN
        WRITE(IUNIT,IFMT) JB0(IB), NATMX(IB),
     1                    KB0(IB), NATMY(IB)
  200   CONTINUE
      WRITE(IUNIT,LFMT) ((SEAMSK(J,K),J=1,K1),K=1,K2)
C
C     Save bucket height (m)
      WRITE(IUNIT,CFMT) 'C', 'Bucket height (m)'
      WRITE(IUNIT,EFMT) ((BUCKHT(J,K),J=1,K1),K=1,K2)
C
C     Save orography height (m)
      WRITE(IUNIT,CFMT) 'C', 'Orography height (m)'
      WRITE(IUNIT,EFMT) ((OROHT(J,K),J=1,K1),K=1,K2)
C
C     Include file "mstate.h", Part 2: Prognostic quantities
C
C     Save atmospheric prognostic quantities
      WRITE(IUNIT,CFMT) 'C', 'Atmosphere'
      DO 300 L=1,L1
        DO 300 IPGQ=JVOR,JPOT+NTRC
          WRITE(IUNIT,EFMT)
     1         ((PGQSP0(M,N,IPGQ,L),M=0,MIN0(N,M1)),N=0,N1)
          WRITE(IUNIT,EFMT)
     1         ((PGQSP1(M,N,IPGQ,L),M=0,MIN0(N,M1)),N=0,N1)
  300     CONTINUE
C     Save prognostic sea surface temperature (K)
      WRITE(IUNIT,CFMT) 'C', 'Sea'
      WRITE(IUNIT,EFMT) ((SST0(J,K),J=1,K1),K=1,K2)
      WRITE(IUNIT,EFMT) ((SST1(J,K),J=1,K1),K=1,K2)
C     Save land surface temperature (K) and water level (m)
      WRITE(IUNIT,CFMT) 'C', 'Land'
      DO 310 IPGQ=JLANT,JLANWL
        WRITE(IUNIT,EFMT) ((LAND0(J,K,IPGQ),J=1,K1),K=1,K2)
        WRITE(IUNIT,EFMT) ((LAND1(J,K,IPGQ),J=1,K1),K=1,K2)
  310   CONTINUE
C
      IF (ICEFLG) THEN
C       Save ice details
        WRITE(IUNIT,CFMT) 'C', 'Ice'
C       Save ice mask
        WRITE(IUNIT,LFMT) ((ICEMSK(J,K),J=1,K1),K=1,K2)
C       Save ice temperature (K) and height (m)
        DO 320 IPGQ=JICET,JICEHT
          WRITE(IUNIT,EFMT) ((ICE0(J,K,IPGQ),J=1,K1),K=1,K2)
          WRITE(IUNIT,EFMT) ((ICE1(J,K,IPGQ),J=1,K1),K=1,K2)
  320     CONTINUE
      ENDIF
C
C     End-of-file comment
      WRITE(IUNIT,CFMT) 'C', 'End-of-file'
C
      RETURN
      END
 
      SUBROUTINE GETTIM( IUNIT )
C Read model timing information from save file (unit IUNIT)
C
C Input:  INTEGER IUNIT -- Fortran unit number from which to read
C Output: (mstate.h, Part 1)
C
#include "mparams.h"
#include "mstate.h"
C
      INTEGER IUNIT
C
C     Include file "mstate.h", Part 1: Time-dependent information
C
      READ(IUNIT,AFMT) INIPRG
      READ(IUNIT,AFMT) INITIM
      READ(IUNIT,*) TIM
      READ(IUNIT,*) IPART, ISTEP
C
C     Check whether save file was written by the same program
      IF (INIPRG(1:12) .NE. PRGNAM) THEN
        CALL ATMERR( 'GETTIM',
     1      'Cannot read save file written by different program')
      ENDIF
C
      RETURN
      END
 
      SUBROUTINE GETPAR( IUNIT )
C Read model parameters from unit IUNIT,
C and perform consistency checks
C
C Input:  INTEGER IUNIT -- Fortran unit number from which to read
C Output: (mparams.h, Part 2)
C
#include "mparams.h"
C
      INTEGER IUNIT
C
      INTEGER IB
      REAL SUM
      CHARACTER CH*1, COMNT*72
C
C     Include file "mparams.h", Part 2: Model parameters
C
      READ(IUNIT,CFMT) CH, COMNT
      READ(IUNIT,AFMT) RTITLE
      READ(IUNIT,AFMT) SEAFIL
      READ(IUNIT,AFMT) OROFIL
      READ(IUNIT,AFMT) SSTFIL
      READ(IUNIT,AFMT) FADFIL
      READ(IUNIT,AFMT) ICEFIL
C
      READ(IUNIT,*) NSPDAY, NDPYR, NSYNC
      READ(IUNIT,*) MMAX, NMAX, NLEV, NTRC
C
      READ(IUNIT,LFMT) COUFLG, SSNFLG, ICEFLG
C
      READ(IUNIT,*) PLID, PTROP
      READ(IUNIT,*) EQPFAC, PPFAC, YPHASE
      READ(IUNIT,*) IMPLCT, REFFAC, D8DAMP, IPRNDL
      READ(IUNIT,*) VISC0,  DMPBOT, DZBOT, DMPTOP, DPTOP
      READ(IUNIT,*) LWFRAC, DLWAIR, DLWH2O, DSWAIR, DSWO3
      READ(IUNIT,*) DLWCO2, MULCO2, MULTIM
      READ(IUNIT,*) ALBSEA, ALBLAN, ALBICE
      READ(IUNIT,*) LANHCP, SATURF, BUKDEP, MIXDEP
#ifdef OLD_VERSION
      SATURF= 0.8
#endif
C
      READ(IUNIT,*) NBASIN, NBCOUP, IRUNOF
C
      IF ( (NBASIN .GT. MAXBAS) .OR. (NBCOUP .GT. NBASIN) ) THEN
        CALL ATMERR( 'GETPAR', 'Error in basin count' )
      ENDIF
C
      DO 100 IB=1,NBASIN
        READ(IUNIT,*) LONBLO(IB), LONBHI(IB),
     1                LATBLO(IB), LATBHI(IB),
     2                ROFAC(IB)
  100   CONTINUE
C
C     Check for unimplemented options
      IF (BUKDEP .NE. 0.) STOP 'BUKDEP'
C
C     Check time-stepping parameters
      IF ( (NSPDAY .LE. 0) .OR. (NDPYR .LE. 0) .OR.
     1     (NSYNC .LE. 0) ) THEN
        CALL ATMERR( 'GETPAR',
     1              'Incorrect time-stepping parameters' )
      ENDIF
C
C     Check that model fits within declared array dimensions
      IF ( (MMAX .GT. M1MAX) .OR. (NMAX .GT. N1MAX) .OR.
     1     (NLEV .GT. L1MAX) ) THEN
        CALL ATMERR( 'GETPAR',
     1              'Recompile model with increased resolution' )
      ELSE IF (NTRC.NE.NTRACE) THEN
        CALL ATMERR( 'GETPAR',
     1              'Incorrect number of tracers specified' )
      ENDIF
C
      IF (NBASIN .GT. 0) THEN
        DO 210 IB=1,NBASIN
C         Check that longitudinal basin boundaries lie within range
          IF ( (LONBLO(IB) .LT.   0.0) .OR.
     1         (LONBLO(IB) .GE. 360.0) .OR.
     2         (LONBHI(IB) .LE.   0.0) .OR.
     3         (LONBHI(IB) .GT. 720.0) .OR.
     4         (LONBLO(IB) .GT. LONBHI(IB)) ) THEN
            CALL ATMERR( 'GETPAR',
     1          'Longitudinal basin boundaries out of bounds')
          ENDIF
  210   CONTINUE
C
        IF ( (IRUNOF .LT. 0) .OR. (IRUNOF .GT. 2) ) THEN
          CALL ATMERR( 'GETPAR', 'Invalid run-off option IRUNOF' )
        ENDIF
        IF (IRUNOF .GT. 0) THEN
C         Check that total run-off is conserved to within .01%
          SUM= 0.0
          DO 220 IB=1,NBASIN
  220       SUM= SUM + ROFAC(IB)
          IF (ABS(SUM - 1.0) .GT. 0.0001) THEN
            CALL ATMERR( 'GETPAR',
     1                  'Run-off fractions do not sum to unity')
          ENDIF
        ENDIF
      ENDIF
C
      IF (COUFLG .AND. (NBCOUP .EQ. 0)) THEN
          CALL ATMERR( 'GETPAR', 'COUFLG set, but no coupled basins')
      ELSE IF ((.NOT.COUFLG) .AND. (NBCOUP .GT. 0)) THEN
          CALL ATMERR( 'GETPAR', 'COUFLG not set for coupled basins')
      ENDIF
C
      RETURN
      END
 
      SUBROUTINE GETMSK( IUNIT )
C Read "masks" from save file (unit IUNIT)
C (sea mask, bucket height, orography, mixed-layer depth)
C
C Input:  INTEGER IUNIT -- Fortran unit number from which to read
C Output: (mparams.h, Part 3)
C
#include "mgrid.h"
#include "mparams.h"
C
      INTEGER IUNIT
C
      INTEGER IB, J, K
      CHARACTER CH*1, COMNT*72
C
C     Include file "mparams.h", Part 3: Model "masks"
C
C     Read sea mask
      READ(IUNIT,CFMT) CH, COMNT
      DO 200 IB=1,NBASIN
        READ(IUNIT,*) JB0(IB), NATMX(IB),
     1                KB0(IB), NATMY(IB)
 200     CONTINUE
      READ(IUNIT,LFMT) ((SEAMSK(J,K),J=1,K1),K=1,K2)
C
C     Read bucket height (m)
      READ(IUNIT,CFMT) CH, COMNT
      READ(IUNIT,EFMT) ((BUCKHT(J,K),J=1,K1),K=1,K2)
C
C     Read orography height (m)
      READ(IUNIT,CFMT) CH, COMNT
      READ(IUNIT,EFMT) ((OROHT(J,K),J=1,K1),K=1,K2)
C
      RETURN
      END
 
      SUBROUTINE GETPGQ( IUNIT )
C Read prognostic model variables from save file (unit IUNIT)
C
C Input:  INTEGER IUNIT -- Fortran unit number from which to read
C Output: (mstate.h, Part 2)
C
#include "mgrid.h"
#include "mparams.h"
#include "mstate.h"
C
      INTEGER IUNIT
C
      INTEGER IPGQ, J, K, L, M, N
      CHARACTER CH*1, COMNT*72
C
C     Include file "mstate.h", Part 2: Prognostic quantities
C
C     Read atmospheric prognostic quantities
      READ(IUNIT,CFMT) CH, COMNT
      DO 300 L=1,L1
        DO 300 IPGQ=JVOR,JPOT+NTRC
          READ(IUNIT,EFMT)
     1         ((PGQSP0(M,N,IPGQ,L),M=0,MIN0(N,M1)),N=0,N1)
          READ(IUNIT,EFMT)
     1         ((PGQSP1(M,N,IPGQ,L),M=0,MIN0(N,M1)),N=0,N1)
  300     CONTINUE
C     Read prognostic sea surface temperature (K)
      READ(IUNIT,CFMT) CH, COMNT
      READ(IUNIT,EFMT) ((SST0(J,K),J=1,K1),K=1,K2)
      READ(IUNIT,EFMT) ((SST1(J,K),J=1,K1),K=1,K2)
C     Save land surface temperature (K) and water level (m)
      READ(IUNIT,CFMT) CH, COMNT
      DO 310 IPGQ=JLANT,JLANWL
        READ(IUNIT,EFMT) ((LAND0(J,K,IPGQ),J=1,K1),K=1,K2)
        READ(IUNIT,EFMT) ((LAND1(J,K,IPGQ),J=1,K1),K=1,K2)
  310   CONTINUE
C
      IF (ICEFLG) THEN
C       Read ice details
        READ(IUNIT,CFMT) CH, COMNT
C       Read ice mask
        READ(IUNIT,LFMT) ((ICEMSK(J,K),J=1,K1),K=1,K2)
C       Save ice temperature (K) and height (m)
        DO 320 IPGQ=JICET,JICEHT
          READ(IUNIT,EFMT) ((ICE0(J,K,IPGQ),J=1,K1),K=1,K2)
          READ(IUNIT,EFMT) ((ICE1(J,K,IPGQ),J=1,K1),K=1,K2)
  320     CONTINUE
      ELSE
C       Set default ice mask, ice temperature and height (no ice)
        DO 330 K=1,K2
          DO 330 J=1,K1
            ICEMSK(J,K)= .FALSE.
            ICE0(J,K,JICET)= 0.0
            ICE1(J,K,JICET)= 0.0
            ICE0(J,K,JICEHT)= 0.0
  330       ICE1(J,K,JICEHT)= 0.0
      ENDIF
C
C     End-of-file comment
      READ(IUNIT,CFMT) CH, COMNT
      IF (COMNT .NE. 'End-of-file') THEN
        CALL ATMERR( 'GETPGQ',
     1              'End-of-file comment not found in save file' )
      ENDIF
C
      RETURN
      END
 
      SUBROUTINE INPAR( IUNIT, RESTRT )
C Initialize model parameters from unit IUNIT
C (for new/restart run)
C
C Input:  INTEGER IUNIT -- Fortran unit number from which to read
C         LOGICAL RESTRT -- Restart run flag
C Output: (mparams.h, Part 2)
C
#include "mgrid.h"
#include "mparams.h"
#include "mstate.h"
C
      INTEGER IUNIT
      LOGICAL RESTRT
C
      INTEGER MOLD, NOLD, NLVOLD
      REAL YPHOLD
C
      IF (RESTRT) THEN
C       Restart run; save old model resolution parameters
        MOLD= MMAX
        NOLD= NMAX
        NLVOLD= NLEV
C       Save old phase of year
        YPHOLD= FLOAT(MOD(ISTEP,NSPDAY*NDPYR))/FLOAT(NSPDAY*NDPYR)
      ENDIF
C
C     Read model parameters from parameter file
      CALL GETPAR( IUNIT )
C
      IF (RESTRT) THEN
C       Restart run; check model resolution parameters
        IF ( (MMAX .NE. MOLD) .OR. (NMAX .NE. NOLD) .OR.
     1       (NLEV .NE. NLVOLD) ) THEN
          CALL ATMERR( 'INPAR',
     1                'Cannot alter model resolution on restart' )
        ENDIF
C       Check whether phase of year agrees to within 1%
        IF (ABS(YPHASE - YPHOLD) .GT. 0.01) THEN
          CALL ATMERR( 'INPAR',
     1                'Cannot alter phase of year on restart' )
        ENDIF
      ENDIF
C
      RETURN
      END
 
