C
C Diagnos $Revision: 6.6 $ created $Date: 1996/09/12 16:49:36 $
C
#ifndef MCONS
#   include "mcons.h"
#endif
#ifndef LENWRD
#   define LENWRD 4
#endif
#ifndef RECUNIT
#   define RECUNIT 1
#endif
#ifndef NLEVMX
#   define NLEVMX 2
#endif
#ifndef NLATMX
#   define NLATMX 4
#endif
C ------ Description of variables in module diacom.h ----------------
C C preprocessor parameters
C LENWRD= length of a machine word (in bytes)
C RECUNIT= units, i.e., no. of bytes, in which direct-access record
C          lengths are measured (usually 1 for most machines).
C NLEVMX= maximum number of levels for horizontal section saves
C NLATMX= maximum number of latitudes for vertical section saves
C
C C preprocessor switches to determine which variables are saved
C to horizontal section files
C
C SAVE_FLX_BOT: If defined, save surface fluxes (TSURF, PRECIP, EVAP, FS
C SAVE_FLX_TOP: If defined, save top fluxes (OLR, ASR)
C SAVE_TUV:     If defined, save T, U, V at each level
C SAVE_Q:       If defined, save Q at each level
C SAVE_EDDY_HI: If defined, save upper level eddy statistics
C                           (TEKE, PSIFR1)
C SAVE_EDDY_LO: If defined, save lower level eddy statistics
C                           (VT1, DIVUT1, DIVUQ1. PSIFR1)
C
C Variables
C IDSET = Diagnostic set number for this run (0,...,9999)
C NTSPD = number of time-steps per model day
C (If negative, represents no. of model days per time-step)
C NYRDAY= number of days in a model year
C IUADG = fortran unit number set aside for diagnostics
C IUATS = fortran unit number set aside for time-series
C IUAHS = fortran unit number set aside for horizontal section data
C IUAVS = fortran unit number set aside for vertical section data
C IUAZS = fortran unit number set aside for zonal-mean section data
C IDSTEP= value of step-count (ISTEP) when sampling is to be initiated
C ISSTEP= value of step-count (ISTEP) when saving is to be initiated
C NDSKIP= diagnostic sampling interval (in units of time-steps)
C NTFREQ= time-series save frequency (in units of NDSKIP time-steps)
C NSFREQ= section data save frequency (in units of NDSKIP time-steps)
C NSAMPL= count of diagnostic samples taken so far
C NTSAMP= count of time-series samples saved so far (<= NSAMPL)
C NSSAMP= count of section samples saved so far (<= NSAMPL)
C LDIAG = level no. for special diagnostics
C NRECTS= direct-access record count for time-series file
C NRECHS= direct-access record count for horizontal section file
C NRECVS= direct-access record count for vertical section file
C NRECZS= direct-access record count for zonal-mean section file
C ISTP0 = time-step count of first sample
C ISTP1 = time-step count of last sample
C DTSTEP= model time-step (in sec)
C TAU0  = model time at beginning of sampling period (in sec)
C TAU1  = model time at end of sampling period (in sec)
C YRPHA = phase of year at beginning of run (time-step 0)
C         (YRPHA=0..1, 0 => Mar 21, 0.75 => Dec 22)
C LANWD = fractional width of land-sector (0..1)
C SAVSRF= surface flag indicating whether section data should be saved
C SAVEL = level flag indicating whether section data should be saved
C SAVEK = latitude flag indicating whether section data should be saved
C
C Time-series data is saved using unit number IUATS. The following
C real values are saved (in order):
C SPEV1, SPEV2, PIEEV, EKEEV, UEQEV, UXTSEV, UXTNEV,
C V01DEV, V02DEV, V03DEV, T00EV, T00SEV, T00DEV,
C SST00E, SST01E, SST02E, LST00E, LST01E, LST02E,
C TR00E, TR00SE, TR00DE (for tracer 1)
C TR00E, TR00SE, TR00DE (for tracer 2)
C ...
C The time-series file is in direct-access IEEE format. Each record
C contains the values of the above quantities averaged over a set of
C NTFREQ successive samples. The first 16 bytes of each record contain
C a character string consisting of the run name (5 bytes), the
C diagnostic data-set number (4 bytes, 0000..9999), the two characters
C "TS" followed by the time-series sample number (00001..99999). This
C character string is followed by at least 21 values of IEEE
C single-precision data. The first value is the number of floating
C point numbers that follow (itself a floating point number, >=20.0).
C The next value is the model time, when the sample was saved, in days.
C The next 19 values contain SPEV1, ..., LST02E. This is followed by
C a triplet of values for each tracer present. The values are TR00E,
C TR00SE, TR00DE.
C (Total record length= 16 + LENWRD*(21 + 3*NTRC) bytes)
C (Note: The very first sample of the run, at time-step 0, will be
C        numbered 00000 )
C
C
C For the levels designated using SAVEL, section data is saved using
C unit number IUAHS. The physical space representations of the five
C fields (T,u,v,omega,EPV) are saved for each sample at the designated
C level. (EPV denotes Ertel's potential vorticity). If any tracers are
C present, their distribution is also saved. The section file is in
C direct-access IEEE format. Each record contains the value of one
C quantity at a particular level. Successive records contain values
C of different quantities at the same level, ordered T,u,v,omega,EPV,
C tracer1,tracer2,... The levels are ordered from top to bottom.
C The first 48 bytes of each record contain a character string
C consisting of a comment (16 bytes), the date ("yyyymmdd"),
C time of day ("hhmm"), level ("??????mb"), and the quantity
C descriptor string (12 bytes). This character string is followed by
C four values of IEEE single-precision data. The first value is
C area-weighted section mean of the quantity. The next two values
C contain K1, K2, respectively, but as floating point numbers.
C The fourth value is unused at the moment. This is followed by K1*K2
C values of IEEE single-precision data, each representing the
C deviations from the global-mean of the quantity on the gaussian grid
C at that level. The longitude varies fastest, ordered West to East,
C starting from the Greenwich meridian. The latitude circles are ordered
C from South to North.
C (Note: The first five characters of the comment contain the run name,
C        and the model year numbers range from yyyy=0000 to 9999)
C
C The following quantity descriptor strings are used:
C "T (K)", "U (M/S)", "V (M/S)", "OMEGA (PA/S)", "EPV (SIU)",
C "Q (KG/KG)", ...
C corresponding to the Temperature (in K), the zonal component of
C velocity (in m/s), the meridional component of velocity (in m/s),
C vertical "pressure" velocity (omega, in Pa/s),
C Ertel's PV (in SI units), water vapour mass mixing ratio (in kg/kg),
C ...
C (Total record length= 48 + LENWRD*(4 + K1*K2) bytes)
C
C If the surface save flag SAVSRF is set, the following surface are also
C saved in the horizontal section file, with the 8-character level
C descriptor string set to "SURFACE ":
C "TSURF (K)", "PRECP KG/SM2", "EVAP KG/SM2", "FSURF (W/M2)",
C "TAUX (PA)", "TAUY (PA)", and "ICEHT (M)",
C corrsponding to the surface temperature (in K), precipitation
C (in kg/(s*m**2)), evaporation (in kg/(s*m**2)), heat flux into
C the surface (in W/m**2), the zonal and the meridional components of
C the surface wind stress (in Pa) and ice height (in metres).
C NOTE: The surface fields will not be available at the very beginning
C of the run (i.e. at time-step 0)
C
C For the latitudes designated using SAVEK, section data is saved using
C unit number IUAVS. The physical space representations of the four
C fields (T,u,v,omega) are saved for each sample at the designated
C latitude. No tracers distributions are saved.
C The vertical section file is in direct-access IEEE format. Each record
C contains the value of one quantity at a particular latitude circle.
C Successive records contain values of different quantities at the same
C latitude, ordered T,u,v, and omega. The latitudes are ordered from
C South to North. The first 48 bytes of each record contain a character
C string consisting of a comment (16 bytes), the date ("yyyymmdd"),
C time of day ("hhmm"), latitude degrees ("???.?deg"),
C and the quantity descriptor string (12 bytes). This character string
C is followed by four values of IEEE single-precision data. The first
C value is mass-weighted section mean of the quantity. The next two
C values contain K1, L1, respectively, but as floating point
C numbers. The fourth value is unused at the moment. This is followed
C by K1*L1 values of IEEE single-precision data, each representing the
C deviations from the global-mean of the quantity on vertical section
C at the latitude circle. The longitude varies fastest, ordered West
C to East, starting from the Greenwich meridian. The levels are ordered
C from top to bottom.
C
C The following quantity descriptor strings are used:
C "T (K)", "U (M/S)", "V (M/S)", and "OMEGA (PA/S)".
C corresponding to the Temperature (in K), the zonal component of
C velocity (in m/s), the meridional component of velocity (in m/s),
C and vertical "pressure" velocity (omega, in Pa/s).
C (Total record length= 48 + LENWRD*(4 + K1*L1) bytes)
C
C If IUAZS > 0, then unit number IUAZS is used to save zonal-mean
C section data. The physical space zonal-mean representations of the
C five fields (Theta,u,v,omega,absolute-vorticity) are saved for each
C sample. If any tracers are present, their zonal-mean distribution is
C also saved. The zonal-mean section file is in direct-access IEEE
C format. Each record contains the zonal-mean value of one quantity.
C Successive records contain zonal-mean values of different quantities,
C ordered Theta, u, v, omega, abs-vor, tracer1, tracer2, ... .
C The first 48 bytes of each record contain a character
C string consisting of a comment (16 bytes), the date ("yyyymmdd"),
C time of day ("hhmm"), the string ("ZONMEAN "),
C and the quantity descriptor string (12 bytes). This character string
C is followed by four values of IEEE single-precision data. The first
C value is mass-weighted section mean of the quantity. The next two
C values contain K2, L1, respectively, but as floating point
C numbers. The fourth value is unused at the moment. This is followed
C by K2*L1 values of IEEE single-precision data, each representing the
C deviations from the global-mean of the quantity on vertical section
C at the latitude circle. The latitude varies fastest, ordered South
C to North. The levels are ordered from top to bottom.
C
C The following quantity descriptor strings are used:
C "THETA (K)", "U (M/S)", "V (M/S)", "OMEGA (PA/S)", "ABSVOR (1/S)"
C "Q (KG/KG)", ...
C corresponding to the pot. temperature (in K), the zonal component of
C velocity (in m/s), the meridional component of velocity (in m/s),
C vertical "pressure" velocity (omega, in Pa/s), absolute vorticity
C (in 1/s), water vapour mass mixing ratio (in kg/kg), ...
C (Total record length= 48 + LENWRD*(4 + K2*L1) bytes)
C
C
C PGQAVG= time-averaged spectral coefficients of all prognostic
C         quantities
C
C Note: The following quantities are used for special diagnostics, or
C       for saving section data
C ULA   = zonal comp. of velocity at current level (in latitude space)
C VLA   = meridional comp. of velocity at current level (in lat. space)
C PTLA  = pot. temperature at current level (in latitude space)
C WLA   = omega (Dp/Dt) at current level (in latitude space)
C PVORPH= Ertel's pot. vorticity at current level (in physical space)
C DUDPL = zonal-mean vertical shear (du/dp) at current level
C DTDPL = zonal-mean dTheta/dp at current level
C AVORL = zonal-mean absolute vorticity at current level
C
C Note: The following are instantaneous diagnostic quantities
C
C VELZON= lat-pressure cross-section of zonal-mean velocity
C POTZON= lat-pressure cross-section of zonal-mean pot. temperature
C WZON  = lat-pressure(L+1/2) cross-section of zonal-mean omega
C KEZON = lat-pressure cross-section of kinetic energy
C PTFLUX= lat-press. cross-section of zonal-mean pot. tem. flux
C PSIIC = streamfunction of induced circulation
C AMFLUX= lat-press. cross-section of angular momentum flux
C EAMFLX= lat-press. cross-section of eddy angular momentum flux
C EPFLUX= lat-press. cross-section of Eliassen-Palm flux
C EFDIV = lat-press. cross-section of E-P flux divergence
C PVZON = lat-press. cross-section of Ertel's potential vorticity
C EPSPEC= m-spectrum of E-P flux at level LDIAG
C KESPEC= m-spectrum of global kinetic energy
C KEPOW = n-spectrum of global kinetic energy
C ENPOW = n-spectrum of global enstrophy
C EAFDIV= lat-press. cross-section of eddy ang-mom flux divergence
C EMMCT = lat-press. cross-section of ang-mom tendency due to
C         Eulerian mean meridional advection
C UFORC = lat-press. cross-section of ang-mom forcing tendency
C TFORC = lat-press. cross-section of Theta forcing tendency
C KEDISS= lat-press. cross-section of KE dissipation rate
C
C KEAVG, PTFAVG, PICAVG, AMFAVG, EAMFAV, EPFAVG, EFDAVG, PVAVG, EPSAVG,
C KESAVG, KEPAVG, ENPAVG, EAFDAV, EMTAVG, UFOAVG, TFOAVG, KEDAVG are
C the respective time-averages of "quadratic" diagnostic quantities over
C the sampling period.
C
C AVZON = lat-pressure cross-section of time-averaged zonal-mean
C         absolute vorticity
C AVGZON= lat-pressure cross-section of meridional gradient of
C         time-averaged zonal-mean absolute vorticity
C WRZON = lat-pressure(L+1/2) cross-section of residual omega
C HDIFFT= lat-press. cross-section of ang-mom tendency due to
C         horizontal hyper-diffusion
C ANGMT0= lat-press. cross-section of zonal mean ang-mom for sample 1
C ANGMOT= lat-press. cross-section of zonal mean ang-mom tendency
C TINIT = lat-press. cross-section of zonal mean T for sample 1
C TTEND = lat-press. cross-section of zonal mean T tendency
C SPSECT= special lat-pressure cross-section (for future use)
C
C The following are horizontal fields at level LDIAG
C VELFLD= velocity (m/s)
C TFLD  = temperature (K)
C
C The following are time-averaged fields at level LDIAG
C KEFAVG = kinetic energy (m**2 / s**2)
C TVAR  = temperature variance (K**2)
C
C (The following are instantaneous values at sampling time)
C PIE   = globally averaged potential+internal energy
C         (with reference to standard vertical stratification)
C EKE   = globally averaged eddy kinetic energy
C UEQ   = diagnostic-level u-bar at equator
C UXTSH = diagnostic-level u-bar averaged between 60S and 30S
C UXTNH = diagnostic-level u-bar averaged between 30N and 60N
C VOR01D= diagnostic-level relative vorticity coefficient of Y[0,1]
C VOR02D= diagnostic-level relative vorticity coefficient of Y[0,2]
C VOR03D= diagnostic-level relative vorticity coefficient of Y[0,3]
C T00   = globally averaged temperature
C T00S  = lowest-level temperature, horizontally averaged
C T00D  = diagnostic-level temperature, horizontally averaged
C SST00 = Sea surface temperature (SST), horizontally averaged
C SST01 = Y[0,1] coefficient of zonally-averaged SST
C SST02 = Y[0,2] coefficient of zonally-averaged SST
C LST00 = Land surface temperature (LST), horizontally averaged
C LST01 = Y[0,1] coefficient of zonally-averaged LST
C LST02 = Y[0,2] coefficient of zonally-averaged LST
C VOR12D= diagnostic-level relative vorticity coefficient of Y[1,2]
C
C (PIEEV, ..., LST02E are the cumulative values of PIE, ..., LST02)
C NEV   = no. of cumulative samples so far for time-series (<=NTFREQ)
C
C Tracers
C TRCZON= lat-pressure cross-section of zonal-mean tracer mixing ratio
C TRFLUX= lat-pressure cross-section of zonal-mean tracer flux
C TRCFLD= tracer mixing ratio at level LDIAG
C TRCVAR= transient variance of tracer mixing ratio at level LDIAG
C TRC00 = globally averaged tracer mixing ratio
C TRC00S= lowest-level tracer mixing ratio, horizontally averaged
C TRC00D= diagnostic-level tracer mixing ratio, horizontally averaged
C (TR00E, TR00SE, TR00DE are the cumulative values of TRC00, TRC00S,
C  TRC00D)
C TRINIT= lat-press. cross-section of zonal mean tracer for sample 1
C TRTEND= lat-press. cross-section of zonal mean tracer tendency
C (TRFAVG is the time-averaged zonal-mean tracer flux)
C
C The following are zonal-mean quantities (all time-averages)
C SOLZON= downward SW radiation at the top of the atmosphere (W/m**2)
C ALBZON= planetary albedo (0 ... 1)
C OSRZON= upward SW radiation at the top of the atmosphere (W/m**2)
C OLRZON= upward LW radiation at the top of the atmosphere (W/m**2)
C ISRZON= net downward SW radiation at the bottom of the atmosphere
C ILRZON= net downward LW radiation at the bottom of the atmosphere
C FEVPZN= latent heat flux (into atmosphere, W/m**2)
C FSNSZN= sensible heat flux (into atmosphere, W/m**2)
C FRADZN= radiative heat flux (into atmosphere, W/m**2)
C FNETZN= net heat flux into land/ocean surface (W/m**2)
C MSAPZN= precipitation(due to moist static adjustment, kg/(s*m**2))
C MCAPZN= precipitation(due to moist convective adjustment, kg/(s*m**2))
C EVAPZN= evaporation (in kg/(s*m**2))
C MERHT = atmospheric meridional heat transport (in Watts)
C TAUOCN= zonal wind stress over ocean (in Pa)
C FOCN  = net heat flux into the ocean (excludes land areas, W/m**2)
C EMPOCN= net freshwater flux out of ocean (E-P, in kg/(s*m**2))
C EMPLAN= net freshwater flux out of land (E-P, in kg/(s*m**2))
C SSTZON= sea surface temperature
C LSTZON= land surface temperature
C
C The following are zonally and time-averaged latitude-pressure sections
C SWHAVG= short-wave heating rate
C LWHAVG= long-wave heating rate
C MSAHAV= moist static adjustment heating rate
C MCAHAV= moist convective adjustment heating rate
C DCAHAV= dry convective adjustment heating rate
C
C The following are horizontal fields at the surface (time-averaged)
C TSRFAV= surface temperature (K)
C ALBAVG= planetary albedo (0..1)
C PCPAVG= precipitation (kg/(s*m**2))
C EVPAVG= evaporation (kg/(s*m**2))
C FNETAV= net downward heat flux (W/m**2)
C OLRAVG= outgoing longwave radiation (W/m**2)
C TAUXAV= surface wind stress in the zonal direction (Pa)
C TAUYAV= surface wind stress in the meridional direction (Pa)
C ICEAVG= ice height (m)
C SPFLD1= special field 1 (for future use)
C SPFLD2= special field 2 (for future use)
C
C Run details
C RUNTTL= title of run
C DAYTM0= string containing date and time when run was initialized
C PREFIX= file-name prefix for diagnostic output
C         (Constructed from the first five characters of RUNTTL)
C
C ---------------------- End of description -------------------------
C
 
      SUBROUTINE DIAINI( ISET, DTIME, NSPDAY, NDPYR, IBEGIN,
     1                   NDASKP, NTSFRQ, NSCFRQ,
     2                   IDAFIL, ITSFIL, IHSFIL, IVSFIL, IZSFIL,
     3                   LSPECL, SAVFST, SFSAV, LEVSAV, LATSAV,
     4                   YPHASE, LANWID, TITLE, INITIM )
C Initializes diagnostic operations
C Input:  INTEGER ISET -- diagnostic data-set count (0,...,9999)
C         REAL DTIME -- time-step (in sec)
C         INTEGER NSPDAY -- no. of time-steps per model day
C         INTEGER NDPYR -- no. of days per model year
C         INTEGER IBEGIN -- time-step no. at which to initiate sampling
C         INTEGER NDASKP -- diagnostic sampling interval (in time-steps)
C         INTEGER NTSFRQ -- Time-series save frequency
C                           (in units of NDASKP time-steps)
C         INTEGER NSCFRQ -- section data sampling interval
C                           (in units of NDASKP time-steps)
C         INTEGER IDAFIL -- unit number for diagnostics file
C         INTEGER ITSFIL -- unit number for time-series file
C         INTEGER IHSFIL -- unit number for horizontal section file
C         INTEGER IVSFIL -- unit number for vertical section file
C         INTEGER IZSFIL -- unit number for zonal-mean section file
C      (Specify IDAFIL/ITSFIL/IHSFIL/IVSFIL/IZSFIL=0 to suppress output)
C         INTEGER LSPECL   -- level no. for special diagnostics
C         LOGICAL SAVFST -- if .TRUE., save first sample
C                           if .FALSE., skip first sample
C         (SAVFST affects time-series and section data output)
C         LOGICAL SFSAV -- surface flag for saving section data
C         LOGICAL LEVSAV(L1) -- level flag for saving section data
C         LOGICAL LATSAV(K2) -- latitude flag for saving section data
C         REAL YPHASE -- phase of year at beginning of run (time-step 0)
C                        (YPHASE=0..1, 0 => Mar 21, 0.75 => Dec 22)
C         REAL LANWID -- fractional width of land-sector (0..1)
C         CHARACTER*(*) TITLE -- title of run
C         CHARACTER*(*) INITIM -- run initialization date and time
C Note: The first five characters of TITLE are used to construct a
C       prefix for diagnostic files. i.e. Diagnostic output is stored
C       in <prefix>.da, where <prefix> consists of the first five
C       characters of TITLE followed by four digits (0000 .. 9999)
C       denoting the data-set count ISET.
C       Time-series data is stored in a file named <prefix>.ts.
C       Horizontal section data is stored in a file named <prefix>.hs
C       Vertical section data is stored in a file named <prefix>.vs
C       Zonal-mean section data is stored in a file named <prefix>.zs
C Output: (Initializes common module diacom.h)
C
#include "mgrid.h"
#include "diacom.h"
      REAL DTIME, YPHASE, LANWID
      INTEGER ISET, NSPDAY, NDPYR, IBEGIN, NDASKP, NTSFRQ, NSCFRQ
      INTEGER IDAFIL, ITSFIL, IHSFIL, IVSFIL, IZSFIL, LSPECL
      LOGICAL SAVFST, SFSAV, LEVSAV(L1), LATSAV(K2)
      CHARACTER*(*) TITLE, INITIM
C
      INTEGER I, J, K, L, M, N, IPGQ, LENREC, NSPYR
      CHARACTER TEMST5*5, FILNAM*12
#if (NTRACE > 0)
      INTEGER ITR
#endif
C
      IDSET= ISET
      DTSTEP= DTIME
      NTSPD= NSPDAY
      NYRDAY= NDPYR
      IDSTEP= IBEGIN
      NDSKIP= MAX0(1,NDASKP)
      NTFREQ= MAX0(1,NTSFRQ)
      NSFREQ= MAX0(1,NSCFRQ)
      IUADG= IDAFIL
      IUATS= ITSFIL
      IUAHS= IHSFIL
      IUAVS= IVSFIL
      IUAZS= IZSFIL
      LDIAG= LSPECL
      YRPHA= YPHASE
      LANWD= LANWID
      RUNTTL= TITLE
      DAYTM0= INITIM
C     Compute no. of time-steps per year
      IF (NTSPD .GT. 0) THEN
        NSPYR= NYRDAY * NTSPD
      ELSE
        NSPYR= NYRDAY/(-NTSPD)
      ENDIF
      IF ((NDSKIP*NSFREQ .GT. 0).AND.(NDSKIP*NSFREQ .LT. NSPYR)) THEN
        IF (MOD(NSPYR,NDSKIP*NSFREQ) .NE. 0) THEN
          WRITE(*,*) 'DIAINI: Error - invalid section save interval'
          STOP 'DIAINI1'
        ENDIF
      ENDIF
C     Determine whether to skip the first sample for save operations
      ISSTEP= IDSTEP
      IF (.NOT.SAVFST) ISSTEP= ISSTEP + NDSKIP
C     Determine file prefix to be used for diagnostic/section output
      TEMST5= RUNTTL(1:5)
      IF (TEMST5.EQ.' ') THEN
        TEMST5='tmp00'
      ELSE
        CALL RJUST( TEMST5 )
      ENDIF
      WRITE(PREFIX,'(4X,I5)') 10000+IDSET
      PREFIX(1:5)= TEMST5
C
C     Copy surface save flag
      SAVSRF= SFSAV
C     Copy level save flags
      DO 10 L=1,L1
   10   SAVEL(L)= LEVSAV(L)
C     Copy latitude save flags
      DO 20 K=1,K2
   20   SAVEK(K)= LATSAV(K)
C     Zero save data accumulators
      CALL SAVZSF
      CALL SAVZRO
C
      IF (IUATS .GT. 0) THEN
C       Open time-series file
        FILNAM= PREFIX//'.ts'
        CALL LJUST( FILNAM )
        LENREC= 16 + LENWRD*(21 + 3*(NPGQ-3))
        OPEN(IUATS, FILE=FILNAM, FORM='UNFORMATTED', STATUS='NEW',
     1              ACCESS='DIRECT', RECL=(LENREC+RECUNIT-1)/RECUNIT )
        WRITE(*,*) 'Saving time-series data to ', FILNAM
C       Zero record count
        NRECTS= 0
C       Initialize time-series accumulators
        CALL TSINI
      ENDIF
      IF (IUAHS .GT. 0) THEN
C       Open horizontal section file
        FILNAM= PREFIX//'.hs'
        CALL LJUST( FILNAM )
        LENREC= 48 + LENWRD*(4 + K1*K2)
        OPEN(IUAHS, FILE=FILNAM, FORM='UNFORMATTED', STATUS='NEW',
     1              ACCESS='DIRECT', RECL=(LENREC+RECUNIT-1)/RECUNIT )
        WRITE(*,*) 'Saving horizontal section data to ', FILNAM
C       Zero record count
        NRECHS= 0
      ENDIF
      IF (IUAVS .GT. 0) THEN
C       Open vertical section file
        FILNAM= PREFIX//'.vs'
        CALL LJUST( FILNAM )
        LENREC= 48 + LENWRD*(4 + K1*L1)
        OPEN(IUAVS, FILE=FILNAM, FORM='UNFORMATTED', STATUS='NEW',
     1              ACCESS='DIRECT', RECL=(LENREC+RECUNIT-1)/RECUNIT )
        WRITE(*,*) 'Saving vertical section data to ', FILNAM
C       Zero record count
        NRECVS= 0
      ENDIF
      IF (IUAZS .GT. 0) THEN
C       Open zonal-mean section file
        FILNAM= PREFIX//'.zs'
        CALL LJUST( FILNAM )
        LENREC= 48 + LENWRD*(4 + K2*L1)
        OPEN(IUAZS, FILE=FILNAM, FORM='UNFORMATTED', STATUS='NEW',
     1              ACCESS='DIRECT', RECL=(LENREC+RECUNIT-1)/RECUNIT )
        WRITE(*,*) 'Saving zonal-mean section data to ', FILNAM
C       Zero record count
        NRECZS= 0
      ENDIF
C
C     Initialize sample counts
      NSAMPL= 0
      NTSAMP= 0
      NSSAMP= 0
C     Initialize accumulators for calculating time-averages
      DO 100 L=1,L1
        DO 100 IPGQ=1,NPGQ
  100     CALL ZEROSP( PGQAVG(0, 0, IPGQ, L), N1 )
C
      DO 110 I=1,2
        DO 110 L=1,L1
          DO 110 K=1,K2
            AMFAVG(K,L,I)= 0.0
            EAMFAV(K,L,I)= 0.0
            EPFAVG(K,L,I)= 0.0
  110       CONTINUE
C
      DO 120 L=1,L1
       DO 120 K=1,K2
         KEAVG(K,L)=  0.0
         PTFAVG(K,L)= 0.0
         PICAVG(K,L)= 0.0
         EMTAVG(K,L)= 0.0
         PVAVG(K,L)= 0.0
         SPSECT(K,L)= 0.0
  120    CONTINUE
C
      DO 130 I=1,2
        DO 130 K=1,K2
          DO 130 M=0,M1
            EPSAVG(M,K,I)= 0.0
  130       CONTINUE
C
      DO 140 M=0,M1
  140   KESAVG(M)= 0.0
C
      DO 150 N=0,N1
        KEPAVG(N)= 0.0
  150   ENPAVG(N)= 0.0
C
      DO 170 L=1,L1
        DO 170 K=1,K2
          UFOAVG(K,L)= 0.0
          TFOAVG(K,L)= 0.0
  170     KEDAVG(K,L)= 0.0
C
      DO 180 K=1,K2
        DO 180 J=1,K1
          TVAR(J,K)= 0.0
  180     KEFAVG(J,K)= 0.0
C
#if (NTRACE > 0)
      DO 190 ITR=JTR1,NPGQ
        DO 190 L=1,L1
          DO 190 K=1,K2
  190       TRFAVG(K,L,ITR)= 0.0
      DO 200 ITR=JTR1,NPGQ
        DO 200 K=1,K2
          DO 200 J=1,K1
  200       TRCVAR(J,K,ITR)= 0.0
#endif
C
C
      DO 210 K=1,K2
        SOLZON(K)= 0.0
        ALBZON(K)= 0.0
        OSRZON(K)= 0.0
        OLRZON(K)= 0.0
        ISRZON(K)= 0.0
        ILRZON(K)= 0.0
        FEVPZN(K)= 0.0
        FSNSZN(K)= 0.0
        FRADZN(K)= 0.0
        FNETZN(K)= 0.0
        MSAPZN(K)= 0.0
        MCAPZN(K)= 0.0
        EVAPZN(K)= 0.0
        MERHT(K)= 0.0
        TAUOCN(K)= 0.0
        FOCN(K)= 0.0
        EMPOCN(K)= 0.0
        EMPLAN(K)= 0.0
        SSTZON(K)= 0.0
        LSTZON(K)= 0.0
  210   CONTINUE
C
      DO 220 K=1,K2
        DO 220 L=1,L1
          SWHAVG(K,L)= 0.0
          LWHAVG(K,L)= 0.0
          MSAHAV(K,L)= 0.0
          MCAHAV(K,L)= 0.0
  220     DCAHAV(K,L)= 0.0
C
      DO 230 K=1,K2
        DO 230 J=1,K1
          TSRFAV(J,K)= 0.0
          ALBAVG(J,K)= 0.0
          PCPAVG(J,K)= 0.0
          EVPAVG(J,K)= 0.0
          FNETAV(J,K)= 0.0
          OLRAVG(J,K)= 0.0
          TAUXAV(J,K)= 0.0
          TAUYAV(J,K)= 0.0
          ICEAVG(J,K)= 0.0
          SPFLD1(J,K)= 0.0
  230     SPFLD2(J,K)= 0.0
      RETURN
      END
 
      SUBROUTINE SAVZSF
C Zero surface data save accumulators
#include "mgrid.h"
#include "diacom.h"
      INTEGER J, K
C
C     Zero surface field save sample count
      NSFSAV= 0
C     Initialize accumulators for save data
      DO 10 K=1,K2
        DO 10 J=1,K1
          TSFSV(J,K)= 0.0
          PCPSV(J,K)= 0.0
          EVAPSV(J,K)= 0.0
          FSFSV(J,K)= 0.0
          TAUXSV(J,K)= 0.0
          TAUYSV(J,K)= 0.0
          ICESV(J,K)= 0.0
          OLRSV(J,K)= 0.0
          ASRSV(J,K)= 0.0
   10     CONTINUE
      RETURN
      END
 
      SUBROUTINE SAVZRO
C Zero horizontal/vertical section data save accumulators
#include "mgrid.h"
#include "diacom.h"
      INTEGER ITR, I, J, K, L
C     Zero horizontal/vertical section save sample count
      NSSAV= 0
C     Initialize accumulators for save data
      DO 5 K=1,K2
        DO 5 J=1,K1
              UTHS(J,K)= 0.0
              VTHS(J,K)= 0.0
              UQHS(J,K)= 0.0
              VQHS(J,K)= 0.0
              UZ2HS(J,K)= 0.0
              VZ2HS(J,K)= 0.0
              UZ1HS(J,K)= 0.0
              VZ1HS(J,K)= 0.0
              KEHS(J,K)= 0.0
   5      CONTINUE
      DO 10 I=1,NLEVMX
        DO 10 K=1,K2
          DO 10 J=1,K1
            UHS(J,K,I)= 0.0
            VHS(J,K,I)= 0.0
            WHS(J,K,I)= 0.0
            PVHS(J,K,I)= 0.0
   10       AVHS(J,K,I)= 0.0
      DO 20 ITR=JPOT,NPGQ
        DO 20 I=1,NLEVMX
          DO 20 K=1,K2
            DO 20 J=1,K1
   20         SCALHS(J,K,I,ITR)= 0.0
      DO 30 I=1,NLATMX
        DO 30 L=1,L1
          DO 30 J=1,K1
            TVS(J,L,I)= 0.0
            UVS(J,L,I)= 0.0
            VVS(J,L,I)= 0.0
   30       WVS(J,L,I)= 0.0
      DO 40 L=1,L1
        DO 40 K=1,K2
          UZS(K,L)= 0.0
          VZS(K,L)= 0.0
          WZS(K,L)= 0.0
   40     AVZS(K,L)= 0.0
      DO 50 ITR=JPOT,NPGQ
        DO 50 L=1,L1
          DO 50 K=1,K2
   50     SCALZS(K,L,ITR)= 0.0
      RETURN
      END
 
      SUBROUTINE TSINI
C Initialize time-series accumulators
#include "mgrid.h"
#include "diacom.h"
#if (NTRACE > 0)
      INTEGER ITR
#endif
      NEV= 0
      SPEV1= 0.0
      SPEV2= 0.0
      PIEEV= 0.0
      EKEEV= 0.0
      UEQEV= 0.0
      UXTSEV= 0.0
      UXTNEV= 0.0
      V01DEV= 0.0
      V02DEV= 0.0
      V03DEV= 0.0
      T00EV= 0.0
      T00SEV= 0.0
      T00DEV= 0.0
      SST00E= 0.0
      SST01E= 0.0
      SST02E= 0.0
      LST00E= 0.0
      LST01E= 0.0
      LST02E= 0.0
#if (NTRACE > 0)
      DO 100 ITR=JTR1,NPGQ
        TR00E(ITR)= 0.0
        TR00SE(ITR)= 0.0
  100   TR00DE(ITR)= 0.0
#endif
      RETURN
      END
 
      SUBROUTINE TSACC
C Accumulate time-series quantities
#include "mgrid.h"
#include "diacom.h"
#if (NTRACE > 0)
      INTEGER ITR
#endif
      NEV= NEV + 1
      SPEV1= SPEV1 + REAL(VOR12D)
      SPEV2= SPEV2 + AIMAG(VOR12D)
      PIEEV= PIEEV + PIE
      EKEEV= EKEEV + EKE
      UEQEV= UEQEV + UEQ
      UXTSEV= UXTSEV + UXTSH
      UXTNEV= UXTNEV + UXTNH
      V01DEV= V01DEV + VOR01D
      V02DEV= V02DEV + VOR02D
      V03DEV= V03DEV + VOR03D
      T00EV= T00EV + T00
      T00SEV= T00SEV + T00S
      T00DEV= T00DEV + T00D
      SST00E= SST00E + SST00
      SST01E= SST01E + SST01
      SST02E= SST02E + SST02
      LST00E= LST00E + LST00
      LST01E= LST01E + LST01
      LST02E= LST02E + LST02
#if (NTRACE > 0)
      DO 100 ITR=JTR1,NPGQ
        TR00E(ITR)= TR00E(ITR) + TRC00(ITR)
        TR00SE(ITR)= TR00SE(ITR) + TRC00S(ITR)
  100   TR00DE(ITR)= TR00DE(ITR) + TRC00D(ITR)
#endif
      RETURN
      END
 
      SUBROUTINE TSWRIT( ISTP )
C Write out time-series quantities.
C ISTP is the current time-step number
#include "mgrid.h"
#include "diacom.h"
      INTEGER ISTP
C
      REAL TAUEV, EVQTY
      CHARACTER TSSTR*16
#if (NTRACE > 0)
      INTEGER ITR
#endif
C
      SPEV1= SPEV1 / NEV
      SPEV2= SPEV2 / NEV
      PIEEV= PIEEV / NEV
      EKEEV= EKEEV / NEV
      UEQEV= UEQEV / NEV
      UXTSEV= UXTSEV / NEV
      UXTNEV= UXTNEV / NEV
      V01DEV= V01DEV / NEV
      V02DEV= V02DEV / NEV
      V03DEV= V03DEV / NEV
      T00EV= T00EV / NEV
      T00SEV= T00SEV / NEV
      T00DEV= T00DEV / NEV
      SST00E= SST00E / NEV
      SST01E= SST01E / NEV
      SST02E= SST02E / NEV
      LST00E= LST00E / NEV
      LST01E= LST01E / NEV
      LST02E= LST02E / NEV
#if (NTRACE > 0)
      DO 100 ITR=JTR1,NPGQ
        TR00E(ITR)= TR00E(ITR) / NEV
        TR00SE(ITR)= TR00SE(ITR) / NEV
  100   TR00DE(ITR)= TR00DE(ITR) / NEV
#endif
      WRITE(TSSTR,'(10X,I6)') 100000+((ISTP-IDSTEP)/(NDSKIP*NTFREQ))
      TSSTR(1:11)= PREFIX//'TS'
C     Current time in days
      IF (NTSPD .GT. 0) THEN
        TAUEV= DBLE(ISTP)/DBLE(NTSPD)
      ELSE
        TAUEV= ISTP*(-NTSPD)
      ENDIF
C
      EVQTY= 20.0 + (NPGQ-3)*3.0
C     Increment sample count
      NTSAMP= NTSAMP + 1
C     Write record in time-series file
      NRECTS= NRECTS + 1
#if (NTRACE > 0)
      WRITE(IUATS,REC=NRECTS) TSSTR, EVQTY, TAUEV,
     1  SPEV1, SPEV2, PIEEV, EKEEV, UEQEV, UXTSEV, UXTNEV,
     2  V01DEV, V02DEV, V03DEV, T00EV, T00SEV, T00DEV,
     3  SST00E, SST01E, SST02E, LST00E, LST01E, LST02E,
     4  (TR00E(ITR), TR00SE(ITR), TR00DE(ITR), ITR=JTR1,NPGQ)
#else
      WRITE(IUATS,REC=NRECTS) TSSTR, EVQTY, TAUEV,
     1  SPEV1, SPEV2, PIEEV, EKEEV, UEQEV, UXTSEV, UXTNEV,
     2  V01DEV, V02DEV, V03DEV, T00EV, T00SEV, T00DEV,
     3  SST00E, SST01E, SST02E, LST00E, LST01E, LST02E
#endif
      RETURN
      END
 
      SUBROUTINE SAVHDR( ISTP, COMNT, MDATE )
C SAVHDR creates the comment string COMNT and the date string MDATE
C for use in save files, given the model time-step count ISTP.
C
#include "mgrid.h"
#include "diacom.h"
      INTEGER ISTP
      CHARACTER COMNT*11, MDATE*13
C
      INTEGER IYEAR, IMONTH, IDAY, IHH, IMM, ITEMP, NDPMON
      REAL TSPHR, TSPMN
      CHARACTER YRSTR*5
C
C     Run comment
      WRITE(COMNT,'(A5,A1,I4,A1)') PREFIX, '/', 1000+N1, '/'
      COMNT(7:7)='T'
C     No. of days per month (arbitrary)
      NDPMON= NYRDAY/12
C     Create date/time string
      IF (NTSPD .GT. 0) THEN
C       No. of time-steps per hour, per minute
        TSPHR= NTSPD/24.0
        TSPMN= TSPHR/60.0
C
        ITEMP= ISTP
        IYEAR= ITEMP/(NTSPD*NYRDAY)
        ITEMP= MOD(ITEMP,NTSPD*NYRDAY)
        IMONTH= 1 + ITEMP/(NTSPD*NDPMON)
        ITEMP= MOD(ITEMP,NTSPD*NDPMON)
        IDAY= 1 + ITEMP/NTSPD
        ITEMP= MOD(ITEMP,NTSPD)
        IHH= INT(ITEMP/TSPHR)
        ITEMP= NINT(ITEMP - IHH*TSPHR)
        IMM= INT(ITEMP/TSPMN)
      ELSE
C       Time-step specified as days per time-step
        ITEMP= ISTP*(-NTSPD)
        IYEAR= ITEMP/NYRDAY
        ITEMP= MOD(ITEMP,NYRDAY)
        IMONTH= 1 + ITEMP/NDPMON
        ITEMP= MOD(ITEMP,NDPMON)
        IDAY= 1 + ITEMP
        IHH= 0
        IMM= 0
      ENDIF
C     Create mmddhhmm string
      WRITE(MDATE,'(4X,I9)') 100000000 + IMONTH*1000000 + IDAY*10000
     1                                 + IHH*100 + IMM
C     Create year string
      IF (IYEAR .LT. 10000) THEN
        WRITE(YRSTR,'(I5)') 10000 + IYEAR
        YRSTR(1:1)= ' '
      ELSE
        WRITE(YRSTR,'(I5)') IYEAR
      ENDIF
      MDATE(1:5)= YRSTR
      RETURN
      END
 
      SUBROUTINE DIASV0( ISTP, QEVAP, MSAPCP, MCAPCP,
     1                   FEVAP, FSENS, FRAD, FNET, FPSSRF, FMSSRF,
     2                   FPLSRF, FMLSRF, FPSTOP, FPLTOP,
     3                   SWHET, LWHET, MSAHET, MCAHET, DCAHET,
     4                   SEAMSK, ICEMSK, SOLRAD, ALBEDO,
     5                   TSURF, STRESX, STRESY, ICEHT )
C DIASV0 writes out model surface section info
C Input:  INTEGER ISTP -- current model step number
C         REAL QEVAP(K1MAX,K2MAX) -- evaporative flux of water vapour
C                                    (in kg/(s*m**2))
C         REAL MSAPCP(K1MAX,K2MAX) -- precipitation due to moist
C                                static adjustment (in kg/(s*m**2))
C         REAL MCAPCP(K1MAX,K2MAX) -- precipitation due to moist
C                                convective adjustment (in kg/(s*m**2))
C         REAL FEVAP(K1MAX,K2MAX) -- heat flux associated with the
C                                    evaporative flux of water vapour
C                                    (in W/m**2)
C         REAL FSENS(K1MAX,K2MAX) -- upward surface sensible heat flux
C                                    (in W/m**2)
C         REAL FRAD(K1MAX,K2MAX) -- upward surface radiative heat flux
C                                   (in W/m**2)
C         REAL FNET(K1MAX,K2MAX) -- Net heat flux into land/sea
C                                   (in W/m**2)
C         REAL FPSSRF(K1MAX,K2MAX) -- upward SW radiation at surface
C         REAL FMSSRF(K1MAX,K2MAX) -- downward SW radiation at surface
C         REAL FPLSRF(K1MAX,K2MAX) -- upward LW radiation at surface
C         REAL FMLSRF(K1MAX,K2MAX) -- downward LW radiation at surface
C         REAL FPSTOP(K1MAX,K2MAX) -- upward SW radiation at model top
C         REAL FPLTOP(K1MAX,K2MAX) -- upward LW radiation at model top
C         REAL SWHET(K2MAX,L1MAX)  -- zonally-averaged vertical profile
C              of heating due to SW radiation (in K/s)
C         REAL LWHET(K2MAX,L1MAX)  -- zonally-averaged vertical profile
C              of heating due to LW radiation (in K/s)
C         REAL MSAHET(K2MAX,L1MAX) -- zonally-averaged vertical profile
C              of heating due to moist static adjustment (in K/s)
C         REAL MCAHET(K2MAX,L1MAX) -- zonally-averaged vertical profile
C              of heating due to moist convective adjustment (in K/s)
C         REAL DCAHET(K2MAX,L1MAX) -- zonally-averaged vertical profile
C              of heating due to dry convective adjustment (in K/s)
C         LOGICAL SEAMSK(K1MAX,K2MAX) -- sea mask (.TRUE. over sea)
C         LOGICAL ICEMSK(K1MAX,K2MAX) -- ice mask (.TRUE. over ice)
C         REAL SOLRAD(K1MAX,K2MAX) -- incident solar radiation (W/m**2)
C         REAL ALBEDO(K1MAX,K2MAX) -- surface albedo (0 .. 1)
C         REAL TSURF(K1MAX,K2MAX) -- surface temperature (in K)
C         REAL STRESX(K1MAX,K2MAX) -- surface wind stress in the
C               zonal direction (Pa)
C         REAL STRESY(K1MAX,K2MAX) -- surface wind stress in the
C               meridional direction (Pa)
C         REAL ICEHT(K1MAX,K2MAX) -- ice height (m)
C
C Output: (Surface sections are saved, if need be)
C
#include "mgrid.h"
#include "diacom.h"
      INTEGER ISTP
      REAL QEVAP(K1MAX,K2MAX)
      REAL MSAPCP(K1MAX,K2MAX), MCAPCP(K1MAX,K2MAX)
      REAL FEVAP(K1MAX,K2MAX), FSENS(K1MAX,K2MAX)
      REAL FRAD(K1MAX,K2MAX), FNET(K1MAX,K2MAX)
      REAL FPSSRF(K1MAX,K2MAX), FMSSRF(K1MAX,K2MAX)
      REAL FPLSRF(K1MAX,K2MAX), FMLSRF(K1MAX,K2MAX)
      REAL FPSTOP(K1MAX,K2MAX), FPLTOP(K1MAX,K2MAX)
      REAL SWHET(K2MAX,L1MAX), LWHET(K2MAX,L1MAX)
      REAL MSAHET(K2MAX,L1MAX), MCAHET(K2MAX,L1MAX)
      REAL DCAHET(K2MAX,L1MAX), SOLRAD(K1MAX,K2MAX)
      REAL ALBEDO(K1MAX,K2MAX), TSURF(K1MAX,K2MAX)
      REAL STRESX(K1MAX,K2MAX), STRESY(K1MAX,K2MAX)
      REAL ICEHT(K1MAX,K2MAX)
      LOGICAL SEAMSK(K1MAX,K2MAX), ICEMSK(K1MAX,K2MAX)
C
C
      INTEGER J, K
      REAL EMPSAV(K1MAX,K2MAX)
      CHARACTER COMNT*11, MDATE*13
      CHARACTER HEDHED*36, HEADER*48
      LOGICAL FIRST
C
C     Check if section data needs to be saved for this time-step
      IF (.NOT.SAVSRF .OR. (IUAHS.LE.0) .OR. (ISTP.LT.ISSTEP)) RETURN
C     Accumulate samples
      NSFSAV= NSFSAV + 1
      DO 10 K=1,K2
        DO 10 J=1,K1
          TSFSV(J,K)= TSFSV(J,K) + TSURF(J,K)
          PCPSV(J,K)= PCPSV(J,K) + MSAPCP(J,K) + MCAPCP(J,K)
          EVAPSV(J,K)= EVAPSV(J,K) + QEVAP(J,K)
          FSFSV(J,K)= FSFSV(J,K) + FNET(J,K)
          TAUXSV(J,K)= TAUXSV(J,K) + STRESX(J,K)
          TAUYSV(J,K)= TAUYSV(J,K) + STRESY(J,K)
          ICESV(J,K)= ICESV(J,K) + ICEHT(J,K)
          OLRSV(J,K)= OLRSV(J,K) + FPLTOP(J,K)
          ASRSV(J,K)= ASRSV(J,K) + SOLRAD(J,K) - FPSTOP(J,K)
   10     CONTINUE
      IF (MOD((ISTP-IDSTEP), NSFREQ*NDSKIP) .EQ. 0) THEN
C       Write to save file; compute averages
        DO 20 K=1,K2
          DO 20 J=1,K1
            TSFSV(J,K)= TSFSV(J,K) / NSFSAV
            PCPSV(J,K)= PCPSV(J,K) / NSFSAV
            EVAPSV(J,K)= EVAPSV(J,K) / NSFSAV
            EMPSAV(J,K)= EVAPSV(J,K) - PCPSV(J,K)
            FSFSV(J,K)= FSFSV(J,K) / NSFSAV
            TAUXSV(J,K)= TAUXSV(J,K) / NSFSAV
            TAUYSV(J,K)= TAUYSV(J,K) / NSFSAV
            ICESV(J,K)= ICESV(J,K) / NSFSAV
            OLRSV(J,K)= OLRSV(J,K) / NSFSAV
            ASRSV(J,K)= ASRSV(J,K) / NSFSAV
   20       CONTINUE
C
C       Set first sample flag
        FIRST= NSSAMP .LE. 1
C       Get comment/date strings
        CALL SAVHDR( ISTP, COMNT, MDATE )
C
C       Save surface diagnostics
        HEDHED= COMNT//'HSEC'//MDATE//'SURFACE '
#ifdef SAVE_FLX_BOT
C
C       Write surface temperature
        HEADER= HEDHED//'TSURF (K)   '
        NRECHS= NRECHS + 1
        CALL QPUTHS( IUAHS, NRECHS, TSFSV, HEADER )
        IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
C
C       Write precipitation
        HEADER= HEDHED//'PRECP KG/SM2'
        NRECHS= NRECHS + 1
        CALL QPUTHS( IUAHS, NRECHS, PCPSV, HEADER )
        IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
C
C       Write evaporation
        HEADER= HEDHED//'EVAP KG/SM2 '
        NRECHS= NRECHS + 1
        CALL QPUTHS( IUAHS, NRECHS, EVAPSV, HEADER )
        IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
C
C       Write heat flux into surface
        HEADER= HEDHED//'FSURF (W/M2)'
        NRECHS= NRECHS + 1
        CALL QPUTHS( IUAHS, NRECHS, FSFSV, HEADER )
        IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
#endif
#ifdef SAVE_FLX_TOP
C
C       Write outgoing longwave radiation
        HEADER= HEDHED//'OLR (W/M2)  '
        NRECHS= NRECHS + 1
        CALL QPUTHS( IUAHS, NRECHS, OLRSV, HEADER )
        IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
C
C       Write absorbed shortwave radiation
        HEADER= HEDHED//'ASR (W/M2)  '
        NRECHS= NRECHS + 1
        CALL QPUTHS( IUAHS, NRECHS, ASRSV, HEADER )
        IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
#endif
C
C       Reset surface save data accumulators
        CALL SAVZSF
      ENDIF
C
      RETURN
      END
 
      SUBROUTINE DIASAV( ISTP, L, UPH, VPH, WPH, SCALPH, AVPH, PVPH,
     1                   DUDP, DTDP, AVORZ )
C DIASAV writes out model interior section info at level L.
C Input:  INTEGER ISTP -- current time-step number
C         (Note: specify ISTP=-1 for time-mean states)
C         INTEGER L -- Level for which data is being saved
C         REAL UPH(K1MAX,K2MAX), VPH(K1MAX,K2MAX) -- Hor. velocity
C                                  components at level L
C         REAL WPH(K1MAX,K2MAX) -- Omega at level L
C         REAL SCALPH(K1MAX,K2MAX,JPOT:NPGQ) -- scalars at level L
C         REAL AVPH(K1MAX,K2MAX) -- Absolute vorticity at level L
C         REAL PVPH(K1MAX,K2MAX) -- Ertel's PV at level L+1/2
C         REAL DUDP(K2MAX) -- Zonal-mean dU/dp at level L
C         REAL DTDP(K2MAX) -- Zonal-mean dTheta/dp at level L
C         REAL AVORZ(K2MAX) -- Zonal-mean abs. vorticity at level L
C Output: (Section data is saved, if need be)
C
#include "mgrid.h"
#include "diacom.h"
      INTEGER ISTP, L
      REAL UPH(K1MAX,K2MAX), VPH(K1MAX,K2MAX), WPH(K1MAX,K2MAX)
      REAL SCALPH(K1MAX,K2MAX,JPOT:NPGQ), PVPH(K1MAX,K2MAX)
      REAL AVPH(K1MAX,K2MAX)
      REAL DUDP(K2MAX), DTDP(K2MAX), AVORZ(K2MAX)
C
      INTEGER ITR, J, K, L0, NLAT
      REAL TPH(K1MAX,K2MAX), UTEM, VTEM, WTEM, TEMP
      CHARACTER COMNT*11, MDATE*13
      CHARACTER HEDHED*36, LVLHED*8, LATHED*8, HEADER*48
      LOGICAL FIRST
C
      COMPLEX FSP1(0:M1MAX,0:N1MAX), FSP2(0:M1MAX,0:N1MAX)
      REAL  UZHS0(K1MAX,K2MAX), VZHS0(K1MAX,K2MAX)
      REAL  DIVP0(K1MAX,K2MAX),  DIVP(K1MAX,K2MAX)
      REAL  KEHS0(K1MAX,K2MAX)
#ifdef SAVE_EDDY_LO
      REAL  UTHS0(K1MAX,K2MAX), VTHS0(K1MAX,K2MAX)
      REAL  UQHS0(K1MAX,K2MAX), VQHS0(K1MAX,K2MAX)
      REAL DIVUT0(K1MAX,K2MAX), DIVUT(K1MAX,K2MAX)
      REAL DIVUQ0(K1MAX,K2MAX), DIVUQ(K1MAX,K2MAX)
      REAL  VTHSE(K1MAX,K2MAX)
#endif
C
C     Check if section data needs to be saved for this time-step
      IF (     ((IUAHS.LE.0).AND.(IUAVS.LE.0).AND.(IUAZS.LE.0))
     1    .OR. (ISTP.LT.ISSTEP)) RETURN
      IF (L .EQ. 1) THEN
C       Initialize save level count for first level
        NSLEV= 0
C       Accumulate samples
        NSSAV= NSSAV + 1
      ENDIF
      IF ( (IUAHS .GT. 0) .AND. SAVEL(L) ) THEN
C       Increment save level count
        NSLEV= NSLEV + 1
        IF (NSLEV .GT. NLEVMX) STOP 'NLEVMX'
C       Accumulate data for this level
        DO 10 K=1,K2
          DO 10 J=1,K1
            UHS(J,K,NSLEV)= UHS(J,K,NSLEV) + UPH(J,K)
            VHS(J,K,NSLEV)= VHS(J,K,NSLEV) + VPH(J,K)
            WHS(J,K,NSLEV)= WHS(J,K,NSLEV) + WPH(J,K)
            PVHS(J,K,NSLEV)= PVHS(J,K,NSLEV) + PVPH(J,K)
            AVHS(J,K,NSLEV)= AVHS(J,K,NSLEV) + AVPH(J,K)
   10       CONTINUE
        DO 20 ITR=JPOT,NPGQ
          DO 20 K=1,K2
            DO 20 J=1,K1
              SCALHS(J,K,NSLEV,ITR)=  SCALHS(J,K,NSLEV,ITR)
     1                              + SCALPH(J,K,ITR)
   20         CONTINUE
        IF (L .EQ. 1) THEN
C         Accumulate fluxes at 250mb
          DO 22 K=1,K2
            DO 22 J=1,K1
              UZ1HS(J,K)= UZ1HS(J,K) + UPH(J,K)*AVPH(J,K)
              VZ1HS(J,K)= VZ1HS(J,K) + VPH(J,K)*AVPH(J,K)
              KEHS(J,K)= KEHS(J,K) + 0.5*(UPH(J,K)**2 + VPH(J,K)**2)
   22         CONTINUE
        ENDIF
        IF (L .EQ. L1) THEN
C         Accumulate fluxes at 750mb
          DO 26 K=1,K2
            DO 26 J=1,K1
              UTHS(J,K)= UTHS(J,K) + UPH(J,K)*SCALPH(J,K,JPOT)
              VTHS(J,K)= VTHS(J,K) + VPH(J,K)*SCALPH(J,K,JPOT)
              UQHS(J,K)= UQHS(J,K) + UPH(J,K)*SCALPH(J,K,JTR1)
              VQHS(J,K)= VQHS(J,K) + VPH(J,K)*SCALPH(J,K,JTR1)
              UZ2HS(J,K)= UZ2HS(J,K) + UPH(J,K)*AVPH(J,K)
              VZ2HS(J,K)= VZ2HS(J,K) + VPH(J,K)*AVPH(J,K)
   26         CONTINUE
        ENDIF
      ENDIF
      IF (IUAVS .GT. 0) THEN
C       Initialize save latitude count
        NLAT= 0
        DO 40 K=1,K2
          IF (SAVEK(K)) THEN
C           Increment save latitude count
            NLAT= NLAT + 1
            IF (NLAT .GT. NLATMX) STOP 'NLATMX'
C           Accumulate data for latitude at this level
            DO 30 J=1,K1
              TVS(J,L,NLAT)= TVS(J,L,NLAT) + SCALPH(J,K,JPOT)*PKLV(L)
              UVS(J,L,NLAT)= UVS(J,L,NLAT) + UPH(J,K)
              VVS(J,L,NLAT)= VVS(J,L,NLAT) + VPH(J,K)
   30         WVS(J,L,NLAT)= WVS(J,L,NLAT) + WPH(J,K)
          ENDIF
   40     CONTINUE
      ENDIF
      IF (IUAZS .GT. 0) THEN
        DO 60 K=1,K2
C         Accumulate zonal-mean data for latitude at this level
          UTEM= 0.0
          VTEM= 0.0
          WTEM= 0.0
          DO 50 J=1,K1
            UTEM= UTEM + UPH(J,K)
            VTEM= VTEM + VPH(J,K)
   50       WTEM= WTEM + WPH(J,K)
          UZS(K,L)= UZS(K,L) + UTEM/K1
          VZS(K,L)= VZS(K,L) + VTEM/K1
          WZS(K,L)= WZS(K,L) + WTEM/K1
          AVZS(K,L)= AVZS(K,L) + AVORZ(K)
   60     CONTINUE
        DO 80 ITR=JPOT,NPGQ
          DO 80 K=1,K2
            TEMP= 0.0
            DO 70 J=1,K1
   70         TEMP= TEMP + SCALPH(J,K,ITR)
   80       SCALZS(K,L,ITR)= SCALZS(K,L,ITR) + TEMP/K1
      ENDIF
      IF (MOD((ISTP-IDSTEP), NSFREQ*NDSKIP) .EQ. 0) THEN
C       Increment sample count (only at the first level)
        IF (L .EQ. 1) NSSAMP= NSSAMP + 1
C       Set first sample flag
        FIRST= NSSAMP .EQ. 1
C       Get comment/date strings
        CALL SAVHDR( ISTP, COMNT, MDATE )
C
        IF ( (IUAHS .GT. 0) .AND. SAVEL(L) ) THEN
C         Compute averages
          DO 110 K=1,K2
            DO 110 J=1,K1
              UHS(J,K,NSLEV)= UHS(J,K,NSLEV) / NSSAV
              VHS(J,K,NSLEV)= VHS(J,K,NSLEV) / NSSAV
              WHS(J,K,NSLEV)= WHS(J,K,NSLEV) / NSSAV
              PVHS(J,K,NSLEV)= PVHS(J,K,NSLEV) / NSSAV
              AVHS(J,K,NSLEV)= AVHS(J,K,NSLEV) / NSSAV
  110         CONTINUE
          IF (L .EQ. 1) THEN
C           Compute average fluxes at 250mb
            DO 112 K=1,K2
              DO 112 J=1,K1
                UZ1HS(J,K)= UZ1HS(J,K) / NSSAV
                VZ1HS(J,K)= VZ1HS(J,K) / NSSAV
                KEHS(J,K)= KEHS(J,K) / NSSAV
  112           CONTINUE
          ENDIF
C
          IF (L .EQ. L1) THEN
C           Compute average fluxes at 750mb
            DO 116 K=1,K2
              DO 116 J=1,K1
                UTHS(J,K)= UTHS(J,K) / NSSAV
                VTHS(J,K)= VTHS(J,K) / NSSAV
                UQHS(J,K)= UQHS(J,K) / NSSAV
                VQHS(J,K)= VQHS(J,K) / NSSAV
                UZ2HS(J,K)= UZ2HS(J,K) / NSSAV
                VZ2HS(J,K)= VZ2HS(J,K) / NSSAV
  116           CONTINUE
          ENDIF
C
          DO 120 ITR=JPOT,NPGQ
            DO 120 K=1,K2
              DO 120 J=1,K1
                SCALHS(J,K,NSLEV,ITR)=  SCALHS(J,K,NSLEV,ITR)
     1                                / NSSAV
  120           CONTINUE
C
C         Save diagnostics for this level
          WRITE(LVLHED,'(I6,A2)') NINT(PLV(L)/100.), 'mb'
          HEDHED= COMNT//'HSEC'//MDATE//LVLHED
C         Convert potential temperature to temperature
          DO 130 K=1,K2
            DO 130 J=1,K1
  130         TPH(J,K)= SCALHS(J,K,NSLEV,JPOT)*PKLV(L)
#ifdef SAVE_TUV
C
C         Write T
          HEADER= HEDHED//'T (K)       '
          NRECHS= NRECHS + 1
          CALL QPUTHS( IUAHS, NRECHS, TPH, HEADER )
          IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
C
C         Write U
          HEADER= HEDHED//'U (M/S)     '
          NRECHS= NRECHS + 1
          CALL QPUTHS( IUAHS, NRECHS, UHS(1,1,NSLEV), HEADER )
          IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
C
C         Write V
          HEADER= HEDHED//'V (M/S)     '
          NRECHS= NRECHS + 1
          CALL QPUTHS( IUAHS, NRECHS, VHS(1,1,NSLEV), HEADER )
          IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
#endif
C
#if (NTRACE > 0)
#ifdef SAVE_Q
C
C         Write Q
          HEADER= HEDHED//'Q (KG/KG)   '
          NRECHS= NRECHS + 1
          CALL QPUTHS( IUAHS, NRECHS, SCALHS(1,1,NSLEV,JTR1), HEADER)
          IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
#endif
#endif
C
#ifdef SAVE_EDDY_HI
          IF (L .EQ. 1) THEN
C           Compute stationary eddy fluxes at 250mb
            DO 140 K=1,K2
              DO 140 J=1,K1
                UZHS0(J,K)= UHS(J,K,NSLEV)*AVHS(J,K,NSLEV)
                VZHS0(J,K)= VHS(J,K,NSLEV)*AVHS(J,K,NSLEV)
                KEHS0(J,K)= 0.5*(UHS(J,K,NSLEV)**2 + VHS(J,K,NSLEV)**2)
  140           CONTINUE
C           Compute stationary streamfunction forcing (250mb)
            CALL ZEROSP( FSP1, N1 )
            CALL DIVERG( FSP1, UZHS0, VZHS0)
            CALL IDELSQ( FSP2, FSP1, N1)
            CALL PHYSIC( DIVP0, FSP2 )
C           Compute total streamfunction forcing (250mb)
            CALL ZEROSP( FSP1, N1 )
            CALL DIVERG( FSP1, UZ1HS, VZ1HS)
            CALL IDELSQ( FSP2, FSP1, N1)
            CALL PHYSIC( DIVP, FSP2 )
C           Subtract stationary component from total for eddy part
            DO 150 K=1,K2
              DO 150 J=1,K1
                KEHS(J,K)= KEHS(J,K) - KEHS0(J,K)
                DIVP(J,K)= DIVP(J,K) - DIVP0(J,K)
  150           CONTINUE
C
C           Write TEKE
            HEADER= HEDHED//'TEKE (M2/S2)'
            NRECHS= NRECHS + 1
            CALL QPUTHS( IUAHS, NRECHS, KEHS, HEADER )
            IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
C
C           Write del<-2> div(U'zeta')
            HEADER= HEDHED//'PSIFR1 M2/S2'
            NRECHS= NRECHS + 1
            CALL QPUTHS( IUAHS, NRECHS, DIVP, HEADER )
            IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
          ENDIF
#endif
C
#ifdef SAVE_EDDY_LO
          IF (L .EQ. L1) THEN
C           Compute stationary eddy fluxes at 750mb
            DO 160 K=1,K2
              DO 160 J=1,K1
                UTHS0(J,K)= UHS(J,K,NSLEV)*SCALHS(J,K,NSLEV,JPOT)
                VTHS0(J,K)= VHS(J,K,NSLEV)*SCALHS(J,K,NSLEV,JPOT)
                UQHS0(J,K)= UHS(J,K,NSLEV)*SCALHS(J,K,NSLEV,JTR1)
                VQHS0(J,K)= VHS(J,K,NSLEV)*SCALHS(J,K,NSLEV,JTR1)
                UZHS0(J,K)= UHS(J,K,NSLEV)*AVHS(J,K,NSLEV)
                VZHS0(J,K)= VHS(J,K,NSLEV)*AVHS(J,K,NSLEV)
  160           CONTINUE
C           Compute stationary potential temperature forcing (750mb)
            CALL ZEROSP( FSP1, N1 )
            CALL DIVERG( FSP1, UTHS0, VTHS0)
            CALL PHYSIC( DIVUT0, FSP1 )
C           Compute stationary moisture forcing (750mb)
            CALL ZEROSP( FSP1, N1 )
            CALL DIVERG( FSP1, UQHS0, VQHS0)
            CALL PHYSIC( DIVUQ0, FSP1 )
C           Compute stationary streamfunction forcing (750mb)
            CALL ZEROSP( FSP1, N1 )
            CALL DIVERG( FSP1, UZHS0, VZHS0)
            CALL IDELSQ( FSP2, FSP1, N1)
            CALL PHYSIC( DIVP0, FSP2 )
C           Compute total potential temperature forcing (750mb)
            CALL ZEROSP( FSP1, N1 )
            CALL DIVERG( FSP1, UTHS, VTHS)
            CALL PHYSIC( DIVUT, FSP1 )
C           Compute total moisture forcing (750mb)
            CALL ZEROSP( FSP1, N1 )
            CALL DIVERG( FSP1, UQHS, VQHS)
            CALL PHYSIC( DIVUQ, FSP1 )
C           Compute total streamfunction forcing (750mb)
            CALL ZEROSP( FSP1, N1 )
            CALL DIVERG( FSP1, UZ2HS, VZ2HS)
            CALL IDELSQ( FSP2, FSP1, N1)
            CALL PHYSIC( DIVP, FSP2 )
C           Subtract stationary component from total for eddy part
            DO 170 K=1,K2
              DO 170 J=1,K1
                VTHSE(J,K)= VTHS(J,K) - VTHS0(J,K)
                DIVUT(J,K)= DIVUT(J,K) - DIVUT0(J,K)
                DIVUQ(J,K)= DIVUQ(J,K) - DIVUQ0(J,K)
                DIVP(J,K)= DIVP(J,K) - DIVP0(J,K)
  170           CONTINUE
C           Convert potential temperature to temperature
            DO 180 K=1,K2
              DO 180 J=1,K1
                VTHSE(J,K)= VTHSE(J,K)*PKLV(L)
                DIVUT0(J,K)= DIVUT0(J,K)*PKLV(L)
                DIVUT(J,K)= DIVUT(J,K)*PKLV(L)
  180           CONTINUE
C
C           Write v'T'
            HEADER= HEDHED//'VT1 (MK/S)  '
            NRECHS= NRECHS + 1
            CALL QPUTHS( IUAHS, NRECHS, VTHSE, HEADER )
            IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
C
C           Write div(U'T')
            HEADER= HEDHED//'DIVUT1 (K/S)'
            NRECHS= NRECHS + 1
            CALL QPUTHS( IUAHS, NRECHS, DIVUT, HEADER )
            IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
C
C           Write div(U'Q')
            HEADER= HEDHED//'DIVUQ1 (1/S)'
            NRECHS= NRECHS + 1
            CALL QPUTHS( IUAHS, NRECHS, DIVUQ, HEADER )
            IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
C
C           Write del<-2> div(U'zeta')
            HEADER= HEDHED//'PSIFR1 M2/S2'
            NRECHS= NRECHS + 1
            CALL QPUTHS( IUAHS, NRECHS, DIVP, HEADER )
            IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
          ENDIF
#endif
        ENDIF
C
        IF ( (IUAVS .GT. 0) .AND. (L.EQ.L1) ) THEN
C         Last level; save diagnostics for all selected latitude circles
          NLAT= 0
          DO 220 K=1,K2
            IF (SAVEK(K)) THEN
              NLAT= NLAT + 1
C             Compute averages
              DO 210 L0=1,L1
                DO 210 J=1,K1
                  TVS(J,L0,NLAT)= TVS(J,L0,NLAT) / NSSAV
                  UVS(J,L0,NLAT)= UVS(J,L0,NLAT) / NSSAV
                  VVS(J,L0,NLAT)= VVS(J,L0,NLAT) / NSSAV
  210             WVS(J,L0,NLAT)= WVS(J,L0,NLAT) / NSSAV
C
              WRITE(LATHED,'(F5.1,A3)') DEGLAT(K), 'deg'
              HEDHED= COMNT//'VSEC'//MDATE//LATHED
C
C             Write T
              HEADER= HEDHED//'T (K)       '
              NRECVS= NRECVS + 1
              CALL QPUTVS( IUAVS, NRECVS, TVS(1,1,NLAT), HEADER )
              IF (FIRST) WRITE(*,*) 'Saving ver-section ', HEADER
C
C             Write U
              HEADER= HEDHED//'U (M/S)     '
              NRECVS= NRECVS + 1
              CALL QPUTVS( IUAVS, NRECVS, UVS(1,1,NLAT), HEADER )
              IF (FIRST) WRITE(*,*) 'Saving ver-section ', HEADER
C
C             Write V
              HEADER= HEDHED//'V (M/S)     '
              NRECVS= NRECVS + 1
              CALL QPUTVS( IUAVS, NRECVS, VVS(1,1,NLAT), HEADER )
              IF (FIRST) WRITE(*,*) 'Saving ver-section ', HEADER
C
C             Write omega
              HEADER= HEDHED//'OMEGA (PA/S)'
              NRECVS= NRECVS + 1
              CALL QPUTVS( IUAVS, NRECVS, WVS(1,1,NLAT), HEADER )
              IF (FIRST) WRITE(*,*) 'Saving ver-section ', HEADER
            ENDIF
  220       CONTINUE
        ENDIF
C
        IF ( (IUAZS .GT. 0) .AND. (L.EQ.L1) ) THEN
C         Last level; save diagnostics for all zonal-mean sections
C         Compute averages
          DO 310 L0=1,L1
            DO 310 K=1,K2
              UZS(K,L0)= UZS(K,L0) / NSSAV
              VZS(K,L0)= VZS(K,L0) / NSSAV
              WZS(K,L0)= WZS(K,L0) / NSSAV
  310         AVZS(K,L0)= AVZS(K,L0) / NSSAV
C
          DO 320 ITR=JPOT,NPGQ
            DO 320 L0=1,L1
              DO 320 K=1,K2
  320           SCALZS(K,L0,ITR)= SCALZS(K,L0,ITR) / NSSAV
C
          HEDHED= COMNT//'MSEC'//MDATE//'ZONMEAN '
C
C         Write Theta
          HEADER= HEDHED//'THETA (K)   '
          NRECZS= NRECZS + 1
          CALL QPUTZS( IUAZS, NRECZS, SCALZS(1,1,JPOT), HEADER )
          IF (FIRST) WRITE(*,*) 'Saving zonal-mean ', HEADER
C
C         Write U
          HEADER= HEDHED//'U (M/S)     '
          NRECZS= NRECZS + 1
          CALL QPUTZS( IUAZS, NRECZS, UZS, HEADER )
          IF (FIRST) WRITE(*,*) 'Saving zonal-mean ', HEADER
C
C         Write V
          HEADER= HEDHED//'V (M/S)     '
          NRECZS= NRECZS + 1
          CALL QPUTZS( IUAZS, NRECZS, VZS, HEADER )
          IF (FIRST) WRITE(*,*) 'Saving zonal-mean ', HEADER
C
C         Write omega
          HEADER= HEDHED//'OMEGA (PA/S)'
          NRECZS= NRECZS + 1
          CALL QPUTZS( IUAZS, NRECZS, WZS, HEADER )
          IF (FIRST) WRITE(*,*) 'Saving zonal-mean ', HEADER
C
C         Write absolute vorticity
          HEADER= HEDHED//'ABSVOR (1/S)'
          NRECZS= NRECZS + 1
          CALL QPUTZS( IUAZS, NRECZS, AVZS, HEADER )
          IF (FIRST) WRITE(*,*) 'Saving zonal-mean ', HEADER
#if (NTRACE > 0)
C
C         Write Q
          HEADER= HEDHED//'Q (KG/KG)   '
          NRECZS= NRECZS + 1
          CALL QPUTZS( IUAZS, NRECZS, SCALZS(1,1,JTR1), HEADER )
          IF (FIRST) WRITE(*,*) 'Saving zonal-mean ', HEADER
#endif
        ENDIF
C
C       Last level; reset save data accumulators
        IF (L .EQ. L1) CALL SAVZRO
      ENDIF
C
      RETURN
      END
 
      SUBROUTINE DIASTP( ISTP, PGQSP, FRCSP )
C DIASTP should be called during every time-marching step.
C It decides whether to sample for dynamical diagnostics or not,
C and carries out the necessary calculations. It also writes out
C time-series and section data.
C Input:  INTEGER ISTP -- current model step number
C         COMPLEX PGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX) --
C            spectral representations of all prognostic quantities
C            at current model time.
C         COMPLEX FRCSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX) --
C            spectral representations of corresponding forcing
C            tendencies at current model time.
C         (FRCSP may be undefined for ISTP.LE.IDSTEP)
C
C Output: (Diagnostic calculations are carried out, if need be)
C
#include "mgrid.h"
#include "diacom.h"
      INTEGER ISTP
      COMPLEX PGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX)
      COMPLEX FRCSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX)
      REAL TIM, TEMP
      INTEGER I, J, K, L, M, N, KMID, IPGQ, ITEMP
#if (NTRACE > 0)
      INTEGER ITR
#endif
C
C     Check if current model step needs to be sampled
      IF ( (ISTP.LT.IDSTEP) .OR.
     1     (MOD((ISTP-IDSTEP), NDSKIP) .NE. 0) )RETURN
C
C     Calculate diagnostic quantities
      CALL DIACAL( ISTP, PGQSP, FRCSP )
C     Compute current model time
      TIM=  DBLE(ISTP)*DBLE(DTSTEP)
      ISTP1= ISTP
      TAU1= TIM
C
      IF (ISTP.EQ.IDSTEP) THEN
C       Initial snapshot
        ISTP0= ISTP
        TAU0= TIM
C       Save initial zonal-mean angular momentum and temperature
        DO 10 L=1,L1
          DO 10 K=1,K2
            ANGMT0(K,L)= VELZON(K,L,JX)*COSPHI(K)
            ANGMOT(K,L)= 0.0
            TINIT(K,L)= POTZON(K,L)*PKLV(L)
   10       TTEND(K,L)= 0.0
#if (NTRACE > 0)
C       Save initial zonal-mean tracer concentration
        DO 20 ITR=JTR1,NPGQ
          DO 20 L=1,L1
            DO 20 K=1,K2
              TRINIT(K,L,ITR)= TRCZON(K,L,ITR)
   20         TRTEND(K,L,ITR)= 0.0
#endif
      ELSE
C       Increment sample counter
        NSAMPL= NSAMPL+1
C       Compute time-averaged angular momentum and T tendency
        DO 30 L=1,L1
          DO 30 K=1,K2
            ANGMOT(K,L)= (  VELZON(K,L,JX)*COSPHI(K)
     1                    - ANGMT0(K,L) )/((NDSKIP*DTSTEP)*NSAMPL)
            TTEND(K,L)=  (  POTZON(K,L)*PKLV(L)
     1                    - TINIT(K,L) )/((NDSKIP*DTSTEP)*NSAMPL)
   30       CONTINUE
#if (NTRACE > 0)
C       Compute time-averaged tracer tendencies
        DO 40 ITR=JTR1,NPGQ
          DO 40 L=1,L1
            DO 40 K=1,K2
              TRTEND(K,L,ITR)= ( TRCZON(K,L,ITR) - TRINIT(K,L,ITR) )
     1                        /( (NDSKIP*DTSTEP)*NSAMPL )
   40         CONTINUE
#endif
C       Accumulate diagnostic quantities (for time-mean values)
        DO 100 L=1,L1
          DO 100 IPGQ=1,NPGQ
            DO 100 N=0,N1
              DO 100 M=0,MIN0(N,M1)
  100           PGQAVG(M,N,IPGQ,L)= PGQAVG(M,N,IPGQ,L)+PGQSP(M,N,IPGQ,L)
C
        DO 110 I=1,2
          DO 110 L=1,L1
            DO 110 K=1,K2
              AMFAVG(K,L,I)= AMFAVG(K,L,I) + AMFLUX(K,L,I)
              EAMFAV(K,L,I)= EAMFAV(K,L,I) + EAMFLX(K,L,I)
              EPFAVG(K,L,I)= EPFAVG(K,L,I) + EPFLUX(K,L,I)
  110         CONTINUE
C
        DO 120 L=1,L1
         DO 120 K=1,K2
           KEAVG(K,L)= KEAVG(K,L) + KEZON(K,L)
           PTFAVG(K,L)= PTFAVG(K,L) + PTFLUX(K,L)
           PICAVG(K,L)= PICAVG(K,L) + PSIIC(K,L)
           EMTAVG(K,L)= EMTAVG(K,L) + EMMCT(K,L)
           PVAVG(K,L)= PVAVG(K,L) + PVZON(K,L)
           SPSECT(K,L)= SPSECT(K,L) + 0.0
  120      CONTINUE
C
        DO 130 I=1,2
          DO 130 K=1,K2
            DO 130 M=0,M1
              EPSAVG(M,K,I)= EPSAVG(M,K,I) + EPSPEC(M,K,I)
  130         CONTINUE
C
        DO 140 M=0,M1
  140     KESAVG(M)= KESAVG(M) + KESPEC(M)
C
        DO 150 N=0,N1
          KEPAVG(N)= KEPAVG(N) + KEPOW(N)
  150     ENPAVG(N)= ENPAVG(N) + ENPOW(N)
C
C       Accumulate forcing diagnostics
        DO 170 L=1,L1
          DO 170 K=1,K2
            UFOAVG(K,L)= UFOAVG(K,L) + UFORC(K,L)
            TFOAVG(K,L)= TFOAVG(K,L) + TFORC(K,L)
  170       KEDAVG(K,L)= KEDAVG(K,L) + KEDISS(K,L)
C
C       Accumulate squared temperature and velocity at LDIAG
        DO 180 K=1,K2
          DO 180 J=1,K1
            TVAR(J,K)= TVAR(J,K) + TFLD(J,K)**2
            KEFAVG(J,K)=  KEFAVG(J,K)
     1                  + 0.5*(VELFLD(J,K,JX)**2 + VELFLD(J,K,JY)**2)
  180       CONTINUE
#if (NTRACE > 0)
C       Accumulate tracer flux
        DO 190 ITR=JTR1,NPGQ
          DO 190 L=1,L1
            DO 190 K=1,K2
  190         TRFAVG(K,L,ITR)= TRFAVG(K,L,ITR) + TRFLUX(K,L,ITR)
C       Accumulate squared tracer concentration at LDIAG
        DO 200 ITR=JTR1,NPGQ
          DO 200 K=1,K2
            DO 200 J=1,K1
  200         TRCVAR(J,K,ITR)= TRCVAR(J,K,ITR) + TRCFLD(J,K,ITR)**2
#endif
      ENDIF
C
C     Compute time-evolution quantities
C
      EKE= 0.0
      PIE= 0.0
      DO 210 L=1,L1
        TEMP= DP(L)/PSURF
        DO 210 K=1,K2
C         Compute globally averaged eddy kinetic energy
          EKE= EKE + TEMP*G(K)*(KEZON(K,L) - 0.5*( VELZON(K,L,JX)**2
     1                                            +VELZON(K,L,JY)**2) )
C         Compute globally averaged potential+internal energy
C         (With reference to standard vertical stratification)
          PIE= PIE + TEMP*CP*PKLV(L)*G(K)*
     1                               (POTZON(K,L) - TSTDLV(L))
  210     CONTINUE
C     Compute diagnostic-level u-bar at equator
      KMID= (K2+1)/2
      IF (MOD(K2,2).EQ.1) THEN
C       Equator is a gaussian latitude
        UEQ= VELZON(KMID,LDIAG,JX)
      ELSE
C       Average over the two latitudes straddling the equator
        UEQ= 0.5*(VELZON(KMID,LDIAG,JX) + VELZON(KMID+1,LDIAG,JX))
      ENDIF
C     Compute average diagnostic-level u-bar between 30S and 60S
      ITEMP= 0
      TEMP= 0.0
      DO 220 K=1,K2
        IF ((DEGLAT(K) .GE. -60.) .AND. (DEGLAT(K) .LE. -30.)) THEN
          ITEMP= ITEMP + 1
          TEMP= TEMP + VELZON(K,LDIAG,JX)
        ENDIF
  220   CONTINUE
      UXTSH= TEMP / ITEMP
C     Compute average diagnostic-level u-bar between 30N and 60N
      ITEMP= 0
      TEMP= 0.0
      DO 230 K=1,K2
        IF ((DEGLAT(K) .GE. 30.) .AND. (DEGLAT(K) .LE. 60.)) THEN
          ITEMP= ITEMP + 1
          TEMP= TEMP + VELZON(K,LDIAG,JX)
        ENDIF
  230   CONTINUE
      UXTNH= TEMP / ITEMP
C
C     Vorticity coefficients
      VOR01D= REAL( PGQSP(0,1,JVOR,LDIAG) )
      VOR02D= REAL( PGQSP(0,2,JVOR,LDIAG) )
      VOR03D= REAL( PGQSP(0,3,JVOR,LDIAG) )
      VOR12D= PGQSP(1,2,JVOR,LDIAG)
C     Temperature coefficients
C     Compute globally averaged temperature
      TEMP= 0.0
      DO 240 L=1,L1
  240   TEMP= TEMP + DP(L)*REAL(PGQSP(0,0,JPOT,L))*PKLV(L)
      T00= TEMP / PSURF
      T00D= REAL( PGQSP(0,0,JPOT,LDIAG) )*PKLV(LDIAG)
      T00S= REAL( PGQSP(0,0,JPOT,L1) )*PKLV(L1)
C
#if (NTRACE > 0)
C     Compute globally averaged tracer concentrations
      DO 260 ITR=JTR1,NPGQ
        TEMP= 0.0
        DO 250 L=1,L1
  250     TEMP= TEMP + DP(L)*REAL(PGQSP(0,0,ITR,L))
        TRC00(ITR)= TEMP / PSURF
        TRC00D(ITR)= REAL( PGQSP(0,0,ITR,LDIAG) )
        TRC00S(ITR)= REAL( PGQSP(0,0,ITR,L1) )
  260   CONTINUE
#endif
C
      IF ((IUATS .GT. 0) .AND. (ISTP .GE. ISSTEP)) THEN
C       Accumulate time-series data
        CALL TSACC
        IF (MOD((ISTP-IDSTEP), NTFREQ*NDSKIP) .EQ. 0) THEN
C         Write time-series data
          CALL TSWRIT( ISTP )
C         Re-initialize time-series accumulators
          CALL TSINI
        ENDIF
      ENDIF
      RETURN
      END
 
      SUBROUTINE DIAFLX( ISTP, PGQSP, FRCSP, QEVAP, MSAPCP, MCAPCP,
     1                   FEVAP, FSENS, FRAD, FNET, FPSSRF, FMSSRF,
     2                   FPLSRF, FMLSRF, FPSTOP, FPLTOP,
     3                   SWHET, LWHET, MSAHET, MCAHET, DCAHET,
     4                   SEAMSK, ICEMSK, SOLRAD, ALBEDO,
     5                   TSURF, STRESX, STRESY, ICEHT )
C DIAFLX should be called during every time-marching step.
C It decides whether to sample for flux diagnostics or not,
C and carries out the necessary calculations.
C (Note:  DIAFLX automatically calls DIASTP.)
C Input:  INTEGER ISTP -- current model step number
C         COMPLEX PGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX) --
C            spectral representations of all prognostic quantities
C            at current model time.
C         COMPLEX FRCSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX) --
C            spectral representations of corresponding forcing
C            tendencies at current model time.
C         (FRCSP may be undefined for ISTP.LE.IDSTEP)
C         REAL QEVAP(K1MAX,K2MAX) -- evaporative flux of water vapour
C                                    (in kg/(s*m**2))
C         REAL MSAPCP(K1MAX,K2MAX) -- precipitation due to moist
C                                static adjustment (in kg/(s*m**2))
C         REAL MCAPCP(K1MAX,K2MAX) -- precipitation due to moist
C                                convective adjustment (in kg/(s*m**2))
C         REAL FEVAP(K1MAX,K2MAX) -- heat flux associated with the
C                                    evaporative flux of water vapour
C                                    (in W/m**2)
C         REAL FSENS(K1MAX,K2MAX) -- upward surface sensible heat flux
C                                    (in W/m**2)
C         REAL FRAD(K1MAX,K2MAX) -- upward surface radiative heat flux
C                                   (in W/m**2)
C         REAL FNET(K1MAX,K2MAX) -- Net heat flux into land/sea
C                                   (in W/m**2)
C         REAL FPSSRF(K1MAX,K2MAX) -- upward SW radiation at surface
C         REAL FMSSRF(K1MAX,K2MAX) -- downward SW radiation at surface
C         REAL FPLSRF(K1MAX,K2MAX) -- upward LW radiation at surface
C         REAL FMLSRF(K1MAX,K2MAX) -- downward LW radiation at surface
C         REAL FPSTOP(K1MAX,K2MAX) -- upward SW radiation at model top
C         REAL FPLTOP(K1MAX,K2MAX) -- upward LW radiation at model top
C         REAL SWHET(K2MAX,L1MAX)  -- zonally-averaged vertical profile
C              of heating due to SW radiation (in K/s)
C         REAL LWHET(K2MAX,L1MAX)  -- zonally-averaged vertical profile
C              of heating due to LW radiation (in K/s)
C         REAL MSAHET(K2MAX,L1MAX) -- zonally-averaged vertical profile
C              of heating due to moist static adjustment (in K/s)
C         REAL MCAHET(K2MAX,L1MAX) -- zonally-averaged vertical profile
C              of heating due to moist convective adjustment (in K/s)
C         REAL DCAHET(K2MAX,L1MAX) -- zonally-averaged vertical profile
C              of heating due to dry convective adjustment (in K/s)
C         LOGICAL SEAMSK(K1MAX,K2MAX) -- sea mask (.TRUE. over sea)
C         LOGICAL ICEMSK(K1MAX,K2MAX) -- ice mask (.TRUE. over ice)
C         REAL SOLRAD(K1MAX,K2MAX) -- incident solar radiation (W/m**2)
C         REAL ALBEDO(K1MAX,K2MAX) -- surface albedo (0 .. 1)
C         REAL TSURF(K1MAX,K2MAX) -- surface temperature (in K)
C         REAL STRESX(K1MAX,K2MAX) -- surface wind stress in the
C               zonal direction (Pa)
C         REAL STRESY(K1MAX,K2MAX) -- surface wind stress in the
C               meridional direction (Pa)
C         REAL ICEHT(K1MAX,K2MAX) -- ice height (m)
C
C Output: (Flux diagnostics are computed out, if need be)
C
#include "mgrid.h"
#include "diacom.h"
      INTEGER ISTP
      COMPLEX PGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX)
      COMPLEX FRCSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX)
      REAL QEVAP(K1MAX,K2MAX)
      REAL MSAPCP(K1MAX,K2MAX), MCAPCP(K1MAX,K2MAX)
      REAL FEVAP(K1MAX,K2MAX), FSENS(K1MAX,K2MAX)
      REAL FRAD(K1MAX,K2MAX), FNET(K1MAX,K2MAX)
      REAL FPSSRF(K1MAX,K2MAX), FMSSRF(K1MAX,K2MAX)
      REAL FPLSRF(K1MAX,K2MAX), FMLSRF(K1MAX,K2MAX)
      REAL FPSTOP(K1MAX,K2MAX), FPLTOP(K1MAX,K2MAX)
      REAL SWHET(K2MAX,L1MAX), LWHET(K2MAX,L1MAX)
      REAL MSAHET(K2MAX,L1MAX), MCAHET(K2MAX,L1MAX)
      REAL DCAHET(K2MAX,L1MAX), SOLRAD(K1MAX,K2MAX)
      REAL ALBEDO(K1MAX,K2MAX), TSURF(K1MAX,K2MAX)
      REAL STRESX(K1MAX,K2MAX), STRESY(K1MAX,K2MAX)
      REAL ICEHT(K1MAX,K2MAX)
      LOGICAL SEAMSK(K1MAX,K2MAX), ICEMSK(K1MAX,K2MAX)
C
      REAL TAUBAR(K2MAX), FBAR(K2MAX), EPBARO(K2MAX), EPBARL(K2MAX)
      REAL TBARO(K2MAX), TBARL(K2MAX)
      COMPLEX FSP(0:M1MAX, 0:N1MAX)
      INTEGER J, K, L, ILAND, ISEA
C
C     Check if current model step needs to be sampled
      IF ( (ISTP.LT.IDSTEP) .OR.
     1     (MOD((ISTP-IDSTEP), NDSKIP) .NE. 0) ) RETURN
C     Save surface quantities, if need be
      CALL DIASV0( ISTP, QEVAP, MSAPCP, MCAPCP,
     1             FEVAP, FSENS, FRAD, FNET, FPSSRF, FMSSRF,
     2             FPLSRF, FMLSRF, FPSTOP, FPLTOP,
     3             SWHET, LWHET, MSAHET, MCAHET, DCAHET,
     4             SEAMSK, ICEMSK, SOLRAD, ALBEDO,
     5             TSURF, STRESX, STRESY, ICEHT )
C
C     Compute flux diagnostics
C     Accumulate land/sea quantities separately
      DO 130 K=1,K2
        TAUBAR(K)= 0.0
        FBAR(K)= 0.0
        EPBARO(K)= 0.0
        EPBARL(K)= 0.0
        TBARO(K)= 0.0
        TBARL(K)= 0.0
        ILAND= 0
        ISEA= 0
        DO 120 J=1,K1
          IF (SEAMSK(J,K)) THEN
C           Sea point
            ISEA= ISEA + 1
            TAUBAR(K)= TAUBAR(K) + STRESX(J,K)
            FBAR(K)= FBAR(K) + FNET(J,K)
            EPBARO(K)= EPBARO(K) + (QEVAP(J,K)-MSAPCP(J,K)-MCAPCP(J,K))
            TBARO(K)= TBARO(K) + TSURF(J,K)
          ELSE
C           Land point
            ILAND= ILAND + 1
            EPBARL(K)= EPBARL(K) + (QEVAP(J,K)-MSAPCP(J,K)-MCAPCP(J,K))
            TBARL(K)= TBARL(K) + TSURF(J,K)
          ENDIF
  120     CONTINUE
        IF (ISEA .GT. 0) THEN
          TAUBAR(K)= TAUBAR(K) / ISEA
          FBAR(K)= FBAR(K) / ISEA
          EPBARO(K)= EPBARO(K) / ISEA
          TBARO(K)= TBARO(K) / ISEA
        ENDIF
        IF (ILAND .GT. 0) THEN
          EPBARL(K)= EPBARL(K) / ILAND
          TBARL(K)= TBARL(K) / ILAND
        ENDIF
  130   CONTINUE
C
C     Accumulate diagnostic quantities (for time-mean values)
C
      DO 220 K=1,K2
        DO 210 J=1,K1
          SOLZON(K)= SOLZON(K) + SOLRAD(J,K)/K1
          ALBZON(K)= ALBZON(K) + ALBEDO(J,K)/K1
          OSRZON(K)= OSRZON(K) + FPSTOP(J,K)/K1
          OLRZON(K)= OLRZON(K) + FPLTOP(J,K)/K1
          ISRZON(K)= ISRZON(K) + (FMSSRF(J,K) - FPSSRF(J,K))/K1
          ILRZON(K)= ILRZON(K) + (FMLSRF(J,K) - FPLSRF(J,K))/K1
          FEVPZN(K)= FEVPZN(K) + FEVAP(J,K)/K1
          FSNSZN(K)= FSNSZN(K) + FSENS(J,K)/K1
          FRADZN(K)= FRADZN(K) + FRAD(J,K)/K1
          FNETZN(K)= FNETZN(K) + FNET(J,K)/K1
          MSAPZN(K)= MSAPZN(K) + MSAPCP(J,K)/K1
          MCAPZN(K)= MCAPZN(K) + MCAPCP(J,K)/K1
          EVAPZN(K)= EVAPZN(K) + QEVAP(J,K)/K1
  210     CONTINUE
        TAUOCN(K)= TAUOCN(K) + TAUBAR(K)
        FOCN(K)= FOCN(K) + FBAR(K)
        EMPOCN(K)= EMPOCN(K) + EPBARO(K)
        EMPLAN(K)= EMPLAN(K) + EPBARL(K)
        SSTZON(K)= SSTZON(K) + TBARO(K)
        LSTZON(K)= LSTZON(K) + TBARL(K)
  220   CONTINUE
C
      DO 230 K=1,K2
        DO 230 L=1,L1
          SWHAVG(K,L)= SWHAVG(K,L) + SWHET(K,L)
          LWHAVG(K,L)= LWHAVG(K,L) + LWHET(K,L)
          MSAHAV(K,L)= MSAHAV(K,L) + MSAHET(K,L)
          MCAHAV(K,L)= MCAHAV(K,L) + MCAHET(K,L)
  230     DCAHAV(K,L)= DCAHAV(K,L) + DCAHET(K,L)
C
      DO 240 K=1,K2
        DO 240 J=1,K1
          TSRFAV(J,K)= TSRFAV(J,K) + TSURF(J,K)
          ALBAVG(J,K)= ALBAVG(J,K) + ALBEDO(J,K)
          PCPAVG(J,K)= PCPAVG(J,K) + MSAPCP(J,K) + MCAPCP(J,K)
          EVPAVG(J,K)= EVPAVG(J,K) + QEVAP(J,K)
          FNETAV(J,K)= FNETAV(J,K) + FNET(J,K)
          OLRAVG(J,K)= OLRAVG(J,K) + FPLTOP(J,K)
          TAUXAV(J,K)= TAUXAV(J,K) + STRESX(J,K)
          TAUYAV(J,K)= TAUYAV(J,K) + STRESY(J,K)
          ICEAVG(J,K)= ICEAVG(J,K) + ICEHT(J,K)
C         Surface T variance
          SPFLD1(J,K)= SPFLD1(J,K) + TSURF(J,K)**2
C         Precipitation variance
          SPFLD2(J,K)= SPFLD2(J,K) + (MSAPCP(J,K) + MCAPCP(J,K))**2
  240     CONTINUE
C
C     Compute time-evolution quantities
C
C     SST coefficients
      CALL ZEROSP( FSP, N1 )
      CALL SPEC0( FSP, TBARO )
      SST00= REAL( FSP(0,0) )
      SST01= REAL( FSP(0,1) )
      SST02= REAL( FSP(0,2) )
C     LST coefficients
      CALL ZEROSP( FSP, N1 )
      CALL SPEC0( FSP, TBARL )
      LST00= REAL( FSP(0,0) )
      LST01= REAL( FSP(0,1) )
      LST02= REAL( FSP(0,2) )
C
C     Compute dynamical diagnostics
      CALL DIASTP( ISTP, PGQSP, FRCSP )
      RETURN
      END
 
      SUBROUTINE DIACAL( ISTP, PGQSP, FRCSP  )
C DIACAL computes basic dynamical diagnostics
C Input:  INTEGER ISTP -- current model step number
C         (Note: specify ISTP=-1 for time-mean states)
C         COMPLEX PGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX) --
C            spectral representations of all prognostic quantities
C            at current model time.
C         COMPLEX FRCSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX) --
C            spectral representations of corresponding forcing
C            tendencies at current model time.
C         (Note: FRCSP values are not used when ISTP=-1)
C Output: (Diagnostics are computed and the results stored in diacom.h)
C
#include "mgrid.h"
#include "diacom.h"
      INTEGER ISTP
      COMPLEX PGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX)
      COMPLEX FRCSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX)
C
      REAL WPH(K1MAX,K2MAX), SCALPH(K1MAX,K2MAX,JPOT:NPGQ,0:1)
      REAL VPH(K1MAX,K2MAX,JX:JY,0:1), WMIDPH(K1MAX,K2MAX)
      REAL AVORPH(K1MAX,K2MAX,0:1), UBARPH(K1MAX,K2MAX)
      REAL AVRBAR(K1MAX,K2MAX), AVRBZ(K2MAX,L1MAX)
      REAL GXPTPH(K1MAX,K2MAX), GYPTPH(K1MAX,K2MAX)
      REAL DIVPH(K1MAX,K2MAX), DIVZON(K2MAX), VUFLXZ(K2MAX,0:1)
      REAL UBARL(K2MAX), PSIICM(0:M1MAX), TEMP
      REAL EPTFLX(K1MAX,K2MAX), FPH(K1MAX,K2MAX), FPH2(K1MAX,K2MAX)
      COMPLEX FSP1(0:M1MAX,0:N1MAX), FSP2(0:M1MAX,0:N1MAX)
      INTEGER LMHALF, LPZERO, LPHALF, LPONE
C     (Mnemonic for L-(1/2), L+0, L+(1/2), L+1)
      INTEGER J, K, L, M, N
      EXTERNAL XSP2LA, YSP2LA
      INTEGER ITR
#ifdef ON_SUN4
      character*80 line
#endif
C
C     Initialize spectral accumulators
      DO 10 M=0,M1
   10   KESPEC(M)= 0.0
      DO 20 N=0,N1
        KEPOW(N)= 0.0
   20   ENPOW(N)= 0.0
C     Initialize omega at level 1/2
      DO 30 K=1,K2
        DO 30 J=1,K1
   30     WPH(J,K)= 0.0
C
      LMHALF= 0
      LPHALF= MOD(LMHALF+1, 2)
      LPZERO= 0
      LPONE=  MOD(LPZERO+1, 2)
C     Initialize vertical momentum flux at level 1/2
      DO 40 K=1,K2
   40   VUFLXZ(K,LPHALF)= 0.0
C     Compute horizontal velocity at level 1
      CALL IDELSQ( FSP1, PGQSP(0,0,JVOR,1), N1)
      CALL IDELSQ( FSP2, PGQSP(0,0,JDIV,1), N1)
      CALL HVELOC( VPH(1,1,JX,LPONE), VPH(1,1,JY,LPONE),
     1             FSP1, FSP2)
C     Compute zonal-mean velocity at level 1
      CALL ZAVGE( VELZON(1,1,JX), VPH(1,1,JX,LPONE) )
      CALL ZAVGE( VELZON(1,1,JY), VPH(1,1,JY,LPONE) )
C     Compute absolute vorticity at level 1
      CALL SPCOPY( FSP1, PGQSP(0,0,JVOR,1), N1)
      FSP1(0,1)= FSP1(0,1) + CMPLX(FSP01, 0.)
      CALL PHYSIC( AVORPH(1,1,LPONE), FSP1 )
C     Compute scalars at level 1
      DO 60 ITR=JPOT,NPGQ
   60   CALL PHYSIC( SCALPH(1,1,ITR,LPONE), PGQSP(0,0,ITR,1) )
C     Compute zonal-mean potential temperature at level 1
      CALL ZAVGE( POTZON(1,1), SCALPH(1,1,JPOT,LPONE) )
#if (NTRACE > 0)
C     Compute zonal-mean tracer values at level 1
      DO 70 ITR=JTR1,NPGQ
   70   CALL ZAVGE( TRCZON(1,1,ITR), SCALPH(1,1,ITR,LPONE) )
#endif
C
      DO 500 L=1,L1
C
        LMHALF= MOD(LMHALF+1, 2)
        LPHALF= MOD(LMHALF+1, 2)
        LPZERO= MOD(LPZERO+1, 2)
        LPONE=  MOD(LPZERO+1, 2)
        IF (L.LT.L1) THEN
C         Compute horizontal velocity at level L+1
          CALL IDELSQ( FSP1, PGQSP(0,0,JVOR,L+1), N1)
          CALL IDELSQ( FSP2, PGQSP(0,0,JDIV,L+1), N1)
          CALL HVELOC( VPH(1,1,JX,LPONE), VPH(1,1,JY,LPONE),
     1                 FSP1, FSP2)
C         Compute zonal-mean velocity at level L+1
          CALL ZAVGE( VELZON(1,L+1,JX), VPH(1,1,JX,LPONE) )
          CALL ZAVGE( VELZON(1,L+1,JY), VPH(1,1,JY,LPONE) )
C         Compute absolute vorticity at level L+1
          CALL SPCOPY( FSP1, PGQSP(0,0,JVOR,L+1), N1)
          FSP1(0,1)= FSP1(0,1) + CMPLX(FSP01, 0.)
          CALL PHYSIC( AVORPH(1,1,LPONE), FSP1 )
C         Compute scalars at level L+1
          DO 80 ITR=JPOT,NPGQ
   80       CALL PHYSIC( SCALPH(1,1,ITR,LPONE), PGQSP(0,0,ITR,L+1) )
C         Compute zonal-mean potential temperature at level L+1
          CALL ZAVGE( POTZON(1,L+1), SCALPH(1,1,JPOT,LPONE) )
#if (NTRACE > 0)
C         Compute zonal-mean tracer values at level L+1
          DO 85 ITR=JTR1,NPGQ
   85       CALL ZAVGE( TRCZON(1,L+1,ITR), SCALPH(1,1,ITR,LPONE) )
#endif
C
        ELSE
C         Use a "no shear" lower boundary condition on momentum
          CALL PHCOPY( VPH(1,1,JX,LPONE), VPH(1,1,JX,LPZERO) )
          CALL PHCOPY( VPH(1,1,JY,LPONE), VPH(1,1,JY,LPZERO) )
          CALL PHCOPY( AVORPH(1,1,LPONE), AVORPH(1,1,LPZERO) )
C         Use a "no curvature" lower b.c. on all scalars
          DO 90 ITR=JPOT,NPGQ
            CALL PHYSIC( FPH, PGQSP(0,0,ITR,L1-1) )
            DO 90 K=1,K2
              DO 90 J=1,K1
                SCALPH(J,K,ITR,LPONE)=  2.0*SCALPH(J,K,ITR,LPZERO)
     1                                 - FPH(J,K)
   90           CONTINUE
        ENDIF
C
        IF (ISTP .GT. -1) THEN
C         Compute forcing diagnostics for current level
C
C         Compute zonal-mean potential temperature forcing
          CALL PHYS0( TFORC(1,L), FRCSP(0,0,JPOT,L) )
C         Compute streamfunction and velocity potential forcing
          CALL IDELSQ( FSP1, FRCSP(0,0,JVOR,L), N1)
          CALL IDELSQ( FSP2, FRCSP(0,0,JDIV,L), N1)
C         Compute horizontal velocity forcing
          CALL HVELOC( FPH, FPH2, FSP1, FSP2 )
C         Compute zonal-mean horizontal velocity forcing
          CALL ZAVGE( UFORC(1,L), FPH )
          DO 100 K=1,K2
C           Compute zonal-mean kinetic energy dissipation
            KEDISS(K,L)= 0.0
            DO 95 J=1,K1
              KEDISS(K,L)= KEDISS(K,L) + FPH(J,K)*VPH(J,K,JX,LPZERO)
     1                                 + FPH2(J,K)*VPH(J,K,JY,LPZERO)
   95         CONTINUE
            KEDISS(K,L)= KEDISS(K,L) / K1
C           Convert u-bar forcing to ang-mom forcing
  100       UFORC(K,L)= UFORC(K,L)*COSPHI(K)
        ENDIF
C
        IF (L.EQ.LDIAG) THEN
C         Copy temperature and velocity fields at level LDIAG
          DO 110 K=1,K2
            DO 110 J=1,K1
  110         TFLD(J,K)= SCALPH(J,K,JPOT,LPZERO)*PKLV(LDIAG)
          CALL PHCOPY( VELFLD(1,1,JX), VPH(1,1,JX,LPZERO) )
          CALL PHCOPY( VELFLD(1,1,JY), VPH(1,1,JY,LPZERO) )
#if (NTRACE > 0)
C         Copy tracer fields at level LDIAG
          DO 120 ITR=JTR1,NPGQ
  120       CALL PHCOPY( TRCFLD(1,1,ITR), SCALPH(1,1,ITR,LPZERO) )
#endif
        ENDIF
C
C       Compute quantities for special diagnostics/ section data
C       Copy hor. velocity and pot. temperature to latitude space
        CALL P2L( ULA, VPH(1,1,JX,LPZERO), K2 )
        CALL P2L( VLA, VPH(1,1,JY,LPZERO), K2 )
        CALL P2L( PTLA, SCALPH(1,1,JPOT,LPZERO), K2 )
C
C       Compute hor. velocity and absolute vorticity at level L+1/2
        DO 130 K=1,K2
          DO 130 J=1,K1
            UBARPH(J,K)= 0.5*(VPH(J,K,JX,LPONE) + VPH(J,K,JX,LPZERO))
            AVRBAR(J,K)= 0.5*(AVORPH(J,K,LPONE) + AVORPH(J,K,LPZERO))
  130       CONTINUE
C       Compute zonal-mean hor. velocity at L+1/2
        CALL ZAVGE( UBARL, UBARPH )
C       Compute zonal-mean absolute vorticity at level L+1/2
        CALL ZAVGE( AVRBZ(1,L), AVRBAR )
C
C       Compute horizontal gradient of pot. temperature at level L+1/2
        IF (L.LT.L1) THEN
          DO 140 N=0,N1
            DO 140 M=0,MIN0(N,M1)
  140         FSP1(M,N)= 0.5*( PGQSP(M,N,JPOT,L+1) + PGQSP(M,N,JPOT,L) )
        ELSE
          CALL SPCOPY( FSP1, PGQSP(0,0,JPOT,L1), L1 )
        ENDIF
        CALL S2P( GXPTPH, FSP1, XSP2LA, N1 )
        CALL MULCOS( GXPTPH, GXPTPH, A0INV, -1 )
        CALL S2P( GYPTPH, FSP1, YSP2LA, N1 )
        CALL MULCOS( GYPTPH, GYPTPH, A0INV, 1 )
C
        IF (L.LT.L1) THEN
C         Compute Ertel's potential vorticity at level L+1/2
          TEMP= -G0/(PLV(L+1) - PLV(L))
          DO 150 K=1,K2
            DO 150 J=1,K1
              PVORPH(J,K)= TEMP*(
     1          AVRBAR(J,K)*(  SCALPH(J,K,JPOT,LPONE)
     2                       - SCALPH(J,K,JPOT,LPZERO)  )
     3        + GYPTPH(J,K)*(VPH(J,K,JX,LPONE) - VPH(J,K,JX,LPZERO))
     4        - GXPTPH(J,K)*(VPH(J,K,JY,LPONE) - VPH(J,K,JY,LPZERO)) )
  150         CONTINUE
        ENDIF
C Note: Potential vorticity at level L1+1/2 is assumed to be same as
C       that at level L1-1/2
C       Compute zonal-mean potential vorticity
        CALL ZAVGE( PVZON(1,L), PVORPH )
C
C       Compute divergence in physical space
        CALL PHYSIC( DIVPH, PGQSP(0,0,JDIV,L) )
C       Compute omega at level L in physical space
        DO 160 K=1,K2
          DO 160 J=1,K1
  160       WMIDPH(J,K)= WPH(J,K) - 0.5*DP(L) * DIVPH(J,K)
C       Convert omega to latitude space
        CALL P2L( WLA, WMIDPH, K2 )
C
C       Compute quantities for special diagnostics/ section data
C       Compute zonal-mean vertical gradient of zonal wind and
C       pot. temperature at level L (using already computed
C       zonal-mean values of u,v,Theta at level L+1)
        IF (L.EQ.1) THEN
          DO 170 K=1,K2
            DUDPL(K)= (VELZON(K,2,JX)-VELZON(K,1,JX))/(PLV(2)-PLV(1))
            DTDPL(K)= (POTZON(K,2)-POTZON(K,1))/(PLV(2)-PLV(1))
  170       CONTINUE
        ELSE IF (L.EQ.L1) THEN
          DO 180 K=1,K2
            DUDPL(K)=   (VELZON(K,L1,JX) - VELZON(K,L1-1,JX))
     1                / (PLV(L1)         - PLV(L1-1))
            DTDPL(K)=   (POTZON(K,L1) - POTZON(K,L1-1))
     1                / (PLV(L1)      - PLV(L1-1))
  180       CONTINUE
        ELSE
          DO 190 K=1,K2
            DUDPL(K)=   (VELZON(K,L+1,JX) - VELZON(K,L-1,JX))
     1                / (PLV(L+1)         - PLV(L-1))
            DTDPL(K)=   (POTZON(K,L+1) - POTZON(K,L-1))
     1                / (PLV(L+1)      - PLV(L-1))
  190       CONTINUE
        ENDIF
C       Compute zonal-mean absolute vorticity at level L
        CALL ZAVGE( AVORL, AVORPH(1,1,LPZERO) )
#ifdef ON_SUN4
C       *** dummy line to avoid compiler bug in SUN f77 SC4.0 ***
        write(line,'(3I10)') ISTP, L, LPZERO
#endif
C
C       Save section data, if need be
        CALL DIASAV( ISTP, L,
     1               VPH(1,1,JX,LPZERO), VPH(1,1,JY,LPZERO), WMIDPH,
     2               SCALPH(1,1,JPOT,LPZERO), AVORPH(1,1,LPZERO),
     3               PVORPH, DUDPL, DTDPL, AVORL  )
C
C       Compute kinetic energy at level L
        DO 210 K=1,K2
          DO 210 J=1,K1
            FPH(J,K)= 0.5*(  VPH(J,K,JX,LPZERO)**2
     1                     + VPH(J,K,JY,LPZERO)**2 )
  210       CONTINUE
C       Compute zonal-mean kinetic energy
        CALL ZAVGE( KEZON(1,L), FPH )
C
C       Convert horizontal velocity to spectral space
C       (Note: This is inaccurate at the highest wavenumber)
        CALL ZEROSP( FSP1, N1 )
        CALL ZEROSP( FSP2, N1 )
        CALL SPECTR( FSP1, VPH(1,1,JX,LPZERO) )
        CALL SPECTR( FSP2, VPH(1,1,JY,LPZERO) )
C
        TEMP= DP(L)/PSURF
C       Accumulate m-spectrum of eddy kinetic energy
        DO 220 M=1,M1
          DO 220 N=M,N1
            KESPEC(M)= KESPEC(M) + TEMP*
     1                 (CABS(FSP1(M,N))**2 + CABS(FSP2(M,N))**2)
  220       CONTINUE
C       Accumulate n-spectrum of eddy kinetic energy
        DO 230 N=1,N1
          DO 230 M=1,MIN0(N,M1)
            KEPOW(N)= KEPOW(N) + TEMP*
     1                (CABS(FSP1(M,N))**2 + CABS(FSP2(M,N))**2)
  230       CONTINUE
C       Accumulate n-spectrum of eddy enstrophy
        DO 240 N=1,N1
          DO 240 M=1,MIN0(N,M1)
            ENPOW(N)= ENPOW(N) + TEMP*CABS(PGQSP(M,N,JVOR,L))**2
  240       CONTINUE
C
C       Compute horizontal fluxes of ang. mom. and pot. tem.
        DO 260 K=1,K2
          AMFLUX(K,L,1)= 0.0
          PTFLUX(K,L)= 0.0
          DO 250 J=1,K1
            AMFLUX(K,L,1)= AMFLUX(K,L,1) +
     1                VPH(J,K,JY,LPZERO)*VPH(J,K,JX,LPZERO)*COSPHI(K)
            PTFLUX(K,L)= PTFLUX(K,L) +
     1                VPH(J,K,JY,LPZERO)*SCALPH(J,K,JPOT,LPZERO)
  250       CONTINUE
          AMFLUX(K,L,1)= AMFLUX(K,L,1) / K1
          PTFLUX(K,L)= PTFLUX(K,L) / K1
C         Eddy part of angular momentum flux
          EAMFLX(K,L,1)=  AMFLUX(K,L,1)
     1                  - VELZON(K,L,JY)*VELZON(K,L,JX)*COSPHI(K)
C         Eddy part of potential temperature flux
          EPTFLX(K,L)=  PTFLUX(K,L)
     1                - VELZON(K,L,JY)*POTZON(K,L)
  260     CONTINUE
#if (NTRACE > 0)
C       Compute horizontal fluxes of tracers
        DO 280 ITR=JTR1,NPGQ
          DO 280 K=1,K2
            TRFLUX(K,L,ITR)= 0.0
            DO 270 J=1,K1
              TRFLUX(K,L,ITR)= TRFLUX(K,L,ITR) +
     1                  VPH(J,K,JY,LPZERO)*SCALPH(J,K,ITR,LPZERO)
  270         CONTINUE
  280     TRFLUX(K,L,ITR)= TRFLUX(K,L,ITR) / K1
#endif
C
C       Compute quantities for special diagnostics/ section data
C       Compute omega at level L+1/2
        DO 320 K=1,K2
          DO 320 J=1,K1
  320       WPH(J,K)= WPH(J,K) - DP(L) * DIVPH(J,K)
C       Compute zonal-mean omega
        CALL ZAVGE( WZON(1,L), WPH )
C       Compute vertical angular momentum flux
        DO 340 K=1,K2
          AMFLUX(K,L,2)= 0.0
          DO 330 J=1,K1
  330       AMFLUX(K,L,2)= AMFLUX(K,L,2)+WPH(J,K)*UBARPH(J,K)*COSPHI(K)
          AMFLUX(K,L,2)= AMFLUX(K,L,2) / K1
C         Eddy part of angular momentum flux
          EAMFLX(K,L,2)=  AMFLUX(K,L,2)
     1                  - WZON(K,L)*UBARL(K)*COSPHI(K)
  340     CONTINUE
C       Compute zonal-mean divergence at level L
        CALL ZAVGE( DIVZON, DIVPH )
C       Compute vertical zonal momentum flux at level L+1/2
        DO 350 K=1,K2
  350     VUFLXZ(K,LPHALF)= -WZON(K,L)*UBARL(K)
C       Compute ang-mom tendency due to Eulerian MMC
        DO 360 K=1,K2
          EMMCT(K,L)= COSPHI(K)*(  AVORL(K)*VELZON(K,L,JY)
     1                           - DIVZON(K)*VELZON(K,L,JX)
     2               + (VUFLXZ(K,LPHALF) - VUFLXZ(K,LMHALF))/DP(L) )
  360     CONTINUE
C
        IF (L.EQ.LDIAG) THEN
C         Compute E-P flux at special diagnostic level
          DO 450 K=1,K2
            EPSPEC(0,K,1)= 0.0
            EPSPEC(0,K,2)= 0.0
            DO 440 M=1,M1
C             Spectrally decomposed induced streamfunction
              PSIICM(M)=  2.*REAL(CONJG(VLA(M,K))*PTLA(M,K))*COSPHI(K)
     1                  / DTDPL(K)
C             Horizontal component of spectrally decomposed E-P flux
              EPSPEC(M,K,1)=-2.*REAL(CONJG(VLA(M,K))*ULA(M,K))*COSPHI(K)
     1                      + PSIICM(M)*DUDPL(K)
C             Vertical component of spectrally decomposed E-P flux
              EPSPEC(M,K,2)=-2.*REAL(CONJG(WLA(M,K))*ULA(M,K))*COSPHI(K)
     1                      + PSIICM(M)*AVORL(K)
  440         CONTINUE
  450       CONTINUE
        ENDIF
C
  500   CONTINUE
C
C     Compute streamfunction of induced circulation at level L+1/2
      DO 610 L=1,L1-1
        DO 610 K=1,K2
          PSIIC(K,L)= COSPHI(K)*0.5*(EPTFLX(K,L+1)+EPTFLX(K,L))
     1               *(PLV(L+1)-PLV(L)) / (POTZON(K,L+1)-POTZON(K,L))
  610     CONTINUE
      DO 620 K=1,K2
  620   PSIIC(K,L1)= PSIIC(K,L1-1)
C
C     Compute zonal-mean cross-section of Eliassen-Palm flux
      DO 790 L=1,L1
C       Compute horizontal component of E-P flux at level L
        DO 710 K=1,K2
  710     EPFLUX(K,L,1)= -EAMFLX(K,L,1)
C
        IF (L.GT.1) THEN
          DO 720 K=1,K2
            EPFLUX(K,L,1)= EPFLUX(K,L,1) + 0.5*PSIIC(K,L-1)
     1          * (VELZON(K,L,JX)-VELZON(K,L-1,JX))/(PLV(L)-PLV(L-1))
  720       CONTINUE
        ENDIF
C
        IF (L.LT.L1) THEN
          DO 730 K=1,K2
            EPFLUX(K,L,1)= EPFLUX(K,L,1) + 0.5*PSIIC(K,L)
     1          * (VELZON(K,L+1,JX)-VELZON(K,L,JX))/(PLV(L+1)-PLV(L))
  730       CONTINUE
        ENDIF
C       Compute vertical component of E-P flux at level L+1/2
        DO 750 K=1,K2
  750     EPFLUX(K,L,2)= -EAMFLX(K,L,2)+PSIIC(K,L)*AVRBZ(K,L)
  790   CONTINUE
C
      RETURN
      END
 
      SUBROUTINE DIAEND( FILNAM )
C DIAEND should be called at the end of the diagnostic sampling period.
C The diagnostic results are written to file PREFIX.da. The file name
C is returned in the variable FILNAM.
C
#include "mgrid.h"
#include "tmarch.h"
#include "diacom.h"
      CHARACTER*(*) FILNAM
      INTEGER I, J, K, L, M, N, IPGQ
      REAL FPH(K1MAX,K2MAX), FPH0(K2MAX), MERFLX, MSETEN
      COMPLEX FSP(0:M1MAX, 0:N1MAX), FSP2(0:M1MAX, 0:N1MAX)
      EXTERNAL YLA2SP, YSP2LA
#if (NTRACE > 0)
      INTEGER ITR
#endif
C
      REAL PI
      PARAMETER (PI=3.141592653589793)
C     Coefficients for water-air systems
C     (R, Cp for dry air (in J/(kg*K)),
C      latent heat of condendation (in J/kg) )
      REAL RDRY, CPDRY, LHEAT
      PARAMETER (RDRY= 287.0, CPDRY=1004.0, LHEAT=2.5E6)
C
      IF (NSAMPL.LT.1)
     1  CALL SPERR( 'DIAEND', 'Insufficient data for time-averages')
C     Calculate time-averages
      DO 100 L=1,L1
        DO 100 IPGQ=1,NPGQ
          DO 100 N=0,N1
            DO 100 M=0,MIN0(N,M1)
  100         PGQAVG(M,N,IPGQ,L)= PGQAVG(M,N,IPGQ,L) / NSAMPL
C
      DO 110 I=1,2
        DO 110 L=1,L1
          DO 110 K=1,K2
            AMFAVG(K,L,I)= AMFAVG(K,L,I) / NSAMPL
            EAMFAV(K,L,I)= EAMFAV(K,L,I) / NSAMPL
            EPFAVG(K,L,I)= EPFAVG(K,L,I) / NSAMPL
  110       CONTINUE
C
      DO 120 L=1,L1
       DO 120 K=1,K2
         KEAVG(K,L)= KEAVG(K,L) / NSAMPL
         PTFAVG(K,L)= PTFAVG(K,L) / NSAMPL
         PICAVG(K,L)= PICAVG(K,L) / NSAMPL
         EMTAVG(K,L)= EMTAVG(K,L) / NSAMPL
         PVAVG(K,L)= PVAVG(K,L) / NSAMPL
         SPSECT(K,L)= SPSECT(K,L) / NSAMPL
  120    CONTINUE
C
      DO 130 I=1,2
        DO 130 K=1,K2
          DO 130 M=0,M1
            EPSAVG(M,K,I)= EPSAVG(M,K,I) / NSAMPL
  130       CONTINUE
C
      DO 140 M=0,M1
  140   KESAVG(M)= KESAVG(M) / NSAMPL
C
      DO 150 N=0,N1
        KEPAVG(N)= KEPAVG(N) / NSAMPL
  150   ENPAVG(N)= ENPAVG(N) / NSAMPL
C
      DO 170 L=1,L1
        DO 170 K=1,K2
          UFOAVG(K,L)= UFOAVG(K,L) / NSAMPL
          TFOAVG(K,L)= TFOAVG(K,L) / NSAMPL
  170     KEDAVG(K,L)= KEDAVG(K,L) / NSAMPL
C
C     Compute diagnostics for time-mean state
      CALL DIACAL( -1, PGQAVG, PGQAVG )
C
C     Compute temperature variance and KE at LDIAG
      DO 180 K=1,K2
        DO 180 J=1,K1
          TVAR(J,K)= TVAR(J,K) / NSAMPL
  180     KEFAVG(J,K)= KEFAVG(J,K) / NSAMPL
#if (NTRACE > 0)
C     Compute time-averaged tracer flux
      DO 190 ITR=JTR1,NPGQ
        DO 190 L=1,L1
          DO 190 K=1,K2
  190       TRFAVG(K,L,ITR)= TRFAVG(K,L,ITR) / NSAMPL
C     Compute transient tracer variance at LDIAG
      DO 200 ITR=JTR1,NPGQ
        DO 200 K=1,K2
          DO 200 J=1,K1
  200       TRCVAR(J,K,ITR)=  TRCVAR(J,K,ITR) / NSAMPL
#endif
C
      DO 270 L=1,L1
C       Compute divergence of horizontal E-P flux at level L
C       First, the time-average
        DO 210 K=1,K2
  210     FPH0(K)= EPFAVG(K,L,1) * COSPHI(K)
        CALL ZEROSP( FSP, N1 )
        CALL MEGRAD( FSP, FPH0 )
        CALL PHYS0( EFDAVG(1,L), FSP )
C       Next, the stationary eddy contribution
        DO 220 K=1,K2
  220     FPH0(K)= EPFLUX(K,L,1) * COSPHI(K)
        CALL ZEROSP( FSP, N1 )
        CALL MEGRAD( FSP, FPH0 )
        CALL PHYS0( EFDIV(1,L), FSP )
C
C       Compute divergence of horizontal eddy ang-mom flux at level L
C       First, the time-average
        DO 230 K=1,K2
  230     FPH0(K)= EAMFAV(K,L,1) * COSPHI(K)
        CALL ZEROSP( FSP, N1 )
        CALL MEGRAD( FSP, FPH0 )
        CALL PHYS0( EAFDAV(1,L), FSP )
C       Next, the stationary contribution
        DO 240 K=1,K2
  240     FPH0(K)= EAMFLX(K,L,1) * COSPHI(K)
        CALL ZEROSP( FSP, N1 )
        CALL MEGRAD( FSP, FPH0 )
        CALL PHYS0( EAFDIV(1,L), FSP )
C
C       Add divergence of vertical E-P and eddy ang-mom flux at level L
        IF (L.GT.1) THEN
          DO 250 K=1,K2
            EFDAVG(K,L)= EFDAVG(K,L)
     1                  +(EPFAVG(K,L,2)-EPFAVG(K,L-1,2))/DP(L)
            EFDIV(K,L) = EFDIV(K,L)
     1                  +(EPFLUX(K,L,2)-EPFLUX(K,L-1,2))/DP(L)
            EAFDAV(K,L)= EAFDAV(K,L)
     1                  +(EAMFAV(K,L,2)-EAMFAV(K,L-1,2))/DP(L)
            EAFDIV(K,L)= EAFDIV(K,L)
     1                  +(EAMFLX(K,L,2)-EAMFLX(K,L-1,2))/DP(L)
  250       CONTINUE
        ELSE
C         Uppermost level
          DO 260 K=1,K2
            EFDAVG(K,L)= EFDAVG(K,L) + EPFAVG(K,L,2)/DP(1)
            EFDIV(K,L) = EFDIV(K,L)  + EPFLUX(K,L,2)/DP(1)
            EAFDAV(K,L)= EAFDAV(K,L) + EAMFAV(K,L,2)/DP(1)
            EAFDIV(K,L)= EAFDIV(K,L) + EAMFLX(K,L,2)/DP(1)
  260       CONTINUE
        ENDIF
  270   CONTINUE
C     Make implicit corrections to ang-mom tendencies
      DO 370 L=1,L1
C       Zero velocity potential tendency
        CALL ZEROSP( FSP2, N1 )
C
        CALL ZEROSP( FSP, N1 )
        CALL MEGRAD( FSP, EFDAVG(1,L) )
        DO 310 N=0,N1
          DO 310 M=0,MIN0(N,M1)
  310       FSP(M,N)= - UD8COR(N) * FSP(M,N)
        CALL IDELSQ( FSP, FSP, N1)
        CALL HVEL0( EFDAVG(1,L), FPH0, FSP, FSP2 )
C
        CALL ZEROSP( FSP, N1 )
        CALL MEGRAD( FSP, EFDIV(1,L) )
        DO 320 N=0,N1
          DO 320 M=0,MIN0(N,M1)
  320       FSP(M,N)= - UD8COR(N) * FSP(M,N)
        CALL IDELSQ( FSP, FSP, N1)
        CALL HVEL0( EFDIV(1,L), FPH0, FSP, FSP2 )
C
        CALL ZEROSP( FSP, N1 )
        CALL MEGRAD( FSP, EAFDAV(1,L) )
        DO 330 N=0,N1
          DO 330 M=0,MIN0(N,M1)
  330       FSP(M,N)= - UD8COR(N) * FSP(M,N)
        CALL IDELSQ( FSP, FSP, N1)
        CALL HVEL0( EAFDAV(1,L), FPH0, FSP, FSP2 )
C
        CALL ZEROSP( FSP, N1 )
        CALL MEGRAD( FSP, EAFDIV(1,L) )
        DO 340 N=0,N1
          DO 340 M=0,MIN0(N,M1)
  340       FSP(M,N)= - UD8COR(N) * FSP(M,N)
        CALL IDELSQ( FSP, FSP, N1)
        CALL HVEL0( EAFDIV(1,L), FPH0, FSP, FSP2 )
C
        CALL ZEROSP( FSP, N1 )
        CALL MEGRAD( FSP, EMTAVG(1,L) )
        DO 350 N=0,N1
          DO 350 M=0,MIN0(N,M1)
  350       FSP(M,N)= - UD8COR(N) * FSP(M,N)
        CALL IDELSQ( FSP, FSP, N1)
        CALL HVEL0( EMTAVG(1,L), FPH0, FSP, FSP2 )
C
C       Convert u-bar tendency to ang-mom tendency
        DO 360 K=1,K2
          EFDAVG(K,L)= EFDAVG(K,L) * COSPHI(K)
          EFDIV(K,L) = EFDIV(K,L)  * COSPHI(K)
          EAFDAV(K,L)= EAFDAV(K,L) * COSPHI(K)
          EAFDIV(K,L)= EAFDIV(K,L) * COSPHI(K)
          EMTAVG(K,L)= EMTAVG(K,L) * COSPHI(K)
  360     CONTINUE
  370   CONTINUE
C
      DO 440 L=1,L1
C       Compute meridional gradient of time-averaged abs. vorticity
C       Copy time-averaged rel. vorticity
        CALL SPCOPY( FSP, PGQAVG(0,0,JVOR,L), N1 )
C       Add planetary vorticity
        FSP(0,1)= FSP(0,1) + CMPLX(FSP01, 0.)
C       Convert zonal-mean absolute vorticity to physical space
        CALL PHYS0( AVZON(1,L), FSP )
C       Compute meridional gradient of absolute vorticity
        CALL S2P( FPH, FSP, YSP2LA, N1 )
        CALL MULCOS( FPH, FPH, A0INV, +1)
C       Compute zonal-average
        CALL ZAVGE( AVGZON(1,L), FPH )
C
C       Compute residual omega
        CALL ZEROSP( FSP, N1 )
        CALL MEGRAD( FSP, PICAVG(1,L) )
        CALL PHYS0( WRZON(1,L), FSP )
        DO 410 K=1,K2
  410     WRZON(K,L)= WZON(K,L) + WRZON(K,L)
C
C       Compute (implicit) horizontal diffusion tendency
        DO 420 N=0,N1
          DO 420 M=0,MIN0(N,M1)
  420       FSP(M,N)= - UD8COR(N) * UD8FAC(N) * PGQAVG(M,N,JVOR,L)
C       Compute streamfunction tendency
        CALL IDELSQ( FSP, FSP, N1)
C       Zero velocity potential tendency
        CALL ZEROSP( FSP2, N1 )
C       Compute u-bar diffusion tendency
        CALL HVEL0( HDIFFT(1,L), FPH0, FSP, FSP2 )
C       Convert u-bar tendency to ang-mom tendency
        DO 430 K=1,K2
  430     HDIFFT(K,L)= HDIFFT(K,L) * COSPHI(K)
  440   CONTINUE
C
C     Compute time-averaged flux quantities
      DO 510 K=1,K2
        SOLZON(K)= SOLZON(K) / NSAMPL
        ALBZON(K)= ALBZON(K) / NSAMPL
        OSRZON(K)= OSRZON(K) / NSAMPL
        OLRZON(K)= OLRZON(K) / NSAMPL
        ISRZON(K)= ISRZON(K) / NSAMPL
        ILRZON(K)= ILRZON(K) / NSAMPL
        FEVPZN(K)= FEVPZN(K) / NSAMPL
        FSNSZN(K)= FSNSZN(K) / NSAMPL
        FRADZN(K)= FRADZN(K) / NSAMPL
        FNETZN(K)= FNETZN(K) / NSAMPL
        MSAPZN(K)= MSAPZN(K) / NSAMPL
        MCAPZN(K)= MCAPZN(K) / NSAMPL
        EVAPZN(K)= EVAPZN(K) / NSAMPL
        TAUOCN(K)= TAUOCN(K) / NSAMPL
        FOCN(K)= FOCN(K) / NSAMPL
        EMPOCN(K)= EMPOCN(K) / NSAMPL
        EMPLAN(K)= EMPLAN(K) / NSAMPL
        SSTZON(K)= SSTZON(K) / NSAMPL
        LSTZON(K)= LSTZON(K) / NSAMPL
  510   CONTINUE
C
      DO 520 K=1,K2
        DO 520 L=1,L1
          SWHAVG(K,L)= SWHAVG(K,L) / NSAMPL
          LWHAVG(K,L)= LWHAVG(K,L) / NSAMPL
          MSAHAV(K,L)= MSAHAV(K,L) / NSAMPL
          MCAHAV(K,L)= MCAHAV(K,L) / NSAMPL
  520     DCAHAV(K,L)= DCAHAV(K,L) / NSAMPL
C
      DO 530 K=1,K2
        DO 530 J=1,K1
          TSRFAV(J,K)= TSRFAV(J,K) / NSAMPL
          ALBAVG(J,K)= ALBAVG(J,K) / NSAMPL
          PCPAVG(J,K)= PCPAVG(J,K) / NSAMPL
          EVPAVG(J,K)= EVPAVG(J,K) / NSAMPL
          FNETAV(J,K)= FNETAV(J,K) / NSAMPL
          OLRAVG(J,K)= OLRAVG(J,K) / NSAMPL
          TAUXAV(J,K)= TAUXAV(J,K) / NSAMPL
          TAUYAV(J,K)= TAUYAV(J,K) / NSAMPL
          ICEAVG(J,K)= ICEAVG(J,K) / NSAMPL
          SPFLD1(J,K)= (SPFLD1(J,K)/NSAMPL - TSRFAV(J,K)**2)
  530     SPFLD2(J,K)= (SPFLD2(J,K)/NSAMPL - PCPAVG(J,K)**2)
C
C     Compute meridional heat transport
      MERFLX= 0.0
      DO 550 K=1,K2
C       Compute vertically integrated moist static energy tendency
        MSETEN= 0.0
        DO 540 L=1,L1
#if (NTRACE > 0)
C         Latent heat contribution
          MSETEN= MSETEN + (DP(L)/G0)*LHEAT*TRTEND(K,L,JTR1)
#endif
C         Temperature contribution
          MSETEN= MSETEN + (DP(L)/G0)*CPDRY*TTEND(K,L)
C         Kinetic energy dissipation contribution (-ve)
  540     MSETEN= MSETEN - (DP(L)/G0)*KEDAVG(K,L)
        MERFLX= MERFLX + 4.0*PI*(A0**2)*G(K)
     1                 *(SOLZON(K)-OSRZON(K)-OLRZON(K)-FNETZN(K)-MSETEN)
  550   MERHT(K)= MERFLX
C
      IF (IUADG .GT. 0) THEN
C       Write out diagnostic results
        FILNAM= PREFIX//'.da'
        CALL LJUST( FILNAM )
        OPEN(IUADG, FILE=FILNAM, STATUS='NEW')
        CALL DIAPUT( IUADG )
        CLOSE( IUADG )
        WRITE(*,*) 'Saved diagnostic data to ', FILNAM
      ENDIF
C
C     Close files
      IF (IUATS .GT. 0) CLOSE (IUATS )
      IF (IUAHS .GT. 0) CLOSE (IUAHS )
      IF (IUAVS .GT. 0) CLOSE (IUAVS )
      IF (IUAZS .GT. 0) CLOSE (IUAZS )
C
      RETURN
      END
 
      SUBROUTINE QPUTHS( IUNIT, IRECD, FPH, HEADER )
C QPUTHS writes physical space values of quantity FPH to unit IUNIT.
C IRECD is the record number, HEADER is a 48 character header string.
C
#include "spgrid.h"
      INTEGER IUNIT, IRECD
      REAL FPH(K1MAX,K2MAX)
      CHARACTER*48 HEADER
C
      INTEGER J, K
      REAL FPRIME(K1MAX,K2MAX), FMEAN
      REAL GAVGE
C
C     Compute area-weighted global-mean of the field
      FMEAN= GAVGE(FPH)
C     Compute deviations from the global mean
      DO 10 K=1,K2
        DO 10 J=1,K1
   10     FPRIME(J,K)= FPH(J,K) - FMEAN
C     Write to fortran unit
      WRITE(IUNIT,REC=IRECD) HEADER, FMEAN, FLOAT(K1), FLOAT(K2),
     1                       0.0, ((FPRIME(J,K),J=1,K1),K=1,K2)
      RETURN
      END
 
      SUBROUTINE QPUTVS( IUNIT, IRECD, FVS, HEADER )
C QPUTVS writes latitude circle values of quantity FVS to unit IUNIT.
C IRECD is the record number, HEADER is a 48 character header string.
C
#include "mgrid.h"
      INTEGER IUNIT, IRECD
      REAL FVS(K1MAX,L1MAX)
      CHARACTER*48 HEADER
C
      INTEGER J, L
      REAL FPRIME(K1MAX,L1MAX), FMEAN
C
C     Compute mass-weighted section-mean of the field
      FMEAN= 0.0
      DO 10 L=1,L1
        DO 10 J=1,K1
   10     FMEAN= FMEAN + (DP(L)/(PSURF*K1))*FVS(J,L)
C     Compute deviations from the global mean
      DO 20 L=1,L1
        DO 20 J=1,K1
   20     FPRIME(J,L)= FVS(J,L) - FMEAN
C     Write to fortran unit
      WRITE(IUNIT,REC=IRECD) HEADER, FMEAN, FLOAT(K1), FLOAT(L1),
     1                       0.0, ((FPRIME(J,L),J=1,K1),L=1,L1)
      RETURN
      END
 
      SUBROUTINE QPUTZS( IUNIT, IRECD, FZS, HEADER )
C QPUTZS writes zonal-mean values of quantity FZS to unit IUNIT.
C IRECD is the record number, HEADER is a 48 character header string.
C
#include "mgrid.h"
      INTEGER IUNIT, IRECD
      REAL FZS(K2MAX,L1MAX)
      CHARACTER*48 HEADER
C
      INTEGER K, L
      REAL FPRIME(K2MAX,L1MAX), FMEAN
C
C     Compute mass-weighted section-mean of the field
      FMEAN= 0.0
      DO 10 L=1,L1
        DO 10 K=1,K2
   10     FMEAN= FMEAN + (DP(L)/PSURF)*G(K)*FZS(K,L)
C     Compute deviations from the global mean
      DO 20 L=1,L1
        DO 20 K=1,K2
   20     FPRIME(K,L)= FZS(K,L) - FMEAN
C     Write to fortran unit
      WRITE(IUNIT,REC=IRECD) HEADER, FMEAN, FLOAT(K2), FLOAT(L1),
     1                       0.0, ((FPRIME(K,L),K=1,K2),L=1,L1)
      RETURN
      END
 
      SUBROUTINE DIAPUT( IUNIT )
C DIAPUT writes out diagnostic results onto unit IUNIT
C
#include "mgrid.h"
#include "diacom.h"
      INTEGER IUNIT
      INTEGER I, J, K, L, M, N
      REAL DAYSEC, TAUINT
      CHARACTER FMT*40, STROUT*72, TIMST0*10, TIMST1*10
#if (NTRACE > 0)
      INTEGER ITR
#endif
C
      FMT= '(1X,1P,5E15.6)'
C     Seconds per day
      IF (NTSPD .GT. 0) THEN
        DAYSEC= NTSPD * DTSTEP
      ELSE
        DAYSEC= DTSTEP / FLOAT(-NTSPD)
      ENDIF
C     Sampling time-interval
      TAUINT= NDSKIP * DTSTEP
C
      WRITE(TIMST0,'(F10.2)') TAU0/DAYSEC
      WRITE(TIMST1,'(F10.2)') TAU1/DAYSEC
C
C     Preface to diagnostic file
      WRITE(IUNIT,'(1X,A)') RUNTTL
      STROUT= 'Diagnostics file saved by DIAPUT version 5.1'
      WRITE(IUNIT,'(1X,A)') STROUT
      STROUT= 'Started on '// DAYTM0
      WRITE(IUNIT,'(1X,A)') STROUT
      STROUT= 'Sampled from model day '//TIMST0//' to '//TIMST1
      WRITE(IUNIT,'(1X,A)') STROUT
      WRITE(IUNIT,'(1X,A,F8.2,A)')
     1            'Sampling interval = ', TAUINT/3600., ' hours'
C
C     Model/planetary parameters
      WRITE(IUNIT,'(A)') 'M1, N1, K1, K2, L1, NTRC'
      WRITE(IUNIT,'(1X,6I12)') M1, N1, K1, K2, L1, NPGQ-3
      WRITE(IUNIT,'(A)')
     1                'IDSET, ISTP0, ISTP1, NTSPD, NYRDAY, LDIAG'
      WRITE(IUNIT,'(1X,6I12)')
     1                 IDSET, ISTP0, ISTP1, NTSPD, NYRDAY, LDIAG
      WRITE(IUNIT,'(A)')
     1                'NSAMPL, NDSKIP, NTFREQ, NSFREQ, NTSAMP, NSSAMP'
      WRITE(IUNIT,'(1X,6I12)')
     1                 NSAMPL, NDSKIP, NTFREQ, NSFREQ, NTSAMP, NSSAMP
      WRITE(IUNIT,'(A)') '0, 0, 0, 0, 0, 0'
      WRITE(IUNIT,'(1X,6I12)') 0, 0, 0, 0, 0, 0
      WRITE(IUNIT,'(A)') 'A0, OMEGA0, F0, FSP01'
      WRITE(IUNIT,FMT) A0, OMEGA0, F0, FSP01
      WRITE(IUNIT,'(A)') 'RGAS, CP, KAPPA, G0, PSURF'
      WRITE(IUNIT,FMT) RGAS, CP, KAPPA, G0, PSURF
      WRITE(IUNIT,'(A)') 'DTSTEP, TAUINT, TAU0, TAU1'
      WRITE(IUNIT,FMT) DTSTEP, TAUINT, TAU0, TAU1
      WRITE(IUNIT,'(A)') 'YRPHA, LANWD, 0., 0., 0.'
      WRITE(IUNIT,FMT) YRPHA, LANWD, 0., 0., 0.
C
      WRITE(IUNIT,'(A)') 'DEGLON(J)'
      WRITE(IUNIT,FMT) (DEGLON(J),J=1,K1)
      WRITE(IUNIT,'(A)') 'DEGLAT(K), MU(K), COSPHI(K), G(K)'
      DO 10 K=1,K2
   10   WRITE(IUNIT,FMT) DEGLAT(K), MU(K), COSPHI(K), G(K)
      WRITE(IUNIT,'(A)') 'PLV(L), PKLV(L), DP(L), TSTDLV(L), ZSTDLV(L)'
      DO 20 L=1,L1
   20   WRITE(IUNIT,FMT) PLV(L), PKLV(L), DP(L), TSTDLV(L), ZSTDLV(L)
      WRITE(IUNIT,'(A)') 'PHLV(L)'
      DO 30 L=0,L1
   30   WRITE(IUNIT,FMT) PHLV(L)
C
C     Descriptive strings for special time-series
      WRITE(IUNIT,'(A)') 'VOR12D-R'
      WRITE(IUNIT,'(A)') 'VOR12D-I'
C     Zonal-mean cross-section of horizontal velocity
      WRITE(IUNIT,'(A)') 'VELZON(K,L,I)'
      WRITE(IUNIT,FMT) (((VELZON(K,L,I),K=1,K2),L=1,L1),I=1,2)
C     Zonal-mean cross-section of potential temperature
      WRITE(IUNIT,'(A)') 'POTZON(K,L)'
      WRITE(IUNIT,FMT) ((POTZON(K,L),K=1,K2),L=1,L1)
C     Zonal-mean cross-section of omega
      WRITE(IUNIT,'(A)') 'WZON(K,L)'
      WRITE(IUNIT,FMT) ((WZON(K,L),K=1,K2),L=1,L1)
C     Zonal-mean cross-section of mer. gradient of absolute vorticity
      WRITE(IUNIT,'(A)') 'AVGZON(K,L)'
      WRITE(IUNIT,FMT) ((AVGZON(K,L),K=1,K2),L=1,L1)
C     Zonal-mean cross-section of kinetic energy
      WRITE(IUNIT,'(A)') 'KEZON(K,L)'
      WRITE(IUNIT,FMT) ((KEZON(K,L),K=1,K2),L=1,L1)
      WRITE(IUNIT,'(A)') 'KEAVG(K,L)'
      WRITE(IUNIT,FMT) ((KEAVG(K,L),K=1,K2),L=1,L1)
C     Zonal-mean cross-section of eddy heat flux
      WRITE(IUNIT,'(A)') 'PTFLUX(K,L)'
      WRITE(IUNIT,FMT) ((PTFLUX(K,L),K=1,K2),L=1,L1)
      WRITE(IUNIT,'(A)') 'PTFAVG(K,L)'
      WRITE(IUNIT,FMT) ((PTFAVG(K,L),K=1,K2),L=1,L1)
C     Zonal-mean cross-section of induced streamfunction
      WRITE(IUNIT,'(A)') 'PSIIC(K,L)'
      WRITE(IUNIT,FMT) ((PSIIC(K,L),K=1,K2),L=1,L1)
      WRITE(IUNIT,'(A)') 'PICAVG(K,L)'
      WRITE(IUNIT,FMT) ((PICAVG(K,L),K=1,K2),L=1,L1)
C     Zonal-mean cross-section of angular momentum flux
      WRITE(IUNIT,'(A)') 'AMFLUX(K,L,I)'
      WRITE(IUNIT,FMT) (((AMFLUX(K,L,I),K=1,K2),L=1,L1),I=1,2)
      WRITE(IUNIT,'(A)') 'AMFAVG(K,L,I)'
      WRITE(IUNIT,FMT) (((AMFAVG(K,L,I),K=1,K2),L=1,L1),I=1,2)
C     Zonal-mean cross-section of Eliassen-Palm flux
      WRITE(IUNIT,'(A)') 'EPFLUX(K,L,I)'
      WRITE(IUNIT,FMT) (((EPFLUX(K,L,I),K=1,K2),L=1,L1),I=1,2)
      WRITE(IUNIT,'(A)') 'EPFAVG(K,L,I)'
      WRITE(IUNIT,FMT) (((EPFAVG(K,L,I),K=1,K2),L=1,L1),I=1,2)
C     Zonal-mean cross-section of E-P flux divergence
      WRITE(IUNIT,'(A)') 'EFDIV(K,L)'
      WRITE(IUNIT,FMT) ((EFDIV(K,L),K=1,K2),L=1,L1)
      WRITE(IUNIT,'(A)') 'EFDAVG(K,L)'
      WRITE(IUNIT,FMT) ((EFDAVG(K,L),K=1,K2),L=1,L1)
C     Zonal-mean cross-section of Ertel's potential vorticity
      WRITE(IUNIT,'(A)') 'PVZON(K,L)'
      WRITE(IUNIT,FMT) ((PVZON(K,L),K=1,K2),L=1,L1)
      WRITE(IUNIT,'(A)') 'PVAVG(K,L)'
      WRITE(IUNIT,FMT) ((PVAVG(K,L),K=1,K2),L=1,L1)
C
C     Spectrally decomposed E-P flux at special diagnostic level
      WRITE(IUNIT,'(A)') 'EPSPEC(M,K,I)'
      WRITE(IUNIT,FMT) (((EPSPEC(M,K,I),M=0,M1),K=1,K2),I=1,2)
      WRITE(IUNIT,'(A)') 'EPSAVG(M,K,I)'
      WRITE(IUNIT,FMT) (((EPSAVG(M,K,I),M=0,M1),K=1,K2),I=1,2)
C     Eddy kinetic energy m-spectrum
      WRITE(IUNIT,'(A)') 'KESPEC(M)'
      WRITE(IUNIT,FMT) (KESPEC(M),M=0,M1)
      WRITE(IUNIT,'(A)') 'KESAVG(M)'
      WRITE(IUNIT,FMT) (KESAVG(M),M=0,M1)
C     Eddy kinetic energy n-spectrum
      WRITE(IUNIT,'(A)') 'KEPAVG(N)'
      WRITE(IUNIT,FMT) (KEPAVG(N),N=0,N1)
C     Eddy enstrophy n-spectrum
      WRITE(IUNIT,'(A)') 'ENPAVG(N)'
      WRITE(IUNIT,FMT) (ENPAVG(N),N=0,N1)
C
C     Zonal-mean cross-section of absolute vorticity
      WRITE(IUNIT,'(A)') 'AVZON(K,L)'
      WRITE(IUNIT,FMT) ((AVZON(K,L),K=1,K2),L=1,L1)
C     Zonal-mean cross-section of ang-mom flux divergence
      WRITE(IUNIT,'(A)') 'EAFDAV(K,L)'
      WRITE(IUNIT,FMT) ((EAFDAV(K,L),K=1,K2),L=1,L1)
C     Ang-mom tendency due to Eulerian mean meridional advection
      WRITE(IUNIT,'(A)') 'EMTAVG(K,L)'
      WRITE(IUNIT,FMT) ((EMTAVG(K,L),K=1,K2),L=1,L1)
C     Angular momentum tendency due to forcing terms
      WRITE(IUNIT,'(A)') 'UFOAVG(K,L)'
      WRITE(IUNIT,FMT) ((UFOAVG(K,L),K=1,K2),L=1,L1)
C     Theta tendency due to forcing terms
      WRITE(IUNIT,'(A)') 'TFOAVG(K,L)'
      WRITE(IUNIT,FMT) ((TFOAVG(K,L),K=1,K2),L=1,L1)
C     Zonal-mean cross-section of residual omega
      WRITE(IUNIT,'(A)') 'WRZON(K,L)'
      WRITE(IUNIT,FMT) ((WRZON(K,L),K=1,K2),L=1,L1)
C     Ang-mom tendency due to horizontal hyper-diffusion
      WRITE(IUNIT,'(A)') 'HDIFFT(K,L)'
      WRITE(IUNIT,FMT) ((HDIFFT(K,L),K=1,K2),L=1,L1)
C
C     Averaged actual ang-mom tendency over the sampling period
      WRITE(IUNIT,'(A)') 'ANGMOT(K,L)'
      WRITE(IUNIT,FMT) ((ANGMOT(K,L),K=1,K2),L=1,L1)
C     Averaged actual temperature tendency over the sampling period
      WRITE(IUNIT,'(A)') 'TTEND(K,L)'
      WRITE(IUNIT,FMT) ((TTEND(K,L),K=1,K2),L=1,L1)
C     Special section
      WRITE(IUNIT,'(A)') 'none'
      WRITE(IUNIT,FMT) ((SPSECT(K,L),K=1,K2),L=1,L1)
C
C     Velocity at special diagnostics level
      WRITE(IUNIT,'(A)') 'VELFLD(J,K)'
      WRITE(IUNIT,FMT) (((VELFLD(J,K,I),J=1,K1),K=1,K2),I=1,2)
C     Kinetic energy at special diagnostics level
      WRITE(IUNIT,'(A)') 'KEFAVG(J,K)'
      WRITE(IUNIT,FMT) ((KEFAVG(J,K),J=1,K1),K=1,K2)
C     Temperature at special diagnostics level
      WRITE(IUNIT,'(A)') 'TFLD(J,K)'
      WRITE(IUNIT,FMT) ((TFLD(J,K),J=1,K1),K=1,K2)
C     Temperature variance at special diagnostics level
      WRITE(IUNIT,'(A)') 'TVAR(J,K)'
      WRITE(IUNIT,FMT) ((TVAR(J,K),J=1,K1),K=1,K2)
C
#if (NTRACE > 0)
C     Zonal-mean cross-section of tracer mixing ratio
      WRITE(IUNIT,'(A)') 'TRCZON(K,L,ITR)'
      WRITE(IUNIT,FMT)
     1     (((TRCZON(K,L,ITR),K=1,K2),L=1,L1),ITR=JTR1,NPGQ)
C     Zonal-mean cross-section of tracer flux
      WRITE(IUNIT,'(A)') 'TRFLUX(K,L,ITR)'
      WRITE(IUNIT,FMT)
     1     (((TRFLUX(K,L,ITR),K=1,K2),L=1,L1),ITR=JTR1,NPGQ)
      WRITE(IUNIT,'(A)') 'TRFAVG(K,L,ITR)'
      WRITE(IUNIT,FMT)
     1     (((TRFAVG(K,L,ITR),K=1,K2),L=1,L1),ITR=JTR1,NPGQ)
C     Averaged actual tracer tendency over the sampling period
      WRITE(IUNIT,'(A)') 'TRTEND(K,L,ITR)'
      WRITE(IUNIT,FMT)
     1     (((TRTEND(K,L,ITR),K=1,K2),L=1,L1),ITR=JTR1,NPGQ)
C     Tracer mixing ratio at special diagnostics level
      WRITE(IUNIT,'(A)') 'TRCFLD(J,K,ITR)'
      WRITE(IUNIT,FMT)
     1     (((TRCFLD(J,K,ITR),J=1,K1),K=1,K2),ITR=JTR1,NPGQ)
C     Transient tracer variance at special diagnostics level
      WRITE(IUNIT,'(A)') 'TRCVAR(J,K,ITR)'
      WRITE(IUNIT,FMT)
     1     (((TRCVAR(J,K,ITR),J=1,K1),K=1,K2),ITR=JTR1,NPGQ)
#endif
C
C     Zonal-mean profile of downward short wave (solar) flux
      WRITE(IUNIT,'(A)') 'SOLZON(K)'
      WRITE(IUNIT,FMT) (SOLZON(K),K=1,K2)
C     Zonal-mean profile of planetary albedo
      WRITE(IUNIT,'(A)') 'ALBZON(K)'
      WRITE(IUNIT,FMT) (ALBZON(K),K=1,K2)
C     Zonal-mean profile of Outgoing Short-wave Radiation
      WRITE(IUNIT,'(A)') 'OSRZON(K)'
      WRITE(IUNIT,FMT) (OSRZON(K),K=1,K2)
C     Zonal-mean profile of Outgoing Long-wave Radiation
      WRITE(IUNIT,'(A)') 'OLRZON(K)'
      WRITE(IUNIT,FMT) (OLRZON(K),K=1,K2)
C     Zonal-mean profile of Into-surface Short-wave Radiation
      WRITE(IUNIT,'(A)') 'ISRZON(K)'
      WRITE(IUNIT,FMT) (ISRZON(K),K=1,K2)
C     Zonal-mean profile of Into-surface Long-wave Radiation
      WRITE(IUNIT,'(A)') 'ILRZON(K)'
      WRITE(IUNIT,FMT) (ILRZON(K),K=1,K2)
C     Zonal-mean profile of upward latent heat flux
      WRITE(IUNIT,'(A)') 'FEVPZN(K)'
      WRITE(IUNIT,FMT) (FEVPZN(K),K=1,K2)
C     Zonal-mean profile of upward sensible heat flux
      WRITE(IUNIT,'(A)') 'FSNSZN(K)'
      WRITE(IUNIT,FMT) (FSNSZN(K),K=1,K2)
C     Zonal-mean profile of upward radiative heat flux
      WRITE(IUNIT,'(A)') 'FRADZN(K)'
      WRITE(IUNIT,FMT) (FRADZN(K),K=1,K2)
C     Zonal-mean profile of net heat flux into surface
      WRITE(IUNIT,'(A)') 'FNETZN(K)'
      WRITE(IUNIT,FMT) (FNETZN(K),K=1,K2)
C     Zonal-mean profile of precipitation due to static adjustment
      WRITE(IUNIT,'(A)') 'MSAPZN(K)'
      WRITE(IUNIT,FMT) (MSAPZN(K),K=1,K2)
C     Zonal-mean profile of precipitation due to convective adjustment
      WRITE(IUNIT,'(A)') 'MCAPZN(K)'
      WRITE(IUNIT,FMT) (MCAPZN(K),K=1,K2)
C     Zonal-mean profile of evaporation
      WRITE(IUNIT,'(A)') 'EVAPZN(K)'
      WRITE(IUNIT,FMT) (EVAPZN(K),K=1,K2)
C     Zonal-mean profile of atmospheric meridional heat transport
      WRITE(IUNIT,'(A)') 'MERHT(K)'
      WRITE(IUNIT,FMT) (MERHT(K),K=1,K2)
C     Sector-mean profile of zonal wind stress over ocean
      WRITE(IUNIT,'(A)') 'TAUOCN(K)'
      WRITE(IUNIT,FMT) (TAUOCN(K),K=1,K2)
C     Sector-mean profile of net heat flux into ocean
      WRITE(IUNIT,'(A)') 'FOCN(K)'
      WRITE(IUNIT,FMT) (FOCN(K),K=1,K2)
C     Sector-mean profile of net freshwater flux out of ocean
      WRITE(IUNIT,'(A)') 'EMPOCN(K)'
      WRITE(IUNIT,FMT) (EMPOCN(K),K=1,K2)
C     Sector-mean profile of net freshwater flux out of land
      WRITE(IUNIT,'(A)') 'EMPLAN(K)'
      WRITE(IUNIT,FMT) (EMPLAN(K),K=1,K2)
C     Sector-mean profile of Sea Surface Temperature
      WRITE(IUNIT,'(A)') 'SSTZON(K)'
      WRITE(IUNIT,FMT) (SSTZON(K),K=1,K2)
C     Sector-mean profile of Land Surface Temperature
      WRITE(IUNIT,'(A)') 'LSTZON(K)'
      WRITE(IUNIT,FMT) (LSTZON(K),K=1,K2)
C
C     Zonal-mean cross-section of short wave heating rates
      WRITE(IUNIT,'(A)') 'SWHAVG(K,L)'
      WRITE(IUNIT,FMT) ((SWHAVG(K,L),K=1,K2),L=1,L1)
C     Zonal-mean cross-section of long wave heating rates
      WRITE(IUNIT,'(A)') 'LWHAVG(K,L)'
      WRITE(IUNIT,FMT) ((LWHAVG(K,L),K=1,K2),L=1,L1)
C     Zonal-mean cross-section of moist static adjustment heating
      WRITE(IUNIT,'(A)') 'MSAHAV(K,L)'
      WRITE(IUNIT,FMT) ((MSAHAV(K,L),K=1,K2),L=1,L1)
C     Zonal-mean cross-section of moist convective adjustment heating
      WRITE(IUNIT,'(A)') 'MCAHAV(K,L)'
      WRITE(IUNIT,FMT) ((MCAHAV(K,L),K=1,K2),L=1,L1)
C     Zonal-mean cross-section of dry convective adjustment heating
      WRITE(IUNIT,'(A)') 'DCAHAV(K,L)'
      WRITE(IUNIT,FMT) ((DCAHAV(K,L),K=1,K2),L=1,L1)
C
C     Surface temperature
      WRITE(IUNIT,'(A)') 'TSRFAV(J,K)'
      WRITE(IUNIT,FMT) ((TSRFAV(J,K),J=1,K1),K=1,K2)
C     Planetary albedo
      WRITE(IUNIT,'(A)') 'ALBAVG(J,K)'
      WRITE(IUNIT,FMT) ((ALBAVG(J,K),J=1,K1),K=1,K2)
C     Precipitation
      WRITE(IUNIT,'(A)') 'PCPAVG(J,K)'
      WRITE(IUNIT,FMT) ((PCPAVG(J,K),J=1,K1),K=1,K2)
C     Evaporation
      WRITE(IUNIT,'(A)') 'EVPAVG(J,K)'
      WRITE(IUNIT,FMT) ((EVPAVG(J,K),J=1,K1),K=1,K2)
C     Net heat flux into surface
      WRITE(IUNIT,'(A)') 'FNETAV(J,K)'
      WRITE(IUNIT,FMT) ((FNETAV(J,K),J=1,K1),K=1,K2)
C     Outgoing Long-wave Radiation
      WRITE(IUNIT,'(A)') 'OLRAVG(J,K)'
      WRITE(IUNIT,FMT) ((OLRAVG(J,K),J=1,K1),K=1,K2)
C     Zonal component of surface wind stress
      WRITE(IUNIT,'(A)') 'TAUXAV(J,K)'
      WRITE(IUNIT,FMT) ((TAUXAV(J,K),J=1,K1),K=1,K2)
C     Meridional component of surface wind stress
      WRITE(IUNIT,'(A)') 'TAUYAV(J,K)'
      WRITE(IUNIT,FMT) ((TAUYAV(J,K),J=1,K1),K=1,K2)
C     Ice height
      WRITE(IUNIT,'(A)') 'ICEAVG(J,K)'
      WRITE(IUNIT,FMT) ((ICEAVG(J,K),J=1,K1),K=1,K2)
C     Special field 1
      WRITE(IUNIT,'(A)') 'TSurf-var'
      WRITE(IUNIT,FMT) ((SPFLD1(J,K),J=1,K1),K=1,K2)
C     Special field 2
      WRITE(IUNIT,'(A)') 'Precip-var'
      WRITE(IUNIT,FMT) ((SPFLD2(J,K),J=1,K1),K=1,K2)
      RETURN
      END
 
      SUBROUTINE PHYS0(FPH0,FSP)
C PHYS0 computes the zonal-mean profile FPH0 in physical space,
C given the full spectral space representation FSP.
C Note: Truncation order is assumed to be N1 (i.e. as for a scalar)
C Input:  COMPLEX FSP(0:M1MAX,0:N1MAX) -- spectral representation of F
C Output: REAL FPH0(K2) -- zonal-mean of F, in physical space
C
C (NOTE: INEFFICIENT IMPLEMENTATION)
C
#include "spgrid.h"
      REAL FPH0(K2)
      COMPLEX FSP(0:M1MAX,0:N1MAX)
      REAL FPH(K1MAX,K2MAX)
C     Convert to physical space
      CALL PHYSIC(FPH, FSP)
C     Compute zonal-average
      CALL ZAVGE(FPH0, FPH)
      RETURN
      END
 
      SUBROUTINE HVEL0(UPH0,VPH0,PSISP,CHISP)
C HVEL0 calculates the zonal-mean of horizontal velocity components
C U, V (UPH0, VPH0) in physical space, given the spectral space
C representation of streamfunction PSISP and velocity-potential CHISP.
C Input:  COMPLEX PSISP(0:M1MAX,0:N1MAX) -- spectral streamfunction
C         COMPLEX CHISP(0:M1MAX,0:N1MAX) -- spectral velocity-potential
C Output: REAL UPH0(K2) -- zonal-mean zonal component of velocity
C         REAL VPH0(K2) -- zonal-mean meridional component of velocity
C
C (NOTE: INEFFICIENT IMPLEMENTATION)
C
#include "spgrid.h"
      REAL UPH0(K2), VPH0(K2)
      COMPLEX PSISP(0:M1MAX,0:N1MAX), CHISP(0:M1MAX,0:N1MAX)
      REAL UPH(K1MAX,K2MAX), VPH(K1MAX,K2MAX)
      COMPLEX ULA(0:M1MAX,K2MAX), VLA(0:M1MAX,K2MAX)
      INTEGER K
C     Calculate velocity components (ULA,VLA) in latitude space
      DO 10 K=1,K2
   10   CALL UVLAK(ULA(0,K),VLA(0,K),PSISP,CHISP,K)
C     Convert to physical space
      CALL L2P(UPH,ULA,K2)
      CALL L2P(VPH,VLA,K2)
C     Compute zonal-averages
      CALL ZAVGE( UPH0, UPH )
      CALL ZAVGE( VPH0, VPH )
      RETURN
      END
 
      SUBROUTINE SPEC0(FSP,FPH0)
C SPEC0 converts the zonal-mean profile FPH0 to spectral space, and
C accumulates in FSP.
C Note: Truncation order is assumed to be N1 (i.e. as for a scalar)
C Input:  COMPLEX FSP(0:M1MAX,0:N1MAX) -- old spectral coefficients
C         REAL FPH0(K2) -- zonal-mean profile F
C         REAL FPH(K1MAX,K2MAX) -- physical space representation of F
C Output: COMPLEX FSP(0:M1MAX,0:N1MAX) -- spectral representation of F
C                                         plus old coefficients
C
#include "spgrid.h"
      COMPLEX FSP(0:M1MAX,0:N1MAX)
      REAL FPH0(K2)
      REAL FPH(K1MAX,K2MAX)
      INTEGER J, K
      EXTERNAL LA2SP
C
C     Copy latitudinal profile to all longitudes
      DO 10 K=1,K2
        DO 10 J=1,K1
   10     FPH(J,K)= FPH0(K)
C     Convert to spectral space
      CALL P2S( FSP, FPH, LA2SP, N1)
      RETURN
      END
 
      SUBROUTINE MEGRAD( FSP, FPH0 )
C MEGRAD computes the "meridional gradient" of zonal-mean profile FPH0
C in spectral space, and accumulates in FSP.
C Note: Truncation order is assumed to be N1 (i.e. as for a scalar)
C Input:  COMPLEX FSP(0:M1MAX,0:N1MAX) -- old spectral coefficients
C         REAL FPH0(K2) -- zonal-mean profile F
C Output: COMPLEX FSP(0:M1MAX,0:N1MAX) -- spectral represention of
C                         " meridional gradient" of F [=(1/A0) * dF/dMU]
C                          plus old coefficients
C
C (NOTE: INEFFICIENT IMPLEMENTATION)
C
#include "spgrid.h"
      COMPLEX FSP(0:M1MAX, 0:N1MAX)
      REAL FPH0(K2)
      REAL FPH(K1MAX,K2MAX)
      INTEGER J, K
      EXTERNAL YLA2SP
C
C     Copy latitudinal profile to all longitudes (dividing by A0)
      DO 10 K=1,K2
        DO 10 J=1,K1
   10     FPH(J,K)= A0INV * FPH0(K)
C     Compute d/dMU of F in spectral space
      CALL P2S( FSP, FPH, YLA2SP, N1 )
      RETURN
      END
