C ICAM (Idealized Coupleable Atmospheric Model)
C misc.F, Version 1.1 created 24 April 1996
C
C Author: R. Saravanan, Climate and Global Dynamics Division
C                       National Center for Atmospheric Research
C                       Boulder, CO 80307-3000, U.S.A.
C                       E-mail: svn@ncar.ucar.edu
C                       URL: http://www.cgd.ucar.edu:/gds/svn/svn.html
C
C misc.F: Fortran source module
C Miscellaneous routines:
C
C ROBERT: Applies ROBERT time-filter
C LJUST:  Left justifies a string
C RJUST:  Right justifies a string
C

      SUBROUTINE ROBERT( Q0, Q1, Q2, NX0, NX, NY, ROBFAC)
C Given the fields Q0, Q1, Q2 at three successive time-steps,
C ROBERT performs a Robert-filter operation on the fields,
C with filter weight ROBFAC.
C
C Input:  REAL Q0(NX0, NY) -- field at time t-dt
C         REAL Q1(NX0, NY) -- field at time t
C         REAL Q2(NX0, NY) -- field at time t+dt
C         INTEGER NX0 -- declared upper bound
C         INTEGER NX, NY -- operative upper bounds
C         REAL ROBFAC -- filter weighting factor
C Output: REAL Q1(NX0, NY) -- filtered field at time t
C
      INTEGER NX0, NX, NY
      REAL Q0(NX0, NY), Q1(NX0, NY), Q2(NX0, NY)
      REAL ROBFAC
C
      INTEGER I, J
      REAL TEMP
C
      TEMP= 1.0 - 2.0 * ROBFAC
      DO 10 J=1,NY
        DO 10 I=1,NX
          Q1(I,J)=  TEMP*Q1(I,J) + ROBFAC*(Q0(I,J) + Q2(I,J))
   10     CONTINUE
C
      RETURN
      END

      SUBROUTINE LJUST( STR )
C LJUST left justifies a string, removing leading spaces.
C (The string must not be longer than 133 characters)
      CHARACTER*(*) STR
      CHARACTER*133 TEMSTR
      IF (STR.EQ.' ') RETURN
   10 IF (STR(1:1).EQ.' ') THEN
        TEMSTR= STR(2:)
        STR= TEMSTR
        GOTO 10
      ENDIF
      RETURN
      END
 
      SUBROUTINE RJUST( STR )
C RJUST right justifies a string, removing trailing spaces.
C (The string must not be longer than 133 characters)
      CHARACTER*(*) STR
      CHARACTER*133 TEMSTR
      INTEGER N
      IF (STR.EQ.' ') RETURN
      N= LEN(STR)
   10 IF (STR(N:N).EQ.' ') THEN
        TEMSTR= ' '//STR(1:N-1)
        STR= TEMSTR
        GOTO 10
      ENDIF
      RETURN
      END
 
C --------------------------------------------------------------------
C For GNU Emacs:
C $Msg-digest-checksum: cacfdf7f777b0927349f661deec110ef $
C --------------------------------------------------------------------
C Local Variables:
C msg-digest-active: t
C End:
C --------------------------------------------------------------------
