C ICAM (Idealized Coupleable Atmospheric Model)
C fluxsrc $Revision: 1.1 $ created $Date: 1996/01/28 20:53:43 $
C
C minterp.h: Fortran include module
C Contains interpolation coefficients for each ocean basin
C
C INTINI: Set to TRUE if interpolation coefficients have been
C         initialized. (Set to FALSE initially by ATMINI)
C CYCLIC: Set to TRUE if basin is cyclic in X-direction
C
C NOCNX:  Number of ocean grid points in X-direction
C NOCNY:  Number of ocean grid points in Y-direction
C
C XSFAC:  Stretching factor in X-direction for oceanic grid
C YSFAC:  Stretching factor in Y-direction for oceanic grid
C
C XFFAC: West/East fold back factors in X-direction,
C        to conserve fluxes at the basin edges
C YFFAC: South/North fold back factors in Y-direction,
C        to conserve fluxes at the basin edges
C
C TGRIDX/Y: Temperature grid-points in X/Y direction
C           (padded with extra values at the end)
C VGRIDX/Y: Velocity grid-points in X/Y direction, including boundaries
C
      LOGICAL INTINI, CYCLIC
      INTEGER NOCNX, NOCNY
      REAL XSFAC, XFFAC, TGRIDX, VGRIDX
      REAL YSFAC, YFFAC, TGRIDY, VGRIDY
 
      COMMON/MPINT/ XSFAC(MAXBAS), XFFAC(2,MAXBAS),
     1              YSFAC(MAXBAS), YFFAC(2,MAXBAS),
     2              TGRIDX(0:JOMAX+1), VGRIDX(0:JOMAX),
     3              TGRIDY(0:KOMAX+1), VGRIDY(0:KOMAX),
     4              NOCNX(MAXBAS), NOCNY(MAXBAS),
     5              INTINI(MAXBAS), CYCLIC(MAXBAS)
C     Coefficient tables for interpolation to ocean grid
C     (These are used by the interpolation routine FLUX1D)
C
#ifdef ZONAVG
C
C     Ocean to atmosphere interpolation (SST)
      REAL FINO2A, DYNO2A, DYO2A, LWTO2A, RWTO2A
      INTEGER MINO2A
      COMMON/MO2A/ FINO2A(0:K2MAX,MAXBAS),
     1             DYNO2A(K2MAX,MAXBAS), DYO2A(0:KOMAX,MAXBAS),
     2             LWTO2A(KOMAX,MAXBAS), RWTO2A(KOMAX,MAXBAS),
     3             MINO2A(0:K2MAX,MAXBAS)
C
C     Atmosphere to ocean interpolation (heat/E-P fluxes; T,S grid)
      REAL FINA2O, DYNA2O, DYA2O, LWTA2O, RWTA2O
      INTEGER MINA2O
      COMMON/MA2O/ FINA2O(0:KOMAX,MAXBAS),
     1             DYNA2O(KOMAX,MAXBAS), DYA2O(0:K2MAX,MAXBAS),
     2             LWTA2O(K2MAX,MAXBAS), RWTA2O(K2MAX,MAXBAS),
     3             MINA2O(0:KOMAX,MAXBAS)
C
C     Atmosphere to ocean interpolation (wind stress; U,V grid)
      REAL FINA2V, DYNA2V, DYA2V, LWTA2V, RWTA2V
      INTEGER MINA2V
      COMMON/MA2V/ FINA2V(0:KOMAX+1,MAXBAS),
     1             DYNA2V(KOMAX+1,MAXBAS), DYA2V(0:K2MAX,MAXBAS),
     2             LWTA2V(K2MAX,MAXBAS), RWTA2V(K2MAX,MAXBAS),
     3             MINA2V(0:KOMAX+1,MAXBAS)
#else
C
C     Ocean to atmosphere interpolation (SST)
      REAL FYO2A, CFYO2A, DYO2A, YWTO2A
      REAL FXO2A, CFXO2A, DXO2A
      INTEGER MYO2A, MXO2A
      COMMON/MO2A/FYO2A(3,0:K2MAX,MAXBAS), CFYO2A(3,0:K2MAX,MAXBAS),
     1            FXO2A(2,0:K1MAX,MAXBAS), CFXO2A(2,0:K1MAX,MAXBAS),
     2            DYO2A(0:KOMAX,MAXBAS),   YWTO2A(0:KOMAX,MAXBAS),
     3            DXO2A(0:JOMAX,MAXBAS),
     4            MYO2A(0:K2MAX,MAXBAS),   MXO2A(0:K1MAX,MAXBAS)
C
C     Atmosphere to ocean interpolation (heat/E-P fluxes; T,S grid)
      REAL FYA2O, CFYA2O, DYA2O, YWTA2O
      REAL FXA2O, CFXA2O, DXA2O
      INTEGER MYA2O, MXA2O
      COMMON/MA2O/FYA2O(3,0:KOMAX,MAXBAS), CFYA2O(3,0:KOMAX,MAXBAS),
     1            FXA2O(2,0:JOMAX,MAXBAS), CFXA2O(2,0:JOMAX,MAXBAS),
     2            DYA2O(0:K2MAX,MAXBAS),   YWTA2O(0:K2MAX,MAXBAS),
     3            DXA2O(0:K1MAX,MAXBAS),
     4            MYA2O(0:KOMAX,MAXBAS),   MXA2O(0:JOMAX,MAXBAS)
C
C     Atmosphere to ocean interpolation (wind stress; U,V grid)
      REAL FYA2V, CFYA2V, DYA2V, YWTA2V
      REAL FXA2V, CFXA2V, DXA2V
      INTEGER MYA2V, MXA2V
      COMMON/MA2V/FYA2V(3,0:KOMAX+1,MAXBAS), CFYA2V(3,0:KOMAX+1,MAXBAS),
     1            FXA2V(2,0:JOMAX+1,MAXBAS), CFXA2V(2,0:JOMAX+1,MAXBAS),
     2            DYA2V(0:K2MAX,MAXBAS),     YWTA2V(0:K2MAX,MAXBAS),
     3            DXA2V(0:K1MAX,MAXBAS),
     4            MYA2V(0:KOMAX+1,MAXBAS),   MXA2V(0:JOMAX+1,MAXBAS)
#endif
 
