C ICAM (Idealized Coupleable Atmospheric Model)
C packetsrc $Revision: 1.6 $ created $Date: 1996/05/03 22:50:18 $
C
C mpacket.h: Fortran include module
C Contains details of data packets used for communication between
C the atmospheric model and ocean model(s)
C
C Parts
C      1: Communication options
C      2: SST packet data
C      3: Flux packet data
C
C ----------------------------------------------------------------------
C Part 1: Communication options
C        (initialized by INIPAK)
C
C IPKOPT: Packet communication option (1 ... 3)
C  [Options 1 => use COMMON blocks for communication;
C           2 => use PVM (not yet implemented);
C           3 => use disk files for communication
C               (COMDIR is assumed to contain the name of
C                the directory where these files reside)]
C COMDIR: Name of disk directory for writing packet files
C         (stored as a right justified string)
C RUNPAK: Run name (used to create packet names, group names etc.)
C (The following are PVM parameters)
C MYTID:  "My" task ID
C OTHTID: "Other" task ID
C MYINST: "My" instance number in the group
C OINST:  "Other" instance number in the group
C RIOPT:  Real data code for integer data
C RROPT:  Real data code for real data
C
C
      INTEGER IPKOPT
      COMMON/PACKC0/ IPKOPT
C
      CHARACTER COMDIR*72, RUNPAK*5
      COMMON/PACKC1/ COMDIR, RUNPAK
C
#ifdef PVM_LINK
      INTEGER MYTID, OTHTID, MYINST, OINST, RIOPT, RROPT
      LOGICAL SSTSYN, SEQFLG
      COMMON/PACKC2/ MYTID, OTHTID, MYINST, OINST, RIOPT, RROPT,
     1               SSTSYN, SEQFLG
#elif defined(MPI_LINK)
      INTEGER ISST_TAG, RSST_TAG, ISST_SIZE, RSST_MAX
      INTEGER IFLX_TAG, RFLX_TAG, IFLX_SIZE, RFLX_MAX
      PARAMETER (ISST_TAG=1, ISST_SIZE=5)
      PARAMETER (RSST_TAG=2, RSST_MAX=2*JOMAX+1+2*KOMAX+1+JOMAX*KOMAX)
      PARAMETER (IFLX_TAG=3, IFLX_SIZE=6)
      PARAMETER (RFLX_TAG=4, RFLX_MAX=NFOCN*JOMAX*KOMAX)
      REAL    RSST_BUF, RFLX_BUF
      INTEGER ISST_BUF, IFLX_BUF
      INTEGER MYID, OTHERID, NUMPROCS
      LOGICAL SSTSYN, SEQFLG
      COMMON/PACKC2/ RSST_BUF(RSST_MAX), RFLX_BUF(RFLX_MAX),
     1               ISST_BUF(ISST_SIZE), IFLX_BUF(IFLX_SIZE),
     2               MYID, OTHERID, NUMPROCS,
     3               SSTSYN, SEQFLG
#endif
C
C ----------------------------------------------------------------------
C Part 2: SST packet data
C        (used by SNDSST/RCVSST)
C
C IPACK0: Packet number (>=0)
C         (A value of -1 implies COMMON block is empty)
C ISTP0:  Time-step number (in ocean model)
C NSYNC0: Synchronization interval (no. of ocean time-steps)
C
C NOCNX0: Number of ocean grid points in X-direction
C NOCNY0: Number of ocean grid points in Y-direction
C
C XOCN:   Longitude values of ocean grid points (degrees)
C YOCN:   Sine-latitude values of ocean grid points
C [Ocean grid-points correspond to the temperature grid]
C XOMID:  Longitude values of ocean grid-box edges (degrees)
C YOMID:  Sine-latitude values of ocean grid-box edges
C [Ocean grid box edges correspond to the velocity grid]
C
C SSTOCN(NOCNX0,NOCNY0,IB): SST values on ocean grid for each basin
C
      INTEGER IPACK0, ISTP0, NSYNC0
      INTEGER NOCNX0, NOCNY0
      REAL XOCN, XOMID, YOCN, YOMID
      REAL SSTOCN
      COMMON/SSTCM1/ IPACK0(MAXBAS), ISTP0(MAXBAS), NSYNC0(MAXBAS)
      COMMON/SSTCM2/ NOCNX0(MAXBAS), NOCNY0(MAXBAS)
      COMMON/SSTCM3/ XOCN(JOMAX,MAXBAS), XOMID(0:JOMAX,MAXBAS),
     1               YOCN(KOMAX,MAXBAS), YOMID(0:KOMAX,MAXBAS)
      COMMON/SSTCM4/ SSTOCN(JOMAX,KOMAX,MAXBAS)
C
C ----------------------------------------------------------------------
C Part 3: Flux packet data
C        (used by SNDFLX/RCVFLX)
C
C IPACK1: Packet number (>=0)
C         (A value of -1 implies COMMON block is empty)
C ISTP1:  Time-step number (in atmospheric model)
C NSYNC1: Synchronization interval (no. of atmospheric time-steps)
C
C NOCNX1: Number of ocean grid points in X-direction
C NOCNY1: Number of ocean grid points in Y-direction
C NFLX1:  Number of different fluxes being sent
C
C FLXOCN(NOCNX1,NOCNY1,IFX,IB): Flux values on ocean grid for each basin
C
C FXTOCN(IFX,IB): Total flux values for each basin
C (FXTOCN is the integrated flux, i.e. flux*area, that the atmosphere
C  thinks is going into the ocean. It can be used to check the
C  interpolation algorithm and make corrections to ensure global
C  flux conservation)
C
      INTEGER IPACK1, ISTP1, NSYNC1
      INTEGER NOCNX1, NOCNY1, NFLX1
      REAL FLXOCN, FXTOCN
      COMMON/FLXCM1/ IPACK1(MAXBAS), ISTP1(MAXBAS), NSYNC1(MAXBAS)
      COMMON/FLXCM2/ NOCNX1(MAXBAS), NOCNY1(MAXBAS), NFLX1(MAXBAS)
      COMMON/FLXCM3/ FLXOCN(JOMAX,KOMAX,NFOCN,MAXBAS)
      COMMON/FLXCM4/ FXTOCN(NFOCN,MAXBAS)
 
