C ICAM (Idealized Coupleable Atmospheric Model)
C icamsrc $Revision: 1.8 $ created $Date: 1996/05/02 21:31:03 $
C
C mparams.h: Fortran include module
C Contains the `static' model paramaters
C (i.e., parameters that are initialized, and do not vary during
C  the course of a model run)
C
C Parts
C      0: Defined parameters
C      1: Useful model constants
C      2: Model parameters
C      3: Model "masks"
C      4: Derived model parameters
C
C ----------------------------------------------------------------------
C Part 0: Defined parameters
C
      REAL PI
      PARAMETER (PI=3.141592653589793)
C
C ----------------------------------------------------------------------
C Part 1: Useful model constants (initialized by INICON)
C
C     Constants
C
C DDAYL, DAYL:  Length of day (sec)
C TIMFIL:       Robert time-filter factor (usually 0.01 or 0.02)
C AEARTH:       Planetary radius (m)
C PBOT:         Surface pressure (Pa)
C SOLFLX:       Solar constant (W/m**2)
C EXCH:         Surface exchange ("drag") coefficient (non-dimensional)
C RHOWAT:       Density of freshwater (kg/m**3)
C OCNRHO:       Density of ocean (kg/m**3)
C OCNCP:        Specific heat of ocean water (J/(kg*K))
C
C [The following constants are additionally available in mgrid.h:
C A0:           Planetary radius (=AEARTH)
C OMEGA0:       Planetary angular velocity (=2*PI/DAYL)
C F0:           Coriolis parameter (=2*OMEGA0),
C RGAS, CP:     Dry air specific heats (J/(kg*K))
C KAPPA:        CP/RGAS
C G0:           Surface gravitational acceleration (m/s**2)
C ]
C
      DOUBLE PRECISION DDAYL
      REAL DAYL, TIMFIL, AEARTH, PBOT, SOLFLX
      REAL EXCH, RHOWAT
      REAL OCNRHO, OCNCP
      COMMON/MPCON0/ DDAYL,
     1               DAYL, TIMFIL, AEARTH, PBOT, SOLFLX,
     2               EXCH, RHOWAT,
     3               OCNRHO, OCNCP
C
C Special strings used by the model
C
C PRGNAM: Program name
C PRGVER: Program version number
C PARDIR: Directory containing model parameter/initialization files
C         (stored as a right justified string)
C
      CHARACTER PRGNAM*12, PRGVER*60, PARDIR*72
      COMMON/MPCON1/ PRGNAM, PRGVER, PARDIR
C
C Fortran unit numbers reserved for the model
C
C IUPAR:  Used to read model parameter file
C IUDAA:  Used for model diagnostics file
C IUTSA:  Used for model time-series file
C IUHSA:  Used for model horizontal section save file
C IUVSA:  Used for model vertical section save file
C IUZSA:  Used for model zonal-mean section save file
C IUTEM:  Used for temporary I/O
C IUPAK:  Used for packet communications
C
      INTEGER IUPAR, IUDAA, IUTSA, IUHSA, IUVSA, IUZSA
      INTEGER IUTEM, IUPAK
      COMMON/MPCON2/ IUPAR, IUDAA, IUTSA, IUHSA, IUVSA, IUZSA,
     1               IUTEM, IUPAK
C
C AFMT: "A" format to be used for save file
C CFMT: Comment format to be used for save file
C EFMT: "E" format to be used for save file
C IFMT: "I" format to be used for save file
C LFMT: "L" format to be used for save file
C
      CHARACTER*72 AFMT, CFMT, EFMT, IFMT, LFMT
      COMMON/MPCON3/ AFMT, CFMT, EFMT, IFMT, LFMT
C
C ----------------------------------------------------------------------
C Part 2: Model parameters (initialized by GETPAR)
C
C Strings
C
C RTITLE: Run title, first 5 characters define run name
C SEAFIL: Name of file containing sea mask
C OROFIL: Name of file containing orography height
C SSTFIL: Name of file containing "climatological" SST coefficients
C FADFIL: Name of file containing flux adjustment coefficients
C ICEFIL: Name of file containing initial ice mask, and ice height
C (NOTE: for the above filenames, if the first two characters are 'no',
C        it is assumed that no file is specified)
C
      CHARACTER*72 RTITLE
      CHARACTER*72 SEAFIL, OROFIL, SSTFIL, FADFIL, ICEFIL
      COMMON/MPARC/ RTITLE,
     1              SEAFIL, OROFIL, SSTFIL, FADFIL, ICEFIL
C
C Integer variables
C
C NSPDAY: No. of time-steps per model day
C NDPYR:  No. of model days per model year
C NSYNC:  No. of time-steps between "synchronizations" (flux exchanges),
C         for coupled integrations (>= 1)
C MMAX:   Maximum zonal wavenumber (e.g. 21 for T21)
C NMAX:   Maximum meridional wavenumber (e.g. 21 for T21)
C NLEV:   No. of model pressure levels (>= 2)
C NTRC:   No. of tracers in the model, including moisture (usually 1)
C         (should be the same as C preprocessor parameter NTRACE)
C
      INTEGER NSPDAY, NDPYR, NSYNC
      INTEGER MMAX, NMAX, NLEV, NTRC
      COMMON/MPARI/ NSPDAY, NDPYR, NSYNC,
     1              MMAX, NMAX, NLEV, NTRC
C
C Logical variables
C
C COUFLG: Set to TRUE for coupled ocean-atmosphere runs
C         (Default is atmosphere-only run)
C SSNFLG: Set to TRUE to switch on seasonal cycle
C         (Default is no seasonal cycle, i.e., "perpetual" season)
C ICEFLG: Set to TRUE switch on interactive ice physics
C         (NOT YET IMPLEMENTED)
C
      LOGICAL COUFLG, SSNFLG, ICEFLG
      COMMON/MPARL/ COUFLG, SSNFLG, ICEFLG
C
C Real variables
C
C PLID:   Pressure-thickness of topmost model level (Pa)
C PTROP:  Model tropopause pressure (Pa)
C EQPFAC: Amplification factor for annual-mean equator-to-pole
C         gradient of insolation [usually 1.0]
C PPFAC:  Amplification factor for the pole-to-pole gradient
C         associated with the seasonal cycle of insolation
C         [usually 1.0]
C YPHASE: Phase of year at start of run, ranging from 0 to 1,
C         with 0 corresponding to to the NH spring equinox (March 21),
C         and 0.75 to the NH winter solstice (December 22).
C IMPLCT: Centering factor for the implicit time-stepping scheme
C         (0.0 => forward, 0.5 => semi-implicit, 1.0 => backward)
C         [usually chosen to be 0.5 or 1.0]
C REFFAC: Steepness factor of background THETA profile for
C         implicit time-stepping (w.r.t U.S. standard atmosphere)
C         [usually about 2.0]
C D8DAMP: Del**8 damping coefficient acting on the smallest resolved
C         spatial scale (1/sec)  [usually about 1/(0.25 days)]
C IPRNDL: Inverse Prandtl number (heat diffusivity / viscosity)
C         [usually 0.0, implying no vertical diffusion of temperature]
C
C VISC0:  Vertical viscosity coefficient (m**2/s)
C DMPBOT: Surface friction coefficient (1/sec)
C          [usually about 1/(3 days)]
C DZBOT:  Height-thickness of surface boundary layer (m)
C         [usually 0.0 km]
C DMPTOP: Sponge layer damping coefficient (1/sec, usually 0.0)
C         [usually 0.0 1/day]
C DPTOP:  Pressure-thickness of sponge layer (Pa)
C         [usually 0.0 mb]
C
C LWFRAC: Longwave window fraction (nondim., usually 0.04)
C DLWAIR: Longwave optical thickness of air (nondim., usually 1.5)
C DLWH2O: Longwave optical thickness of H2O (nondim., usually 600.)
C DSWAIR: Shortwave optical thickness of air (nondim., usually 0.1)
C DSWO3:  Shortwave optical thickness of ozone (nondim., usually 0.0)
C
C DLWCO2: Longwave optical thickness of "extra" CO2 (nondim.)
C         [usually 1.0 for a control run]
C MULCO2: Amplification factor for "CO2" optical thickness (nondim.)
C         [usually 1.0 for a control run, i.e., no CO2 increase]
C MULTIM: Transition time-scale for logarithmic increase of "CO2" (sec)
C         [usually about 100 years]
C
C ALBSEA: Sea surface albedo  (usually 0.25)
C ALBLAN: Land surface albedo (usually 0.25)
C ALBICE: Ice surace albedo   (usually 0.65)
C LANHCP: Land heat capacity  (J/(K*m**2), usually 1e7)
C SATURF: Water vapour saturation threshold fraction (<= 1)
C         (for moist adiabatic adjustment)
C BUKDEP: Depth of land surface "bucket" (m)
C MIXDEP: Depth of mixed-layer (m)
C
      REAL PLID, PTROP
      REAL EQPFAC, PPFAC, YPHASE
      REAL IMPLCT, REFFAC, D8DAMP, IPRNDL
      REAL VISC0, DMPBOT, DZBOT, DMPTOP, DPTOP
      REAL LWFRAC, DLWAIR, DLWH2O, DSWAIR, DSWO3
      REAL DLWCO2, MULCO2, MULTIM
      REAL ALBSEA, ALBLAN, ALBICE
      REAL LANHCP, SATURF, BUKDEP, MIXDEP
      COMMON/MPARF/ PLID, PTROP,
     1              EQPFAC, PPFAC, YPHASE,
     2              IMPLCT, REFFAC, D8DAMP, IPRNDL,
     3              VISC0, DMPBOT, DZBOT, DMPTOP, DPTOP,
     4              LWFRAC, DLWAIR, DLWH2O, DSWAIR, DSWO3,
     5              DLWCO2, MULCO2, MULTIM,
     6              ALBSEA, ALBLAN, ALBICE,
     7              LANHCP, SATURF, BUKDEP, MIXDEP
C
C Ocean basin details
C
C NBASIN: No. of (rectangular) ocean basins
C NBCOUP: No. of coupled ocean basins,
C         i.e., those that interact with the atmosphere
C        (<= NBASIN, the remaining basins have prescribed cycle of SST)
C IRUNOF: Run-off distribution option
C (0: E-P is balanced over each basin by adjusting the basin-average;
C  1: E-P is globally averaged, and distributed according to ROFAC;
C  2: E-P is latitudinally averaged, and distributed according to ROFAC)
C
C LONBLO, LONBHI: Longitudinal boundaries of basin (degrees)
C (   0. <= LONBLO < 360., LONBLO < LONBHI,   0. < LONBHI <= 360.)
C LATBLO, LATBHI: Latitudinal boundaries of basin (degrees)
C ( -90. <= LATBLO <  90., LATBLO < LATBHI, -90. < LATBHI <=  90.)
C ROFAC: Fraction of run-off that flows into each basin
C        (Total must equal 1.0; this is checked for)
C [The above basin boundaries correspond to the velocity grid-points
C  at the edge of the basin. It is best to choose basin boundaries
C  to lie midway between atmospheric grid-points, such that the
C  basin edges coincide with atmospheric grid-box edges.]
C
      REAL LONBLO, LONBHI, LATBLO, LATBHI, ROFAC
      INTEGER NBASIN, NBCOUP, IRUNOF
      COMMON/MPBAS0/ LONBLO(MAXBAS), LONBHI(MAXBAS),
     1               LATBLO(MAXBAS), LATBHI(MAXBAS),
     2               ROFAC(MAXBAS),
     3               NBASIN, NBCOUP, IRUNOF
C
C Part 3: Model "masks" (initialized by GETMSK/INIMSK)
C
C JB0, KB0: atmospheric longitude/latitude index offset
C             (i.e., lowest index - 1) corresponding to the
C             western/southern boundary of each ocean basin
C NATMX, NATMY: number of atmospheric grid-points in X/Y direction
C               in each ocean basin
C
C SEAMSK: Sea mask (TRUE over ocean/sea ice, FALSE over land)
C        (NOTE: Ice is assumed to be part of the atmospheric model)
C
      INTEGER JB0, KB0, NATMX, NATMY
      COMMON/MPBAS1/ JB0(MAXBAS), NATMX(MAXBAS),
     1               KB0(MAXBAS), NATMY(MAXBAS)
      LOGICAL SEAMSK
      COMMON/MPMASK/ SEAMSK(K1MAX,K2MAX)
C
C Other "masks"
C
C BUCKHT: Land surface bucket height (m)
C OROHT:  Orography height (m) *FOR FUTURE USE*
C
      REAL BUCKHT, OROHT
      COMMON/MPFLD/ BUCKHT(K1MAX,K2MAX), OROHT(K1MAX,K2MAX)
C
C ----------------------------------------------------------------------
C Part 4: Derived model parameters (initialized by ATMINI/INIPR2)
C
C Timing parameters
C
C DYEARL, YEARL: length of model year (in seconds)
C DT1:           model time-step (in seconds)
C
      DOUBLE PRECISION DYEARL
      REAL YEARL, DT1
      COMMON/MPTIM2/ DYEARL,
     1              YEARL, DT1
C
C Flags (derived from filename parameters OROFIL,SSTFIL,FADFIL,ICEFIL)
C
C SEASPF: Set to TRUE if the sea mask is specified through
C         the filename parameter SEAFIL
C      (Default is to fill the entire rectangular basin with sea points)
C OROSPF: Set to TRUE if the orographic height is specified through
C         the filename parameter OROFIL (NOT YET IMPLEMENTED)
C         (Default is zero height for orography)
C SSTSPF: Set to TRUE if the "climatological" annual cycle of SST
C         is specified through filename parameter SSTFIL
C         (Default is to have a simple zonally-symmetric profile)
C FADSPF: Set to TRUE if the annual cycle of heat/E-P flux adjustments
C         is specified through filename parameter FADFIL
C         (Default is to have no flux adjustment)
C ICESPF: Set to true if initial ice distribution is specified
C         (Default is to have no ice initially)
C
      LOGICAL SEASPF, OROSPF, SSTSPF, FADSPF, ICESPF
      COMMON/MPFIL2/ SEASPF, OROSPF, SSTSPF, FADSPF, ICESPF
C
C Atmospheric grid parameters
C
C DGATM:  Longitude values of atmospheric grid points (degrees)
C         (Range: 0 to 720)
C MUAMID: Sine-latitude (mu) values of atmospheric grid box "edges"
C DGAMID: Longitude values of edges of atmospheric grid boxes (degrees)
C         (Range: 0- to 720+)
C MTROPP:  Model tropopause level number (initialized by ATMINI/INIGRD)
C
      REAL DGATM, MUAMID, DGAMID
      INTEGER MTROPP
      COMMON/MPATM2/ DGATM(2*K1MAX+1), MUAMID(0:K2MAX),
     1               DGAMID(0:2*K1MAX+1), MTROPP
C
C Auxiliary ocean basin parameters
C
C PROSST: Prognostic SST mask (TRUE over those sea points where
C         the SST is determined prognostically; over other sea
C         points, the SST is prescribed)
C NOCNP:  Number of oceanic (not land) grid-points at each
C         latitude circle for each (irregular) ocean basin
C
      LOGICAL PROSST
      INTEGER NOCNP
      COMMON/MPBAS2/ PROSST(K1MAX,K2MAX),
     1               NOCNP(KBMAX,MAXBAS)
#ifdef SLAB_OCEAN
C Internal (mixed-layer) ocean parameters
C
C MIXHCP: Mixed-layer heat capacity (J/m2K)
C
      REAL MIXHCP
      COMMON/MPMIX2/ MIXHCP(K1MAX,K2MAX)
#endif
