#ifndef OCONS
#   include "ocons.h"
#endif

C C preprocessor parameters
C
C LENWRD:  Length of a data word, in bytes (default 4)
C RECUNIT:  Units, i.e., no. of bytes, in which direct-access record
C          lengths are measured (usually 1 for most machines).
C DPRECISION: If defined, implies the single precision => REAL*8 
C ON_CRAY: If defined, implies program is running on a Cray
C GRAPHICS: If defined, graphics is displayed
C

#ifndef LENWRD
#   define LENWRD 4
#endif

#ifndef RECUNIT
#   define RECUNIT 1
#endif

#ifdef DPRECISION
#   define REAL8 REAL
#else
#   define REAL8 REAL*8
#endif

C     "Enumeration literals"
#define JX 1
#define JY 2

      PROGRAM OCN2D
C Two-dimensional (meridional plane) ocean model
C [The formulation is described in Thual and McWilliams, 1992
C  and Saravanan and McWilliams, 1995]
C 
C Version 7.4 created 15 July 96
C
C Author: R. Saravanan, Climate and Global Dynamics Division
C                       National Center for Atmospheric Research
C                       Boulder, CO 80307-3000, U.S.A.
C                       E-mail: svn@ncar.ucar.edu
C                       URL: http://www.cgd.ucar.edu:/gds/svn/svn.html
C
#include "ogrid.h"
C
C ---------------------- module ocons.h ----------------------------
C C preprocessor parameters
C NOCNQ = no. of prognostic quantities, usually 3 (vorticity, T, S)
C NXOMAX = maximum value of NXOCN, usually 0
C NYOMAX = maximum value of NYOCN
C NZOMAX = maximum value of NZOCN
C
C JOCV   = 1  (denotes vorticity)
C JOCT   = 2  (denotes temperature)
C JOCS   = 3  (denotes salinity)
C JOCTR1 = 4  (denotes passive tracer 1)
C JOCTR1 = 5  (denotes passive tracer 2)
C
C (The following two parameters are used only by the Laplacian inversion
C  subroutines, and by the Arakawa jacobian subroutine, for reserving
C  space for temporary variables)
C NXMAX  = NYOMAX
C NYMAX  = NZOMAX
C
C ---------------------- module ogrid.h ----------------------------
C
C      INTEGER NXOCN, NYOCN, NZOCN, NTROCN
C      COMMON/OGRID1/ NXOCN, NYOCN, NZOCN, NTROCN
C
C      REAL AOCN, OCNLEN, OCNCP, OCNRHO, TXCOEF, SXCOEF
C      REAL OCNDEP, OCNT0, OCNS0, OCNTXF, OCNWID
C      REAL TDIFFC, HDFAC, PRANDL, LEWIS
C      REAL DXOCN, DYOCN, DZOCN
C      COMMON/OGRID2/ AOCN, OCNLEN, OCNCP, OCNRHO, TXCOEF, SXCOEF,
C     1               OCNDEP, OCNT0, OCNS0, OCNTXF, OCNWID,
C     2               TDIFFC, HDFAC, PRANDL, LEWIS,
C     3               DXOCN, DYOCN, DZOCN
C      REAL ODIFFY, ODIFFZ
C      REAL LONOCN, LATOCN, PHIOCN, ZOCN
C      REAL WTXOCN, WTYOCN, WTZOCN
C      COMMON/OGRID3/ ODIFFY(NOCNQ), ODIFFZ(NOCNQ),
C     1       LONOCN(0:NXOMAX), LATOCN(0:NYOMAX),
C     2       PHIOCN(0:NYOMAX), ZOCN(0:NZOMAX),
C     3       WTXOCN(0:NXOMAX), WTYOCN(0:NYOMAX), WTZOCN(0:NZOMAX)
C
C      Descripton of variables in "ogrid.h"
C
C NXOCN     = number of grid intervals in the zonal direction
C             (= no. of grid points - 1)
C NYOCN     = number of grid intervals in the meridional direction
C NZOCN     = number of grid intervals in the vertical direction
C NTROCN    = number of passive tracers (>= 0)
C AOCN      = planetary radius (m)
C OCNLEN    = meridional extent of ocean (m)
C OCNCP     = specific heat of ocean water (J/(kg*K))
C OCNRHO    = density of ocean water (kg/m**3)
C TXCOEF    = reference thermal buoyancy coefficient
C             (including gravity term)
C SXCOEF    = saline buoyancy coefficient
C             (including gravity term)
C OCNDEP    = ocean depth (m)
C OCNT0     = reference ocean temperature (K)
C OCNS0     = reference ocean salinity (kg/kg)
C OCNTXF    = thermal buoyancy coefficient attenuation factor
C OCNWID    = ocean width (0..1, fraction of circumference)
C TDIFFC    = vertical thermal diffusion coefficient (m2/s)
C HDFAC     = anisotropy factor (0..1)
C             (i.e. hor. diff. coef. =
C                   ver. diff. coef. * (HDFAC*OCNLEN/OCNDEP)**2 )
C PRANDL    = Prandtl number (ratio of momentum to thermal diffusion)
C LEWIS     = Lewis number (ratio of saline to thermal diffusion)
C DXOCN     = nominal zonal grid spacing
C DYOCN     = nominal meridional grid spacing
C DZOCN     = vertical grid spacing
C
C ODIFFY    = meridional diffusion coefficient for momentum, tracers
C ODIFFZ    = vertical diffusion coefficient for momentum, tracers
C LONOCN    = longitude values (in degrees)
C LATOCN    = latitude values (in degrees)
C PHIOCN    = latitude values (in radians)
C ZOCN      = height values (<= 0, in metres)
C WTXOCN    = grid-box weights in the zonal direction
C             (assumed to be equal for all grid-boxes)
C WTYOCN    = grid-box weights in the meridional direction
C             (proportional to area occupied by latitude band)
C WTZOCN    = grid-box weights in the vertical direction
C             (proportional to level thickness)
C (Note: WTXOCN, WTYOCN, WTZOCN should all sum to unity)
C
C
C NXOCN+1 = no. of grid-points in X direction
C NYOCN+1 = no. of grid-points in Y direction
C NZOCN+1 = no. of grid-points in Z direction
C NTROCN  = no. of tracers (excluding T & S)
C
C
C
      REAL PI
      PARAMETER (PI=3.141592653589793)
C     Stefan's constant (in W/(m**2*K**4))
      REAL STEFAN
      PARAMETER (STEFAN= 5.669E-8)
C
C     Gravitational acceleration (m/s2), length of day (sec),
C     length of year (sec), Robert time-filter factor
      REAL GRAVIT, DAYL, YEARL, ROBFAC
C
C     Current model time (sec), current phase of year (0..1, 0=>Mar 21),
C     current CO2 concentration
      REAL TIM, CURPHA, CURCO2
C
C     Time-steps per day (NSPDAY <0 => days per time-step), days per year
      INTEGER NSPDAY, NDPYR
C
C     Atmospheric temperature, equator-to-pole factor,
C     pole-to-pole factor (for SST specification)
      REAL T0TOP, EQPFAC, PPFAC, YPHASE
C
C     Atmospheric CO2 parameters
      REAL DLWCO2, MULCO2, MULTIM
C
C     Land width (fractional area, 0..1)
      REAL LANWID
C
C     Freshwater flux perturbation, freshwater flux time interval,
C     freshwater minimum latitude, freshwater flux maximum latitude
      REAL FWFLUX, FWFTIM, FWLMIN, FWLMAX
C
C     Oceanic CO2 parameters
      REAL CO2CF1, CO2CF2, CO2CF3, CO2CF4, CO2CF5
C
C     Seasonal cycle flag
      LOGICAL SSNFLG
C
C     Copies of current model time, length of year, length of day
      DOUBLE PRECISION DTIM, DYEARL, DDAYL
C
C     Surface T, surface S
      REAL TTOP(0:NYOMAX), SOCN(0:NYOMAX)
C
C     Surface T, surface S restoring time-scales (seconds)
      REAL TAUT, TAUS
C
C     Mean heat flux into ocean, E-P
      REAL FTOCNM(0:NYOMAX), EMINPM(0:NYOMAX)
C
C     Average surface T, average surface S
      REAL TTOPAV, SOCNAV
C
C     Total heat flux into ocean, E-P
      REAL FTOCNT(0:NYOMAX), EMINPT(0:NYOMAX)
C
C     Diagnosed heat flux into ocean, diagnosed E-P
      REAL FTOCN0(0:NYOMAX), EMINP0(0:NYOMAX)
C
C     Atmospheric model (ICAM) variables:
C     Error flag
      LOGICAL ERRFLG
C     Current atmospheric model time-step count
      INTEGER IASTEP
C     Communication option string, run name
      CHARACTER COMSTR*72, RUNNAM*5
C     Reserved Fortran unit numbers string
      INTEGER IUNITR
C     Atmospheric model parameter file directory, old run name
      CHARACTER DIRSTR*72, OLDRUN*5
C     Old run part, oceanic/atmospheric synchronization intervals
      INTEGER IOPART, NOSYNC, NASYNC
C     Diagnostic sample interval, save frequency
      INTEGER NSAMPL, NSAVE
C     Longitudinal grid-points, sine-latitude mid-points
      REAL LONMID(-1:NXOMAX), MUOMID(-1:NYOMAX)
C     Oceanic SST array (in K)
      REAL SSTOCN(0:NYOMAX)
C
C     Ocean prognostic quantities at time levels 0, 1, 2
      REAL OCNQ0(0:NYOMAX,0:NZOMAX,NOCNQ)
      REAL OCNQ1(0:NYOMAX,0:NZOMAX,NOCNQ)
      REAL OCNQ2(0:NYOMAX,0:NZOMAX,NOCNQ)
C
C     Time tendency of prognostic quantities
      REAL DOCNQ(0:NYOMAX,0:NZOMAX,NOCNQ)
C
C     Y grid spacing for inverting diffusion operator, sine-latitude
C     grid spacing, sine-latitude coordinate, Y coordinate
      REAL DYISO, DMUOCN, MUOCN(0:NYOMAX), YOCN(0:NYOMAX)
C
C     Freshwater flux area, freshwater flux rate,
C     surface salinity perturbation (SH, NH)
      REAL FWAREA, FWRATE, SPERTS, SPERTN
C
C     Downward diffusive flux of prognostic quantities at surface,
C     Wind stress
      REAL OCNFLX(0:NYOMAX,NOCNQ), OCNTAU(0:NYOMAX,JX:JY)
C
C     Downward diffusive flux of prognostic quantities at surface
C     (in model units)
      REAL DIFFLX(0:NYOMAX,NOCNQ)
C
C     Streamfunction
      REAL PSIOCN(0:NYOMAX,0:NZOMAX)
C     Nondimensional density
      REAL RHOOCN(0:NYOMAX,0:NZOMAX)
C     Column tracer mixing ratios
      REAL TRCCOL(0:NZOMAX,JOCT:NOCNQ)
      REAL TRCCL0(JOCT:NOCNQ), TRCCL1(JOCT:NOCNQ)
C
C     Components of velocity: U,V,W (used for diagnostics only)
      REAL UOCN(0:NYOMAX,0:NZOMAX), VOCN(0:NYOMAX,0:NZOMAX)
      REAL WOCN(0:NYOMAX,0:NZOMAX)
#ifdef GRAPHICS
      REAL FPRF(0:NYOMAX)
#endif
C
C     Model time-step (sec), CFL time-step, effective time-step,
C     modified phase of the year, temporary variable
      REAL DT, DTCFL, DTEFF, YRDAY, TEMP
C
C     Symmetric/asymmetric streamfuntion perturbation, vertical
C     wavenumber of perturbation
      REAL PSISYM, PSIASY, MVERT
C
C     Staggered (mid-level) Y,Z coordinate values for old and new
C     grids (used for interpolation)
      REAL YOMID(-1:NYOMAX), YNMID(-1:NYOMAX)
      REAL ZOMID(-1:NZOMAX), ZNMID(-1:NZOMAX)
C
C     Extended Y,Z coordinate values for old grid, interpolation
C     coefficients
      REAL YOLD(-1:NYOMAX+1), YFRAC(-1:NYOMAX)
      REAL ZOLD(-1:NZOMAX+1), ZFRAC(-1:NZOMAX)
C
C     More interpolation coefficients
      REAL DYNEW(0:NYOMAX), DYOLD(-1:NYOMAX)
      REAL DZNEW(0:NZOMAX), DZOLD(-1:NZOMAX)
      REAL YLWT(0:NYOMAX), YRWT(0:NYOMAX)
      REAL ZLWT(0:NZOMAX), ZRWT(0:NZOMAX)
      INTEGER IYMAP(-1:NYOMAX), IZMAP(-1:NZOMAX)
C
C     Copies of no. of Y/Z grid-points, no. of tracers,
C     no. of time-steps per day
      INTEGER NYOCN0, NZOCN0, NTROC0, NYOCN2, NZOCN2, NSPDA2
C
C     Run part number, run time-step count,
      INTEGER IPART, ISTEP
C
C     No. of parts, no. of time-steps per part,
C     skip interval for diagnostics/display, last part
      INTEGER NPART, NSTEP, NSKIP, LPART
C
C     Run part number, time-series save frequency,
C     section data save frequency (used for diagnostics)
      INTEGER ISET, NTSFR2, NOSAVE
C
C     Unit numbers: temporary, save file, diagnostics file, horizontal
C                   section file, vertical section file, zonal mean
C                   section file
      INTEGER IUTEM, IUSAV, IUDAO, IUTSO, IUHSO, IUVSO, IUZSO
C
C     Copies of unit numbers
      INTEGER IDAFL2, ITSFL2, IHSFL2, IVSFL2, IZSFL2
C
C     Loop indices: prognostic quantities (IPGQ=JOCV..NOCNQ),
C                   latitude (K=0,NYOCN), level (L=0,NZOCN),
C                   time-step (IT)
      INTEGER IPGQ, K, L, IT
C
C     Minimum/maximum latitude indices for freshwater flux perturbation
      INTEGER KFWLO, KFWHI
C
C     Restore surface temperature flag, restore surface salinity flag,
C     coupling flag, convective adjustment flag
C     (RSTT/RSTS = .TRUE. => restoring boundary conditions)
C     (RSTT/RSTS = .FALSE. => flux boundary conditions)
C     (COUFLG = .TRUE. => coupled ocean-atmosphere integration)
      LOGICAL RSTS, RSTT, COUFLG, CAFLAG
C
C     Restart flag, synchronous integration flag
      LOGICAL RESTRT, SYNFLG
C
C     Level save flag, latitude save flag, surface save flag
C     (used for diagnostics)
      LOGICAL OLEVSV(0:NZOMAX), OLATSV(0:NYOMAX), OSRFSV
C
C     Output format, program name, filename, savefile, run title
C     run initialization time, temporary
      CHARACTER*72 FMT, PRGSTR, FILNAM, SAVFIL, RTITLE, INITIM, TEMSTR
C     Work arrays
      REAL8 WCOEF(NYOMAX-1, NZOMAX-1), WCOEF2(NYOMAX-1, NZOMAX-1)
      REAL WFST(5*(NYOMAX/2)+15), WFST2(5*(NYOMAX/2)+15)
C
      INTEGER IADJ, IADLEV
#ifdef ON_CRAY
      CHARACTER*8 DATE0, CLOCK0
#endif
C
C     Initialize I/O unit numbers
      IUTEM= 11
      IUSAV= 20
      IUDAO= 21
      IUTSO= 22
      IUHSO= 23
      IUVSO= 24
      IUZSO= 25
C
C     Reserve units 30-40 for use by atmospheric model
      IUNITR= 30
C
C     Format for saving model state
      FMT= '(1X,1P,5E15.6)'
C
C     Program description
      PRGSTR= 'OCN2D v7.4 created 15 July 96'
C     Initialize planetary parameters (in S.I. units)
C     Length of day
      DDAYL= 86400.
      DAYL= DDAYL
C     Surface gravity
      GRAVIT= 9.81
C     Planetary radius
      AOCN= 6.4E6
C     Ocean length: ocean extends from y= -(pi/2)A to +(pi/2)A
      OCNLEN= PI*AOCN
C     Specific heat of ocean water (J/(kg*K))
      OCNCP= 4200.0
C     Density of ocean (kg/m**3)
      OCNRHO= 1000.0
C     Reference thermal buoyancy coefficient (including gravity term)
      TXCOEF= 2.E-4 * GRAVIT
C     Reference saline buoyancy coefficient (including gravity term)
      SXCOEF= -8.E-1 * GRAVIT
C     Robert time-filter factor
      ROBFAC= 0.02
#ifdef GRAPHICS
C     Initialize graphics
      CALL GSETUP
#endif
      WRITE(*,*)
      WRITE(*,*) PRGSTR
      WRITE(*,*)
      WRITE(*,*) 'COMSTR (communication option:<space>/PVM/directory)?'
      READ(*,'(A)') COMSTR
      WRITE(*,*) 'DIRSTR (model parameter file directory)?'
      READ(*,'(A)') DIRSTR
   10 WRITE(*,*)'Continue filename? ("new" for new run, blank to quit)'
      READ(*,'(A)') SAVFIL
      IF (SAVFIL .EQ. ' ') GOTO 990
      RESTRT= .FALSE.
      IF (SAVFIL.NE.'new') THEN
        WRITE(*,*) 'Reading model state from ', SAVFIL
        OPEN(IUSAV, FILE=SAVFIL )
        READ(IUSAV,'(1X,A72)') RTITLE
        READ(IUSAV,'(1X,A72)') TEMSTR
        READ(IUSAV,'(1X,A72)') INITIM
        READ(IUSAV,'(1X,1P,3E15.6)') TIM, CURPHA, CURCO2
        READ(IUSAV,'(1X,2I12)') IPART, ISTEP
        READ(IUSAV,'(1X,L2)') SSNFLG
        READ(IUSAV,'(1X,2I12)') NSPDAY, NDPYR
        READ(IUSAV,'(1X,1P,4E15.6)') T0TOP, EQPFAC, PPFAC, YPHASE
        READ(IUSAV,'(1X,1P,3E15.6)') DLWCO2, MULCO2, MULTIM
        READ(IUSAV,'(1X,1P,E15.6)') LANWID
        READ(IUSAV,'(1X,3I12)') NYOCN, NZOCN, NTROCN
        READ(IUSAV,'(1X,1P,4E15.6)') OCNDEP, OCNT0, OCNS0, OCNTXF
        READ(IUSAV,'(1X,1P,4E15.6)') TDIFFC, HDFAC, PRANDL, LEWIS
        READ(IUSAV,'(1X,1P,4E15.6)') FWFLUX, FWFTIM, FWLMIN, FWLMAX
        READ(IUSAV,'(1X,1P,5E15.6)')
     1                      CO2CF1, CO2CF2, CO2CF3, CO2CF4, CO2CF5
        DTIM= TIM
        NYOCN0= NYOCN
        NZOCN0= NZOCN
        NTROC0= NTROCN
        WRITE(*,*) RTITLE
        WRITE(*,*) TEMSTR
        WRITE(*,'(1X,A,A)') 'Run initialized at ', INITIM
        WRITE(*,*)
        WRITE(*,'(1X,A,F12.2,A)') 'Current model time = ',
     1                             DTIM/DDAYL,  ' days'
        WRITE(*,'(1X,3A14)')  'TIM(days)', 'CURPHA', 'CURCO2'
        WRITE(*,'(1X,3G14.4)') DTIM/DDAYL, CURPHA, CURCO2
        WRITE(*,'(1X,2A12)') 'IPART', 'ISTEP'
        WRITE(*,'(1X,2I12)') IPART, ISTEP
        WRITE(*,'(1X,A12)') 'SSNFLG'
        WRITE(*,'(1X,L12)') SSNFLG
        WRITE(*,'(1X,2A12)') 'NSPDAY', 'NDPYR'
        WRITE(*,'(1X,2I12)') NSPDAY, NDPYR
        WRITE(*,'(1X,4A14)') 'T0TOP', 'EQPFAC', 'PPFAC', 'YPHASE'
        WRITE(*,'(1X,4G14.4)') T0TOP, EQPFAC, PPFAC, YPHASE
        WRITE(*,'(1X,3A14)')
     1        'DLWCO2', 'MULCO2', 'MULTIM(yrs)'
        WRITE(*,'(1X,3G14.4)')
     1         DLWCO2, MULCO2, MULTIM/(NDPYR*DAYL)
        WRITE(*,'(1X,A14)') 'LANWID(fra)'
        WRITE(*,'(1X,G14.4)') LANWID
        WRITE(*,'(1X,3A12)') 'NYOCN', 'NZOCN', 'NTROCN'
        WRITE(*,'(1X,3I12)') NYOCN, NZOCN, NTROCN
        WRITE(*,'(1X,4A14)')
     1      'OCNDEP(km)', 'OCNT0(K)', 'OCNS0(g/kg)', 'OCNTXF'
        WRITE(*,'(1X,4G14.4)')
     1       OCNDEP/1000., OCNT0, OCNS0*1000., OCNTXF
        WRITE(*,'(1X,4A14)')
     1          'TDIFFC(m2/s)', 'HDFAC', 'PRANDL', 'LEWIS'
        WRITE(*,'(1X,4G14.4)')
     1           TDIFFC, HDFAC, PRANDL, LEWIS
        WRITE(*,'(1X,4A14)')
     1           'FWFLUX(Sv)', 'FWFTIM(yrs)',
     2           'FWLMIN(deg)', 'FWLMAX(deg)'
        WRITE(*,'(1X,4G14.4)')
     1           FWFLUX/1.0E6, FWFTIM/(NDPYR*DAYL), FWLMIN, FWLMAX
        WRITE(*,'(1X,5A14)')
     1          'CO2CF1', 'CO2CF2', 'CO2CF3', 'CO2CF4', 'CO2CF5'
        WRITE(*,'(1X,5G14.4)')
     1           CO2CF1, CO2CF2, CO2CF3, CO2CF4, CO2CF5
        WRITE(*,*)
        WRITE(*,*) 'Restart run? (T/F)'
        READ(*,*) RESTRT
      ENDIF
C
      IF (RESTRT) THEN
C       Restart atmospheric run
        OLDRUN= SAVFIL(1:5)
        IOPART= IPART
      ELSE
C       Not a restart atmospheric run
        OLDRUN= ' '
        IOPART= 0
      ENDIF
C
      IF ( (SAVFIL.EQ.'new') .OR. RESTRT ) THEN
C       New/restart run
        WRITE(*,*) 'Title of run?'
        READ(*,'(A72)') RTITLE
        WRITE(*,*)
C        WRITE(*,*) 'SSNFLG=?'
C        READ(*,*) SSNFLG
        SSNFLG= .FALSE.
C        WRITE(*,*) 'NSPDAY=?, NDPYR=?'
C        READ(*,*) NSPDAY, NDPYR
        WRITE(*,*) 'NSPDAY=?'
        READ(*,*) NSPDAY
        NDPYR= 360
        IF (RESTRT) THEN
C         Thermal forcing parameters
C          WRITE(*,'(A)') '(YPHASE=0 => continuation of phase of year)'
C          WRITE(*,*)
C          WRITE(*,*) 'T0TOP(K)=?, EQPFAC=?, PPFAC=?, YPHASE=?'
C          READ(*,*) T0TOP, EQPFAC, PPFAC, TEMP
          T0TOP= 300.
          EQPFAC= 1.0
          PPFAC= 1.0
          TEMP= 0.0
          IF ((TEMP .NE. 0.) .AND. (TEMP .NE. CURPHA)) THEN
            WRITE(*,*) 'Restart error: cannot alter phase of year'
            STOP
          ENDIF
C         Reset initial yearly phase of run
          YPHASE= CURPHA
        ELSE
C         Thermal forcing parameters
C          WRITE(*,'(A)') '(YPHASE=0..1, 0 => Mar 21, 0.75 => Dec 22)'
C          WRITE(*,*)
C          WRITE(*,*) 'T0TOP(K)=?, EQPFAC=?, PPFAC=?, YPHASE=?'
C          READ(*,*) T0TOP, EQPFAC, PPFAC, YPHASE
          T0TOP= 300.
          EQPFAC= 1.0
          PPFAC= 1.0
          YPHASE= 0.0
        ENDIF
C        WRITE(*,*) 'LANWID(frac)=?'
C        READ(*,*) LANWID
         LANWID= 0.7
C        WRITE(*,*) 'NYOCN=?, NZOCN=?, NTROCN=?'
C        READ(*,*) NYOCN, NZOCN, NTROCN
        WRITE(*,*) 'NYOCN=?, NZOCN=?'
        READ(*,*) NYOCN, NZOCN
        NTROCN= 0
        IF ((NYOCN .GT. NYOMAX) .OR. (NZOCN .GT. NZOMAX).OR.
     1      ((3+NTROCN) .GT. NOCNQ)) STOP 'OCN-RESN'
        IF (RESTRT .AND. ((NYOCN.NE.NYOCN0) .OR. (NZOCN.NE.NZOCN0)))
     1    STOP 'NY/NZOCN'
C        WRITE(*,*)'OCNDEP(km)=?, OCNT0(K)=?, OCNS0(g/kg)=?, OCNTXF=?'
C        READ(*,*) OCNDEP, OCNT0, OCNS0, OCNTXF
        WRITE(*,*) 'OCNT0(K)=?, OCNTXF=?'
        READ(*,*) OCNT0, OCNTXF
        OCNDEP= 5.
        OCNS0= 35.
        OCNDEP= OCNDEP * 1000.
        OCNS0= OCNS0 / 1000.
        WRITE(*,*) 'TDIFFC(m2/s)=?, HDFAC=?, PRANDL=?, LEWIS=?'
        READ(*,*) TDIFFC, HDFAC, PRANDL, LEWIS
        WRITE(*,*)
     1  'FWFLUX(Sv)=?, FWFTIM(yrs)=?, FWLMIN(deg)=?, FWLMAX(deg)=?'
        READ(*,*) FWFLUX, FWFTIM, FWLMIN, FWLMAX
        FWFLUX= FWFLUX * 1.0E6
        FWFTIM= FWFTIM * (NDPYR*DAYL)
C        WRITE(*,*) 'CO2CF1=?, CO2CF2=?, CO2CF3=?, CO2CF4=?, CO2CF5=?'
C        READ(*,*) CO2CF1, CO2CF2, CO2CF3, CO2CF4, CO2CF5
        CO2CF1= 0.0
        CO2CF2= 0.0
        CO2CF3= 0.0
        CO2CF4= 0.0
        CO2CF5= 0.0
        WRITE(*,*)
#ifdef ON_CRAY
        CALL DATE( DATE0 )
        CALL CLOCK( CLOCK0 )
        INITIM= DATE0//' '//CLOCK0
#else
        INITIM= 'sometime'
#endif
C       Initialize time-counter
        IPART= 0
        ISTEP= 0
      ENDIF
C     Derived parameters
C     Time-step
      IF (NSPDAY .GT. 0) THEN
C       Time-steps per day
        DT= DAYL / NSPDAY
      ELSE
C       Days per time-step
        DT= DAYL * (-NSPDAY)
      ENDIF
C     Re-compute current model time
      DTIM= DBLE(ISTEP)*DBLE(DT)
      TIM= DTIM
C     Length of "year"
      DYEARL= NDPYR * DDAYL
      YEARL= DYEARL
C
C     Initialize ocean model parameters
C     (Zonal-mean ocean => only one grid point in X direction)
C
C     Grid-points in X-direction
      NXOCN= 0
      WTXOCN(0)= 1.0
      LONMID(-1)= 120.9375
      LONMID(0)=  233.4375
      LONOCN(0)=  0.5*(LONMID(-1) + LONMID(0))
C     Ocean width (fractional area, 0..1)
      OCNWID= 1.0 - LANWID
C
      DXOCN= OCNWID*2.0*PI*AOCN
C     Initialize zonal velocity values to zero
      DO 55 L=0,NZOCN
        DO 55 K=0,NYOCN
   55     UOCN(K,L)= 0.0
C       Grid spacing (MU = sine(latitude) = -1 ... +1)
C     ( Ocean extends from y= -(Pi/2)a to +(pi/2)a )
      DMUOCN= 2.0 / (NYOCN+1)
      DYOCN= 0.5*OCNLEN*DMUOCN
      DZOCN= OCNDEP / (NZOCN+1)
      IF (HDFAC .GT. 0.) THEN
C       Compute horizontal grid-size for isotropic diffusion operator
        DYISO= DYOCN*OCNDEP/(HDFAC*OCNLEN)
      ELSE
C       No horizontal diffusion
        DYISO= 0.0
      ENDIF
C     Coordinate values, weights, wind-stress, freshwater flux region
      KFWLO= 0
      KFWHI= 0
      DO 60 K=0,NYOCN
        MUOCN(K)= -1.0 + 0.5*DMUOCN + K*DMUOCN
        YOCN(K)= 0.5*OCNLEN*MUOCN(K)
        PHIOCN(K)= SIGN(ASIN( ABS(MUOCN(K)) ), MUOCN(K))
        LATOCN(K)= PHIOCN(K) * 180./PI
        OCNTAU(K,JX)= 0.0
        OCNTAU(K,JY)= 0.0
        IF (LATOCN(K) .LT. FWLMIN) KFWLO= K+1
        IF (LATOCN(K) .LE. FWLMAX) KFWHI= K
   60   WTYOCN(K)= 1.0 / (NYOCN+1)
C
C       "Velocity" grid-points in Y-direction
        MUOMID(-1)= -1.0
        MUOMID(NYOCN)= 1.0
        DO 65 K=0,NYOCN-1
   65     MUOMID(K)= 0.5*(MUOCN(K) + MUOCN(K+1))
C
C     Compute area over which freshwater flux is input
      FWAREA= OCNWID*(4.0*PI*AOCN**2)*FLOAT(KFWHI-KFWLO+1)
     1                               /FLOAT(NYOCN+1)
      IF (FWLMAX .GT. FWLMIN) THEN
C       Compute (uniform) freshwater flux pulse input rate (kg/s*m**2)
        FWRATE=  OCNRHO*FWFLUX / FWAREA
      ELSE
C       No freshwater flux
        FWRATE=  0.0
      ENDIF
      DO 70 L=0,NZOCN
        ZOCN(L)= -OCNDEP + 0.5*DZOCN + L*DZOCN
   70   WTZOCN(L)= 1.0 / (NZOCN+1)
C     Anisotropic diffusion coefficients for momentum, T, S
      ODIFFZ(JOCT)= TDIFFC
      ODIFFZ(JOCV)= PRANDL*TDIFFC
      ODIFFZ(JOCS)= LEWIS*TDIFFC
C     Tracer diffusion coefficients (same as for S)
      DO 85 IPGQ=JOCTR1,JOCTR1+NTROCN-1
   85   ODIFFZ(IPGQ)= LEWIS*TDIFFC
C     HDFAC is the anisotropy in diffusive time-scales over the basin
      DO 90 IPGQ=1,NOCNQ
   90   ODIFFY(IPGQ)= ODIFFZ(IPGQ)*(HDFAC*OCNLEN/OCNDEP)**2
C     Initialize coefficient matrices for streamfunction computation
      CALL INVLP0(WCOEF, WFST, NYOCN, NZOCN, DYOCN, DZOCN)
C     Initialize coefficient matrices for diffusive vorticity balance
      CALL INVLP0(WCOEF2, WFST2, NYOCN, NZOCN, DYISO, DZOCN)
C
C     Compute the maximum stable time-step (for T, S, other tracers)
      DTCFL= (DZOCN**2)/TDIFFC
      DO 95 IPGQ=JOCT,NOCNQ
        IF (ODIFFY(IPGQ) .GT. 0.)
     1    DTCFL= AMIN1( DTCFL, (DYOCN**2)/ODIFFY(IPGQ) )
        IF (ODIFFZ(IPGQ) .GT. 0.)
     1    DTCFL= AMIN1( DTCFL, (DZOCN**2)/ODIFFZ(IPGQ) )
   95   CONTINUE
C
C     Ocean model details
      WRITE(*,*) 'Freshwater (DEGLO, DEGHI):',
     1           LATOCN(KFWLO), LATOCN(KFWHI)
      WRITE(*,*) 'Freshwater flux area (m**2)=', FWAREA
      WRITE(*,*) 'Freshwater flux rate (m/yr)=', FWRATE*YEARL/OCNRHO
      WRITE(*,*) 'DT-diffusive (days)=', DTCFL/DAYL
      WRITE(*,*)
C
      WRITE(*,*) 'File containing inward heat flux (W/m**2)?'
      READ(*,'(A)') FILNAM
      IF (FILNAM .NE. ' ') THEN
        OPEN(IUTEM, FILE=FILNAM)
        READ(IUTEM,*) (FTOCNM(K),K=0,NYOCN)
        CLOSE(IUTEM)
      ELSE
        DO 110 K=0,NYOCN
  110     FTOCNM(K)= 0.0
      ENDIF
C     Remove the global-average part of heat flux
      TEMP= 0.0
      DO 120 K=0,NYOCN
  120   TEMP= TEMP + WTYOCN(K)*FTOCNM(K)
      DO 125 K=0,NYOCN
  125   FTOCNM(K)= FTOCNM(K) - TEMP
C
      WRITE(*,*) 'File containing E-P (m/year)?'
      READ(*,'(A)') FILNAM
      IF (FILNAM .NE. ' ') THEN
        OPEN(IUTEM, FILE=FILNAM)
        READ(IUTEM,*) (EMINPM(K),K=0,NYOCN)
        CLOSE(IUTEM)
      ELSE
        DO 130 K=0,NYOCN
  130     EMINPM(K)= 0.0
      ENDIF
C     Remove the global-average part of E-P,
C     and convert to SI units (kg/s*m**2)
      TEMP= 0.0
      DO 140 K=0,NYOCN
  140   TEMP= TEMP + WTYOCN(K)*EMINPM(K)
      DO 145 K=0,NYOCN
  145   EMINPM(K)= (EMINPM(K) - TEMP)*1000./YEARL
C
C     Read convective adjustment flag
      WRITE(*,*) 'Convective adjustment flag? (T/F)'
      READ(*,*) CAFLAG
C
      WRITE(*,*) 'Coupling (to atmosphere) flag? (T/F)'
      READ(*,*) COUFLG
      IF (COUFLG) THEN
C       Coupled ocean-atmosphere integration
        RSTT= .FALSE.
        RSTS= .FALSE.
        WRITE(*,*) 'NOSYNC, NASYNC, NSAMPL, NSAVE?'
        READ(*,*) NOSYNC, NASYNC, NSAMPL, NSAVE
C
C       Synchronous integration flag
        SYNFLG= (NOSYNC .EQ. 1) .AND. (NASYNC .EQ. 1)
C       Initialize atmospheric model for coupled integration
        RUNNAM= RTITLE(1:5)
        CALL ATMBGN( ERRFLG, IASTEP, COMSTR, RUNNAM, IPART, IUNITR,
     1               DIRSTR, OLDRUN, IOPART, NSAMPL, NSAVE )
        IF (ERRFLG) STOP 'ERRFLG'
        IF ((IASTEP .NE. -1) .AND. (IASTEP .NE. ISTEP)) STOP 'IASTEP'
      ELSE
C       Ocean-only integration;
C       read boundary conditions
        WRITE(*,*) 'Restoring-T flag, restoring-S flag? (T/F)'
        READ(*,*) RSTT, RSTS
        WRITE(*,*) 'T, S restoring time constants (days)?'
        READ(*,*) TAUT, TAUS
C       Convert restoring time constants from days to seconds
        TAUT= TAUT * DAYL
        TAUS= TAUS * DAYL
C
        WRITE(*,*) 'File containing upper b.c. for T (K)?'
        READ(*,'(A)') FILNAM
        IF (FILNAM .NE. ' ') THEN
          OPEN(IUTEM, FILE=FILNAM)
          READ(IUTEM,*) (TTOP(K),K=0,NYOCN)
          CLOSE(IUTEM)
        ELSE
C         Surface temperature (in phase with solar radiation)
          YRDAY= YPHASE - 0.75
          DO 150 K=0,NYOCN
            TTOP(K)=  292.0
     1              - 3.0*COS(2.0*PI*YRDAY)*MUOCN(K)
     2              - 19.0*0.5*(3.0*MUOCN(K)**2 - 1.0)
  150       CONTINUE
        ENDIF
C
#ifdef GRAPHICS
C       Display forcing functions
        IF (RSTT) THEN
          DO 180 K=0,NYOCN
  180       FPRF(K)= TTOP(K)
          CALL GLABEL( 'lat', 3, 'T(K)', 4, 'T-surf', 6 )
          CALL GXYRNG(-90.0, 90.0, 6, -1, 200., 350., 6, -1)
          CALL GDRAW
          CALL GCURV( NYOCN+1, LATOCN, FPRF, 1)
        ELSE
          DO 190 K=0,NYOCN
  190     FPRF(K)= FTOCNM(K)
          CALL GLABEL( 'lat', 3, 'F(W/m2)', 7, 'F-ocean', 7 )
          CALL GXYRNG(-90.0, 90.0, 6, -1, -100., 100., 0, -1)
          CALL GDRAW
        CALL GCURV( NYOCN+1, LATOCN, FPRF, 1)
        ENDIF
        CALL ZFRAME
#endif
        WRITE(*,*) 'File containing upper b.c. for S (g/kg)?'
        READ(*,'(A)') FILNAM
        IF (FILNAM .NE. ' ') THEN
          OPEN(IUTEM, FILE=FILNAM)
          READ(IUTEM,*) (SOCN(K),K=0,NYOCN)
          CLOSE(IUTEM)
        ELSE
          DO 200 K=0,NYOCN
  200       SOCN(K)= OCNS0*1000. + 1.0*COS(2.0*YOCN(K)/AOCN)
        ENDIF
C       Convert S values to natural units (kg/kg), compute averages
        SOCNAV= 0.0
        TTOPAV= 0.0
        DO 210 K=0,NYOCN
          SOCN(K)= SOCN(K) / 1000.
          SOCNAV= SOCNAV + WTYOCN(K)*SOCN(K)
  210     TTOPAV= TTOPAV + WTYOCN(K)*TTOP(K)
C
#ifdef GRAPHICS
        IF (RSTS) THEN
        DO 250 K=0,NYOCN
  250       FPRF(K)= SOCN(K) * 1000.
          CALL GLABEL( 'lat', 3, 'S', 7, 'S(g/kg)', 7 )
          CALL GXYRNG(-90.0, 90.0, 6, -1, 30., 40., 0, -1)
          CALL GDRAW
          CALL GCURV( NYOCN+1, LATOCN, FPRF, 1)
        ELSE
          DO 260 K=0,NYOCN
  260       FPRF(K)= EMINPM(K)*YEARL/1000.
          CALL GLABEL( 'lat', 3, 'E-P(m/y)', 8, 'Evap-Precip', 11 )
          CALL GXYRNG(-90.0, 90.0, 6, -1, -0.5, 0.5, 0, -1)
          CALL GDRAW
          CALL GCURV( NYOCN+1, LATOCN, FPRF, 1)
        ENDIF
        CALL ZFRAME
#endif
      ENDIF
C
      IF (SAVFIL .NE. 'new') THEN
C       Read in initial state
        READ(IUSAV,FMT)
     1       (((OCNQ0(K,L,IPGQ),K=0,NYOCN),L=0,NZOCN),IPGQ=JOCV,JOCS)
        READ(IUSAV,FMT)
     1       (((OCNQ1(K,L,IPGQ),K=0,NYOCN),L=0,NZOCN),IPGQ=JOCV,JOCS)
        IF (NTROCN .GT. 0) THEN
C       Read ocean tracers
          READ(IUSAV,FMT)
     1       (((OCNQ0(K,L,IPGQ),K=0,NYOCN),L=0,NZOCN),
     2          IPGQ=JOCTR1,JOCTR1+NTROCN-1)
          READ(IUSAV,FMT)
     1       (((OCNQ1(K,L,IPGQ),K=0,NYOCN),L=0,NZOCN),
     2          IPGQ=JOCTR1,JOCTR1+NTROCN-1)
        ENDIF
        CLOSE(IUSAV)
      ELSE
C       Initialize isothermal/isosaline ocean at rest
        DO 265 L=0,NZOCN
          DO 265 K=0,NYOCN
            OCNQ1(K,L,JOCV)= 0.0
            OCNQ1(K,L,JOCT)= 0.0
  265       OCNQ1(K,L,JOCS)= 0.0
C       Initialize tracer distributions
        DO 270 IPGQ=JOCTR1,JOCTR1+NTROCN-1
          DO 270 L=0,NZOCN
            DO 270 K=0,NYOCN
  270         OCNQ1(K,L,IPGQ)= 0.0
      ENDIF
C
C     Add salinity perturbation in top layer
      WRITE(*,*)
     1 'Surface salinity perturbation (48 to 90 deg)=(SH,NH)? (o/oo)'
      READ(*,*) SPERTS, SPERTN
      DO 273 K=0,(NYOCN/8)
        OCNQ0(K,NZOCN,JOCS)= OCNQ0(K,NZOCN,JOCS) + SPERTS*1.0E-3
  273   OCNQ1(K,NZOCN,JOCS)= OCNQ1(K,NZOCN,JOCS) + SPERTS*1.0E-3
      DO 277 K=7*(NYOCN/8),NYOCN
        OCNQ0(K,NZOCN,JOCS)= OCNQ0(K,NZOCN,JOCS) + SPERTN*1.0E-3
  277   OCNQ1(K,NZOCN,JOCS)= OCNQ1(K,NZOCN,JOCS) + SPERTN*1.0E-3
C
C     Save history data at selected levels/latitudes
      IHSFL2= 0
      IVSFL2= 0
      IZSFL2= 0
C
      OSRFSV= .FALSE.
      DO 290 L=0,NZOCN
  290   OLEVSV(L)= .FALSE.
      DO 295 K=0,NYOCN
  295   OLATSV(K)= .FALSE.
C
C     (One jump to this statement)
C     Read number of time-steps, salinity forcing, perturbation
  300 WRITE(*,*) 'NSTEP, NSKIP, NOSAVE, PSISYM, PSIASY'
      READ(*,*) NSTEP, NSKIP, NOSAVE, PSISYM, PSIASY
      IF (NSTEP .LE. 0) GOTO 900
      IF (COUFLG) THEN
        IF (MOD(NSTEP,NOSYNC) .NE. 0) THEN
          WRITE(*,*) 'Error: NSTEP should be a multiple of NOSYNC'
          STOP
        ENDIF
      ENDIF
C
      IF (NSKIP .GT. 0) THEN
C       Diagnostic sampling switched on
        IDAFL2= IUDAO
      ELSE
C       No diagnostic sampling
        NSKIP= -NSKIP
        IDAFL2= 0
      ENDIF
C
      IF (NOSAVE .GT. 0) THEN
C       Switch on zonal mean save files/time-series files
        IZSFL2= IUZSO
        NTSFR2= NOSAVE
        ITSFL2= IUTSO
      ELSE
C       No zonal-mean save files/time-series files
        IZSFL2= 0
        NTSFR2= 0
        ITSFL2= 0
      ENDIF
C
C     Compute advective CFL time-step
      TEMP= 2.0*AMAX1( 0.5*ABS(PSISYM), ABS(PSIASY) )
      IF (TEMP .GT. 0.) DTCFL= AMIN1( DTCFL, DZOCN*OCNLEN/TEMP )
      TEMP= 2.0*AMAX1( ABS(PSISYM), ABS(PSIASY) )
      IF (TEMP .GT. 0.) DTCFL= AMIN1( DTCFL, DYOCN*OCNDEP/TEMP )
      WRITE(*,*) 'DT-minimum (days)=', DTCFL/DAYL
C     Add perturbation
      MVERT= PI/OCNDEP
      DO 310 L=0,NZOCN
        DO 310 K=0,NYOCN
C         Perturbation vorticity (actually ignored)
          OCNQ1(K,L,JOCV)= OCNQ1(K,L,JOCV)
     1                    + SIN(MVERT*ZOCN(L))
     2                     *(  PSIASY*(MVERT**2 + 1.0/AOCN**2)
     3                               *COS(YOCN(K)/AOCN)
     4                       + PSISYM*(MVERT**2 + 4.0/AOCN**2)
     5                               *SIN(2.0*YOCN(K)/AOCN) )
C         Perturbation temperature
          OCNQ1(K,L,JOCT)= OCNQ1(K,L,JOCT) + 0.0
C         Perturbation salinity (in "balance" with vorticity)
          OCNQ1(K,L,JOCS)= OCNQ1(K,L,JOCS)
     1                   + (PRANDL*TDIFFC/SXCOEF)
     2                    *SIN(MVERT*ZOCN(L))
     3                    *(  PSIASY*( ( (MVERT**2 + 1.0/AOCN**2)
     4                                  *(1.0+HDFAC**2)*MVERT**2 )
     5                                *AOCN)*SIN(YOCN(K)/AOCN)
     6                      - PSISYM*( ( (MVERT**2 + 4.0/AOCN**2)
     7                                  *(1.0+4.0*HDFAC**2)*MVERT**2 )
     8                                *AOCN/2.0)*COS(2.0*YOCN(K)/AOCN))
  310     CONTINUE
      IF (RSTT .AND. (TAUT .EQ. 0.)) THEN
C       Specified temperature at the top level (b.c.)
        DO 320 K=0,NYOCN
          OCNQ0(K,NZOCN,JOCT)= TTOP(K) - OCNT0
  320     OCNQ1(K,NZOCN,JOCT)= TTOP(K) - OCNT0
      ENDIF
      IF (RSTS .AND. (TAUS .EQ. 0.)) THEN
C       Specified salinity at the top level (b.c.)
        DO 330 K=0,NYOCN
          OCNQ0(K,NZOCN,JOCS)= SOCN(K) - OCNS0
  330     OCNQ1(K,NZOCN,JOCS)= SOCN(K) - OCNS0
      ENDIF
C     Zero all surface fluxes at the top
      DO 340 IPGQ=1,NOCNQ
        DO 340 K=0,NYOCN
  340     OCNFLX(K,IPGQ)= 0.0
C
C     New set of parts
  350 WRITE(*,*) 'NPART=?'
      READ(*,*) NPART
      IF (NPART.LE.0) GOTO 300
C     Compute last part number
      LPART= IPART + NPART - 1
      WRITE(*,*)
      WRITE(*,'(1X,A,F10.2,A)') 'Starting at ', DTIM/DDAYL, ' days'
      WRITE(*,*)
C
C     New run part; initialize diagnostics
  360 ISET= MOD(IPART,10000)
      IF (IDAFL2 .NE. 0) THEN
        CALL ODIINI( ISET, DT, NSPDAY, NDPYR, ISTEP,
     1               NSKIP, NTSFR2, NOSAVE,
     2               IDAFL2, ITSFL2, IHSFL2, IVSFL2, IZSFL2,
     3               .FALSE., OSRFSV, OLEVSV, OLATSV,
     4               YPHASE, RTITLE, INITIM )
C       Sample initial condition; compute velocity components
        CALL INVLAP( PSIOCN, OCNQ1(0,0,JOCV), NYOMAX,
     1               NYOCN, NZOCN, DYOCN, DZOCN, WCOEF, WFST)
        DO 380 L=1,NZOCN
          DO 370 K=1,NYOCN
            VOCN(K,L)= -(PSIOCN(K,L) - PSIOCN(K,L-1))
     1                 /(0.5*PI*DZOCN*COS(PHIOCN(K)))
            WOCN(K,L)=  (PSIOCN(K,L) - PSIOCN(K-1,L))
     1                 /(0.5*PI*DYOCN*COS(PHIOCN(K)))
  370       CONTINUE
          VOCN(0,L)= 0.0
  380     WOCN(0,L)= WOCN(1,L)
        DO 390 K=1,NYOCN
          VOCN(K,0)= VOCN(K,1)
          WOCN(K,0)=  (PSIOCN(K,0) - PSIOCN(K-1,0))
     1               /(0.5*PI*DYOCN*COS(PHIOCN(K)))
  390     CONTINUE
        VOCN(0,0)= 0.0
        WOCN(0,0)= 0.0
        CALL ODISTP( ISTEP, UOCN, VOCN, WOCN, OCNQ1(0,0,JOCT) )
      ENDIF
C
      DO 700 IT=1,NSTEP
        IF (ISTEP.EQ.0) THEN
C         First time-step
C         Copy prognostic quantities to "fool" time-stepping routine
          DO 410 IPGQ=1,NOCNQ
            CALL QCOPY( OCNQ0(0,0,IPGQ), OCNQ1(0,0,IPGQ),
     1                  NYOMAX+1, NYOCN+1, NZOCN+1 )
  410     CONTINUE
C         Effective time-step value
          DTEFF= 0.5*DT
        ELSE
C         Not the first time-step
C         Effective time-step value
          DTEFF= DT
        ENDIF
C
        IF (COUFLG) THEN
C         Coupled ocean-atmosphere integration;
          IF (MOD(ISTEP,NOSYNC) .EQ. 0) THEN
C           Copy/compute SST values (in K)
            DO 420 K=0,NYOCN
  420         SSTOCN(K)= OCNQ0(K,NZOCN,JOCT)+OCNT0
C
            IF (.NOT. SYNFLG .AND. (ISTEP .EQ. 0)) THEN
C             Asynchronous integration; send initial SSTs
              CALL ATMINT( SYNFLG, -1, NOSYNC, NASYNC,
     1                     NXOMAX+1, NXOCN+1, NYOCN+1,
     2                     LONOCN(0), LONMID(-1), MUOCN(0), MUOMID(-1),
     3                     SSTOCN, FTOCNT, EMINPT,
     4                     OCNTAU(0,JX), OCNTAU(0,JY), ERRFLG)
              IF (ERRFLG) STOP 'ERRFLG'
            ENDIF
C           Send SSTs to atmospheric model and obtain surface fluxes
            CALL ATMINT( SYNFLG, ISTEP, NOSYNC, NASYNC,
     1                   NXOMAX+1, NXOCN+1, NYOCN+1,
     2                   LONOCN(0), LONMID(-1), MUOCN(0), MUOMID(-1),
     3                   SSTOCN, FTOCNT, EMINPT,
     4                   OCNTAU(0,JX), OCNTAU(0,JY), ERRFLG)
            IF (ERRFLG) STOP 'ERRFLG'
C           Add mean fluxes to fluxes from atmospheric model
            DO 430 K=0,NYOCN
              FTOCNT(K)= FTOCNT(K) + FTOCNM(K)
  430         EMINPT(K)= EMINPT(K) + EMINPM(K)
          ENDIF
        ELSE
C         Uncoupled integration
          IF (RSTT .AND. (TAUT .GT. 0.)) THEN
C           Compute "restoring" heat flux
            DO 440 K=0,NYOCN
  440         FTOCNT(K)= -(DZOCN*OCNRHO/TAUT)*OCNCP*OCNQ0(K,NZOCN,JOCT)
          ELSE
C           Copy mean heat flux
            DO 445 K=0,NYOCN
  445         FTOCNT(K)= FTOCNM(K)
          ENDIF
          IF (RSTS .AND. (TAUS .GT. 0.)) THEN
C           Compute "restoring" freshwater flux
            DO 450 K=0,NYOCN
  450         EMINPT(K)= -(DZOCN*OCNRHO/TAUS)*OCNQ0(K,NZOCN,JOCS)/OCNS0
          ELSE
C           Copy mean freshwater flux
            DO 455 K=0,NYOCN
  455         EMINPT(K)= EMINPM(K)
          ENDIF
        ENDIF
C
C       Compute buoyancy tendency
        DO 460 L=0,NZOCN
          DO 460 K=0,NYOCN
  460       DOCNQ(K,L,JOCV)= 0.0
        CALL BUOYAN( DOCNQ(0,0,JOCV), OCNQ1(0,0,JOCT),
     1               OCNQ1(0,0,JOCS), NYOMAX, NYOCN, NZOCN,
     1               DYOCN, OCNTXF*TXCOEF, SXCOEF )
C       Convert buoyancy tendency to del-squared(vorticity)
C         (assuming infinite Prandtl number diffusive balance)
        DO 470 L=0,NZOCN
          DO 470 K=0,NYOCN
  470       DOCNQ(K,L,JOCV)= -DOCNQ(K,L,JOCV)/ODIFFZ(JOCV)
C       Compute vorticity from buoyancy tendency
        CALL INVLAP( OCNQ2(0,0,JOCV), DOCNQ(0,0,JOCV), NYOMAX,
     1               NYOCN, NZOCN, DYISO, DZOCN, WCOEF2, WFST2)
C       Compute streamfunction from vorticity
        CALL INVLAP( PSIOCN, OCNQ2(0,0,JOCV), NYOMAX,
     1               NYOCN, NZOCN, DYOCN, DZOCN, WCOEF, WFST)
C
C       Compute nondimensional density (convert to ppt)
        DO 480 L=0,NZOCN
          DO 480 K=0,NYOCN
            RHOOCN(K,L)= (  OCNTXF*(TXCOEF/GRAVIT)*OCNQ1(K,L,JOCT)
     1                    + (SXCOEF/GRAVIT)*OCNQ1(K,L,JOCS) ) * 1000.
  480       CONTINUE
        IF (MOD(IT-1,NSKIP) .EQ. 0) THEN
#ifdef GRAPHICS
          WRITE(*,*) 'Time (days)= ', DTIM/DDAYL
          IF (CAFLAG) THEN
            CALL ZSPLIT( 1, 2, 0)
            CALL SELWIN(1)
            CALL DISOCN( RHOOCN, 0, 0.0, 0.0, 'rho' )
            CALL SELWIN(2)
          ENDIF
          CALL DISOCN( PSIOCN, 0, 0.0, 0.0, 'psi' )
          CALL ZFRAME
#endif
        ENDIF
C       Compute advective tendencies for T, S, and other tracers
        CALL ARAKWA( DOCNQ(0,0,JOCT), OCNQ1(0,0,JOCT), PSIOCN,
     1               NYOMAX, NZOMAX, NYOCN, NZOCN,
     2               NOCNQ-JOCT+1, DYOCN, DZOCN)
C
        IF (.NOT. RSTT) THEN
C         Specified heat flux at the top
          DO 520 K=0,NYOCN
  520       OCNFLX(K,JOCT)= FTOCNT(K)
        ENDIF
        IF (.NOT. RSTS) THEN
C         Specified salinity flux at the top
          DO 530 K=0,NYOCN
  530       OCNFLX(K,JOCS)= EMINPT(K)
          IF (TIM .LT. FWFTIM) THEN
C           Add freshwater flux in localized region
            DO 535 K=KFWLO,KFWHI
  535         OCNFLX(K,JOCS)= OCNFLX(K,JOCS) + FWRATE
          ENDIF
        ENDIF
C
C       Compute diffusive fluxes at ocean surface
        DO 540 K=0,NYOCN
C         No vorticity flux at top of ocean
          DIFFLX(K,JOCV)= 0.0
C         Convert heat flux to diffusive temperature flux
C         (units: K*m/s)
          DIFFLX(K,JOCT)= OCNFLX(K,JOCT) / (OCNRHO * OCNCP)
C         Convert E-P to salinity flux (units: m/s)
C         (use global-mean, rather than surface salinity)
          DIFFLX(K,JOCS)= OCNFLX(K,JOCS) * OCNS0 / OCNRHO
  540     CONTINUE
C
C       Compute diffusive tendencies for T, S, and other tracers
        DO 550 IPGQ=JOCT,NOCNQ
          CALL DIFUSE( DOCNQ(0,0,IPGQ), OCNQ0(0,0,IPGQ),
     1                 NYOMAX, NYOCN, NZOCN, DYOCN, DZOCN,
     2                 ODIFFY(IPGQ), ODIFFZ(IPGQ), DIFFLX(0,IPGQ))
  550     CONTINUE
        IF (CAFLAG) THEN
C         Carry out convective adjustment,
C          and add the convective adjustment tendency
C
C         Compute interim values for T, S, and other tracers
          DO 551 IPGQ=JOCT,NOCNQ
            DO 551 L=0,NZOCN
              DO 551 K=0,NYOCN
                OCNQ2(K,L,IPGQ)=  OCNQ0(K,L,IPGQ)
     1                          + 2.0 * DTEFF * DOCNQ(K,L,IPGQ)
  551           CONTINUE
C         Compute convective adjustment tendency
          DO 557 K=0,NYOCN
C           Copy T, S, and other tracers in a column
            DO 553 IPGQ=JOCT,NOCNQ
              TRCCL0(IPGQ)= 0.0
              DO 553 L=0,NZOCN
                TRCCOL(L,IPGQ)= OCNQ2(K,L,IPGQ)
                TRCCL0(IPGQ)= TRCCL0(IPGQ) + TRCCOL(L,IPGQ)*WTZOCN(L)
  553           CONTINUE
CC            IF (K .EQ. NYOCN-3) THEN
CC              WRITE(*,*) 'TRCCL0:',(TRCCL0(IPGQ),IPGQ=JOCT,NOCNQ)
CC            ENDIF
C           Convectively adjust the column
            CALL CONVAD( TRCCOL, OCNTXF*(TXCOEF/GRAVIT),
     1                  -(SXCOEF/GRAVIT), IADJ, IADLEV )
C           Add convective adjustment tendency to other tendencies
            DO 555 IPGQ=JOCT,NOCNQ
              TRCCL1(IPGQ)= 0.0
              DO 555 L=0,NZOCN
                DOCNQ(K,L,IPGQ) =  DOCNQ(K,L,IPGQ) +
     1           (TRCCOL(L,IPGQ) - OCNQ2(K,L,IPGQ)) / (2.0*DTEFF)
                TRCCL1(IPGQ)= TRCCL1(IPGQ) + TRCCOL(L,IPGQ)*WTZOCN(L)
  555           CONTINUE
CC            IF (K .EQ. NYOCN-3) THEN
CC              WRITE(*,*) 'IADJ, IADLEV:', IADJ, IADLEV
CC              WRITE(*,*) 'TRCCL1:',(TRCCL1(IPGQ),IPGQ=JOCT,NOCNQ)
CC              WRITE(*,*) 'T:',(TRCCOL(L,JOCT),L=NZOCN,NZOCN-11,-1)
CC              WRITE(*,*) 'S:',(TRCCOL(L,JOCS),L=NZOCN,NZOCN-11,-1)
CC            ENDIF
  557       CONTINUE
        ENDIF
C
        IF (RSTT .AND. (TAUT .EQ. 0.)) THEN
          DO 560 K=0,NYOCN
C           Compute net heat flux into ocean
            FTOCN0(K)= -OCNRHO * OCNCP * DZOCN * DOCNQ(K,NZOCN,JOCT)
C           Set top-level temperature tendency to zero (b.c.)
  560       DOCNQ(K,NZOCN,JOCT)= 0.0
        ELSE
C         Copy net heat flux into ocean
          DO 565 K=0,NYOCN
  565       FTOCN0(K)= OCNFLX(K,JOCT)
        ENDIF
        IF (RSTS .AND. (TAUS .EQ. 0.)) THEN
          DO 570 K=0,NYOCN
C           Compute net E - P
            EMINP0(K)= -OCNRHO * DZOCN * DOCNQ(K,NZOCN,JOCS) / OCNS0
C           Set top-level salinity tendency to zero (b.c.)
  570       DOCNQ(K,NZOCN,JOCS)= 0.0
        ELSE
C         Copy E - P
          DO 575 K=0,NYOCN
  575       EMINP0(K)= OCNFLX(K,JOCS)
        ENDIF
C
C       Leap-frog scheme for T, S, and other tracers
        DO 610 IPGQ=JOCT,NOCNQ
          DO 610 L=0,NZOCN
            DO 610 K=0,NYOCN
              OCNQ2(K,L,IPGQ)=  OCNQ0(K,L,IPGQ)
     1                        + 2.0 * DTEFF * DOCNQ(K,L,IPGQ)
  610         CONTINUE
C
C       Ensure that the global mean salinity remains constant
        TEMP= 0.0
        DO 620 L=0,NZOCN
          DO 620 K=0,NYOCN
  620       TEMP= TEMP + WTYOCN(K)*WTZOCN(L)*OCNQ2(K,L,JOCS)
        DO 630 L=0,NZOCN
          DO 630 K=0,NYOCN
  630       OCNQ2(K,L,JOCS)= OCNQ2(K,L,JOCS) - TEMP
C
C       Compute velocity components in the meridional plane
        DO 650 L=1,NZOCN
          DO 640 K=1,NYOCN
            VOCN(K,L)= -(PSIOCN(K,L) - PSIOCN(K,L-1))
     1                 /(0.5*PI*DZOCN*COS(PHIOCN(K)))
            WOCN(K,L)=  (PSIOCN(K,L) - PSIOCN(K-1,L))
     1                 /(0.5*PI*DYOCN*COS(PHIOCN(K)))
  640       CONTINUE
          VOCN(0,L)= 0.0
  650     WOCN(0,L)= WOCN(1,L)
        DO 660 K=1,NYOCN
          VOCN(K,0)= VOCN(K,1)
          WOCN(K,0)=  (PSIOCN(K,0) - PSIOCN(K-1,0))
     1               /(0.5*PI*DYOCN*COS(PHIOCN(K)))
  660     CONTINUE
        VOCN(0,0)= 0.0
        WOCN(0,0)= 0.0
C
C       Apply Robert filter on T, S, and other tracers
        DO 670 IPGQ=JOCT,NOCNQ
          CALL ROBERT( OCNQ0(0,0,IPGQ), OCNQ1(0,0,IPGQ),
     1                 OCNQ2(0,0,IPGQ),
     2                 NYOMAX+1, NYOCN+1, NZOCN+1, ROBFAC)
  670     CONTINUE
C
        IF (RSTT .AND. (TAUT .EQ. 0.)) THEN
C         Reset surface temperature
          DO 680 K=0,NYOCN
            OCNQ1(K,NZOCN,JOCT)= TTOP(K) - OCNT0
  680       OCNQ2(K,NZOCN,JOCT)= TTOP(K) - OCNT0
        ENDIF
        IF (RSTS .AND. (TAUS .EQ. 0.)) THEN
C         Reset surface salinity
          DO 685 K=0,NYOCN
            OCNQ1(K,NZOCN,JOCS)= SOCN(K) - OCNS0
  685       OCNQ2(K,NZOCN,JOCS)= SOCN(K) - OCNS0
        ENDIF
C
C       Exchange storage for all quantities
        DO 690 IPGQ=1,NOCNQ
          CALL QCOPY( OCNQ0(0,0,IPGQ), OCNQ1(0,0,IPGQ),
     1                NYOMAX+1, NYOCN+1, NZOCN+1 )
          CALL QCOPY( OCNQ1(0,0,IPGQ), OCNQ2(0,0,IPGQ),
     1                NYOMAX+1, NYOCN+1, NZOCN+1 )
  690     CONTINUE
C
C       Increment time-step counter
        ISTEP= ISTEP + 1
C       Re-compute current model time
        DTIM= DBLE(ISTEP)*DBLE(DT)
        TIM= DTIM
C
        IF (IDAFL2 .NE. 0) THEN
C         Diagnostics
          CALL ODIFLX( ISTEP, UOCN, VOCN, WOCN, OCNQ1(0,0,JOCT),
     1                 OCNTAU(0,JX), OCNTAU(0,JY),
     2                 FTOCN0, EMINP0 )
        ENDIF
  700   CONTINUE
      IF (NSTEP.GE.NSKIP) THEN
C       Write out diagnostics
        IF (IDAFL2 .NE. 0) CALL ODIEND( FILNAM )
      ENDIF
C     Next run part
      IPART= IPART + 1
C     Check if all run parts have completed
      IF (IPART .LE. LPART) THEN
C       Write diagnostics file for this part
        IF (COUFLG) CALL ATMDIA
C       Next run part
        GOTO 360
      ENDIF
C     All run parts have been completed
      WRITE(*,*)
      WRITE(*,'(1X,A,F10.2,A)') 'Finished at ', DTIM/DDAYL, ' days'
      WRITE(*,*)
#ifdef GRAPHICS
      write(*,*) 'Press <RETURN> to continue'
      read(*,*)
      CALL DISOCN( OCNQ1(0,0,JOCV), 0, 0.0, 0.0, 'vor' )
      CALL ZFRAME
      read(*,*)
      CALL DISOCN( OCNQ1(0,0,JOCT), 0, 0.0, 0.0, 'T' )
      CALL ZFRAME
      read(*,*)
      CALL DISOCN( OCNQ1(0,0,JOCS), 0, 0.0, 0.0, 'S' )
      CALL ZFRAME
      read(*,*)
      IF (RSTT) THEN
        CALL GLABEL( 'lat', 3, 'W/m2', 7, ' ', 1 )
        CALL GXYRNG(-90.0, 90.0, 6, -1, -100., 100., 0, -1)
        CALL GDRAW
        CALL GCURV( NYOCN+1, LATOCN, FTOCN0, 1)
        CALL GCURV( NYOCN+1, LATOCN, FTOCNM, 2)
        WRITE(*,*) 'Average FTOCN (W/m**2)'
        WRITE(*,'(10(F7.2,'',''))')(FTOCN0(K),K=0,NYOCN)
      ELSE IF (.NOT. COUFLG) THEN
C       Compute average surface temperature
        TEMP= 0.0
        DO 710 K=0,NYOCN
  710     TEMP= TEMP + WTYOCN(K)*OCNQ1(K,NZOCN,JOCT)
C       Adjust model average to "observations"
        DO 720 K=0,NYOCN
  720     FPRF(K)= OCNQ1(K,NZOCN,JOCT) - TEMP + TTOPAV
        CALL GLABEL( 'lat', 3, 'T(K)', 4, 'T-surf', 6 )
        CALL GXYRNG(-90.0, 90.0, 6, -1, 200., 350., 6, -1)
        CALL GDRAW
        CALL GCURV( NYOCN+1, LATOCN, FPRF, 1)
        CALL GCURV( NYOCN+1, LATOCN, TTOP, 3)
        WRITE(*,*) 'Surface average T diff (K) = ',
     1             TEMP+OCNT0-TTOPAV
        WRITE(*,*) 'Average TOCN (K)'
        WRITE(*,'(10(F7.2,'',''))')(FPRF(K),K=0,NYOCN)
      ENDIF
      CALL ZFRAME
      read(*,*)
      IF (RSTS) THEN
        DO 740 K=0,NYOCN
  740     FPRF(K)= EMINP0(K)*YEARL/1000.
        CALL GLABEL( 'lat', 3, 'E-P(m/y)', 8, 'Evap-Precip', 11 )
        CALL GXYRNG(-90.0, 90.0, 6, -1, -0.5, 0.5, 0, -1)
        CALL GDRAW
        CALL GCURV( NYOCN+1, LATOCN, FPRF, 1)
        WRITE(*,*) 'Average E-P (m/year)'
        WRITE(*,'(10(F7.2,'',''))')(FPRF(K),K=0,NYOCN)
        DO 760 K=0,NYOCN
  760     FPRF(K)= EMINPM(K)*YEARL/1000.
        CALL GCURV( NYOCN+1, LATOCN, FPRF, 3)
      ELSE IF (.NOT. COUFLG) THEN
C       Compute average surface salinity
        TEMP= 0.0
        DO 770 K=0,NYOCN
  770     TEMP= TEMP + WTYOCN(K)*OCNQ1(K,NZOCN,JOCS)
C       Adjust model average to "observations"
        DO 780 K=0,NYOCN
  780     FPRF(K)= (OCNQ1(K,NZOCN,JOCS) - TEMP + SOCNAV)* 1000.
        CALL GLABEL( 'lat', 3, 'S', 7, 'S(g/kg)', 7 )
        CALL GXYRNG(-90.0, 90.0, 6, -1, 30., 40., 0, -1)
        CALL GDRAW
        CALL GCURV( NYOCN+1, LATOCN, FPRF, 1)
        WRITE(*,*) 'Surface average S diff (g/kg) = ',
     1             (TEMP+OCNS0-SOCNAV)*1000.
        WRITE(*,*) 'Average SOCN (g/kg)'
        WRITE(*,'(10(F7.2,'',''))')(FPRF(K),K=0,NYOCN)
        DO 790 K=0,NYOCN
  790     FPRF(K)= SOCN(K) * 1000.
        CALL GCURV( NYOCN+1, LATOCN, FPRF, 3)
      ENDIF
      CALL ZFRAME
      read(*,*)
#endif
      GOTO 300
C
C     Terminate run
C     Save ocean model parameters/final state
C     (One jump to this statement)
  900 IF (COUFLG) THEN
C       Terminate atmospheric model integration
        CALL ATMEND
      ENDIF
      WRITE(*,*) 'Save file name? (type "interp.so" to interpolate)'
      READ(*,'(A)') SAVFIL
      IF (SAVFIL .EQ. 'interp.so') THEN
C       Interpolate to new resolution
        WRITE(*,*) 'NYOCN2, NZOCN2, NSPDA2?'
        READ(*,*) NYOCN2, NZOCN2, NSPDA2
C
C       Initialize y-coordinate arrays and new weights
        YOLD(-1)= -1.0
        YOLD(NYOCN)= 1.0 - 1.0/FLOAT(NYOCN+1)
        YOLD(NYOCN+1)= 1.0
        YOMID(-1)= -1.0
        YOMID(NYOCN)= 1.0
        DO 910 K=0,NYOCN-1
          YOLD(K)= -1.0 + 2.0*((K+0.5)/FLOAT(NYOCN+1))
  910     YOMID(K)= -1.0 + 2.0*(FLOAT(K+1)/FLOAT(NYOCN+1))
        DO 920 K=-1,NYOCN2
  920     YNMID(K)= -1.0 + 2.0*(FLOAT(K+1)/FLOAT(NYOCN2+1))
C
C       Initialize z-coordinate arrays and new weights
        ZOLD(-1)= -OCNDEP
        ZOLD(NZOCN)= - 0.5*OCNDEP/FLOAT(NZOCN+1)
        ZOLD(NZOCN+1)= 0.0
        ZOMID(-1)= -OCNDEP
        ZOMID(NZOCN)= 0.0
        DO 930 L=0,NZOCN-1
          ZOLD(L)= -OCNDEP + OCNDEP*((L+0.5)/FLOAT(NZOCN+1))
  930     ZOMID(L)= -OCNDEP + OCNDEP*(FLOAT(L+1)/FLOAT(NZOCN+1))
        DO 940 L=-1,NZOCN2
  940     ZNMID(L)= -OCNDEP + OCNDEP*(FLOAT(L+1)/FLOAT(NZOCN2+1))
C       Initialize horizontal interpolation arrays
        CALL FLXIN1( IYMAP, YFRAC, DYNEW, DYOLD, YLWT, YRWT,
     1               YNMID, NYOCN2+1, YOMID, YOLD, NYOCN+1 )
C       Initialize vertical interpolation arrays
        CALL FLXIN1( IZMAP, ZFRAC, DZNEW, DZOLD, ZLWT, ZRWT,
     1               ZNMID, NZOCN2+1, ZOMID, ZOLD, NZOCN+1 )
C       Interpolate to new resolution
        DO 970 IPGQ=1,NOCNQ
          CALL OCNINT( OCNQ0(0,0,IPGQ), OCNQ0(0,0,IPGQ),
     1                 NYOCN, NYOCN2, NZOCN, NZOCN2,
     2                 IYMAP, YFRAC, DYNEW, DYOLD, YLWT, YRWT,
     3                 IZMAP, ZFRAC, DZNEW, DZOLD, ZLWT, ZRWT )
          CALL OCNINT( OCNQ1(0,0,IPGQ), OCNQ1(0,0,IPGQ),
     1                 NYOCN, NYOCN2, NZOCN, NZOCN2,
     2                 IYMAP, YFRAC, DYNEW, DYOLD, YLWT, YRWT,
     3                 IZMAP, ZFRAC, DZNEW, DZOLD, ZLWT, ZRWT )
  970     CONTINUE
C       Change resolution parameters
        NYOCN= NYOCN2
        NZOCN= NZOCN2
        IF (NSPDA2 .NE. 0) THEN
C         Change time-steps per day
          NSPDAY= NSPDA2
C         Reset time-step
          IF (NSPDAY .GT. 0) THEN
            DT= DAYL / NSPDAY
          ELSE
            DT= DAYL * (-NSPDAY)
          ENDIF
C         Reset current model time-step count
          ISTEP= NINT(DTIM/DBLE(DT))
        ENDIF
      ENDIF
      IF (SAVFIL.NE.' ') THEN
C       Write save file
        OPEN(IUSAV, FILE=SAVFIL )
        IF (SSNFLG) THEN
          CURPHA= ABS(YPHASE + DTIM/DYEARL - INT(YPHASE + DTIM/DYEARL))
        ELSE
          CURPHA= YPHASE
        ENDIF
C       Compute CO2 concentraton
        IF (TIM .GE. MULTIM) THEN
          CURCO2= DLWCO2 * MULCO2
        ELSE
          CURCO2= DLWCO2 * EXP( (TIM/MULTIM) * ALOG(MULCO2) )
        ENDIF
        WRITE(IUSAV,'(1X,A72)') RTITLE
        WRITE(IUSAV,'(1X,A72)') PRGSTR
        WRITE(IUSAV,'(1X,A72)') INITIM
        WRITE(IUSAV,'(1X,1P,3E15.6)') TIM, CURPHA, CURCO2
        WRITE(IUSAV,'(1X,2I12)') IPART, ISTEP
        WRITE(IUSAV,'(1X,L2)') SSNFLG
        WRITE(IUSAV,'(1X,2I12)') NSPDAY, NDPYR
        WRITE(IUSAV,'(1X,1P,4E15.6)') T0TOP, EQPFAC, PPFAC, YPHASE
        WRITE(IUSAV,'(1X,1P,3E15.6)') DLWCO2, MULCO2, MULTIM
        WRITE(IUSAV,'(1X,1P,E15.6)') LANWID
        WRITE(IUSAV,'(1X,3I12)') NYOCN, NZOCN, NTROCN
        WRITE(IUSAV,'(1X,1P,4E15.6)') OCNDEP, OCNT0, OCNS0, OCNTXF
        WRITE(IUSAV,'(1X,1P,4E15.6)') TDIFFC, HDFAC, PRANDL, LEWIS
        WRITE(IUSAV,'(1X,1P,4E15.6)') FWFLUX, FWFTIM, FWLMIN, FWLMAX
        WRITE(IUSAV,'(1X,1P,5E15.6)')
     1                         CO2CF1, CO2CF2, CO2CF3, CO2CF4, CO2CF5
        WRITE(IUSAV,FMT)
     1       (((OCNQ0(K,L,IPGQ),K=0,NYOCN),L=0,NZOCN),IPGQ=JOCV,JOCS)
        WRITE(IUSAV,FMT)
     1       (((OCNQ1(K,L,IPGQ),K=0,NYOCN),L=0,NZOCN),IPGQ=JOCV,JOCS)
        IF (NTROCN .GT. 0) THEN
C         Write out ocean tracers
          WRITE(IUSAV,FMT)
     1       (((OCNQ0(K,L,IPGQ),K=0,NYOCN),L=0,NZOCN),
     2          IPGQ=JOCTR1,JOCTR1+NTROCN-1)
          WRITE(IUSAV,FMT)
     1       (((OCNQ1(K,L,IPGQ),K=0,NYOCN),L=0,NZOCN),
     2          IPGQ=JOCTR1,JOCTR1+NTROCN-1)
        ENDIF
        CLOSE( IUSAV )
        WRITE(*,*) 'Ocean parameters/final state saved to '//SAVFIL
      ENDIF
      GOTO 10
C
  990 CONTINUE
#ifdef GRAPHICS
C     De-initialize graphics
      CALL GEND
#endif
      STOP
      END

      SUBROUTINE QCOPY(Q2, Q1, NX0, NX, NY)
C QCOPY copies matrix Q1 to Q2
C Input:  REAL Q1(NX0, NY) -- matrix
C         INTEGER NX0 -- declared upper bound
C         INTEGER NX, NY -- operative upper bounds
C Output: REAL Q2(NX0, NY) -- copy of Q1.
C
      INTEGER NX0, NX, NY
      REAL Q2(NX0, NY), Q1(NX0, NY)
C
      INTEGER I, J
      DO 10 J=1,NY
        DO 10 I=1,NX
   10     Q2(I,J)= Q1(I,J)
      RETURN
      END

      SUBROUTINE OCNINT( FNEW, FOLD,
     1                   NYOCN, NYOCN2, NZOCN, NZOCN2,
     2                   IYMAP, YFRAC, DYNEW, DYOLD, YLWT, YRWT,
     3                   IZMAP, ZFRAC, DZNEW, DZOLD, ZLWT, ZRWT )
C OCNINT interpolates oceanic quantities from resolution (NYOCN,NZOCN)
C to (NYOCN2,NZOCN2). IYMAP, YFRAC, DYNEW, DYOLD, YLWT, YRWT,
C IZMAP, ZFRAC, DZNEW, DZOLD, ZLWT, ZRWT are the horizontal/vertical
C interpolation coefficients returned by FLXIN1.
C (Note: FNEW may be the same array as FOLD)
C
      REAL FNEW(0:NYOMAX,0:NZOMAX), FOLD(0:NYOMAX,0:NZOMAX)
      INTEGER NYOCN, NYOCN2, NZOCN, NZOCN2
      INTEGER IYMAP(-1:NYOMAX), IZMAP(-1:NZOMAX)
      REAL YFRAC(-1:NYOMAX)
      REAL ZFRAC(-1:NZOMAX)
      REAL DYNEW(0:NYOMAX), DYOLD(-1:NYOMAX)
      REAL DZNEW(0:NZOMAX), DZOLD(-1:NZOMAX)
      REAL YLWT(0:NYOMAX), YRWT(0:NYOMAX)
      REAL ZLWT(0:NZOMAX), ZRWT(0:NZOMAX)
C
      REAL FTEM(0:NYOMAX,0:NZOMAX)
      REAL FTRANS(0:NZOMAX,0:NYOMAX), FTRAN2(0:NZOMAX,0:NYOMAX)
      INTEGER K, L
C
C     Interpolate to new horizontal grid
      DO 10 L=0,NZOCN
        CALL FLUX1D( FTEM(0,L), NYOCN2+1, FOLD(0,L), NYOCN+1,
     1               IYMAP, YFRAC, DYNEW, DYOLD, YLWT, YRWT )
   10   CONTINUE
C     Transpose array
      DO 20 L=0,NZOCN
        DO 20 K=0,NYOCN2
   20     FTRANS(L,K)= FTEM(K,L)
C     Interpolate to new vertical grid
      DO 30 K=0,NYOCN2
        CALL FLUX1D( FTRAN2(0,K), NZOCN2+1, FTRANS(0,K), NZOCN+1,
     1               IZMAP, ZFRAC, DZNEW, DZOLD, ZLWT, ZRWT )
   30   CONTINUE
C     Untranspose array for output
      DO 40 K=0,NYOCN2
        DO 40 L=0,NZOCN2
   40     FNEW(K,L)= FTRAN2(L,K)
      RETURN
      END

      SUBROUTINE CONVAD( TRCCOL, ALPHA, BETA, IADJ, IADLEV )
C DCADJ performs dry adiabatic adjustment in a single column.
C Input:  REAL TRCCOL(0:NZOMAX,JOCT:NOCNQ) -- column tracer values
C         REAL ALPHA -- Thermal expansion coefficient (1/K)
C         REAL BETA  -- Haline contraction coefficient (nondimensional)
C Output: REAL TRCCOL(0:NZOMAX,JOCT:NOCNQ) -- column tracer values,
C          adjusted so thhat the whole column is statically stable.
C           The adjustment is carried out so as to conserve sum of
C           TRCCOL(L,ITR)*WTZOCN(L).
C         INTEGER IADJ -- no. of adjustment iterations carried out
C                         (>= 0, useful for diagnostic purposes)
C         INTEGER IADLEV -- total no. of levels adjusted in all
C                            iterations (>=0)
C
#include "ogrid.h"
C
      INTEGER IADJ, IADLEV
      REAL ALPHA, BETA, TRCCOL(0:NZOMAX,JOCT:NOCNQ)
C
      REAL CUWT, RHOBAR, RHOCOL(0:NZOMAX)
      REAL TRCTOT(JOCT:NOCNQ), TRCBAR(JOCT:NOCNQ)
      INTEGER ITR, L, LLOW, LHIGH
      LOGICAL EXTEND
C
C ----------------------------------------------------------------------
C (NOTE: It is important for the adjustment not to re-adjust
C        previously adjusted regions, unless they have been extended
C        vertically. Otherwise, infinite-looping might occur. The
C        current adjustment scheme results in the exact equality
C        RHOCOL(L).EQ.RHOCOL(L-1) in the adjusted regions. If the
C        adjustment results in adjoining regions becoming unstable,
C        it should simply lead to adjustment over an extended
C        region in the next iteration. So the above equality may
C        be interpreted either as denoting stability, or instability,
C        depending on the context.)
C ----------------------------------------------------------------------
C
C     Compute nondimensional density using equation of state
      DO 110 L=0,NZOCN
  110   RHOCOL(L)= -(ALPHA*TRCCOL(L,JOCT) - BETA*TRCCOL(L,JOCS))
C     Beginning of iteration
      IADJ= 0
      IADLEV= -1
      LLOW= 0
C     Scan upward for statically unstable region
C     (1 jump to this statement)
  200 DO 210 L=LLOW,NZOCN-1
        IF (RHOCOL(L+1).GT.RHOCOL(L)) GOTO 300
  210   CONTINUE
C
C     Whole column statically stable
      RETURN
C
C     Unstable region encountered
C     (1 jump to this statement)
  300 LLOW= L
C     Determine vertical extent of unstable region,
      CUWT= WTZOCN(LLOW)
      DO 310 ITR=JOCT,JOCS+NTROCN
  310   TRCTOT(ITR)= TRCCOL(LLOW,ITR)*WTZOCN(LLOW)
      DO 330 L=LLOW+1,NZOCN
        IF (RHOCOL(L).GE.RHOCOL(L-1)) THEN
C         Accumulate tracer content, and cumulative weight factor
          LHIGH= L
          CUWT= CUWT + WTZOCN(L)
          DO 320 ITR=JOCT,JOCS+NTROCN
  320       TRCTOT(ITR)= TRCTOT(ITR) + TRCCOL(L,ITR)*WTZOCN(L)
        ELSE
C         Reached stable region
          GOTO 400
        ENDIF
  330   CONTINUE
C
C     Compute mean tracer concentrations
C     (2 jumps to this statement)
  400 DO 410 ITR=JOCT,JOCS+NTROCN
  410   TRCBAR(ITR)= TRCTOT(ITR)/CUWT
C     Compute mean nondimensional density (ignoring pressure effects)
      RHOBAR= -(ALPHA*TRCBAR(JOCT) - BETA*TRCBAR(JOCS))
      EXTEND= .FALSE.
      IF (LHIGH.LT.NZOCN) THEN
C       Unstable region above bottom of model
        IF (RHOBAR.LT.RHOCOL(LHIGH+1)) THEN
C         Adjustment will destabilize bottom of unstable region
C         Extend region downward
          LHIGH= LHIGH+1
          EXTEND= .TRUE.
          CUWT= CUWT + WTZOCN(LHIGH)
          DO 420 ITR=JOCT,JOCS+NTROCN
  420       TRCTOT(ITR)= TRCTOT(ITR) + TRCCOL(LHIGH,ITR)*WTZOCN(LHIGH)
        ENDIF
      ENDIF
      IF (LLOW.GT.0) THEN
C       Unstable region below top of model
        IF (RHOCOL(LLOW-1).LT.RHOBAR) THEN
C         Adjustment will destabilize top of unstable region
C         Extend region upward
          LLOW= LLOW-1
          EXTEND= .TRUE.
          CUWT= CUWT + WTZOCN(LLOW)
          DO 430 ITR=JOCT,JOCS+NTROCN
  430       TRCTOT(ITR)= TRCTOT(ITR) + TRCCOL(LLOW,ITR)*WTZOCN(LLOW)
        ENDIF
      ENDIF
C     Check if unstable region has been extended
      IF (EXTEND) GOTO 400
C
      DO 500 L=LLOW,LHIGH
C       Adjust density profile to be (exactly) uniform
        RHOCOL(L)= RHOBAR
C       Adjust tracer profiles to be (exactly) uniform
        DO 500 ITR=JOCT,JOCS+NTROCN
  500     TRCCOL(L,ITR)= TRCBAR(ITR)
C     Adjustment diagnostics
      IADJ= IADJ+1
      IADLEV= IADLEV + (LHIGH - LLOW + 1)
      LLOW= MIN0(NZOCN, LHIGH+1)
C     End of iteration
      GOTO 200
      END

#ifdef GRAPHICS
      SUBROUTINE DISOCN( OCNFLD, NCDIV, CMIN, CMAX, STR )
C DISOCN displays a height-latitude ocean field OCNFLD described by
C string STR. NCDIV is the number of contour divcisions, CMIN and CMAX
C are the minimum/maximum contour values.
C (specifying NCDIV=0 implies that CMIN is the contour interval; CMAX is
C  ignored in this case)
C
#include "ogrid.h"
      INTEGER NCDIV
      REAL OCNFLD(0:NYOMAX,0:NZOMAX), CMIN, CMAX
      CHARACTER*(*) STR
      REAL CINT, FMIN, FMAX
C
      IF (NCDIV.EQ.0) THEN
        CINT= CMIN
        CALL GCNDAT( FMIN, FMAX, CINT,
     1               NYOCN+1, NZOCN+1, NYOMAX+1, OCNFLD, -1, 0.0, 1 )
      ELSE
        CINT= (CMAX - CMIN) / NCDIV
        CALL GCNRNG( NCDIV, CMIN, CMAX, -1, 0.0, 1 )
      ENDIF
      CALL GXYRN2(  -90.0, 90.0, 6, -1, -OCNDEP, 0.0,
     1              NINT(OCNDEP/1000.), -1, 'lat', 'z', STR )
      CALL GDRAW
C
      CALL GCNBOX( NYOCN+1, NZOCN+1, LATOCN, ZOCN, NYOMAX+1,
     1             OCNFLD, CINT, ' ')
      CALL GCONT( NYOCN+1, NZOCN+1, LATOCN, ZOCN, NYOMAX+1,
     1            OCNFLD )
      RETURN
      END
#endif

C --------------------------------------------------------------------
C For GNU Emacs:
C $Msg-digest-checksum: 0895cab667245627841b44fbcf8a095e $
C --------------------------------------------------------------------
C Local Variables:
C msg-digest-active: t
C End:
C --------------------------------------------------------------------
