C Diagnostic routines: version 1.2 created 6 Sep 1995
#ifndef OCONS
#   include "ocons.h"
#endif
#ifndef LENWRD
#   define LENWRD 4
#endif
#ifndef RECUNIT
#   define RECUNIT 1
#endif
#ifndef NLEVMX
#   define NLEVMX 2
#endif
#ifndef NLATMX
#   define NLATMX 4
#endif
C     "Enumeration literals"
#define JX 1
#define JY 2
C ------ Description of variables in module odicom.h ----------------
C C preprocessor parameters
C LENWRD= length of a machine word (in bytes)
C RECUNIT= units, i.e., no. of bytes, in which direct-access record
C          lengths are measured (usually 1 for most machines).
C NLEVMX= maximum number of levels for horizontal section saves
C NLATMX= maximum number of latitudes for vertical section saves
C
C Variables
C IDSET = Diagnostic set number for this run (0,...,9999)
C NTSPD = number of time-steps per model day
C (If negative, represents no. of model days per time-step)
C NYRDAY= number of days in a model year
C IUODG = fortran unit number set aside for diagnostics
C IUOTS = fortran unit number set aside for time-series
C IUOHS = fortran unit number set aside for horizontal section data
C IUOVS = fortran unit number set aside for vertical section data
C IUOZS = fortran unit number set aside for zonal-mean section data
C IDSTEP= value of step-count (ISTEP) when sampling is to be initiated
C ISSTEP= value of step-count (ISTEP) when saving is to be initiated
C NDSKIP= diagnostic sampling interval (in units of time-steps)
C NTFREQ= time-series save frequency (in units of NDSKIP time-steps)
C NSFREQ= section data save frequency (in units of NDSKIP time-steps)
C NSAMPL= count of diagnostic samples taken so far
C NTSAMP= count of time-series samples saved so far (<= NSAMPL)
C NSSAMP= count of section samples saved so far (<= NSAMPL)
C NRECTS= direct-access record count for time-series file
C NRECHS= direct-access record count for horizontal section file
C NRECVS= direct-access record count for vertical section file
C NRECZS= direct-access record count for zonal-mean section file
C ISTP0 = time-step count of first sample
C ISTP1 = time-step count of last sample
C DTSTEP= model time-step (in sec)
C TAU0  = model time at beginning of sampling period (in sec)
C TAU1  = model time at end of sampling period (in sec)
C YRPHA = phase of year at beginning of run (time-step 0)
C         (YRPHA=0..1, 0 => Mar 21, 0.75 => Dec 22)
C SAVSRF= surface flag indicating whether section data should be saved
C SAVEL = level flag indicating whether section data should be saved
C SAVEK = latitude flag indicating whether section data should be saved
C
C Time-series data is saved using unit number IUOTS. The following
C real values are saved (in order):
C SPEV1, SPEV2, KEEV, PSIEQE, PSI01E, PSI02E, PSIMXE,
C TR00E(JOCT), TR00SE(JOCT), TR01SE(JOCT), TR02SE(JOCT),
C TRCHME(2,JOCT), TRCVME(4,JOCT), TRCHFE(3,JOCT), TRCVFE(4,JOCT),
C ... (next tracer)
C The time-series file is in direct-access IEEE format. Each record
C contains the values of the above quantities averaged over a set of
C NTFREQ successive samples. The first 16 bytes of each record contain
C a character string consisting of the run name (5 bytes), the
C diagnostic data-set number (4 bytes, 0000..9999), the two characters
C "TO" followed by the time-series sample number (00001..99999). This
C character string is followed by at least 46 values of IEEE
C single-precision data. The first value is the number of floating
C point numbers that follow (itself a floating point number, >=45.0).
C The next value is the model time, when the sample was saved, in days.
C The remaining numbers contain the time-series data.
C (Total record length= 16 + LENWRD*(46 + 17*NTROCN) bytes)
C (Note: The very first sample of the run, at time-step 0, will be
C        numbered 00000 )
C
C
C For the levels designated using SAVEL, section data is saved using
C unit number IUOHS. The physical space representations of the five
C fields (u,v,w,T,S) are saved for each sample at the designated
C level. If any tracers are present, their distribution is also saved.
C The section file is in direct-access IEEE format. Each record contains
C the value of one quantity at a particular level. Successive records
C contain values of different quantities at the same level, ordered
C u,v,W,T,S, tracer1,tracer2,... The levels are ordered from top to
C bottom. The first 48 bytes of each record contain a character string
C consisting of a comment (16 bytes), the date ("yyyymmdd"),
C time of day ("hhmm"), level ("???????m"), and the quantity
C descriptor string (12 bytes). This character string is followed by
C four values of IEEE single-precision data. The first value is
C area-weighted section mean of the quantity. The next two values
C contain NXOCN+1, NYOCN+1, respectively, but as floating point numbers.
C The fourth value is unused at the moment. This is followed by
C (NXOCN+1)*(NYOCN+1) values of IEEE single-precision data, each
C representing the deviations from the global-mean of the quantity
C on the grid at that level. The longitude varies fastest, ordered West
C to East, starting from the western boundary. The latitude circles are
C ordered from South to North.
C (Note: The first five characters of the comment contain the run name,
C        and the model year numbers range from yyyy=0000 to 9999)
C
C The following quantity descriptor strings are used:
C "U (M/S)", "V (M/S)", "W (M/S)", "T (K)", "S (KG/KG)", ...
C corresponding to the zonal component of velocity (in m/s),
C the meridional component of velocity (in m/s),
C vertical component of velocity (in m/s), Temperature (in K),
C salinity (in kg/kg), ...
C (Total record length= 48 + LENWRD*(4 + (NXOCN+1)*(NYOCN+1)) bytes)
C
C If the surface save flag SAVSRF is set, the following surface fields
C are also saved in the horizontal section file, with the 8-character
C level descriptor string set to "SURFACE ":
C "TAUX (PA)", "TAUY (PA)", "FSURF (W/M2)", "E-P (KG/SM2)"
C corrsponding to the zonal and the meridional components of
C the surface wind stress (in Pa), heat flux into the surface
C (in W/m**2), and evaporation - precipitation (in kg/(s*m**2)).
C NOTE: The surface fields will not be available at the very beginning
C of the run (i.e. at time-step 0)
C
C For the latitudes designated using SAVEK, section data is saved using
C unit number IUOVS. The physical space representations of the five
C fields (u,v,w,T,S) are saved for each sample at the designated
C latitude. If any tracers are present, their distributon is also saved.
C The vertical section file is in direct-access IEEE format. Each record
C contains the value of one quantity at a particular latitude circle.
C Successive records contain values of different quantities at the same
C latitude, ordered u,v,w,T,S, tracer1,tracer2,... . The latitudes are
C ordered from South to North. The first 48 bytes of each record contain
C a character string consisting of a comment (16 bytes), the date
C ("yyyymmdd"), time of day ("hhmm"), latitude degrees ("???.?deg"),
C and the quantity descriptor string (12 bytes). This character string
C is followed by four values of IEEE single-precision data. The first
C value is mass-weighted section mean of the quantity. The next two
C values contain NXOCN+1, NZOCN+1, respectively, but as floating point
C numbers. The fourth value is unused at the moment. This is followed
C by (NXOCN+1)*(NZOCN+1) values of IEEE single-precision data, each
C representing the deviations from the global-mean of the quantity on
C vertical section at the latitude circle. The longitude varies fastest,
C ordered West to East, starting from the Greenwich meridian. The levels
C are ordered from bottom to top.
C
C The following quantity descriptor strings are used:
C "U (M/S)", "V (M/S)", "W (M/S)", "T (K)", "S (KG/KG)", ...
C corresponding to the the zonal component of velocity (in m/s),
C the meridional component of velocity (in m/s),
C vertical component of velocity (in m/s), Temperature (in K),
C salinity (in kg/kg), ...
C (Total record length= 48 + LENWRD*(4 + (NXOCN+1)*(NZOCN+1)) bytes)
C
C If IUOZS > 0, then unit number IUOZS is used to save zonal-mean
C section data. The physical space zonal-mean representations of the
C five fields (u,v,w,T,S) are saved for each sample. If any tracers are
C present, their zonal-mean distribution is also saved. The zonal-mean
C section file is in direct-access IEEE format. Each record contains the
C zonal-mean value of one quantity. Successive records contain
C zonal-mean values of different quantities, ordered u,v,w,T,S, ... .
C The first 48 bytes of each record contain a character
C string consisting of a comment (16 bytes), the date ("yyyymmdd"),
C time of day ("hhmm"), the string ("ZONMEAN "),
C and the quantity descriptor string (12 bytes). This character string
C is followed by four values of IEEE single-precision data. The first
C value is mass-weighted section mean of the quantity. The next two
C values contain NYOCN+1, NZOCN+1, respectively, but as floating point
C numbers. The fourth value is unused at the moment. This is followed
C by (NYOCN+1)*(NZOCN+1) values of IEEE single-precision data, each
C representing the deviations from the global-mean of the quantity on
C vertical section at the latitude circle. The latitude varies fastest,
C ordered South to North. The levels are ordered from bottom to top.
C
C The following quantity descriptor strings are used:
C "U (M/S)", "V (M/S)", "W (M/S)", "T (K)", "S (KG/KG)", ...
C corresponding to the the zonal component of velocity (in m/s),
C the meridional component of velocity (in m/s),
C vertical component of velocity (in m/s), Temperature (in K),
C salinity (in kg/kg), ...
C (Total record length= 48 + LENWRD*(4 + (NYOCN+1)*(NZOCN+1)) bytes)
C
C
C UAVG  = accumulator for zonal velocity
C VAVG  = accumulator for meridional velocity
C WAVG  = accumulator for vertical velocity
C TRCAVG= accumulators for T, S, other tracers
C
C Note: The following are instantaneous diagnostic quantities
C UZON  = lat-pressure section of zonal-mean u
C VZON  = lat-pressure section of zonal-mean v
C WZON  = lat-pressure section of zonal-mean w
C PSIZON= lat-pressure section of MMC streamfunction
C TRCZON= lat-pressure section of tracer values
C TRFLUX= lat-pressure section of zonal-mean advective tracer flux
C VELSRF= horizontal velocity at the surface
C TRSURF= tracer values at the surface
C
C (TRFAVG is the time-average of TRFLUX)
C
C UZVAR = accumulator for zonally-averaged u-variance
C VZVAR = accumulator for zonally-averaged v-variance
C WZVAR = accumulator for zonally-averaged w-variance
C TRZVAR= accumulator for zonally-averaged tracer-variance
C
C KESAVG= accumulator for surface kinetic energy
C TRSVAR= accumulator for surface tracer variance
C TAUAVG= accumulator for surface wind stress
C TRCIAV= accumulators for downward tracer fluxes at surface
C         ( JOCT corresponds to heat flux, in Watts/m2.
C           JOCS corresponds to E-P, in kg/(s*m**2).
C           All the other fluxes are tracer mass fluxes in kg/(s*m**2))
C
C PSIZIN= lat-press. section of MMC streamfunction for sample 1
C TRCZIN= lat-press. section of zonal-mean tracer value for sample 1
C
C PSITEN= lat-press. section of MMC streamfunction tendency
C TRTEND= lat-press. section of zonal-mean tracer tendency
C
C MERFLX= vertically and zonally averaged total meridonal tracer flux
C         (JOCT corresponds to heat flux, in Watts. The other fluxes,
C          including that of salt, are in kg/s)
C ADVFLX= Like MERFLX, but only the advective component
C
C SPFLD1= special field 1 (for future use)
C SPFLD2= special field 2 (for future use)
C
C (The following are instantaneous values at sampling time)
C KE    = volume-averaged kinetic energy
C PSIEQ = value of MMC streamfunction at equator at mid-level
C PSI01 = MMC streamfunction coefficient of Y[0,1] at mid-level
C PSI02 = MMC streamfunction coefficient of Y[0,2] at mid-level
C PSIMX(1)= extreme value of MMC streamfunction between 90S and 30S
C PSIMX(2)= extreme value of MMC streamfunction between 30S and Equator
C PSIMX(3)= extreme value of MMC streamfunction between Equator and 30N
C PSIMX(4)= extreme value of MMC streamfunction between 30N and 90N
C TR00  = volume-averaged tracer value
C TR00S = area-averaged surface tracer value
C TR01S = Y[0,1] coefficient of zonally-averaged surface tracer value
C TR02S = Y[0,2] coefficient of zonally-averaged surface tracer value
C TRCVM(1,)= vertically-averaged tracer value between 90S and 30S
C TRCVM(2,)= vertically-averaged tracer value between 30S and Equator
C TRCVM(3,)= vertically-averaged tracer value between Equator and 30N
C TRCVM(4,)= vertically-averaged tracer value between 30N and 90N
C TRCHM(1,)= horizontally-averaged tracer value for lower-ocean
C TRCHM(2,)= horizontally-averaged tracer value for upper-ocean
C TRCVF(1,)= horizontally-averaged value of TRFLUX between 90S and 30S
C TRCVF(2,)= horizontally-averaged value of TRFLUX between 30S and Eq.
C TRCVF(3,)= horizontally-averaged value of TRFLUX between Eq. and 30N
C TRCVF(4,)= horizontally-averaged value of TRFLUX between 30N and 90N
C TRCHF(1,)= vertically-averaged value of TRFLUX at 30S
C TRCHF(2,)= vertically-averaged value of TRFLUX at equator
C TRCHF(3,)= vertically-averaged value of TRFLUX at 30N
C
C (KEEV, ..., TRCHFE are the cumulative values of KE, ..., TRCHF)
C NEV   = no. of cumulative samples so far for time-series (<=NTFREQ)
C
C Run details
C RUNTTL= title of run
C DAYTM0= string containing date and time when run was initialized
C PREFIX= file-name prefix for diagnostic output
C         (Constructed from the first five characters of RUNTTL)
C
C ---------------------- End of description -------------------------
C
 
      SUBROUTINE ODIINI( ISET, DTIME, NSPDAY, NDPYR, IBEGIN,
     1                   NDASKP, NTSFRQ, NSCFRQ,
     2                   IDAFIL, ITSFIL, IHSFIL, IVSFIL, IZSFIL,
     3                   SAVFST, SFSAV, LEVSAV, LATSAV,
     4                   YPHASE, TITLE, INITIM )
C Initializes diagnostic operations
C Input:  INTEGER ISET -- diagnostic data-set count (0,...,9999)
C         REAL DTIME -- time-step (in sec)
C         INTEGER NSPDAY -- no. of time-steps per model day
C         INTEGER NDPYR -- no. of days per model year
C         INTEGER IBEGIN -- time-step no. at which to initiate sampling
C         INTEGER NDASKP -- diagnostic sampling interval (in time-steps)
C         INTEGER NTSFRQ -- Time-series save frequency
C                           (in units of NDASKP time-steps)
C         INTEGER NSCFRQ -- section data sampling interval
C                           (in units of NDASKP time-steps)
C         INTEGER IDAFIL -- unit number for diagnostics file
C         INTEGER ITSFIL -- unit number for time-series file
C         INTEGER IHSFIL -- unit number for horizontal section file
C         INTEGER IVSFIL -- unit number for vertical section file
C         INTEGER IZSFIL -- unit number for zonal-mean section file
C      (Specify IDAFIL/ITSFIL/IHSFIL/IVSFIL/IZSFIL=0 to suppress output)
C         LOGICAL SAVFST -- if .TRUE., save first sample
C                           if .FALSE., skip first sample
C         (SAVFST affects time-series and section data output)
C         LOGICAL SFSAV -- surface flag for saving section data
C         LOGICAL LEVSAV(0:NZOCN) -- level flag for saving section data
C         LOGICAL LATSAV(0:NYOCN) -- latitude flag
C                                    for saving section data
C         REAL YPHASE -- phase of year at beginning of run (time-step 0)
C                        (YPHASE=0..1, 0 => Mar 21, 0.75 => Dec 22)
C         CHARACTER*(*) TITLE -- title of run
C         CHARACTER*(*) INITIM -- run initialization date and time
C Note: The first five characters of TITLE are used to construct a
C       prefix for diagnostic files. i.e. Diagnostic output is stored
C       in <prefix>.da, where <prefix> consists of the first five
C       characters of TITLE followed by four digits (0000 .. 9999)
C       denoting the data-set count ISET.
C       Time-series data is stored in a file named <prefix>.to.
C       Horizontal section data is stored in a file named <prefix>.ho
C       Vertical section data is stored in a file named <prefix>.vo
C       Zonal-mean section data is stored in a file named <prefix>.zo
C Output: (Initializes common module odicom.h)
C
#include "ogrid.h"
#include "odicom.h"
      REAL DTIME, YPHASE
      INTEGER ISET, NSPDAY, NDPYR, IBEGIN, NDASKP, NTSFRQ, NSCFRQ
      INTEGER IDAFIL, ITSFIL, IHSFIL, IVSFIL, IZSFIL
      LOGICAL SAVFST, SFSAV, LEVSAV(0:NZOCN), LATSAV(0:NYOCN)
      CHARACTER*(*) TITLE, INITIM
C
      INTEGER I, J, K, L, ITR, LENREC
      CHARACTER TEMST5*5, FILNAM*12
C
      IDSET= ISET
      DTSTEP= DTIME
      NTSPD= NSPDAY
      NYRDAY= NDPYR
      IDSTEP= IBEGIN
      NDSKIP= MAX0(1,NDASKP)
      NTFREQ= MAX0(1,NTSFRQ)
      NSFREQ= MAX0(1,NSCFRQ)
      IUODG= IDAFIL
      IUOTS= ITSFIL
      IUOHS= IHSFIL
      IUOVS= IVSFIL
      IUOZS= IZSFIL
      YRPHA= YPHASE
      RUNTTL= TITLE
      DAYTM0= INITIM
C     Determine whether to skip the first sample for save operations
      ISSTEP= IDSTEP
      IF (.NOT.SAVFST) ISSTEP= ISSTEP + NDSKIP
C     Determine file prefix to be used for diagnostic/section output
      TEMST5= RUNTTL(1:5)
      IF (TEMST5.EQ.' ') THEN
        TEMST5='tmp00'
      ELSE
        CALL RJUST2( TEMST5 )
      ENDIF
      WRITE(PREFIX,'(4X,I5)') 10000+IDSET
      PREFIX(1:5)= TEMST5
C
C     Copy surface save flag
      SAVSRF= SFSAV
C     Copy level save flags
      DO 10 L=0,NZOCN
   10   SAVEL(L)= LEVSAV(L)
C     Copy latitude save flags
      DO 20 K=0,NYOCN
   20   SAVEK(K)= LATSAV(K)
C     Zero save data accumulators
      CALL OSVZSF
      CALL OSVZRO
C
      IF (IUOTS .GT. 0) THEN
C       Open time-series file
        FILNAM= PREFIX//'.to'
        CALL LJUST2( FILNAM )
        LENREC= 16 + LENWRD*(46 + 17*(NOCNQ-JOCS))
        OPEN(IUOTS, FILE=FILNAM, FORM='UNFORMATTED', STATUS='NEW',
     1              ACCESS='DIRECT', RECL=(LENREC+RECUNIT-1)/RECUNIT )
        WRITE(*,*) 'Saving time-series data to ', FILNAM
C       Zero record count
        NRECTS= 0
C       Initialize time-series accumulators
        CALL TOINI
      ENDIF
      IF (IUOHS .GT. 0) THEN
C       Open horizontal section file
        FILNAM= PREFIX//'.ho'
        CALL LJUST2( FILNAM )
        LENREC= 48 + LENWRD*(4 + (NXOCN+1)*(NYOCN+1))
        OPEN(IUOHS, FILE=FILNAM, FORM='UNFORMATTED', STATUS='NEW',
     1              ACCESS='DIRECT', RECL=(LENREC+RECUNIT-1)/RECUNIT )
        WRITE(*,*) 'Saving horizontal section data to ', FILNAM
C       Zero record count
        NRECHS= 0
      ENDIF
      IF (IUOVS .GT. 0) THEN
C       Open vertical section file
        FILNAM= PREFIX//'.vo'
        CALL LJUST2( FILNAM )
        LENREC= 48 + LENWRD*(4 + (NXOCN+1)*(NZOCN+1))
        OPEN(IUOVS, FILE=FILNAM, FORM='UNFORMATTED', STATUS='NEW',
     1              ACCESS='DIRECT', RECL=(LENREC+RECUNIT-1)/RECUNIT )
        WRITE(*,*) 'Saving vertical section data to ', FILNAM
C       Zero record count
        NRECVS= 0
      ENDIF
      IF (IUOZS .GT. 0) THEN
C       Open zonal-mean section file
        FILNAM= PREFIX//'.zo'
        CALL LJUST2( FILNAM )
        LENREC= 48 + LENWRD*(4 + (NYOCN+1)*(NZOCN+1))
        OPEN(IUOZS, FILE=FILNAM, FORM='UNFORMATTED', STATUS='NEW',
     1              ACCESS='DIRECT', RECL=(LENREC+RECUNIT-1)/RECUNIT )
        WRITE(*,*) 'Saving zonal-mean section data to ', FILNAM
C       Zero record count
        NRECZS= 0
      ENDIF
C
C     Initialize sample counts
      NSAMPL= 0
      NTSAMP= 0
      NSSAMP= 0
C     Initialize accumulators for calculating time-averages
      DO 100 L=0,NZOCN
        DO 100 K=0,NYOCN
          DO 100 J=0,NXOCN
            UAVG(J,K,L)= 0.0
            VAVG(J,K,L)= 0.0
            WAVG(J,K,L)= 0.0
  100       CONTINUE
      DO 110 ITR=JOCT,NOCNQ
        DO 110 L=0,NZOCN
          DO 110 K=0,NYOCN
            DO 110 J=0,NXOCN
  110         TRCAVG(J,K,L,ITR)= 0.0
C
      DO 120 L=0,NZOCN
        DO 120 K=0,NYOCN
          UZVAR(K,L)= 0.0
          VZVAR(K,L)= 0.0
          WZVAR(K,L)= 0.0
  120     CONTINUE
      DO 130 ITR=JOCT,NOCNQ
        DO 130 L=0,NZOCN
          DO 130 K=0,NYOCN
  130       TRZVAR(K,L,ITR)= 0.0
C
      DO 140 ITR=JOCT,NOCNQ
        DO 140 I=1,2
          DO 140 L=0,NZOCN
            DO 140 K=0,NYOCN
  140         TRFAVG(K,L,I,ITR)= 0.0
C
      DO 150 K=0,NYOCN
        DO 150 J=0,NXOCN
          KESAVG(J,K)= 0.0
          SPFLD1(J,K)= 0.0
          SPFLD2(J,K)= 0.0
  150     CONTINUE
      DO 160 I=JX,JY
        DO 160 K=0,NYOCN
          DO 160 J=0,NXOCN
  160       TAUAVG(J,K,I)= 0.0
      DO 170 ITR=JOCT,NOCNQ
        DO 170 K=0,NYOCN
          DO 170 J=0,NXOCN
            TRSVAR(J,K,ITR)= 0.0
  170       TRCIAV(J,K,ITR)= 0.0
      RETURN
      END
 
      SUBROUTINE OSVZSF
C Zero surface data save accumulators
#include "ogrid.h"
#include "odicom.h"
      INTEGER I, ITR, J, K
C
C     Zero surface field save sample count
      NSFSAV= 0
C     Initialize accumulators for save data
      DO 10 I=1,2
        DO 10 K=0,NYOCN
          DO 10 J=0,NXOCN
   10       TAUSV(J,K,I)= 0.0
      DO 20 ITR=JOCT,NOCNQ
        DO 20 K=0,NYOCN
          DO 20 J=0,NXOCN
   20       TRCISV(J,K,ITR)= 0.0
      RETURN
      END
 
      SUBROUTINE OSVZRO
C Zero horizontal/vertical section data save accumulators
#include "ogrid.h"
#include "odicom.h"
      INTEGER ITR, I, J, K, L
C     Zero horizontal/vertical section save sample count
      NSSAV= 0
C     Initialize accumulators for save data
      DO 10 I=1,NLEVMX
        DO 10 K=0,NYOCN
          DO 10 J=0,NXOCN
            UHS(J,K,I)= 0.0
            VHS(J,K,I)= 0.0
   10       WHS(J,K,I)= 0.0
      DO 20 ITR=JOCT,NOCNQ
        DO 20 I=1,NLEVMX
          DO 20 K=0,NYOCN
            DO 20 J=0,NXOCN
   20         TRCHS(J,K,I,ITR)= 0.0
      DO 30 I=1,NLATMX
        DO 30 L=0,NZOCN
          DO 30 J=0,NXOCN
            UVS(J,L,I)= 0.0
            VVS(J,L,I)= 0.0
   30       WVS(J,L,I)= 0.0
      DO 40 ITR=JOCT,NOCNQ
        DO 40 I=1,NLATMX
          DO 40 L=0,NZOCN
            DO 40 J=0,NXOCN
   40         TRCVS(J,L,I,ITR)= 0.0
      DO 50 L=0,NZOCN
        DO 50 K=0,NYOCN
          UZS(K,L)= 0.0
          VZS(K,L)= 0.0
   50     WZS(K,L)= 0.0
      DO 60 ITR=JOCT,NOCNQ
        DO 60 L=0,NZOCN
          DO 60 K=0,NYOCN
   60       TRCZS(K,L,ITR)= 0.0
      RETURN
      END
 
      SUBROUTINE TOINI
C Initialize time-series accumulators
#include "ogrid.h"
#include "odicom.h"
      INTEGER ITR
      NEV= 0
      SPEV1= 0.0
      SPEV2= 0.0
      KEEV= 0.0
      PSIEQE= 0.0
      PSI01E= 0.0
      PSI02E= 0.0
      PSIMXE(1)= 0.0
      PSIMXE(2)= 0.0
      PSIMXE(3)= 0.0
      PSIMXE(4)= 0.0
      DO 10 ITR=JOCT,NOCNQ
        TR00E(ITR)= 0.0
        TR00SE(ITR)= 0.0
        TR01SE(ITR)= 0.0
        TR02SE(ITR)= 0.0
C
        TRCHME(1,ITR)= 0.0
        TRCHME(2,ITR)= 0.0
C
        TRCVME(1,ITR)= 0.0
        TRCVME(2,ITR)= 0.0
        TRCVME(3,ITR)= 0.0
        TRCVME(4,ITR)= 0.0
C
        TRCHFE(1,ITR)= 0.0
        TRCHFE(2,ITR)= 0.0
        TRCHFE(3,ITR)= 0.0
C
        TRCVFE(1,ITR)= 0.0
        TRCVFE(2,ITR)= 0.0
        TRCVFE(3,ITR)= 0.0
        TRCVFE(4,ITR)= 0.0
   10   CONTINUE
      RETURN
      END
 
      SUBROUTINE TOACC
C Accumulate time-series quantities
#include "ogrid.h"
#include "odicom.h"
      INTEGER ITR
      NEV= NEV + 1
      SPEV1= SPEV1 + TRSURF(0,3*(NYOCN/4),JOCT)
      SPEV2= SPEV2 + TRSURF(0,3*(NYOCN/4),JOCS)
      KEEV= KEEV + KE
      PSIEQE= PSIEQE + PSIEQ
      PSI01E= PSI01E + PSI01
      PSI02E= PSI02E + PSI02
      PSIMXE(1)= PSIMXE(1) + PSIMX(1)
      PSIMXE(2)= PSIMXE(2) + PSIMX(2)
      PSIMXE(3)= PSIMXE(3) + PSIMX(3)
      PSIMXE(4)= PSIMXE(4) + PSIMX(4)
      DO 10 ITR=JOCT,NOCNQ
        TR00E(ITR)= TR00E(ITR) + TR00(ITR)
        TR00SE(ITR)= TR00SE(ITR) + TR00S(ITR)
        TR01SE(ITR)= TR01SE(ITR) + TR01S(ITR)
        TR02SE(ITR)= TR02SE(ITR) + TR02S(ITR)
C
        TRCHME(1,ITR)= TRCHME(1,ITR) + TRCHM(1,ITR)
        TRCHME(2,ITR)= TRCHME(2,ITR) + TRCHM(2,ITR)
C
        TRCVME(1,ITR)= TRCVME(1,ITR) + TRCVM(1,ITR)
        TRCVME(2,ITR)= TRCVME(2,ITR) + TRCVM(2,ITR)
        TRCVME(3,ITR)= TRCVME(3,ITR) + TRCVM(3,ITR)
        TRCVME(4,ITR)= TRCVME(4,ITR) + TRCVM(4,ITR)
C
        TRCHFE(1,ITR)= TRCHFE(1,ITR) + TRCHF(1,ITR)
        TRCHFE(2,ITR)= TRCHFE(2,ITR) + TRCHF(2,ITR)
        TRCHFE(3,ITR)= TRCHFE(3,ITR) + TRCHF(3,ITR)
C
        TRCVFE(1,ITR)= TRCVFE(1,ITR) + TRCVF(1,ITR)
        TRCVFE(2,ITR)= TRCVFE(2,ITR) + TRCVF(2,ITR)
        TRCVFE(3,ITR)= TRCVFE(3,ITR) + TRCVF(3,ITR)
        TRCVFE(4,ITR)= TRCVFE(4,ITR) + TRCVF(4,ITR)
   10   CONTINUE
      RETURN
      END
 
      SUBROUTINE TOWRIT( ISTP )
C Write out time-series quantities.
C ISTP is the current time-step number
#include "ogrid.h"
#include "odicom.h"
      INTEGER ISTP
C
      REAL TAUEV, EVQTY
      CHARACTER TSSTR*16
      INTEGER I, ITR
C
      SPEV1= SPEV1 / NEV
      SPEV2= SPEV2 / NEV
      KEEV= KEEV / NEV
      PSIEQE= PSIEQE / NEV
      PSI01E= PSI01E / NEV
      PSI02E= PSI02E / NEV
      PSIMXE(1)= PSIMXE(1) / NEV
      PSIMXE(2)= PSIMXE(2) / NEV
      PSIMXE(3)= PSIMXE(3) / NEV
      PSIMXE(4)= PSIMXE(4) / NEV
      DO 10 ITR=JOCT,NOCNQ
        TR00E(ITR)= TR00E(ITR) / NEV
        TR00SE(ITR)= TR00SE(ITR) / NEV
        TR01SE(ITR)= TR01SE(ITR) / NEV
        TR02SE(ITR)= TR02SE(ITR) / NEV
C
        TRCHME(1,ITR)= TRCHME(1,ITR) / NEV
        TRCHME(2,ITR)= TRCHME(2,ITR) / NEV
C
        TRCVME(1,ITR)= TRCVME(1,ITR) / NEV
        TRCVME(2,ITR)= TRCVME(2,ITR) / NEV
        TRCVME(3,ITR)= TRCVME(3,ITR) / NEV
        TRCVME(4,ITR)= TRCVME(4,ITR) / NEV
C
        TRCHFE(1,ITR)= TRCHFE(1,ITR) / NEV
        TRCHFE(2,ITR)= TRCHFE(2,ITR) / NEV
        TRCHFE(3,ITR)= TRCHFE(3,ITR) / NEV
C
        TRCVFE(1,ITR)= TRCVFE(1,ITR) / NEV
        TRCVFE(2,ITR)= TRCVFE(2,ITR) / NEV
        TRCVFE(3,ITR)= TRCVFE(3,ITR) / NEV
        TRCVFE(4,ITR)= TRCVFE(4,ITR) / NEV
   10   CONTINUE
      WRITE(TSSTR,'(10X,I6)') 100000+((ISTP-IDSTEP)/(NDSKIP*NTFREQ))
      TSSTR(1:11)= PREFIX//'TO'
C     Current time in days
      IF (NTSPD .GT. 0) THEN
        TAUEV= DBLE(ISTP)/DBLE(NTSPD)
      ELSE
        TAUEV= ISTP*(-NTSPD)
      ENDIF
C
      EVQTY= 45.0 + (NOCNQ-JOCS)*17.0
C     Increment sample count
      NTSAMP= NTSAMP + 1
C     Write record in time-series file
      NRECTS= NRECTS + 1
      WRITE(IUOTS,REC=NRECTS) TSSTR, EVQTY, TAUEV,
     1  SPEV1, SPEV2, KEEV, PSIEQE, PSI01E, PSI02E,
     2  (PSIMXE(I),I=1,4),
     3  (TR00E(ITR), TR00SE(ITR), TR01SE(ITR), TR02SE(ITR),
     4   (TRCHME(I,ITR),I=1,2), (TRCVME(I,ITR),I=1,4),
     5   (TRCHFE(I,ITR),I=1,3), (TRCVFE(I,ITR),I=1,4),
     6   ITR=JOCT,NOCNQ)
      RETURN
      END
 
      SUBROUTINE LJUST2( STR )
C LJUST2 left justifies a string, removing leading spaces.
C (The string must not be longer than 133 characters)
      CHARACTER*(*) STR
      CHARACTER*133 TEMSTR
      IF (STR.EQ.' ') RETURN
   10 IF (STR(1:1).EQ.' ') THEN
        TEMSTR= STR(2:)
        STR= TEMSTR
        GOTO 10
      ENDIF
      RETURN
      END
 
      SUBROUTINE RJUST2( STR )
C RJUST2 right justifies a string, removing trailing spaces.
C (The string must not be longer than 133 characters)
      CHARACTER*(*) STR
      CHARACTER*133 TEMSTR
      INTEGER N
      IF (STR.EQ.' ') RETURN
      N= LEN(STR)
   10 IF (STR(N:N).EQ.' ') THEN
        TEMSTR= ' '//STR(1:N-1)
        STR= TEMSTR
        GOTO 10
      ENDIF
      RETURN
      END
 
      SUBROUTINE OSVHDR( ISTP, COMNT, MDATE )
C OSVHDR creates the comment string COMNT and the date string MDATE
C for use in save files, given the model time-step count ISTP.
C
#include "ogrid.h"
#include "odicom.h"
      INTEGER ISTP
      CHARACTER COMNT*11, MDATE*13
C
      INTEGER IYEAR, IMONTH, IDAY, IHH, IMM, ITEMP, NDPMON
      REAL TSPHR, TSPMN
      CHARACTER YRSTR*5
C
C     Run comment
      WRITE(COMNT,'(A5,I3,I3)') PREFIX, 100+(NYOCN+1), 100+(NZOCN+1)
      IF ((NYOCN+1) .LT. 100) COMNT(6:6)= '/'
      COMNT(9:9)='x'
C     No. of days per month (arbitrary)
      NDPMON= NYRDAY/12
C     Create date/time string
      IF (NTSPD .GT. 0) THEN
C       No. of time-steps per hour, per minute
        TSPHR= NTSPD/24.0
        TSPMN= TSPHR/60.0
C
        ITEMP= ISTP
        IYEAR= ITEMP/(NTSPD*NYRDAY)
        ITEMP= MOD(ITEMP,NTSPD*NYRDAY)
        IMONTH= 1 + ITEMP/(NTSPD*NDPMON)
        ITEMP= MOD(ITEMP,NTSPD*NDPMON)
        IDAY= 1 + ITEMP/NTSPD
        ITEMP= MOD(ITEMP,NTSPD)
        IHH= INT(ITEMP/TSPHR)
        ITEMP= NINT(ITEMP - IHH*TSPHR)
        IMM= INT(ITEMP/TSPMN)
      ELSE
C       Time-step specified as days per time-step
        ITEMP= ISTP*(-NTSPD)
        IYEAR= ITEMP/NYRDAY
        ITEMP= MOD(ITEMP,NYRDAY)
        IMONTH= 1 + ITEMP/NDPMON
        ITEMP= MOD(ITEMP,NDPMON)
        IDAY= 1 + ITEMP
        IHH= 0
        IMM= 0
      ENDIF
C     Create mmddhhmm string
      WRITE(MDATE,'(4X,I9)') 100000000 + IMONTH*1000000 + IDAY*10000
     1                                 + IHH*100 + IMM
C     Create year string
      IF (IYEAR .LT. 10000) THEN
        WRITE(YRSTR,'(I5)') 10000 + IYEAR
        YRSTR(1:1)= ' '
      ELSE
        WRITE(YRSTR,'(I5)') IYEAR
      ENDIF
      MDATE(1:5)= YRSTR
      RETURN
      END
 
      SUBROUTINE ODISV0( ISTP, STRESX, STRESY, FNET, TRCFIN )
C ODISV0 writes out model surface section info
C Input:  INTEGER ISTP -- current model step number
C         REAL STRESX(0:NXOMAX,0:NYOMAX) -- surface wind stress in the
C                                           zonal direction (Pa)
C         REAL STRESY(0:NXOMAX,0:NYOMAX) -- surface wind stress in the
C                                           meridional direction (Pa)
C         REAL FNET(0:NXOMAX,0:NYOMAX) -- Net heat flux into ocean
C                                         (in W/m**2)
C         REAL TRCFIN(0:NXOMAX,0:NYOMAX,JOCS:NOCNQ) -- other tracer
C                                   mass fluxes (in kg/(s*m**2))
C         (TRCFIN(,,JOCS) contains E-P in kg/(s*m**2))
C
C Output: (Surface sections are saved, if need be)
C
#include "ogrid.h"
#include "odicom.h"
      INTEGER ISTP
      REAL STRESX(0:NXOMAX,0:NYOMAX), STRESY(0:NXOMAX,0:NYOMAX)
      REAL FNET(0:NXOMAX,0:NYOMAX)
      REAL TRCFIN(0:NXOMAX,0:NYOMAX,JOCS:NOCNQ)
C
C
      INTEGER ITR, J, K
      CHARACTER COMNT*11, MDATE*13
      CHARACTER HEDHED*36, HEADER*48
      LOGICAL FIRST
C
C     Check if section data needs to be saved for this time-step
      IF (.NOT.SAVSRF .OR. (IUOHS.LE.0) .OR. (ISTP.LT.ISSTEP)) RETURN
C     Accumulate samples
      NSFSAV= NSFSAV + 1
      DO 10 K=0,NYOCN
        DO 10 J=0,NXOCN
          TAUSV(J,K,JX)= TAUSV(J,K,JX) + STRESX(J,K)
          TAUSV(J,K,JY)= TAUSV(J,K,JY) + STRESY(J,K)
          TRCISV(J,K,JOCT)= TRCISV(J,K,JOCT) + FNET(J,K)
   10     CONTINUE
      DO 20 ITR=JOCS,NOCNQ
        DO 20 K=0,NYOCN
          DO 20 J=0,NXOCN
   20       TRCISV(J,K,ITR)= TRCISV(J,K,ITR) + TRCFIN(J,K,ITR)
      IF (MOD((ISTP-IDSTEP), NSFREQ*NDSKIP) .EQ. 0) THEN
C       Write to save file; compute averages
        DO 30 K=0,NYOCN
          DO 30 J=0,NXOCN
            TAUSV(J,K,JX)= TAUSV(J,K,JX) / NSFSAV
            TAUSV(J,K,JY)= TAUSV(J,K,JY) / NSFSAV
            TRCISV(J,K,JOCT)= TRCISV(J,K,JOCT) / NSFSAV
   30       CONTINUE
        DO 40 ITR=JOCS,NOCNQ
          DO 40 K=0,NYOCN
            DO 40 J=0,NXOCN
   40         TRCISV(J,K,ITR)= TRCISV(J,K,ITR) / NSFSAV
C
C       Set first sample flag
        FIRST= NSSAMP .LE. 1
C       Get comment/date strings
        CALL OSVHDR( ISTP, COMNT, MDATE )
C
C       Save surface diagnostics
        HEDHED= COMNT//'HSEC'//MDATE//'SURFACE '
C
C       Write zonal component of surface wind stress
        HEADER= HEDHED//'TAUX (PA)   '
        NRECHS= NRECHS + 1
        CALL QPUTHO( IUOHS, NRECHS, TAUSV(0,0,JX), HEADER )
        IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
C
C       Write meridional component of surface wind stress
        HEADER= HEDHED//'TAUY (PA)   '
        NRECHS= NRECHS + 1
        CALL QPUTHO( IUOHS, NRECHS, TAUSV(0,0,JY), HEADER )
        IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
C
C       Write heat flux into surface
        HEADER= HEDHED//'FSURF (W/M2)'
        NRECHS= NRECHS + 1
        CALL QPUTHO( IUOHS, NRECHS, TRCISV(0,0,JOCT), HEADER )
        IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
C
C       Write E-P values
        HEADER= HEDHED//'E-P (KG/SM2)'
        NRECHS= NRECHS + 1
        CALL QPUTHO( IUOHS, NRECHS, TRCISV(0,0,JOCS), HEADER )
        IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
C
C       Reset surface save data accumulators
        CALL OSVZSF
      ENDIF
C
      RETURN
      END
 
      SUBROUTINE ODISAV( ISTP, U, V, W, TRC )
C ODISAV writes out model interior section info.
C Input:  INTEGER ISTP -- current time-step number
C         (Note: specify ISTP=-1 for time-mean states)
C         REAL U(0:NXOMAX,0:NYOMAX,0:NZOMAX) -- Zonal velocity
C         REAL V(0:NXOMAX,0:NYOMAX,0:NZOMAX) -- Meridional velocity
C         REAL W(0:NXOMAX,0:NYOMAX,0:NZOMAX) -- Vertical velocity
C         REAL TRC(0:NXOMAX,0:NYOMAX,0:NZOMAX,JOCT:NOCNQ) -- tracers
C Output: (Section data is saved, if need be)
C
#include "ogrid.h"
#include "odicom.h"
      INTEGER ISTP
      REAL U(0:NXOMAX,0:NYOMAX,0:NZOMAX), V(0:NXOMAX,0:NYOMAX,0:NZOMAX)
      REAL W(0:NXOMAX,0:NYOMAX,0:NZOMAX)
      REAL TRC(0:NXOMAX,0:NYOMAX,0:NZOMAX,JOCT:NOCNQ)
      REAL PSIZS(0:NYOMAX,0:NZOMAX)
C
      INTEGER ITR, J, K, L, NLEV, NLAT
      CHARACTER COMNT*11, MDATE*13
      CHARACTER HEDHED*36, LVLHED*8, LATHED*8, HEADER*48
      LOGICAL FIRST
C
C     Check if section data needs to be saved for this time-step
      IF (     ((IUOHS.LE.0).AND.(IUOVS.LE.0).AND.(IUOZS.LE.0))
     1    .OR. (ISTP.LT.ISSTEP)) RETURN
C     Accumulate samples
      NSSAV= NSSAV + 1
      IF (IUOHS .GT. 0) THEN
C       Initialize save level count
        NLEV= 0
        DO 30 L=0,NZOCN
          IF ( SAVEL(L) ) THEN
C           Increment save level count
            NLEV= NLEV + 1
            IF (NLEV .GT. NLEVMX) STOP 'NLEVMX'
C           Accumulate data for this level
            DO 10 K=0,NYOCN
              DO 10 J=0,NXOCN
                UHS(J,K,NLEV)= UHS(J,K,NLEV) + U(J,K,L)
                VHS(J,K,NLEV)= VHS(J,K,NLEV) + V(J,K,L)
                WHS(J,K,NLEV)= WHS(J,K,NLEV) + W(J,K,L)
   10           CONTINUE
            DO 20 ITR=JOCT,NOCNQ
              DO 20 K=0,NYOCN
                DO 20 J=0,NXOCN
                  TRCHS(J,K,NLEV,ITR)=  TRCHS(J,K,NLEV,ITR)
     1                                + TRC(J,K,L,ITR)
   20             CONTINUE
          ENDIF
   30     CONTINUE
      ENDIF
      IF (IUOVS .GT. 0) THEN
C       Initialize save latitude count
        NLAT= 0
        DO 60 K=0,NYOCN
          IF (SAVEK(K)) THEN
C           Increment save latitude count
            NLAT= NLAT + 1
            IF (NLAT .GT. NLATMX) STOP 'NLATMX'
C           Accumulate data for latitude at this level
            DO 40 L=0,NZOCN
              DO 40 J=0,NXOCN
                UVS(J,L,NLAT)= UVS(J,L,NLAT) + U(J,K,L)
                VVS(J,L,NLAT)= VVS(J,L,NLAT) + V(J,K,L)
   40           WVS(J,L,NLAT)= WVS(J,L,NLAT) + W(J,K,L)
            DO 50 ITR=JOCT,NOCNQ
              DO 50 L=0,NZOCN
                DO 50 J=0,NXOCN
                  TRCVS(J,L,NLAT,ITR)=  TRCVS(J,L,NLAT,ITR)
     1                                + TRC(J,K,L,ITR)
   50             CONTINUE
            ENDIF
   60       CONTINUE
      ENDIF
      IF (IUOZS .GT. 0) THEN
C       Accumulate zonal-mean data for latitude at this level
        DO 70 L=0,NZOCN
          DO 70 K=0,NYOCN
            DO 70 J=0,NXOCN
              UZS(K,L)= UZS(K,L) + WTXOCN(J)*U(J,K,L)
              VZS(K,L)= VZS(K,L) + WTXOCN(J)*V(J,K,L)
   70         WZS(K,L)= WZS(K,L) + WTXOCN(J)*W(J,K,L)
        DO 80 ITR=JOCT,NOCNQ
          DO 80 L=0,NZOCN
            DO 80 K=0,NYOCN
              DO 80 J=0,NXOCN
                TRCZS(K,L,ITR)=  TRCZS(K,L,ITR)
     1                         + WTXOCN(J)*TRC(J,K,L,ITR)
   80           CONTINUE
      ENDIF
      IF (MOD((ISTP-IDSTEP), NSFREQ*NDSKIP) .EQ. 0) THEN
C       Increment save sample count
        NSSAMP= NSSAMP + 1
C       Set first sample flag
        FIRST= NSSAMP .EQ. 1
C       Get comment/date strings
        CALL OSVHDR( ISTP, COMNT, MDATE )
C
        IF (IUOHS .GT. 0) THEN
C         Initialize save level count
          NLEV= 0
          DO 140 L=0,NZOCN
            IF (SAVEL(L)) THEN
C             Increment save level count
              NLEV= NLEV + 1
C             Compute averages
              DO 110 K=0,NYOCN
                DO 110 J=0,NXOCN
                  UHS(J,K,NLEV)= UHS(J,K,NLEV) / NSSAV
                  VHS(J,K,NLEV)= VHS(J,K,NLEV) / NSSAV
                  WHS(J,K,NLEV)= WHS(J,K,NLEV) / NSSAV
  110             CONTINUE
              DO 120 ITR=JOCT,NOCNQ
                DO 120 K=0,NYOCN
                  DO 120 J=0,NXOCN
                    TRCHS(J,K,NLEV,ITR)=  TRCHS(J,K,NLEV,ITR)
     1                                  / NSSAV
  120               CONTINUE
C             Save diagnostics for this level
              WRITE(LVLHED,'(I7,A1)') -NINT(ZOCN(L)), 'm'
              HEDHED= COMNT//'HSEC'//MDATE//LVLHED
C
C             Write U
              HEADER= HEDHED//'U (M/S)     '
              NRECHS= NRECHS + 1
              CALL QPUTHO( IUOHS, NRECHS, UHS(0,0,NLEV), HEADER )
              IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
C
C             Write V
              HEADER= HEDHED//'V (M/S)     '
              NRECHS= NRECHS + 1
              CALL QPUTHO( IUOHS, NRECHS, VHS(0,0,NLEV), HEADER )
              IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
C
C             Write W
              HEADER= HEDHED//'W (M/S)     '
              NRECHS= NRECHS + 1
              CALL QPUTHO( IUOHS, NRECHS, WHS(0,0,NLEV), HEADER )
              IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
C
C             Write T
              HEADER= HEDHED//'T (K)       '
              NRECHS= NRECHS + 1
              CALL QPUTHO(IUOHS, NRECHS, TRCHS(0,0,NLEV,JOCT), HEADER)
              IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
C
C             Write S
              HEADER= HEDHED//'S (KG/KG)   '
              NRECHS= NRECHS + 1
              CALL QPUTHO(IUOHS, NRECHS, TRCHS(0,0,NLEV,JOCS), HEADER)
              IF (FIRST) WRITE(*,*) 'Saving hor-section ', HEADER
            ENDIF
  140       CONTINUE
        ENDIF
C
        IF (IUOVS .GT. 0) THEN
C         Initialize save latitude count
          NLAT= 0
          DO 230 K=0,NYOCN
            IF (SAVEK(K)) THEN
C             Increment save latitude count
              NLAT= NLAT + 1
C             Compute averages
              DO 210 L=0,NZOCN
                DO 210 J=0,NXOCN
                  UVS(J,L,NLAT)= UVS(J,L,NLAT) / NSSAV
                  VVS(J,L,NLAT)= VVS(J,L,NLAT) / NSSAV
  210             WVS(J,L,NLAT)= WVS(J,L,NLAT) / NSSAV
              DO 220 ITR=JOCT,NOCNQ
                DO 220 L=0,NZOCN
                  DO 220 J=0,NXOCN
  220               TRCVS(J,L,NLAT,ITR)= TRCVS(J,L,NLAT,ITR) / NSSAV
C
              WRITE(LATHED,'(F5.1,A3)') LATOCN(K), 'deg'
              HEDHED= COMNT//'VSEC'//MDATE//LATHED
C
C             Write U
              HEADER= HEDHED//'U (M/S)     '
              NRECVS= NRECVS + 1
              CALL QPUTVO( IUOVS, NRECVS, UVS(0,0,NLAT), HEADER )
              IF (FIRST) WRITE(*,*) 'Saving ver-section ', HEADER
C
C             Write V
              HEADER= HEDHED//'V (M/S)     '
              NRECVS= NRECVS + 1
              CALL QPUTVO( IUOVS, NRECVS, VVS(0,0,NLAT), HEADER )
              IF (FIRST) WRITE(*,*) 'Saving ver-section ', HEADER
C
C             Write W
              HEADER= HEDHED//'W (M/S)     '
              NRECVS= NRECVS + 1
              CALL QPUTVO( IUOVS, NRECVS, WVS(0,0,NLAT), HEADER )
              IF (FIRST) WRITE(*,*) 'Saving ver-section ', HEADER
C
C             Write T
              HEADER= HEDHED//'T (K)       '
              NRECVS= NRECVS + 1
              CALL QPUTVO(IUOVS, NRECVS, TRCVS(0,0,NLAT,JOCT), HEADER)
              IF (FIRST) WRITE(*,*) 'Saving ver-section ', HEADER
C
C             Write S
              HEADER= HEDHED//'S (KG/KG)   '
              NRECVS= NRECVS + 1
              CALL QPUTVO(IUOVS, NRECVS, TRCVS(0,0,NLAT,JOCS), HEADER)
              IF (FIRST) WRITE(*,*) 'Saving ver-section ', HEADER
            ENDIF
  230       CONTINUE
        ENDIF
C
        IF (IUOZS .GT. 0) THEN
C         Save diagnostics for zonal-mean sections
C         Compute averages
          DO 310 L=0,NZOCN
            DO 310 K=0,NYOCN
              UZS(K,L)= UZS(K,L) / NSSAV
              VZS(K,L)= VZS(K,L) / NSSAV
  310         WZS(K,L)= WZS(K,L) / NSSAV
C
          DO 320 ITR=JOCT,NOCNQ
            DO 320 L=0,NZOCN
              DO 320 K=0,NYOCN
  320           TRCZS(K,L,ITR)= TRCZS(K,L,ITR) / NSSAV
C         Compute MMC streamfunction
          DO 325 K=0,NYOCN
            PSIZS(K,0)= 0.0
            DO 325 L=1,NZOCN
              PSIZS(K,L)=  PSIZS(K,L-1)
     1                    - DZOCN*VZS(K,L)*COS(PHIOCN(K))
  325         CONTINUE
C
          HEDHED= COMNT//'MSEC'//MDATE//'ZONMEAN '
C
C         Write PSI
          HEADER= HEDHED//'PSI (M2/S)  '
          NRECZS= NRECZS + 1
          CALL QPUTZO( IUOZS, NRECZS, PSIZS, HEADER )
          IF (FIRST) WRITE(*,*) 'Saving zonal-mean ', HEADER
C
C         Write T
          HEADER= HEDHED//'T (K)       '
          NRECZS= NRECZS + 1
          CALL QPUTZO( IUOZS, NRECZS, TRCZS(0,0,JOCT), HEADER )
          IF (FIRST) WRITE(*,*) 'Saving zonal-mean ', HEADER
C
C         Write S
          HEADER= HEDHED//'S (KG/KG)   '
          NRECZS= NRECZS + 1
          CALL QPUTZO( IUOZS, NRECZS, TRCZS(0,0,JOCS), HEADER )
          IF (FIRST) WRITE(*,*) 'Saving zonal-mean ', HEADER
        ENDIF
C
C       Reset save data accumulators
        CALL OSVZRO
      ENDIF
C
      RETURN
      END
 
      SUBROUTINE ODISTP( ISTP, U, V, W, TRC )
C ODISTP should be called during every time-marching step.
C It decides whether to sample for dynamical diagnostics or not,
C and carries out the necessary calculations. It also writes out
C time-series and section data.
C Input:  INTEGER ISTP -- current model step number
C         REAL U(0:NXOMAX,0:NYOMAX,0:NZOMAX) -- Zonal velocity
C         REAL V(0:NXOMAX,0:NYOMAX,0:NZOMAX) -- Meridional velocity
C         REAL W(0:NXOMAX,0:NYOMAX,0:NZOMAX) -- Vertical velocity
C         REAL TRC(0:NXOMAX,0:NYOMAX,0:NZOMAX,JOCT:NOCNQ) -- tracers
C Output: (Diagnostic calculations are carried out, if need be)
C
#include "ogrid.h"
#include "odicom.h"
      INTEGER ISTP
      REAL U(0:NXOMAX,0:NYOMAX,0:NZOMAX), V(0:NXOMAX,0:NYOMAX,0:NZOMAX)
      REAL W(0:NXOMAX,0:NYOMAX,0:NZOMAX)
      REAL TRC(0:NXOMAX,0:NYOMAX,0:NZOMAX,JOCT:NOCNQ)
C
      REAL TIM, P01, P02, WT
      INTEGER ITR, I, J, K, L, KST, KNT, KEQ1, KEQ2, LMID
C
C     Check if current model step needs to be sampled
      IF ( (ISTP.LT.IDSTEP) .OR.
     1     (MOD((ISTP-IDSTEP), NDSKIP) .NE. 0) )RETURN
C
C     Calculate diagnostic quantities
      CALL ODICAL( ISTP, U, V, W, TRC )
C     Save interior quantities, if need be
      CALL ODISAV( ISTP, U, V, W, TRC )
C     Compute current model time
      TIM= DBLE(ISTP)*DBLE(DTSTEP)
      ISTP1= ISTP
      TAU1= TIM
C
      IF (ISTP.EQ.IDSTEP) THEN
C       Initial snapshot
        ISTP0= ISTP
        TAU0= TIM
C       Save initial MMC streamfunction
        DO 10 L=0,NZOCN
          DO 10 K=0,NYOCN
            PSIZIN(K,L)= PSIZON(K,L)
   10       PSITEN(K,L)= 0.0
C       Save initial zonal-mean tracer concentration
        DO 20 ITR=JOCT,NOCNQ
          DO 20 L=0,NZOCN
            DO 20 K=0,NYOCN
              TRCZIN(K,L,ITR)= TRCZON(K,L,ITR)
   20         TRTEND(K,L,ITR)= 0.0
      ELSE
C       Increment sample counter
        NSAMPL= NSAMPL+1
C       Compute time-averaged MMC streamfunction tendency
        DO 30 L=0,NZOCN
          DO 30 K=0,NYOCN
            PSITEN(K,L)=   ( PSIZON(K,L) - PSIZIN(K,L) )
     1                   / ((NDSKIP*DTSTEP)*NSAMPL)
   30       CONTINUE
C       Compute time-averaged tracer tendencies
        DO 40 ITR=JOCT,NOCNQ
          DO 40 L=0,NZOCN
            DO 40 K=0,NYOCN
              TRTEND(K,L,ITR)= ( TRCZON(K,L,ITR) - TRCZIN(K,L,ITR) )
     1                        /( (NDSKIP*DTSTEP)*NSAMPL )
   40         CONTINUE
C       Accumulate quantities for calculating time-averages
        DO 100 L=0,NZOCN
          DO 100 K=0,NYOCN
            DO 100 J=0,NXOCN
              UAVG(J,K,L)= UAVG(J,K,L) + U(J,K,L)
              VAVG(J,K,L)= VAVG(J,K,L) + V(J,K,L)
              WAVG(J,K,L)= WAVG(J,K,L) + W(J,K,L)
  100         CONTINUE
        DO 110 ITR=JOCT,NOCNQ
          DO 110 L=0,NZOCN
            DO 110 K=0,NYOCN
              DO 110 J=0,NXOCN
  110           TRCAVG(J,K,L,ITR)= TRCAVG(J,K,L,ITR) + TRC(J,K,L,ITR)
C
        DO 120 L=0,NZOCN
          DO 120 K=0,NYOCN
            DO 120 J=0,NXOCN
              UZVAR(K,L)= UZVAR(K,L) + WTXOCN(J)*U(J,K,L)**2
              VZVAR(K,L)= VZVAR(K,L) + WTXOCN(J)*V(J,K,L)**2
              WZVAR(K,L)= WZVAR(K,L) + WTXOCN(J)*W(J,K,L)**2
  120         CONTINUE
        DO 130 ITR=JOCT,NOCNQ
          DO 130 L=0,NZOCN
            DO 130 K=0,NYOCN
              DO 130 J=0,NXOCN
                TRZVAR(K,L,ITR)=  TRZVAR(K,L,ITR)
     1                          + WTXOCN(J)*TRC(J,K,L,ITR)**2
  130           CONTINUE
C
        DO 140 ITR=JOCT,NOCNQ
          DO 140 I=1,2
            DO 140 L=0,NZOCN
              DO 140 K=0,NYOCN
  140           TRFAVG(K,L,I,ITR)= TRFAVG(K,L,I,ITR)+TRFLUX(K,L,I,ITR)
C
        DO 150 K=0,NYOCN
          DO 150 J=0,NXOCN
            KESAVG(J,K)= KESAVG(J,K) + 0.5*(  VELSRF(J,K,JX)**2
     1                                      + VELSRF(J,K,JY)**2 )
            SPFLD1(J,K)= SPFLD1(J,K) + 0.0
            SPFLD2(J,K)= SPFLD2(J,K) + 0.0
  150       CONTINUE
C
        DO 170 ITR=JOCT,NOCNQ
          DO 170 K=0,NYOCN
            DO 170 J=0,NXOCN
  170         TRSVAR(J,K,ITR)= TRSVAR(J,K,ITR) + TRSURF(J,K,ITR)**2
      ENDIF
C
C     Compute time-evolution quantities
C
C     Mid-ocean vertical coordinate index
      LMID= NZOCN/2
C     Compute location of equator, 30 deg. latitude circles
      KST= 0
      KNT= 0
      DO 210 K=0,NYOCN
        IF (LATOCN(K) .LT. -30.) KST= MIN0(K+1, NYOCN)
  210   IF (LATOCN(K) .LE. 30.) KNT= K
      KEQ1= NYOCN/2
      KEQ2= (NYOCN+1)/2
C     Compute globally averaged kinetic energy
      KE= 0.0
      DO 220 L=0,NZOCN
        DO 220 K=0,NYOCN
          DO 220 J=0,NXOCN
            KE= KE + WTXOCN(J)*WTYOCN(K)*WTZOCN(L)*0.5*( U(J,K,L)**2
     1                                                  +V(J,K,L)**2)
  220       CONTINUE
C     Compute volume-averaged tracer values
      DO 230 ITR=JOCT,NOCNQ
        TR00(ITR)= 0.0
        DO 230 L=0,NZOCN
          DO 230 K=0,NYOCN
            DO 230 J=0,NXOCN
              TR00(ITR)= TR00(ITR) + WTXOCN(J)*WTYOCN(K)*WTZOCN(L)
     1                              *TRC(J,K,L,ITR)
  230         CONTINUE
C
C     Compute legendre coefficients of MMC streamfunction, tracers
      PSI01= 0.0
      PSI02= 0.0
      DO 240 ITR=JOCT,NOCNQ
        TR00S(ITR)= 0.0
        TR01S(ITR)= 0.0
  240   TR02S(ITR)= 0.0
C
      DO 250 K=0,NYOCN
        P01= SQRT(3.0)*SIN(PHIOCN(K))
        P02= 0.5*SQRT(5.0)*(3.0*SIN(PHIOCN(K))**2 - 1.0)
        PSI01= PSI01 + WTYOCN(K)*P01*PSIZON(K,LMID)
        PSI02= PSI02 + WTYOCN(K)*P02*PSIZON(K,LMID)
        DO 250 ITR=JOCT,NOCNQ
          TR00S(ITR)= TR00S(ITR) + WTYOCN(K)*TRCZON(K,NZOCN,ITR)
          TR01S(ITR)= TR01S(ITR) + WTYOCN(K)*P01*TRCZON(K,NZOCN,ITR)
  250     TR02S(ITR)= TR02S(ITR) + WTYOCN(K)*P02*TRCZON(K,NZOCN,ITR)
C
C     Compute equatorial value of mid-level MMC streamfunction
      PSIEQ= 0.5*( PSIZON(KEQ1,LMID) + PSIZON(KEQ2,LMID) )
C
C     Compute values south of 30S
C     Extreme value of MMC streamfunction
      PSIMX(1)= 0.0
      DO 310 L=0,NZOCN
        DO 310 K=0,KST-1
          IF (ABS(PSIZON(K,L)).GT.ABS(PSIMX(1))) PSIMX(1)= PSIZON(K,L)
  310     CONTINUE
C     Weight
      WT= 0.0
      DO 320 K=0,KST-1
  320   WT= WT + WTYOCN(K)
C     Vertically-averaged value of tracer
      DO 330 ITR=JOCT,NOCNQ
        TRCVM(1,ITR)= 0.0
        DO 330 L=0,NZOCN
          DO 330 K=0,KST-1
            TRCVM(1,ITR)=  TRCVM(1,ITR)
     1                   + (WTYOCN(K)*WTZOCN(L)/WT)*TRCZON(K,L,ITR)
  330       CONTINUE
C     Vertical flux of tracer at mid-level
      DO 340 ITR=JOCT,NOCNQ
        TRCVF(1,ITR)= 0.0
        DO 340 K=0,KST-1
          TRCVF(1,ITR)=  TRCVF(1,ITR)
     1                 + (WTYOCN(K)/WT)*TRFLUX(K,LMID,2,ITR)
  340     CONTINUE
C
C     Compute values between 30S and the Equator
C     Extreme value of MMC streamfunction
      PSIMX(2)= 0.0
      DO 410 L=0,NZOCN
        DO 410 K=KST,KEQ1
          IF (ABS(PSIZON(K,L)).GT.ABS(PSIMX(2))) PSIMX(2)= PSIZON(K,L)
  410     CONTINUE
C     Weight
      WT= 0.0
      DO 420 K=KST,KEQ1
  420   WT= WT + WTYOCN(K)
C     Vertically-averaged value of tracer
      DO 430 ITR=JOCT,NOCNQ
        TRCVM(2,ITR)= 0.0
        DO 430 L=0,NZOCN
          DO 430 K=KST,KEQ1
            TRCVM(2,ITR)=  TRCVM(2,ITR)
     1                   + (WTYOCN(K)*WTZOCN(L)/WT)*TRCZON(K,L,ITR)
  430       CONTINUE
C     Vertical flux of tracer at mid-level
      DO 440 ITR=JOCT,NOCNQ
        TRCVF(2,ITR)= 0.0
        DO 440 K=KST,KEQ1
          TRCVF(2,ITR)=  TRCVF(2,ITR)
     1                 + (WTYOCN(K)/WT)*TRFLUX(K,LMID,2,ITR)
  440     CONTINUE
C
C     Compute values between the Equator and 30N
      PSIMX(3)= 0.0
      DO 510 L=0,NZOCN
        DO 510 K=KEQ2,KNT
          IF (ABS(PSIZON(K,L)).GT.ABS(PSIMX(3))) PSIMX(3)= PSIZON(K,L)
  510     CONTINUE
C     Weight
      WT= 0.0
      DO 520 K=KEQ2,KNT
  520   WT= WT + WTYOCN(K)
C     Vertically-averaged value of tracer
      DO 530 ITR=JOCT,NOCNQ
        TRCVM(3,ITR)= 0.0
        DO 530 L=0,NZOCN
          DO 530 K=KEQ2,KNT
            TRCVM(3,ITR)=  TRCVM(3,ITR)
     1                   + (WTYOCN(K)*WTZOCN(L)/WT)*TRCZON(K,L,ITR)
  530       CONTINUE
C     Vertical flux of tracer at mid-level
      DO 540 ITR=JOCT,NOCNQ
        TRCVF(3,ITR)= 0.0
        DO 540 K=KEQ2,KNT
          TRCVF(3,ITR)=  TRCVF(3,ITR)
     1                 + (WTYOCN(K)/WT)*TRFLUX(K,LMID,2,ITR)
  540     CONTINUE
C
C     Compute values north of 30N
      PSIMX(4)= 0.0
      DO 610 L=0,NZOCN
        DO 610 K=KNT+1,NYOCN
          IF (ABS(PSIZON(K,L)).GT.ABS(PSIMX(4))) PSIMX(4)= PSIZON(K,L)
  610     CONTINUE
C     Weight
      WT= 0.0
      DO 620 K=KNT+1,NYOCN
  620   WT= WT + WTYOCN(K)
C     Vertically-averaged value of tracer
      DO 630 ITR=JOCT,NOCNQ
        TRCVM(4,ITR)= 0.0
        DO 630 L=0,NZOCN
          DO 630 K=KNT+1,NYOCN
            TRCVM(4,ITR)=  TRCVM(4,ITR)
     1                   + (WTYOCN(K)*WTZOCN(L)/WT)*TRCZON(K,L,ITR)
  630       CONTINUE
C     Vertical flux of tracer at mid-level
      DO 640 ITR=JOCT,NOCNQ
        TRCVF(4,ITR)= 0.0
        DO 640 K=KNT+1,NYOCN
          TRCVF(4,ITR)=  TRCVF(4,ITR)
     1                 + (WTYOCN(K)/WT)*TRFLUX(K,LMID,2,ITR)
  640     CONTINUE
C
C     Compute mean lower ocean tracer values
      WT= 0.0
      DO 710 L=0,LMID
  710   WT= WT + WTZOCN(L)
C
      DO 720 ITR=JOCT,NOCNQ
        TRCHM(1,ITR)= 0.0
        DO 720 L=0,LMID
          DO 720 K=0,NYOCN
            TRCHM(1,ITR)=  TRCHM(1,ITR)
     1                   + (WTYOCN(K)*WTZOCN(L)/WT)*TRCZON(K,L,ITR)
  720     CONTINUE
C     Compute mean upper ocean tracer values
      WT= 0.0
      DO 730 L=LMID,NZOCN
  730   WT= WT + WTZOCN(L)
C
      DO 740 ITR=JOCT,NOCNQ
        TRCHM(2,ITR)= 0.0
        DO 740 L=LMID,NZOCN
          DO 740 K=0,NYOCN
            TRCHM(2,ITR)=  TRCHM(2,ITR)
     1                   + (WTYOCN(K)*WTZOCN(L)/WT)*TRCZON(K,L,ITR)
  740     CONTINUE
C
C     Compute tracer fluxes at tropics/extratropics interface
      DO 750 ITR=JOCT,NOCNQ
        TRCHF(1,ITR)= 0.0
        TRCHF(2,ITR)= 0.0
        TRCHF(3,ITR)= 0.0
        DO 750 L=0,NZOCN
          TRCHF(1,ITR)= TRCHF(1,ITR) + WTZOCN(L)*TRFLUX(KST,L,1,ITR)
          TRCHF(2,ITR)=  TRCHF(2,ITR)
     1                 + WTZOCN(L)*0.5*(  TRFLUX(KEQ1,L,1,ITR)
     2                                  + TRFLUX(KEQ2,L,1,ITR) )
          TRCHF(3,ITR)= TRCHF(3,ITR) + WTZOCN(L)*TRFLUX(KNT,L,1,ITR)
  750     CONTINUE
      IF ((IUOTS .GT. 0) .AND. (ISTP .GE. ISSTEP)) THEN
C       Accumulate time-series data
        CALL TOACC
        IF (MOD((ISTP-IDSTEP), NTFREQ*NDSKIP) .EQ. 0) THEN
C         Write time-series data
          CALL TOWRIT( ISTP )
C         Re-initialize time-series accumulators
          CALL TOINI
        ENDIF
      ENDIF
      RETURN
      END
 
      SUBROUTINE ODIFLX( ISTP, U, V, W, TRC,
     1                   STRESX, STRESY, FNET, TRCFIN )
C ODIFLX should be called during every time-marching step.
C It decides whether to sample for flux diagnostics or not,
C and carries out the necessary calculations.
C (Note:  ODIFLX automatically calls ODISTP.)
C Input:  INTEGER ISTP -- current model step number
C         REAL U(0:NXOMAX,0:NYOMAX,0:NZOMAX) -- Zonal velocity
C         REAL V(0:NXOMAX,0:NYOMAX,0:NZOMAX) -- Meridional velocity
C         REAL W(0:NXOMAX,0:NYOMAX,0:NZOMAX) -- Vertical velocity
C         REAL TRC(0:NXOMAX,0:NYOMAX,0:NZOMAX,JOCT:NOCNQ) -- tracers
C         REAL STRESX(0:NXOMAX,0:NYOMAX) -- surface wind stress in the
C                                           zonal direction (Pa)
C         REAL STRESY(0:NXOMAX,0:NYOMAX) -- surface wind stress in the
C                                           meridional direction (Pa)
C         REAL FNET(0:NXOMAX,0:NYOMAX) -- Net heat flux into ocean
C                                        (in W/m**2)
C         REAL TRCFIN(0:NXOMAX,0:NYOMAX,JOCS:NOCNQ) -- other tracer
C                                   mass fluxes (in kg/(s*m**2))
C         (TRCFIN(,,JOCS) contains E-P in kg/(s*m**2))
C Output: (Diagnostic calculations are carried out, if need be)
C
#include "ogrid.h"
#include "odicom.h"
      INTEGER ISTP
      REAL U(0:NXOMAX,0:NYOMAX,0:NZOMAX), V(0:NXOMAX,0:NYOMAX,0:NZOMAX)
      REAL W(0:NXOMAX,0:NYOMAX,0:NZOMAX)
      REAL TRC(0:NXOMAX,0:NYOMAX,0:NZOMAX,JOCT:NOCNQ)
      REAL STRESX(0:NXOMAX,0:NYOMAX), STRESY(0:NXOMAX,0:NYOMAX)
      REAL FNET(0:NXOMAX,0:NYOMAX)
      REAL TRCFIN(0:NXOMAX,0:NYOMAX,JOCS:NOCNQ)
C
      INTEGER ITR, J, K
C
C     Check if current model step needs to be sampled
      IF ( (ISTP.LT.IDSTEP) .OR.
     1     (MOD((ISTP-IDSTEP), NDSKIP) .NE. 0) )RETURN
C     Save surface quantities, if need be
      CALL ODISV0( ISTP, STRESX, STRESY, FNET, TRCFIN )
C
C     Accumulate flux diagnostics
C
      DO 150 K=0,NYOCN
        DO 150 J=0,NXOCN
          TAUAVG(J,K,JX)= TAUAVG(J,K,JX) + STRESX(J,K)
          TAUAVG(J,K,JY)= TAUAVG(J,K,JY) + STRESY(J,K)
  150     TRCIAV(J,K,JOCT)= TRCIAV(J,K,JOCT) + FNET(J,K)
C
      DO 170 ITR=JOCS,NOCNQ
        DO 170 K=0,NYOCN
          DO 170 J=0,NXOCN
  170       TRCIAV(J,K,ITR)= TRCIAV(J,K,ITR) + TRCFIN(J,K,ITR)
C
C     Compute dynamical diagnostics
      CALL ODISTP( ISTP, U, V, W, TRC )
      RETURN
      END
 
      SUBROUTINE ODICAL( ISTP, U, V, W, TRC )
C Computes diagnostics, and stores results in common module odicom.h.
C PGQSP is the spectral representation of prognostic quantities.
C ISTP is the current time-step number
C (Note: specify ISTP=-1 for time-mean states)
C Input:  INTEGER ISTP -- current model step number
C         (Note: specify ISTP=-1 for time-mean states)
C         REAL U(0:NXOMAX,0:NYOMAX,0:NZOMAX) -- Zonal velocity
C         REAL V(0:NXOMAX,0:NYOMAX,0:NZOMAX) -- Meridional velocity
C         REAL W(0:NXOMAX,0:NYOMAX,0:NZOMAX) -- Vertical velocity
C         REAL TRC(0:NXOMAX,0:NYOMAX,0:NZOMAX,JOCT:NOCNQ) -- tracers
C
#include "ogrid.h"
#include "odicom.h"
      INTEGER ISTP
      REAL U(0:NXOMAX,0:NYOMAX,0:NZOMAX), V(0:NXOMAX,0:NYOMAX,0:NZOMAX)
      REAL W(0:NXOMAX,0:NYOMAX,0:NZOMAX)
      REAL TRC(0:NXOMAX,0:NYOMAX,0:NZOMAX,JOCT:NOCNQ)
C
      INTEGER ITR, J, K, L
C
C     Compute zonal-mean wind components
      DO 110 L=0,NZOCN
        DO 110 K=0,NYOCN
          UZON(K,L)= 0.0
          VZON(K,L)= 0.0
          WZON(K,L)= 0.0
          DO 110 J=0,NXOCN
            UZON(K,L)= UZON(K,L) + WTXOCN(J)*U(J,K,L)
            VZON(K,L)= VZON(K,L) + WTXOCN(J)*V(J,K,L)
            WZON(K,L)= WZON(K,L) + WTXOCN(J)*W(J,K,L)
  110       CONTINUE
C     Compute MMC streamfunction
      DO 120 K=0,NYOCN
        PSIZON(K,0)= 0.0
        DO 120 L=1,NZOCN
  120     PSIZON(K,L)= PSIZON(K,L-1) - DZOCN*VZON(K,L)*COS(PHIOCN(K))
C     Compute zonal-mean tracer values, and fluxes
      DO 130 ITR=JOCT,NOCNQ
        DO 130 L=0,NZOCN
          DO 130 K=0,NYOCN
            TRCZON(K,L,ITR)= 0.0
            TRFLUX(K,L,1,ITR)= 0.0
            TRFLUX(K,L,2,ITR)= 0.0
            DO 130 J=0,NXOCN
              TRCZON(K,L,ITR)=  TRCZON(K,L,ITR)
     1                        + WTXOCN(J)*TRC(J,K,L,ITR)
              TRFLUX(K,L,1,ITR)=  TRFLUX(K,L,1,ITR)
     1                          + WTXOCN(J)*V(J,K,L)*TRC(J,K,L,ITR)
              TRFLUX(K,L,2,ITR)=  TRFLUX(K,L,2,ITR)
     1                          + WTXOCN(J)*W(J,K,L)*TRC(J,K,L,ITR)
  130         CONTINUE
C     Copy surface velocity values
      DO 140 K=0,NYOCN
        DO 140 J=0,NXOCN
          VELSRF(J,K,JX)= U(J,K,NZOCN)
  140     VELSRF(J,K,JY)= V(J,K,NZOCN)
C     Copy surface tracer values
      DO 150 ITR=JOCT,NOCNQ
        DO 150 K=0,NYOCN
          DO 150 J=0,NXOCN
  150       TRSURF(J,K,ITR)= TRC(J,K,NZOCN,ITR)
      RETURN
      END
 
      SUBROUTINE ODIEND( FILNAM )
C ODIEND should be called at the end of the diagnostic sampling period.
C The diagnostic results are written to file PREFIX.do. The file name
C is returned in the variable FILNAM.
C
#include "ogrid.h"
#include "odicom.h"
      CHARACTER*(*) FILNAM
      REAL MERACC(JOCT:NOCNQ), TFLXAV(JOCT:NOCNQ)
      REAL TTENAV(JOCT:NOCNQ), TINPAV(JOCT:NOCNQ)
      INTEGER ITR, I, J, K, L
C
      REAL PI
      PARAMETER (PI=3.141592653589793)
C
      IF (NSAMPL.LT.1) THEN
        WRITE(*,*) 'Insufficient data for time-averages'
        STOP 'ODIEND'
      ENDIF
C     Calculate time-averages
      DO 100 L=0,NZOCN
        DO 100 K=0,NYOCN
          DO 100 J=0,NXOCN
            UAVG(J,K,L)= UAVG(J,K,L) / NSAMPL
            VAVG(J,K,L)= VAVG(J,K,L) / NSAMPL
            WAVG(J,K,L)= WAVG(J,K,L) / NSAMPL
  100       CONTINUE
      DO 110 ITR=JOCT,NOCNQ
        DO 110 L=0,NZOCN
          DO 110 K=0,NYOCN
            DO 110 J=0,NXOCN
  110         TRCAVG(J,K,L,ITR)= TRCAVG(J,K,L,ITR) / NSAMPL
C
C     Compute diagnostics for time-mean state
      CALL ODICAL( -1, UAVG, VAVG, WAVG, TRCAVG )
C
      DO 120 L=0,NZOCN
        DO 120 K=0,NYOCN
          UZVAR(K,L)= UZVAR(K,L) / NSAMPL
          VZVAR(K,L)= VZVAR(K,L) / NSAMPL
          WZVAR(K,L)= WZVAR(K,L) / NSAMPL
  120     CONTINUE
      DO 130 ITR=JOCT,NOCNQ
        DO 130 L=0,NZOCN
          DO 130 K=0,NYOCN
  130       TRZVAR(K,L,ITR)= TRZVAR(K,L,ITR) / NSAMPL
C
      DO 140 ITR=JOCT,NOCNQ
        DO 140 I=1,2
          DO 140 L=0,NZOCN
            DO 140 K=0,NYOCN
  140         TRFAVG(K,L,I,ITR)= TRFAVG(K,L,I,ITR) / NSAMPL
C
      DO 150 K=0,NYOCN
        DO 150 J=0,NXOCN
          KESAVG(J,K)= KESAVG(J,K) / NSAMPL
          SPFLD1(J,K)= SPFLD1(J,K) / NSAMPL
          SPFLD2(J,K)= SPFLD2(J,K) / NSAMPL
  150     CONTINUE
      DO 155 ITR=JOCT,NOCNQ
        DO 155 K=0,NYOCN
          DO 155 J=0,NXOCN
  155       TRSVAR(J,K,ITR)= TRSVAR(J,K,ITR) / NSAMPL
C
C     Compute time-averaged flux quantities
      DO 160 I=JX,JY
        DO 160 K=0,NYOCN
          DO 160 J=0,NXOCN
  160       TAUAVG(J,K,I)= TAUAVG(J,K,I) / NSAMPL
      DO 170 ITR=JOCT,NOCNQ
        DO 170 K=0,NYOCN
          DO 170 J=0,NXOCN
  170       TRCIAV(J,K,ITR)= TRCIAV(J,K,ITR) / NSAMPL
C
C     Initialize accumulators for meridional integration
      DO 510 ITR=JOCT,NOCNQ
  510   MERACC(ITR)= 0.0
C     Compute meridional tracer transport
      DO 580 K=0,NYOCN
        DO 560 ITR=JOCT,NOCNQ
C         Compute vertically-averaged tracer flux and tendency
          TFLXAV(ITR)= 0.0
          TTENAV(ITR)= 0.0
          DO 540 L=0,NZOCN
            TFLXAV(ITR)= TFLXAV(ITR) + WTZOCN(L)*TRFAVG(K,L,1,ITR)
  540       TTENAV(ITR)= TTENAV(ITR) + WTZOCN(L)*TRTEND(K,L,ITR)
C         Compute zonally-averaged surface input tracer flux
          TINPAV(ITR)= 0.0
          DO 550 J=0,NXOCN
  550       TINPAV(ITR)= TINPAV(ITR) + WTXOCN(J)*TRCIAV(J,K,ITR)
  560     CONTINUE
C       Integrate heat flux meridionally (in Watts)
        MERACC(JOCT)= MERACC(JOCT) + (4.0*PI*AOCN**2)
     1               *OCNWID*WTYOCN(K)
     2               *(TINPAV(JOCT) - OCNDEP*OCNRHO*OCNCP*TTENAV(JOCT))
        MERFLX(K,JOCT)= MERACC(JOCT)
C       Compute advective heat flux (in Watts)
        ADVFLX(K,JOCT)= 2.0*PI*AOCN*OCNWID*OCNDEP*COS(PHIOCN(K))
     1                 *OCNRHO*OCNCP*TFLXAV(JOCT)
C
C       Convert E-P to input salt flux
        TINPAV(JOCS)= TINPAV(JOCS) * OCNS0
C       Handle other tracers
        DO 570 ITR=JOCS,NOCNQ
C         Integrate tracer flux meridionally (in kg/s)
          MERACC(ITR)= MERACC(ITR) + (4.0*PI*AOCN**2)
     1                 *OCNWID*WTYOCN(K)
     2                 *(TINPAV(ITR) - OCNDEP*OCNRHO*TTENAV(ITR))
          MERFLX(K,ITR)= MERACC(ITR)
C         Compute advective tracer flux (in kg/s)
          ADVFLX(K,ITR)= 2.0*PI*AOCN*OCNWID*OCNDEP*COS(PHIOCN(K))
     1                   *OCNRHO*TFLXAV(ITR)
  570     CONTINUE
  580   CONTINUE
C
      IF (IUODG .GT. 0) THEN
C       Write out diagnostic results
        FILNAM= PREFIX//'.do'
        CALL LJUST2( FILNAM )
        OPEN(IUODG, FILE=FILNAM, STATUS='NEW')
        CALL ODIPUT( IUODG )
        CLOSE( IUODG )
        WRITE(*,*) 'Saved diagnostic data to ', FILNAM
      ENDIF
C
C     Close files
      IF (IUOTS .GT. 0) CLOSE (IUOTS )
      IF (IUOHS .GT. 0) CLOSE (IUOHS )
      IF (IUOVS .GT. 0) CLOSE (IUOVS )
      IF (IUOZS .GT. 0) CLOSE (IUOZS )
C
      RETURN
      END
 
      SUBROUTINE QPUTHO( IUNIT, IRECD, FPH, HEADER )
C QPUTHO writes physical space values of quantity FPH to unit IUNIT.
C IRECD is the record number, HEADER is a 48 character header string.
C
#include "ogrid.h"
      INTEGER IUNIT, IRECD
      REAL FPH(0:NXOMAX,0:NYOMAX)
      CHARACTER*48 HEADER
C
      INTEGER J, K
      REAL FPRIME(0:NXOMAX,0:NYOMAX), FMEAN
C
C     Compute area-weighted global-mean of the field
      FMEAN= 0.0
      DO 10 K=0,NYOCN
        DO 10 J=0,NXOCN
   10     FMEAN= FMEAN + WTXOCN(J)*WTYOCN(K)*FPH(J,K)
C     Compute deviations from the global mean
      DO 20 K=0,NYOCN
        DO 20 J=0,NXOCN
   20     FPRIME(J,K)= FPH(J,K) - FMEAN
C     Write to fortran unit
      WRITE(IUNIT,REC=IRECD) HEADER, FMEAN, FLOAT(NXOCN+1),
     1                       FLOAT(NYOCN+1), 0.0,
     2                       ((FPRIME(J,K),J=0,NXOCN),K=0,NYOCN)
      RETURN
      END
 
      SUBROUTINE QPUTVO( IUNIT, IRECD, FVS, HEADER )
C QPUTVO writes latitude circle values of quantity FVS to unit IUNIT.
C IRECD is the record number, HEADER is a 48 character header string.
C
#include "ogrid.h"
      INTEGER IUNIT, IRECD
      REAL FVS(0:NXOMAX,0:NZOMAX)
      CHARACTER*48 HEADER
C
      INTEGER J, L
      REAL FPRIME(0:NXOMAX,0:NZOMAX), FMEAN
C
C     Compute mass-weighted section-mean of the field
      FMEAN= 0.0
      DO 10 L=0,NZOCN
        DO 10 J=0,NXOCN
   10     FMEAN= FMEAN + WTXOCN(J)*WTZOCN(L)*FVS(J,L)
C     Compute deviations from the global mean
      DO 20 L=0,NZOCN
        DO 20 J=0,NXOCN
   20     FPRIME(J,L)= FVS(J,L) - FMEAN
C     Write to fortran unit
      WRITE(IUNIT,REC=IRECD) HEADER, FMEAN, FLOAT(NXOCN+1),
     1                       FLOAT(NZOCN+1), 0.0,
     2                       ((FPRIME(J,L),J=0,NXOCN),L=0,NZOCN)
      RETURN
      END
 
      SUBROUTINE QPUTZO( IUNIT, IRECD, FZS, HEADER )
C QPUTZO writes zonal-mean values of quantity FZS to unit IUNIT.
C IRECD is the record number, HEADER is a 48 character header string.
C
#include "ogrid.h"
      INTEGER IUNIT, IRECD
      REAL FZS(0:NYOMAX,0:NZOMAX)
      CHARACTER*48 HEADER
C
      INTEGER K, L
      REAL FPRIME(0:NYOMAX,0:NZOMAX), FMEAN
C
C     Compute mass-weighted section-mean of the field
      FMEAN= 0.0
      DO 10 L=0,NZOCN
        DO 10 K=0,NYOCN
   10     FMEAN= FMEAN + WTYOCN(K)*WTZOCN(L)*FZS(K,L)
C     Compute deviations from the global mean
      DO 20 L=0,NZOCN
        DO 20 K=0,NYOCN
   20     FPRIME(K,L)= FZS(K,L) - FMEAN
C     Write to fortran unit
      WRITE(IUNIT,REC=IRECD) HEADER, FMEAN, FLOAT(NYOCN+1),
     1                       FLOAT(NZOCN+1), 0.0,
     2                       ((FPRIME(K,L),K=0,NYOCN),L=0,NZOCN)
      RETURN
      END
 
      SUBROUTINE ODIPUT( IUNIT )
C ODIPUT writes out diagnostic results onto unit IUNIT
C
#include "ogrid.h"
#include "odicom.h"
      INTEGER IUNIT
      INTEGER I, J, K, L, ITR
      REAL DAYSEC, TAUINT
      CHARACTER FMT*40, STROUT*72, TIMST0*10, TIMST1*10
C
      FMT= '(1X,1P,5E15.6)'
C     Seconds per day
      IF (NTSPD .GT. 0) THEN
        DAYSEC= NTSPD * DTSTEP
      ELSE
        DAYSEC= DTSTEP / FLOAT(-NTSPD)
      ENDIF
C     Sampling time-interval
      TAUINT= NDSKIP * DTSTEP
C
      WRITE(TIMST0,'(F10.2)') TAU0/DAYSEC
      WRITE(TIMST1,'(F10.2)') TAU1/DAYSEC
C
C     Preface to diagnostic file
      WRITE(IUNIT,'(1X,A)') RUNTTL
      STROUT= 'Diagnostics file saved by ODIPUT version 1.0'
      WRITE(IUNIT,'(1X,A)') STROUT
      STROUT= 'Started on '// DAYTM0
      WRITE(IUNIT,'(1X,A)') STROUT
      STROUT= 'Sampled from model day '//TIMST0//' to '//TIMST1
      WRITE(IUNIT,'(1X,A)') STROUT
      WRITE(IUNIT,'(1X,A,F8.2,A)')
     1            'Sampling interval = ', TAUINT/3600., ' hours'
C
C     Model/planetary parameters
      WRITE(IUNIT,'(A)') 'NXOCN, NYOCN, NZOCN, NTROCN'
      WRITE(IUNIT,'(1X,6I12)') NXOCN, NYOCN, NZOCN, NTROCN
      WRITE(IUNIT,'(A)')
     1                'IDSET, ISTP0, ISTP1, NTSPD, NYRDAY'
      WRITE(IUNIT,'(1X,6I12)')
     1                 IDSET, ISTP0, ISTP1, NTSPD, NYRDAY
      WRITE(IUNIT,'(A)')
     1                'NSAMPL, NDSKIP, NTFREQ, NSFREQ, NTSAMP, NSSAMP'
      WRITE(IUNIT,'(1X,6I12)')
     1                 NSAMPL, NDSKIP, NTFREQ, NSFREQ, NTSAMP, NSSAMP
      WRITE(IUNIT,'(A)') '0, 0, 0, 0, 0, 0'
      WRITE(IUNIT,'(1X,6I12)') 0, 0, 0, 0, 0, 0
      WRITE(IUNIT,'(A)') 'OCNCP, OCNRHO, TXCOEF, SXCOEF, AOCN'
      WRITE(IUNIT,FMT) OCNCP, OCNRHO, TXCOEF, SXCOEF, AOCN
      WRITE(IUNIT,'(A)') 'OCNT0, OCNS0, OCNTXF'
      WRITE(IUNIT,FMT) OCNT0, OCNS0, OCNTXF
      WRITE(IUNIT,'(A)') 'OCNDEP, OCNLEN, DXOCN, DYOCN, DZOCN'
      WRITE(IUNIT,FMT) OCNDEP, OCNLEN, DXOCN, DYOCN, DZOCN
      WRITE(IUNIT,'(A)') 'DTSTEP, TAUINT, TAU0, TAU1'
      WRITE(IUNIT,FMT) DTSTEP, TAUINT, TAU0, TAU1
      WRITE(IUNIT,'(A)') 'YRPHA, OCNWID, 0.0, 0.0, 0.0'
      WRITE(IUNIT,FMT) YRPHA, OCNWID, 0.0, 0.0, 0.0
C
      WRITE(IUNIT,'(A)') 'ODIFFY(ITR)'
      WRITE(IUNIT,FMT) (ODIFFY(ITR),ITR=JOCV,NOCNQ)
      WRITE(IUNIT,'(A)') 'ODIFFZ(ITR)'
      WRITE(IUNIT,FMT) (ODIFFZ(ITR),ITR=JOCV,NOCNQ)
C
      WRITE(IUNIT,'(A)') 'LONOCN(J), WTXOCN(J)'
      DO 10 J=0,NXOCN
   10   WRITE(IUNIT,FMT)  LONOCN(J), WTXOCN(J)
      WRITE(IUNIT,'(A)') 'LATOCN(K), PHIOCN(K), WTYOCN(K)'
      DO 20 K=0,NYOCN
   20   WRITE(IUNIT,FMT)  LATOCN(K), PHIOCN(K), WTYOCN(K)
      WRITE(IUNIT,'(A)') 'ZOCN(L), WTZOCN(L)'
      DO 30 L=0,NZOCN
   30   WRITE(IUNIT,FMT)  ZOCN(L), WTZOCN(L)
C
C     Descriptive strings for special time-series
      WRITE(IUNIT,'(A)') 'T-s(45N)'
      WRITE(IUNIT,'(A)') 'S-s(45N)'
C     Zonal-mean velocity components and variance
      WRITE(IUNIT,'(A)') 'UZON(K,L)'
      WRITE(IUNIT,FMT) ((UZON(K,L),K=0,NYOCN),L=0,NZOCN)
      WRITE(IUNIT,'(A)') 'UZVAR(K,L)'
      WRITE(IUNIT,FMT) ((UZVAR(K,L),K=0,NYOCN),L=0,NZOCN)
      WRITE(IUNIT,'(A)') 'VZON(K,L)'
      WRITE(IUNIT,FMT) ((VZON(K,L),K=0,NYOCN),L=0,NZOCN)
      WRITE(IUNIT,'(A)') 'VZVAR(K,L)'
      WRITE(IUNIT,FMT) ((VZVAR(K,L),K=0,NYOCN),L=0,NZOCN)
      WRITE(IUNIT,'(A)') 'WZON(K,L)'
      WRITE(IUNIT,FMT) ((WZON(K,L),K=0,NYOCN),L=0,NZOCN)
      WRITE(IUNIT,'(A)') 'WZVAR(K,L)'
      WRITE(IUNIT,FMT) ((WZVAR(K,L),K=0,NYOCN),L=0,NZOCN)
C     MMC streamfunction
      WRITE(IUNIT,'(A)') 'PSIZON(K,L)'
      WRITE(IUNIT,FMT) ((PSIZON(K,L),K=0,NYOCN),L=0,NZOCN)
C     Zonal-mean tracer fields and variance
      WRITE(IUNIT,'(A)') 'TRCZON(K,L,ITR)'
      WRITE(IUNIT,FMT)
     1     (((TRCZON(K,L,ITR),K=0,NYOCN),L=0,NZOCN),ITR=JOCT,NOCNQ)
      WRITE(IUNIT,'(A)') 'TRZVAR(K,L,ITR)'
      WRITE(IUNIT,FMT)
     1     (((TRZVAR(K,L,ITR),K=0,NYOCN),L=0,NZOCN),ITR=JOCT,NOCNQ)
C     MMC streamfunction tendency
      WRITE(IUNIT,'(A)') 'PSITEN(K,L)'
      WRITE(IUNIT,FMT) ((PSITEN(K,L),K=0,NYOCN),L=0,NZOCN)
C     Zonal-mean tracer tendencies
      WRITE(IUNIT,'(A)') 'TRTEND(K,L,ITR)'
      WRITE(IUNIT,FMT)
     1     (((TRTEND(K,L,ITR),K=0,NYOCN),L=0,NZOCN),ITR=JOCT,NOCNQ)
C     Zonal-mean tracer flux
      WRITE(IUNIT,'(A)') 'TRFLUX(K,L,I,ITR)'
      WRITE(IUNIT,FMT)
     1     ((((TRFLUX(K,L,I,ITR),K=0,NYOCN),L=0,NZOCN),I=1,2),
     2                           ITR=JOCT,NOCNQ)
      WRITE(IUNIT,'(A)') 'TRFAVG(K,L,I,ITR)'
      WRITE(IUNIT,FMT)
     1     ((((TRFAVG(K,L,I,ITR),K=0,NYOCN),L=0,NZOCN),I=1,2),
     2                           ITR=JOCT,NOCNQ)
C
C     Vertically-averaged total tracer flux
      WRITE(IUNIT,'(A)') 'MERFLX(K,ITR)'
      WRITE(IUNIT,FMT)
     1     ((MERFLX(K,ITR),K=0,NYOCN),ITR=JOCT,NOCNQ)
C     Vertically-averaged advective tracer flux
      WRITE(IUNIT,'(A)') 'ADVFLX(K,ITR)'
      WRITE(IUNIT,FMT)
     1     ((ADVFLX(K,ITR),K=0,NYOCN),ITR=JOCT,NOCNQ)
C
C     Surface velocity field
      WRITE(IUNIT,'(A)') 'VELSRF(J,K,I)'
      WRITE(IUNIT,FMT) (((VELSRF(J,K,I),J=0,NXOCN),K=0,NYOCN),I=JX,JY)
C     Surface tracer fields
      WRITE(IUNIT,'(A)') 'TRSURF(J,K,ITR)'
      WRITE(IUNIT,FMT)
     1     (((TRSURF(J,K,ITR),J=0,NXOCN),K=0,NYOCN),ITR=JOCT,NOCNQ)
C     Surface kinetic energy
      WRITE(IUNIT,'(A)') 'KESAVG(J,K)'
      WRITE(IUNIT,FMT) ((KESAVG(J,K),J=0,NXOCN),K=0,NYOCN)
C     Surface tracer variance
      WRITE(IUNIT,'(A)') 'TRSVAR(J,K,ITR)'
      WRITE(IUNIT,FMT)
     1     (((TRSVAR(J,K,ITR),J=0,NXOCN),K=0,NYOCN),ITR=JOCT,NOCNQ)
C     Surface wind stress
      WRITE(IUNIT,'(A)') 'TAUAVG(J,K,I)'
      WRITE(IUNIT,FMT) (((TAUAVG(J,K,I),J=0,NXOCN),K=0,NYOCN),I=JX,JY)
C     Surface tracer input flux
      WRITE(IUNIT,'(A)') 'TRCIAV(J,K,ITR)'
      WRITE(IUNIT,FMT)
     1     (((TRCIAV(J,K,ITR),J=0,NXOCN),K=0,NYOCN),ITR=JOCT,NOCNQ)
C     Special field 1
      WRITE(IUNIT,'(A)') 'SPFLD1(J,K)'
      WRITE(IUNIT,FMT) ((SPFLD1(J,K),J=0,NXOCN),K=0,NYOCN)
C     Special field 2
      WRITE(IUNIT,'(A)') 'SPFLD2(J,K)'
      WRITE(IUNIT,FMT) ((SPFLD2(J,K),J=0,NXOCN),K=0,NYOCN)
      RETURN
      END
