#ifndef MCONS
#   include "mcons.h"
#endif
C Prognos version 4.2 created 19 May 1993
C         A MECHANISTIC SPECTRAL PRIMITIVE EQUATION MODEL,
C                   USING PRESSURE COORDINATES
C
C Author: R. Saravanan, D.A.M.T.P., University of Cambridge,
C         Silver Street, Cambridge CB3 9EW, U.K.
C         E-mail: svn@atm.amtp.cam.ac.uk
C (Please send bug-reports/comments to above address)
C
C This software is distributed free for research purposes. It comes with
C absolutely no warranties whatsoever. It may be freely copied, and may
C also be modified, provided the modified software continues to be
C freely available under the same terms. This software may not be used
C for any commercial purpose.
C
C ---------------------------------------------------------------------
C Major modifications since version 3.5
C
C 1. The common variable HDP has been eliminated. The restriction that
C    the `full' pressure levels should lie exactly half-way between the
C    `half' pressure levels has been removed. Subroutine VERINI now
C    takes two array arguments PFULL and PHALF, giving the location of
C    the `full' and `half' model levels, intead of taking a single
C    array argument PTHICK giving the pressure level thicknesses.
C
C Major modifications since version 2.2
C
C 1. A major bug in subroutine DDTFRC, where the forcing of zonal-mean
C    potential temperature was inadvertently applied on all zonal
C    wavenumbers, has been corrected.
C 2. Subroutines DDTPGQ and DDTIMP have been modified to allow the
C    time-evolution of an arbitrary number of tracers to be followed.
C 3. The scalar parameter TSTEEP to subroutine VERINI has been replaced
C    by TZREF, which is an array of dimension NLEV. TZREF contains the
C    reference values of potential temperature, which are required for
C    "linearization" in the semi-implicit time-stepping scheme.
C 4. The parameter SFRIC to subroutine VERINI has been eliminated, and
C    the parameter VERVIS is now an array of dimension NLEV. This allows
C    a vertically varying kinematic viscosity profile to be specified.
C    The last element of array VERVIS is now effectively a surface
C    drag coefficient.
C 5. A new parameter IPRNDL (inverse Prandtl number) has been introduced
C    for subroutine VERINI, to allow vertical diffusion of temperature.
C 6. Subroutine DDTFRC now computes the forcing/damping tendencies
C    without adding them to the total tendencies (to facilitate split-
C    time stepping schemes). It also accepts only one set of prognostic
C    variables PGQSP, rather than PGQSP0 and PGQSP1, for calculating
C    forcing/damping terms.
C
C ------------- Description of major subroutines -----------------------
C
C PLINI  -- Initializes planetary parameters
C VERINI -- Initializes details of vertical truncation
C DDTINI -- Initializes the semi-implicit time-stepping scheme
C DDTPGQ -- Computes the "adiabatic" time-tendencies of prognostic
C           quantities
C DDTFRC -- Computes mechanistic forcing/damping time-tendencies
C DDTIMP -- Makes implicit corrections to explicit time-tendencies
C ROBFIL -- Applies the Robert time-filter on prognostic quantities
C
C ------------- Description of C preprocessor parameters ---------------
C The following C preprocessor parameters are used by this module
C They may be redefined by the user.
C
C L1MAX    -- maximum no. of pressure-levels (>=2)
C             (Default value is 20)
C NTRACE   -- number of "passive" tracers (>= 0)
C             (Default value is 0)
C FIXTRUNC -- This parameter, if defined, fixes the vertical truncation
C             to be equal to L1MAX. This may lead to more efficient
C             execution in some cases.
C(Note: This parameter also affects the spectral transform module)
C
C DEBUG    -- This parameter, if defined, produces debugging diagnostics
C
C The following are derived parameters:
C
C NPGQ     -- number of prognostic quantities per level (3+NTRACE)
C
C The following are "enumeration literals":
C
C Prognostic quantities
C JVOR   -- denotes relative vorticity
C JDIV   -- denotes horizontal divergence
C JPOT   -- denotes potential temperature
C JTR1   -- denotes tracer #1
C
C Vector components
C JX     -- denotes zonal direction
C JY     -- denotes meridional direction
C
C ------------- Description of common module mgrid.h -------------------
C
C     Planetary parameters
C     REAL OMEGA0, F0, FSP01, RGAS, CP, KAPPA, G0
C     COMMON/PLPARM/ OMEGA0, F0, FSP01, RGAS, CP, KAPPA, G0
C
C OMEGA0 -- angular velocity of planetary rotation (in 1/sec)
C F0     -- coriolis parameter (= 2 * OMEGA0)
C FSP01  -- F0 / SQRT(3.0)
C          (F0 expressed as coefficient of spherical harmonic Y[0,1])
C RGAS   -- specific gas constant (in J/(kg*K))
C CP     -- specific heat at constant pressure (in J/(kg*K))
C KAPPA  -- RGAS / CP (an adiabatic exponent)
C G0     -- gravitational acceleration (in m/s**2)
C
#ifndef FIXTRUNC
C     INTEGER L1
C     COMMON/VGRID1/ L1
#endif
C     REAL PSURF, DP, PLV, PHLV, PKLV, PKCHLV
C     REAL TSTDLV, ZSTDLV, TREFLV
C     COMMON/VGRID2/ PSURF, DP(L1MAX), PLV(L1MAX),
C    1               PHLV(0:L1MAX), PKLV(L1MAX), PKCHLV(L1MAX-1),
C    2               TSTDLV(L1MAX), ZSTDLV(L1MAX), TREFLV(L1MAX)
C
C     REAL QHDAMP, UD8FAC, TD8FAC
C     REAL DCAP2D, D2DCAP, D2W, D2TT, T2GPCP, NNT2DT, IMPCOR
C     COMMON/VGRID3/ QHDAMP, UD8FAC(0:N1MAX), TD8FAC(0:N1MAX),
C    1  DCAP2D(L1MAX,L1MAX-1), D2DCAP(L1MAX-1,L1MAX),
C    2  D2W(L1MAX-1,L1MAX), D2TT(L1MAX,L1MAX),
C    3  T2GPCP(L1MAX-1,L1MAX), NNT2DT(L1MAX,L1MAX),
C    4  IMPCOR(L1MAX,L1MAX,0:N1MAX)
C
C L1       -- number of pressure-levels ( >= 2)
C PSURF    -- surface pressure (in N/m**2)
C DP(L)    -- pressure-thickness of L-th level
C PLV(L)   -- pressure value of the L-th level
C PHLV(L)  -- pressure values at half-levels
C             (PHLV(0)= 0.0, PHLV(L1)= PSURF)
C PKLV(L)  -- (PLV(L)/PSURF)**KAPPA at L-th pressure level
C PKCHLV(L)-- vertical differential of PKLV (PK-cap)
C             {i.e. (1/2)*(PKLV(L+1) - PKLV(L)) }
C TSTDLV(L)-- standard value of Theta at L-th level
C ZSTDLV(L)-- standard height above surface (in metres) of L-th level
C             (computed using the standard value of Theta)
C TREFLV(L)-- reference value of Theta at L-th level
C             (used to linearize Theta for semi-implicit stepping)
C QHDAMP   -- Del**8 diffusion coefficient (in units of A0**8/sec)
C UD8FAC(N)-- Del**8 damping factor for momentum
C TD8FAC(N)-- Del**8 damping factor for pot. temperature
C DCAP2D   -- DCAP to D conversion matrix
C D2DCAP   -- D to DCAP conversion matrix
C D2W      -- D to W conversion matrix
C D2TT     -- D to Theta tendency conversion matrix
C T2GPCP   -- Theta to GeoPotential-CAP conversion matrix
C NNT2DT   -- Theta to D-tendency (without -Del**2 factor) conversion
C              matrix (= DCAP2D . T2GPCP)
C IMPCOR   -- implicit correction matrix
C
C ------------- Description of common module tmarch.h ------------------
C
C     REAL IMPFAC, DT, DTFAC, UD8COR, TD8COR
C     COMMON/TMAR1/ IMPFAC, DT, DTFAC, UD8COR(0:N1MAX), TD8COR(0:N1MAX)
C     REAL INVPNO, URLXLV, VVISC, TRLXLV, TMNLV
C     COMMON/CDAMP/ INVPNO, URLXLV(L1MAX), VVISC(L1MAX),
C    1              TRLXLV(L1MAX), TMNLV(0:N1MAX,L1MAX)
C
C IMPFAC   -- implicitness factor (0.0=explicit ... 1.0=fully implicit)
C DT       -- time-step (in seconds)
C DTFAC    -- IMPFAC * 2.0 * DT
C UD8COR(N)-- Del**8 implicit correction factor for vorticity/divergence
C TD8COR(N)-- Del**8 implicit correction factor for Theta
C INVPNO   -- inverse Prandtl number (conductivity/viscosity)
C URLXLV(L)-- vertical profile of Rayleigh damping coefficient
C VVISC(L) -- vertical viscosity coefficient at level L+1/2
C             (in units of (pressure)**2/time)
C             (VVISC(L1) is effectively a surface drag coefficient)
C TRLXLV(L)-- vertical profile of Newtonian damping coefficient
C TMNLV(N,L)--spectral Legendre-pressure coeffs. of zonally symmetric
C             mean Theta distribution
C
C ------------------- End of comments ---------------------------------
C
      SUBROUTINE PLINI(OMEGA, R, CPRESS, GRAVIT)
C PLINI initializes the common block /PLPARM/ which contains the
C planetary parameters.
C Input:  REAL OMEGA -- angular velocity of planetary rotation (in 1/s)
C         REAL R -- gas constant {in J/(kg*K)}
C         REAL CPRESS -- specific heat at const. pressure {in J/(kg*K)}
C         REAL GRAVIT -- gravitational acceleration (in m/s**2)
C Output: Common block /PLPARM/ is initialized
C
C
#include "mgrid.h"
      REAL OMEGA, R, CPRESS, GRAVIT
C
      OMEGA0= OMEGA
      F0= 2.0 * OMEGA0
      FSP01= F0 / SQRT(3.0)
      RGAS= R
      CP= CPRESS
      KAPPA= RGAS / CP
      G0= GRAVIT
      RETURN
      END
 
      SUBROUTINE VERINI( NLEV, PFULL, PHALF, TZSTD, TZREF, DEL8DF,
     1                   TRADEQ, TDAMP, UDAMP, IPRNDL, VERVIS)
C
C VERINI initializes the vertical truncation for a spectral pressure
C coordinate model (using the Lorenz (1960) finite-differencing, with
C vertically averaged divergence set to zero).
C
C Note: Prior to calling VERINI, the horizontal truncation should have
C       been initialized by a call to routine SPINI in module sptran.F.
C       Also, the planetary parameters should have been initialized by
C       a call to routine PLINI in this module.
C
C The prognostic quantities are vorticity, divergence, and
C potential temperature (Theta) in each level.
C
C NLEV is the number of pressure levels in the vertical truncation.
C    (NLEV >= 2)
C PFULL(NLEV) are the pressure values at model levels (in SI units).
C PHALF(NLEV) are the pressure values at half-levels (in SI units).
C    (Note: The upper boundary boundary is always assumed to be at zero
C     pressure. This means that PHALF(1) is the thickness of the
C     uppermost level, and PHALF(NLEV) is the `surface' pressure.
C     Usually, PFULL(L)= 0.5*(PHALF(L) + PHALF(L-1)), i.e. centred-
C     differencing in pressure coordinates.)
C TZSTD(NLEV) is the pressure profile of the mean Theta distribution.
C    (Note: TZSTD is used to calculate standard height-levels
C     corresponding to the pressure levels, for diagnostic purposes)
C TZREF(NLEV) is the pressure profile of the reference Theta
C    distribution, to be used to linearize Theta for semi-implicit
C    time-stepping. (TZREF would typically be steeper than TZSTD.)
C DEL8DF is the del-8 (quad-harmonic) diffusion coefficient
C    (in units of A0**8/sec, where A0 is the planetary radius)
C TRADEQ(K2MAX,NLEV) is the latitude-pressure field of the "radiative
C    equilibrium" Theta distribution.
C TDAMP(NLEV) is the pressure dependent radiative damping coefficient
C    (in units of 1/sec) that determines relaxation of Theta back to
C    TRADEQ.
C UDAMP(NLEV) is the pressure dependent Rayleigh friction coefficient
C    (in units of  1/sec) that damps velocity.
C IPRNDL is the inverse Prandtl number (non-dimensional).
C    (i.e. heat diffusivity/viscosity).
C VERVIS(NLEV) is the pressure-dependent vertical viscosity coefficient
C    (in units of m**2/s). The last element of this array is effectively
C    a surface drag coefficient.
C
C Lorenz, E.N. 1960: Energy and numerical weather prediction,
C    TELLUS,vol.12,pp.364-373.
C
#include "mgrid.h"
#include "tmarch.h"
      INTEGER NLEV
      REAL PFULL(NLEV), PHALF(NLEV), TZSTD(NLEV), TZREF(NLEV)
      REAL DEL8DF, TRADEQ(K2MAX, NLEV), TDAMP(NLEV), UDAMP(NLEV)
      REAL IPRNDL, VERVIS(NLEV)
      INTEGER J, K, L, L0, N
      REAL W2TT(L1MAX,L1MAX-1), FPH(K1MAX,K2MAX)
      COMPLEX FSP(0:M1MAX,0:N1MAX)
#ifdef DEBUG
      REAL W2D(L1MAX,L1MAX-1), WK(L1MAX,L1MAX)
#endif
 
#ifdef FIXTRUNC
      IF (NLEV.NE.L1)
     1  CALL SPERR( 'VERINI', 'Incompatible vertical truncation')
#else
 
      IF ( (NLEV.LT.2) .OR. (NLEV.GT.L1MAX) )
     1  CALL SPERR( 'VERINI', 'Too few/too many pressure levels')
      L1= NLEV
#endif
 
C     Initialize pressure related parameters
      PHLV(0)= 0.0
      DO 10 L=1,L1
        PLV(L)= PFULL(L)
        PHLV(L)= PHALF(L)
        DP(L)= PHLV(L) - PHLV(L-1)
   10   CONTINUE
      PSURF= PHLV(L1)
C     Coordinate PK, and standard/reference vertical Theta profiles
      DO 20 L=1,L1
        PKLV(L)= (PLV(L)/PSURF)**KAPPA
        TSTDLV(L)= TZSTD(L)
        TREFLV(L)= TZREF(L)
   20   CONTINUE
C     Compute PK-cap
      DO 30 L=1,L1-1
   30   PKCHLV(L)= 0.5 * (PKLV(L+1) - PKLV(L))
C     Compute standard height values, using the hydrostatic relation.
      ZSTDLV(L1)= CP*(1.0-PKLV(L1))*TSTDLV(L1) / G0
      DO 40 L=L1-1,1,-1
        ZSTDLV(L)= ZSTDLV(L+1) +
     1             CP*PKCHLV(L)*(TSTDLV(L+1)+TSTDLV(L)) / G0
   40   CONTINUE
#ifdef DEBUG
      CALL MATOUT(PLV, 1, 1, L1, 'PLV')
      CALL MATOUT(PHLV, 1, 1, L1+1, 'PHLV')
      CALL MATOUT(PKLV, 1, 1, L1, 'PKLV')
      CALL MATOUT(ZSTDLV, 1, 1, L1, 'ZSTDLV')
      CALL MATOUT(TREFLV, 1, 1, L1, 'TREFLV')
#endif
C     DCAP to D conversion matrix
      DO 50 L=1,L1
        DO 50 L0=1,L1-1
          IF (L0.LT.L) THEN
            DCAP2D(L,L0)= 2.0 *  PHLV(L0)/PHLV(L1)
          ELSE
            DCAP2D(L,L0)= 2.0 * (PHLV(L0)/PHLV(L1) - 1.0)
          ENDIF
   50     CONTINUE
C     D to DCAP conversion matrix ("vertical derivative")
      DO 60 L=1,L1-1
        DO 60 L0=1,L1
          IF (L0.EQ.(L+1)) THEN
            D2DCAP(L,L0)=  0.5
          ELSE IF (L0.EQ.L) THEN
            D2DCAP(L,L0)= -0.5
          ELSE
            D2DCAP(L,L0)=  0.0
          ENDIF
   60     CONTINUE
#ifdef DEBUG
      CALL MATOUT(D2DCAP, L1MAX-1, L1-1, L1,   'D2DCAP')
      CALL MATOUT(DCAP2D, L1MAX,   L1,   L1-1, 'DCAP2D')
      CALL MATMUL(WK(1,1), L1MAX, D2DCAP, L1MAX-1, DCAP2D, L1MAX,
     1            L1-1, L1, L1-1 )
      CALL MATOUT(WK, L1MAX, L1-1, L1-1, 'D2DCAP*DCAP2D')
#endif
C     D to W conversion matrix
      DO 70 L=1,L1-1
        DO 70 L0=1,L1
          IF (L0.LE.L) THEN
            D2W(L,L0)= -DP(L0)
          ELSE
            D2W(L,L0)= 0.0
          ENDIF
   70     CONTINUE
#ifdef DEBUG
C     W to D conversion matrix
      DO 75 L=1,L1
        DO 75 L0=1,L1-1
          IF (L0.EQ.(L-1)) THEN
            W2D(L,L0)=  1.0/DP(L)
          ELSE IF (L0.EQ.L) THEN
            W2D(L,L0)= -1.0/DP(L)
          ELSE
            W2D(L,L0)=  0.0
          ENDIF
   75     CONTINUE
      CALL MATOUT(D2W, L1MAX-1, L1-1, L1,   'D2W')
      CALL MATOUT(W2D, L1MAX,   L1,   L1-1, 'W2D')
      CALL MATMUL(WK(1,1), L1MAX, D2W, L1MAX-1, W2D, L1MAX,
     1            L1-1, L1, L1-1 )
      CALL MATOUT(WK, L1MAX, L1-1, L1-1, 'D2W*W2D')
#endif
C     W to Theta tendency conversion matrix (Static stability)
      DO 80 L=1,L1
        DO 80 L0=1,L1-1
          IF (L0.EQ.(L-1)) THEN
            W2TT(L,L0)= -0.5 * (TREFLV(L)-TREFLV(L-1)) / DP(L)
          ELSE IF (L0.EQ.L) THEN
            W2TT(L,L0)= -0.5 * (TREFLV(L+1)-TREFLV(L)) / DP(L)
          ELSE
            W2TT(L,L0)= 0.0
          ENDIF
   80     CONTINUE
C     D to Theta-tendency conversion matrix
C     (Composite of W2TT and D2W)
      CALL MATMUL(D2TT, L1MAX, W2TT, L1MAX, D2W, L1MAX-1,
     1            L1, L1-1, L1)
C     Theta to GeoPotential-CAP conversion matrix (Hydrostatic relation)
      DO 90 L=1,L1-1
        DO 90 L0=1,L1
          IF ( (L0.EQ.L) .OR. (L0.EQ.(L+1)) ) THEN
            T2GPCP(L,L0)= -0.5 * CP * PKCHLV(L)
          ELSE
            T2GPCP(L,L0)=  0.0
          ENDIF
   90     CONTINUE
C     Theta to D-tendency (without -Del**2 factor) conversion matrix
C     (Composite of DCAP2D and T2GPCP)
      CALL MATMUL(NNT2DT, L1MAX, DCAP2D, L1MAX, T2GPCP, L1MAX-1,
     1            L1, L1-1, L1)
#ifdef DEBUG
      CALL MATOUT(W2TT,    L1MAX,   L1,   L1-1, 'W2TT'  )
      CALL MATOUT(D2TT,    L1MAX,   L1,   L1,   'D2TT'  )
      CALL MATOUT(T2GPCP,  L1MAX-1, L1-1, L1,   'T2GPCP')
      CALL MATOUT(NNT2DT,  L1MAX,   L1,   L1,   'NNT2DT')
#endif
C     Del**8 sub-grid-scale diffusion
      QHDAMP= DEL8DF
      DO 100 N=0,N1
C       Scale-dependent del**8 damping factor
C       (Note: QHDAMP is in units such that A0 = 1)
        TD8FAC(N) =  QHDAMP * FLOAT( N*(N+1)   )**4
        UD8FAC(N) =  QHDAMP * FLOAT( N*(N+1)-2 )**4
  100   CONTINUE
#ifdef DEBUG
      CALL MATOUT(TD8FAC(0), 1, 1, N1+1, 'TD8FAC')
      CALL MATOUT(UD8FAC(0), 1, 1, N1+1, 'UD8FAC')
#endif
C     Mean thermal state, and Rayleigh/Newtonian damping coefficients
      DO 130 L=1,L1
        URLXLV(L)= UDAMP(L)
        TRLXLV(L)= TDAMP(L)
C       Compute spectral coefficients of zonally symmetric mean Theta
        DO 110 K=1,K2
          DO 110 J=1,K1
  110       FPH(J,K)= TRADEQ(K,L)
        CALL ZEROSP(FSP, N1)
        CALL SPECTR(FSP, FPH)
        DO 120 N=0,N1
  120     TMNLV(N,L)= REAL( FSP(0,N) )
  130   CONTINUE
C     Inverse Prandtl number
      INVPNO= IPRNDL
C     Convert viscosity coefficient from z-coordinate to p-coordinate,
C     using the standard height values
      DO 140 L=1,L1-1
        VVISC(L)= VERVIS(L) * ( (PLV(L+1)    - PLV(L)   ) /
     1                          (ZSTDLV(L+1) - ZSTDLV(L))   )**2
  140   CONTINUE
C
      VVISC(L1)= VERVIS(L1) * ((PHLV(L1)-PLV(L1))/ZSTDLV(L1))**2
#ifdef DEBUG
      CALL MATOUT(VVISC, 1, 1, L1, 'VVISC')
#endif
      RETURN
      END
 
      SUBROUTINE MATMUL(A, NA0, B, NB0, C, NC0, NROWB, NCOLB, NCOLC)
C MATMUL multiplies two matrices together to produce a third matrix.
C  A = B . C
C Input:  INTEGER NA0, NB0, NC0 -- inner dimension of matrices A, B, C
C         INTEGER NROWB, NCOLB -- no. of rows and columns of matrix B
C         INTEGER NCOLC -- no. of columns of matrix C
C         REAL B(NB0,NCOLB), C(NC0,NCOLC) -- matrices to be multiplied
C Output: REAL A(NA0,NCOLC) -- matrix A = B . C (with NROWB rows)
C
      INTEGER NA0, NB0, NC0, NROWB, NCOLB, NCOLC
      REAL A(NA0,NCOLC), B(NB0,NCOLB), C(NC0,NCOLC)
      INTEGER I, J, K
      REAL TEMP
      DO 20 I=1,NROWB
        DO 20 K=1,NCOLC
          TEMP= 0.0
          DO 10 J=1,NCOLB
   10       TEMP= TEMP + B(I,J) * C(J,K)
          A(I,K)= TEMP
   20     CONTINUE
      RETURN
      END
 
      SUBROUTINE MATINV(AINV, NAINV0, A, NA0, NA)
C MATINV finds the inverse of a square matrix A.
C Input:  INTEGER NAINV0, NA0 -- inner dimension of matrices AINV, A
C         INTEGER NA -- no. of rows/columns in A
C (Note: NA must be <= L1MAX)
C         REAL A(NA0,NA) -- matrix A
C Output: REAL AINV(NAINV0,NA) -- inverse of A
C
      INTEGER NAINV0, NA0, NA
      REAL AINV(NAINV0,NA), A(NA0,NA)
#include "mgrid.h"
      REAL8 A2(L1MAX,L1MAX), B(L1MAX,L1MAX), C(L1MAX,L1MAX)
      REAL8 WKSPCE(L1MAX), AA(L1MAX,L1MAX), BB(L1MAX,L1MAX)
      INTEGER I, J, IFAIL
#ifdef DEBUG
      REAL WK(L1MAX,L1MAX)
#endif
C
C     IF (NA.GT.L1MAX) CALL SPERR( 'MATINV', 'Matrix too large')
C     Initialize temporary identity matrix
      DO 10 I=1,NA
        DO 10 J=1,NA
C         Copy matrix to correct precision
          A2(I,J)= A(I,J)
C         Create Identity matrix for RHS
          IF (I.EQ.J) THEN
            B(I,J)= 1.0
          ELSE
            B(I,J)= 0.0
          ENDIF
   10     CONTINUE
C     Call Nag routine
      IFAIL= 0
      CALL F04AEF(A2, L1MAX, B, L1MAX, NA, NA, C, L1MAX,
     1            WKSPCE, AA, L1MAX, BB, L1MAX, IFAIL)
      IF (IFAIL.NE.0)
     1  CALL SPERR( 'MATINV', 'Unable to invert matrix')
C     Copy inverse back to original precision
      DO 20 I=1,NA
        DO 20 J=1,NA
          AINV(I,J)= C(I,J)
   20     CONTINUE
#ifdef DEBUG
C     Check that the inverse is actually the inverse
      CALL MATMUL(WK, L1MAX, A, NA0, AINV, NAINV0, NA, NA, NA)
      CALL MATOUT(WK, L1MAX, NA, NA, 'A*AINV')
#endif
      RETURN
      END
 
#ifdef DEBUG
      SUBROUTINE MATOUT(A, NA0, NROWA, NCOLA, STR)
C MATOUT prints out a matrix A(NA0,NCOLA), with NROWA rows on
C standard output. STR is the name of the matrix.
C (This routine is used for debugging purposes)
      INTEGER NA0, NROWA, NCOLA
      REAL A(NA0,NCOLA)
      CHARACTER*(*) STR
      INTEGER I, J
      WRITE(*,'(1X,A,A,A,I3,A,I3,A)') 'Matrix ', STR,
     1                 '(', NROWA, 'x', NCOLA, ')'
      DO 10 I=1,NROWA
        WRITE(*,'(1X,1P,5E15.6)') (A(I,J), J=1,NCOLA)
        IF (NCOLA.GT.5) WRITE(*,*)
   10   CONTINUE
      WRITE(*,*)
      RETURN
      END
#endif
 
      SUBROUTINE DDTINI( DT1, IMPLCT )
C DDTINI sets up coefficients for time integration using routine
C DDTPGQ. DT1 is the time-step (in seconds). IMPLCT is the fractional
C implicitness (between 0.0 and 1.0) for time-stepping. The coefficients
C are stored in common module tmarch.h.
C **Note: Normally DDTPGQ uses a partially-implicit leap-frog scheme
C         for time-stepping (using PGQSP0, PGQSP1). But for start-up
C         purposes, you can set DT1 = 0.5 DT, and set PGQSP0 = PGQSP1
C         (i.e. half the actual time step) to get the effect of a
C         partially-implicit forward scheme.
#include "mgrid.h"
#include "tmarch.h"
      REAL DT1, IMPLCT
      REAL RTEMP
      REAL TMPMAT(L1MAX,L1MAX), TMPMA2(L1MAX,L1MAX), TMPMA3(L1MAX,L1MAX)
      INTEGER L, L0, N
#ifdef DEBUG
      INTEGER IFAIL
      REAL8 DET, WK(L1MAX,L1MAX), WK2(L1MAX)
#endif
C     Store time-step
      DT= DT1
C     Store implicitness factor
      IMPFAC= IMPLCT
C     Fractional implicit time-step
      DTFAC= IMPFAC * 2.0 * DT
C     Compute effective ( (-(Del**2)*Dcap) -> (d**2)Dcap/dt**2) matrix
      CALL MATMUL( TMPMA2, L1MAX, T2GPCP, L1MAX-1, D2TT, L1MAX,
     1             L1-1, L1, L1)
      CALL MATMUL( TMPMAT, L1MAX, TMPMA2, L1MAX, DCAP2D, L1MAX,
     1             L1-1, L1, L1-1)
#ifdef DEBUG
      CALL MATOUT(TMPMAT,  L1MAX,   L1-1,   L1-1,   'TMPMAT')
#endif
C     Initialize useful wave-no dependent quantities
      DO 30 N=0,N1
C       Del**8 damping correction factor
        TD8COR(N) =  1.0 / (1.0 + DTFAC * TD8FAC(N))
        UD8COR(N) =  1.0 / (1.0 + DTFAC * UD8FAC(N))
C
C       Compute implicit correction matrix
        RTEMP= (A0QINV*N*(N+1)) * UD8COR(N) * TD8COR(N) * DTFAC**2
        DO 20 L=1,L1-1
          DO 10 L0=1,L1-1
            TMPMA2(L,L0)= -RTEMP * TMPMAT(L,L0)
   10       CONTINUE
C         Add identity matrix
          TMPMA2(L,L)= TMPMA2(L,L) + 1.0
   20     CONTINUE
C       Invert matrix
        CALL MATINV( TMPMA3, L1MAX, TMPMA2, L1MAX, L1-1 )
#ifdef DEBUG
        CALL MATOUT(TMPMA3,  L1MAX,   L1-1,   L1-1,   'TMPINV')
        DO 25 L=1,L1-1
          DO 25 L0=1,L1-1
   25       WK(L,L0)= TMPMA3(L,L0)
        IFAIL= 0
        CALL F03AAF( WK, L1MAX, L1-1, DET, WK2, IFAIL )
        WRITE(*,'(A,I3,A,1PD15.6,A,I2)') 'Determinant of IMPCOR(,,',
     1                  N, ')= ', DET, '   ;IFAIL= ', IFAIL
#endif
        CALL MATMUL( TMPMA2, L1MAX, TMPMA3, L1MAX, D2DCAP, L1MAX-1,
     1               L1-1, L1-1, L1)
        CALL MATMUL( IMPCOR(1,1,N), L1MAX, DCAP2D, L1MAX, TMPMA2, L1MAX,
     1               L1, L1-1, L1)
   30   CONTINUE
      RETURN
      END
 
      SUBROUTINE DDTPGQ( DPGQSP, PGQSP0, PGQSP1 )
C DDTPGQ calculates DPGQSP, the "adiabatic" time tendency of prognostic
C quantities in spectral space at some model time. (i.e. The only
C damping effect included in these calculations is the scale-selective
C Del**8 damping. Any other damping effects will need to be evaluated
C separately)
C Note: The coefficients required for time stepping must be initialized
C        by calling DDTINI before the first call to DDTPGQ.
C Input:  COMPLEX PGQSP0(0:M1MAX,0:N1MAX,NPGQ,L1MAX) -- spectral
C         representation of prognostic quantity at (current time) - DT1
C         COMPLEX PGQSP1(0:M1MAX,0:N1MAX,NPGQ,L1MAX) -- spectral
C         representation of prognostic quantity at current model time
C Output: COMPLEX DPGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX) -- spectral
C                   "adiabatic" time-tendency of prognostic quantities
C
#include "mgrid.h"
#include "tmarch.h"
      COMPLEX DPGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX)
      COMPLEX PGQSP0(0:M1MAX,0:N1MAX,NPGQ,L1MAX)
      COMPLEX PGQSP1(0:M1MAX,0:N1MAX,NPGQ,L1MAX)
      REAL U(K1MAX, K2MAX, JX:JY, 0:1), TP(K1MAX, K2MAX, 0:1)
      REAL VUFLX(K1MAX, K2MAX, JX:JY, 0:1), VTPFLX(K1MAX, K2MAX, 0:1)
      REAL W(K1MAX, K2MAX, 0:1)
      REAL ABSVOR(K1MAX, K2MAX), DIV(K1MAX, K2MAX)
      REAL FLXPH(K1MAX, K2MAX, JX:JY), FPH(K1MAX, K2MAX), TEMP
      COMPLEX FSP1(0:M1MAX,0:N1MAX), FSP2(0:M1MAX,0:N1MAX)
      INTEGER LMHALF, LPZERO, LPHALF, LPONE
C     (Mnemonic for L-(1/2), L+0, L+(1/2), L+1)
      INTEGER I, J, K, L, L0, M, N, IPGQ
#if (NTRACE > 0)
      INTEGER ITR
      REAL TRC(K1MAX, K2MAX, JTR1:NPGQ, 0:1)
      REAL VTRCFX(K1MAX, K2MAX, JTR1:NPGQ, 0:1)
#endif
C ----------------------------------------------------------------------
C     Initialize all spectral time-tendencies to zero
      DO 10 L=1,L1
        DO 10 IPGQ=1,NPGQ
          CALL ZEROSP( DPGQSP(0,0,IPGQ,L), N1)
   10     CONTINUE
C ----------------------------------------------------------------------
      LMHALF= 0
      LPZERO= 0
C **Note Values of prognostic quantities at current model time
C        (PGQSP1) are used in the following calculations. i.e. The
C        treatment is leap-frog in time.
      DO 500 L=0,L1
        LMHALF= MOD(LMHALF+1, 2)
        LPHALF= MOD(LMHALF+1, 2)
        LPZERO= MOD(LPZERO+1, 2)
        LPONE=  MOD(LPZERO+1, 2)
        IF (L.LT.L1) THEN
C ----------------------------------------------------------------------
C         Compute hor. velocity and "perturbation" Theta at level L+1
C
C         First compute streamfunction PSI and velocity potential CHI
          CALL IDELSQ( FSP1, PGQSP1(0,0,JVOR,L+1), N1)
          CALL IDELSQ( FSP2, PGQSP1(0,0,JDIV,L+1), N1)
C         Then compute horizontal velocity from PSI and CHI
          CALL HVELOC( U(1,1,JX,LPONE), U(1,1,JY,LPONE), FSP1, FSP2 )
C
          CALL SPCOPY( FSP1, PGQSP1(0,0,JPOT,L+1), N1 )
C         Subtract reference value of Theta
          FSP1(0,0)= FSP1(0,0) - TREFLV(L+1)
C         Compute "perturbation" Theta in physical space (at level L+1)
          CALL PHYSIC( TP(1,1,LPONE), FSP1 )
#if (NTRACE > 0)
C         Compute tracer concentrations in physical space (at level L+1)
          DO 110 ITR=JTR1,NPGQ
  110       CALL PHYSIC( TRC(1,1,ITR,LPONE), PGQSP1(0,0,ITR,L+1) )
#endif
C ----------------------------------------------------------------------
        ENDIF
C
        IF (L.GT.0) THEN
C ----------------------------------------------------------------------
C         Compute divergence at level L
          CALL PHYSIC( DIV, PGQSP1(0,0,JDIV,L) )
C ----------------------------------------------------------------------
        ENDIF
C
C       Compute omega and vertical fluxes at level L+1/2
        IF ( (L.EQ.0) .OR. (L.EQ.L1) ) THEN
C ----------------------------------------------------------------------
C         Omega is zero at the boundary half-levels 1/2 and L1+1/2
C         => All vertical fluxes are also zero
          DO 200 K=1,K2
            DO 200 J=1,K1
              W(J,K,LPHALF)= 0.0
              VTPFLX(J,K,LPHALF)= 0.0
  200         CONTINUE
          DO 210 I=JX,JY
            DO 210 K=1,K2
              DO 210 J=1,K1
  210           VUFLX(J,K,I,LPHALF)= 0.0
#if (NTRACE > 0)
          DO 220 ITR=JTR1,NPGQ
            DO 220 K=1,K2
              DO 220 J=1,K1
                VTRCFX(J,K,ITR,LPHALF)= 0.0
  220           CONTINUE
#endif
C ----------------------------------------------------------------------
        ELSE
C ----------------------------------------------------------------------
C         L+1/2 is not at the boundaries. Compute omega and vertical
C         fluxes at level L+1/2.
C
C         Compute "pressure velocity" omega at L+1/2, using DIV at L
          DO 230 K=1,K2
            DO 230 J=1,K1
              W(J,K,LPHALF)= W(J,K,LMHALF) - DP(L)*DIV(J,K)
  230         CONTINUE
C         Compute vertical fluxes at level L+1/2
          DO 240 I=JX,JY
            DO 240 K=1,K2
              DO 240 J=1,K1
                VUFLX(J,K,I,LPHALF)=    - W(J,K,LPHALF) *
     1                          0.5 * (U(J,K,I,LPZERO) + U(J,K,I,LPONE))
  240           CONTINUE
          DO 250 K=1,K2
            DO 250 J=1,K1
              VTPFLX(J,K,LPHALF)=       - W(J,K,LPHALF) *
     1                          0.5 * ( TP(J,K,LPZERO) +  TP(J,K,LPONE))
  250         CONTINUE
#if (NTRACE > 0)
          DO 260 ITR=JTR1,NPGQ
            DO 260 K=1,K2
              DO 260 J=1,K1
                VTRCFX(J,K,ITR,LPHALF)= - W(J,K,LPHALF) *
     1                    0.5 * (TRC(J,K,ITR,LPZERO)+TRC(J,K,ITR,LPONE))
  260           CONTINUE
#endif
C ----------------------------------------------------------------------
        ENDIF
C
        IF (L.GT.0) THEN
C ----------------------------------------------------------------------
C         Computing vorticity/divergence tendencies at level L
C
C         Computing absolute vorticity at level L
          CALL SPCOPY( FSP1, PGQSP1(0,0,JVOR,L), N1 )
C         Add planetary vorticity to relative vorticity
          FSP1(0,1)= FSP1(0,1) + CMPLX(FSP01, 0.)
          CALL PHYSIC( ABSVOR, FSP1 )
C         Computing horizontal "flux convergence" of momentum at level L
          DO 310 K=1,K2
            DO 310 J=1,K1
              FLXPH(J,K,JX)= + ABSVOR(J,K) * U(J,K,JY,LPZERO)
     1                          - DIV(J,K) * U(J,K,JX,LPZERO)
              FLXPH(J,K,JY)= - ABSVOR(J,K) * U(J,K,JX,LPZERO)
     1                          - DIV(J,K) * U(J,K,JY,LPZERO)
  310         CONTINUE
C         Add contribution from vertical flux divergence
          TEMP= 1.0 / DP(L)
          DO 320 I=JX,JY
            DO 320 K=1,K2
              DO 320 J=1,K1
                FLXPH(J,K,I)= FLXPH(J,K,I) +
     1               TEMP * (VUFLX(J,K,I,LPHALF)-VUFLX(J,K,I,LMHALF))
  320           CONTINUE
C         Computing vorticity/divergence tendencies from "flux conv."
          CALL CURLZ( DPGQSP(0,0,JVOR,L), FLXPH(1,1,JX), FLXPH(1,1,JY))
          CALL DIVERG(DPGQSP(0,0,JDIV,L), FLXPH(1,1,JX), FLXPH(1,1,JY))
C
C         Compute kinetic energy at level L
          DO 330 K=1,K2
            DO 330 J=1,K1
              FPH(J,K)= 0.5*(  U(J,K,JX,LPZERO)**2
     1                       + U(J,K,JY,LPZERO)**2 )
  330         CONTINUE
C         Convert to spectral space
          CALL ZEROSP( FSP1, N1 )
          CALL SPECTR( FSP1, FPH )
C         Compute the Laplacian, and add contribution to D tendency
          CALL DELSQ( FSP1, FSP1, N1 )
          DO 340 N=0,N1
            DO 340 M=0,MIN0(N,M1)
              DPGQSP(M,N,JDIV,L)= DPGQSP(M,N,JDIV,L) - FSP1(M,N)
  340         CONTINUE
C
C **Note: Contribution from geopotential still to be added to D tendency
C
C         Computing Theta tendency at level L
C
C         Compute horizontal "flux" of "perturbation" Theta
          DO 350 I=JX,JY
            DO 350 K=1,K2
              DO 350 J=1,K1
                FLXPH(J,K,I)= - TP(J,K,LPZERO) * U(J,K,I,LPZERO)
  350           CONTINUE
C         Compute divergence of horizontal "flux"
          CALL DIVERG(DPGQSP(0,0,JPOT,L), FLXPH(1,1,JX), FLXPH(1,1,JY))
C
C         Compute vertical "perturbation" Theta flux convergence
          TEMP= 1.0 / DP(L)
          DO 360 K=1,K2
            DO 360 J=1,K1
              FPH(J,K)= TEMP * (VTPFLX(J,K,LPHALF)-VTPFLX(J,K,LMHALF))
  360         CONTINUE
C         Convert to spectral space and add to Theta tendency
          CALL SPECTR( DPGQSP(0,0,JPOT,L), FPH )
C
C     All adiabatic tendency terms, excluding the geopotential in the
C     D tendency equation and the linearized heating term in the Theta
C     tendency equation, have now been computed for level L.
C
#if (NTRACE > 0)
C         Compute tracer time tendency at level L
C
          DO 430 ITR=JTR1,NPGQ
C           Calculate "flux" of tracer
            DO 410 I=JX,JY
              DO 410 K=1,K2
                DO 410 J=1,K1
                  FLXPH(J,K,I)= - TRC(J,K,ITR,LPZERO) * U(J,K,I,LPZERO)
  410             CONTINUE
C           Add horizontal convergence of tracer flux to the tendency
            CALL DIVERG(DPGQSP(0,0,ITR,L), FLXPH(1,1,JX), FLXPH(1,1,JY))
C           Compute vertical tracer flux convergence
            TEMP= 1.0 / DP(L)
            DO 420 K=1,K2
              DO 420 J=1,K1
                FPH(J,K)= TEMP *
     1                   (VTRCFX(J,K,ITR,LPHALF)-VTRCFX(J,K,ITR,LMHALF))
  420           CONTINUE
C           Convert to spectral space and add to Theta tendency
            CALL SPECTR( DPGQSP(0,0,ITR,L), FPH )
  430       CONTINUE
#endif
C ----------------------------------------------------------------------
        ENDIF
C
  500   CONTINUE
C
C     Now we compute the remaining terms of the tendency equation,
C     including the damping terms.
C **Note: Values of prognostic quantities at model time TIM - DT1
C        (PGQSP0) are used in the following calculations. i.e. The
C        treatment is explicit in time.
C
      DO 700 L=1,L1
        DO 700 N=0,N1
C ----------------------------------------------------------------------
C         Add geopotential contribution to D tendency
          TEMP= A0QINV*N*(N+1)
          DO 610 L0=1,L1
            DO 610 M=0,MIN0(N,M1)
              DPGQSP(M,N,JDIV,L)= DPGQSP(M,N,JDIV,L)
     1              + TEMP * NNT2DT(L,L0) * PGQSP0(M,N,JPOT,L0)
  610         CONTINUE
C
C         Add contribution from reference static stability to Theta
C         tendency
          DO 620 L0=1,L1
            DO 620 M=0,MIN0(N,M1)
              DPGQSP(M,N,JPOT,L)= DPGQSP(M,N,JPOT,L)
     1             + D2TT(L,L0) * PGQSP0(M,N,JDIV,L0)
  620         CONTINUE
C
C         Add Del**8 damping terms to VOR/DIV/POT tendencies
          DO 630 M=0,MIN0(N,M1)
            DPGQSP(M,N,JVOR,L)= DPGQSP(M,N,JVOR,L)
     1            - UD8FAC(N) * PGQSP0(M,N,JVOR,L)
            DPGQSP(M,N,JDIV,L)= DPGQSP(M,N,JDIV,L)
     1            - UD8FAC(N) * PGQSP0(M,N,JDIV,L)
            DPGQSP(M,N,JPOT,L)= DPGQSP(M,N,JPOT,L)
     1            - TD8FAC(N) * PGQSP0(M,N,JPOT,L)
  630       CONTINUE
#if (NTRACE > 0)
C
C         Add Del**8 damping term to tracer tendency
          DO 640 ITR=JTR1,NPGQ
            DO 640 M=0,MIN0(N,M1)
              DPGQSP(M,N,ITR,L)= DPGQSP(M,N,ITR,L)
     1             - TD8FAC(N) * PGQSP0(M,N,ITR,L)
  640       CONTINUE
#endif
C ----------------------------------------------------------------------
  700     CONTINUE
C
      RETURN
      END
 
      SUBROUTINE DDTFRC( DFRCSP, PGQSP )
C DDTFRC computes the spectral forcing/damping tendencies DFRCSP, given
C the values of prognostic quantities PGQSP.
C Input:  COMPLEX PGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX) --
C          spectral representation of prognostic quantities at all
C          levels at some appropriate model time
C Output: COMPLEX DFRCSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX) --
C         spectral forcing/damping time-tendencies (using PGQSP)
C
#include "mgrid.h"
#include "tmarch.h"
      COMPLEX DFRCSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX)
      COMPLEX PGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX)
      REAL TEMP
      COMPLEX STRESS(0:M1MAX, 0:N1MAX, 0:1)
      INTEGER L, M, N, IPGQ
      INTEGER LMHALF, LPHALF
C     (Mnemonic for L-(1/2), L+(1/2))
C
C ----------------------------------------------------------------------
C     First, compute the linear relaxation tendencies
      DO 200 L=1,L1
        DO 120 N=0,N1
C         Rayleigh friction & Newtonian cooling tendencies
          DO 110 M=0,MIN0(N,M1)
            DFRCSP(M,N,JVOR,L)= - URLXLV(L) * PGQSP(M,N,JVOR,L)
            DFRCSP(M,N,JDIV,L)= - URLXLV(L) * PGQSP(M,N,JDIV,L)
            DFRCSP(M,N,JPOT,L)= - TRLXLV(L) * PGQSP(M,N,JPOT,L)
  110       CONTINUE
C         Include forcing of zonal-mean potential temperature
          DFRCSP(0,N,JPOT,L)= DFRCSP(0,N,JPOT,L)
     1          + TRLXLV(L) * CMPLX(TMNLV(N,L), 0.0)
  120     CONTINUE
C       No relaxation of other prognostic quantities (if any)
        DO 130 IPGQ=JPOT+1,NPGQ
  130     CALL ZEROSP( DFRCSP(0,0,IPGQ,L), N1 )
  200   CONTINUE
C
C ----------------------------------------------------------------------
C     Add effects of vertical viscous damping (for vorticity/divergence)
      DO 400 IPGQ=JVOR,JDIV
        LMHALF= 1
        LPHALF= 0
C       No stress at uppermost level
        CALL ZEROSP( STRESS(0,0,LPHALF), N1 )
        DO 400 L=1,L1
          LMHALF= MOD(LMHALF+1, 2)
          LPHALF= MOD(LMHALF+1, 2)
          IF (L.LT.L1) THEN
C           Compute stress at level L+1/2
            TEMP= VVISC(L) / (PLV(L+1) - PLV(L))
            DO 310 N=0,N1
              DO 310 M=0,MIN0(N,M1)
                STRESS(M,N,LPHALF)= TEMP *
     1                       (PGQSP(M,N,IPGQ,L+1) - PGQSP(M,N,IPGQ,L))
  310           CONTINUE
          ELSE
C           Compute stress at surface (level L1+1/2)
            TEMP= VVISC(L1) / (PHLV(L1) - PLV(L1))
            DO 320 N=0,N1
              DO 320 M=0,MIN0(N,M1)
                STRESS(M,N,LPHALF)= -TEMP * PGQSP(M,N,IPGQ,L1)
  320           CONTINUE
          ENDIF
C         Add contribution from viscous stress divergence
          TEMP= 1.0 / DP(L)
          DO 330 N=0,N1
            DO 330 M=0,MIN0(N,M1)
              DFRCSP(M,N,IPGQ,L)= DFRCSP(M,N,IPGQ,L) +
     1                 TEMP * (STRESS(M,N,LPHALF) - STRESS(M,N,LMHALF))
  330         CONTINUE
C
  400   CONTINUE
C
C ----------------------------------------------------------------------
C     Add effects of vertical diffusion of temperature (not potential)
      LMHALF= 1
      LPHALF= 0
C     Compute stress at level 1+1/2
      TEMP= INVPNO * VVISC(1) / (PLV(2) - PLV(1))
      DO 510 N=0,N1
        DO 510 M=0,MIN0(N,M1)
          STRESS(M,N,LPHALF)= TEMP *
     1         (PKLV(2)*PGQSP(M,N,JPOT,2) - PKLV(1)*PGQSP(M,N,JPOT,1))
  510     CONTINUE
C     Thermal diffusion acts only on the inner (1 < L < L1) levels
      DO 540 L=2,L1-1
        LMHALF= MOD(LMHALF+1, 2)
        LPHALF= MOD(LMHALF+1, 2)
C       Compute stress at level L+1/2
        TEMP= INVPNO * VVISC(L) / (PLV(L+1) - PLV(L))
        DO 520 N=0,N1
          DO 520 M=0,MIN0(N,M1)
            STRESS(M,N,LPHALF)= TEMP *
     1     (PKLV(L+1)*PGQSP(M,N,JPOT,L+1) - PKLV(L)*PGQSP(M,N,JPOT,L))
  520       CONTINUE
C       Add contribution from divergence of diffusive heat "flux"
        TEMP= 1.0 / (DP(L)*PKLV(L))
        DO 530 N=0,N1
          DO 530 M=0,MIN0(N,M1)
            DFRCSP(M,N,JPOT,L)= DFRCSP(M,N,JPOT,L) +
     1               TEMP * (STRESS(M,N,LPHALF) - STRESS(M,N,LMHALF))
  530       CONTINUE
C
  540   CONTINUE
C ----------------------------------------------------------------------
      RETURN
      END
 
      SUBROUTINE DDTIMP( DPGQSP )
C DDTIMP makes implicit corrections to the spectral time tendency DPGQSP
C Input:  COMPLEX DPGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX) --
C          explicit spectral time-tendency of prognostic quantities
C Output: COMPLEX DPGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX) --
C          implicit spectral time-tendency of prognostic quantities
C
#include "mgrid.h"
#include "tmarch.h"
      COMPLEX DPGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX)
      INTEGER L0, L, M, N
      REAL RTEMP
      COMPLEX TEM(0:M1MAX,L1MAX)
#if (NTRACE > 0)
      INTEGER ITR
#endif
C
      DO 500 N=0,N1
C
C       Making implicit corrections to D tendency
        DO 200 L=1,L1
C         Contribution from explicit D tendency, with Del**8 correction
          DO 110 M=0,MIN0(N,M1)
  110       TEM(M,L)= UD8COR(N) * DPGQSP(M,N,JDIV,L)
C         Add contribution from explicit Theta tendency
          RTEMP= UD8COR(N) * (A0QINV*N*(N+1)) * TD8COR(N) * DTFAC
          DO 120 L0=1,L1
            DO 120 M=0,MIN0(N,M1)
  120         TEM(M,L)=TEM(M,L) + RTEMP*NNT2DT(L,L0)*DPGQSP(M,N,JPOT,L0)
  200     CONTINUE
C
C       Multiply TEM by implicit correction matrix to find implicit D
C       tendency
        DO 300 L=1,L1
          DO 210 M=0,MIN0(N,M1)
  210       DPGQSP(M,N,JDIV,L)= (0., 0.)
          DO 220 L0=1,L1
            DO 220 M=0,MIN0(N,M1)
  220         DPGQSP(M,N,JDIV,L)=  DPGQSP(M,N,JDIV,L)
     1                           + IMPCOR(L,L0,N)*TEM(M,L0)
  300     CONTINUE
C
        DO 400 L=1,L1
C
C         Make implicit correction to Theta tendency due to
C         implicit D tendency
          DO 310 L0=1,L1
            DO 310 M=0,MIN0(N,M1)
              DPGQSP(M,N,JPOT,L)= DPGQSP(M,N,JPOT,L)
     1              + DTFAC * D2TT(L,L0) * DPGQSP(M,N,JDIV,L0)
  310         CONTINUE
C
C         Make implicit correction due to Del**8 damping of Theta
          DO 320 M=0,MIN0(N,M1)
  320       DPGQSP(M,N,JPOT,L)= TD8COR(N) * DPGQSP(M,N,JPOT,L)
C
C         Make implicit correction due to Del**8 damping of vorticity
          DO 330 M=0,MIN0(N,M1)
  330       DPGQSP(M,N,JVOR,L)= UD8COR(N) * DPGQSP(M,N,JVOR,L)
#if (NTRACE > 0)
C
C         Make implicit correction due to Del**8 damping of tracer
          DO 340 ITR=JTR1,NPGQ
            DO 340 M=0,MIN0(N,M1)
  340         DPGQSP(M,N,ITR,L)= TD8COR(N) * DPGQSP(M,N,ITR,L)
#endif
  400     CONTINUE
  500   CONTINUE
      RETURN
      END
 
      SUBROUTINE ROBFIL( FSP0, FSP1, FSP2, ROBFAC )
C Given the spectral states FSP0, FSP1, FSP2 at three successive
C time-steps, ROBFIL performs a Robert-filter operation on the
C states, with filter weight ROBFAC.
C Input:  COMPLEX FSP0(0:M1MAX,0:N1MAX) -- spectral
C         representation of prognostic quantity at model time TIM-DT
C         COMPLEX FSP1(0:M1MAX,0:N1MAX) -- spectral
C         representation of prognostic quantity at model time TIM
C         COMPLEX FSP2(0:M1MAX,0:N1MAX) -- spectral
C         representation of prognostic quantity at model time TIM+DT
C         REAL ROBFAC -- filter weighting factor
C Output: (Filtered versions of FSP0, FSP1, FSP2)
C
#include "spgrid.h"
      COMPLEX FSP0(0:M1MAX,0:N1MAX)
      COMPLEX FSP1(0:M1MAX,0:N1MAX), FSP2(0:M1MAX,0:N1MAX)
      REAL ROBFAC
      INTEGER M, N
      REAL TEMP
      TEMP= 1.0 - 2.0 * ROBFAC
      DO 10 N=0,N1
        DO 10 M=0,MIN0(N,M1)
          FSP1(M,N)=  TEMP  *  FSP1(M,N)
     1              + ROBFAC * (FSP0(M,N) + FSP2(M,N))
   10     CONTINUE
      RETURN
      END
 
#ifdef DEBUG
      SUBROUTINE PGNORM(PGQSP, STR)
C PGNORM displays mass-weighted global mean and standard
C deviation of all prognostic variables. STR is a descriptive
C string.
C Input:  COMPLEX PGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX) --
C          spectral representation of prognostic quantities at all
C          levels at some appropriate model time
C         CHARACTER*(*) STR -- descriptive string
C Output: (Mean and std. deviation of PGQSP, averaged over all levels,
C          are displayed on standard output)
C
#include "mgrid.h"
      COMPLEX PGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX)
      CHARACTER*(*) STR
C
      REAL GMEAN(NPGQ), GVAR(NPGQ), GSDEV(NPGQ), TEMP
      INTEGER IPGQ, L, M, N
C
      DO 200 IPGQ=1,NPGQ
C       Initialize accumulators
        GMEAN(IPGQ)= 0.0
        GVAR(IPGQ)= 0.0
        DO 100 L=1,L1
C         Mass-weighting factor
          TEMP= DP(L) / PSURF
C         Compute mass-weighted global mean
          GMEAN(IPGQ)= GMEAN(IPGQ) + TEMP*PGQSP(0,0,IPGQ,L)
C         Compute mass-weighted global variance
          DO 100 N=1,N1
            GVAR(IPGQ)=  GVAR(IPGQ)
     1                 + TEMP*CABS(PGQSP(0,N,IPGQ,L))**2
            DO 100 M=1,MIN0(N,M1)
              GVAR(IPGQ)=  GVAR(IPGQ)
     1                   + 2.0*TEMP*CABS(PGQSP(M,N,IPGQ,L))**2
  100         CONTINUE
C       Compute standard deviation
        GSDEV(IPGQ)= SQRT(GVAR(IPGQ))
  200   CONTINUE
      WRITE(*,'(1X,A)') STR
      WRITE(*,'(7X,1P,4E18.9)') (GMEAN(IPGQ),IPGQ=1,NPGQ)
      WRITE(*,'(7X,1P,4E18.9)') (GSDEV(IPGQ),IPGQ=1,NPGQ)
      RETURN
      END
 
      SUBROUTINE SPNORM(FSP, L, STR)
C SPNORM displays horizontal mean and std. deviation of spectral field
C FSP. L is a descriptive integer, and STR is a descriptive string.
C Input:  COMPLEX FSP(0:M1MAX,0:N1MAX) --
C          spectral representation of a variable
C         INTEGER L -- descriptive integer (usually level no.)
C         CHARACTER*(*) STR -- descriptive string
C Output: (Mean and std. deviation of FSP are displayed on standard
C          output)
C
#include "spgrid.h"
      COMPLEX FSP(0:M1MAX,0:N1MAX)
      INTEGER L
      CHARACTER*(*) STR
C
      REAL HMEAN, HVAR, HSDEV
      INTEGER M, N
C
      HMEAN= FSP(0,0)
      HVAR= 0.0
      DO 100 N=1,N1
        HVAR= HVAR + CABS(FSP(0,N))**2
        DO 100 M=1,MIN0(N,M1)
          HVAR= HVAR + 2.0*CABS(FSP(M,N))**2
  100     CONTINUE
      HSDEV= SQRT(HVAR)
      WRITE(*,'(1X,A)') STR
      WRITE(*,'(1X,I5,1X,1P,2E18.9)') L, HMEAN, HSDEV
      RETURN
      END
 
      SUBROUTINE PHNORM(FPH, L, STR)
C PHNORM displays horizontal mean and std. deviation of physical field
C FPH. L is a descriptive integer, and STR is a descriptive string.
C Input:  REAL FPH(K1MAX,K2MAX) --
C          physical representation of a variable
C         INTEGER L -- descriptive integer (usually level no.)
C         CHARACTER*(*) STR -- descriptive string
C Output: (Mean and std. deviation of FPH are displayed on standard
C          output)
C
#include "spgrid.h"
      REAL FPH(K1MAX,K2MAX)
      INTEGER L
      CHARACTER*(*) STR
C
      REAL HMEAN, HVAR, HSDEV, TEMP
      INTEGER J, K
      REAL GAVGE
C
      HMEAN= GAVGE( FPH )
      HVAR= 0.0
      DO 100 K=1,K2
C       Latitude-weighting factor
        TEMP= G(K) / K1
        DO 100 J=1,K1
          HVAR= HVAR + TEMP*(FPH(J,K)-HMEAN)**2
  100     CONTINUE
      HSDEV= SQRT(HVAR)
      WRITE(*,'(1X,A)') STR
      WRITE(*,'(1X,I5,1X,1P,2E18.9)') L, HMEAN, HSDEV
      RETURN
      END
#endif
