C
C C preprocessor parameters
C
C NLON: No. of longitudes
C
C NLAT: No. of latitudes
C
C LENWRD: length of a machine word (in bytes)
C
C UDFVAL: Value used to represent missing/undefined data values
C
C

#ifndef NLON
#   define NLON 64
#endif

#ifndef NLAT
#   define NLAT 32
#endif

#ifndef LENWRD
#   define LENWRD 4
#endif

#ifndef UDFVAL
#   define UDFVAL 1.E36
#endif

      PROGRAM H2IEEE
C Converts horizontal-section save files to raw IEEE format
C
C Longitudes are ordered East to West, from the Greenwich meridian
C Latitudes are ordered South to North
C
      INTEGER I, J, IUINP, IUOUT, IRECD, NRECD
      REAL FMEAN, FK1, FK2, FDUMMY, FIELD(NLON,NLAT)
      CHARACTER INFIL*40, OUTFIL*40, HEADER*48
      INTEGER IDATE, ITIME
      CHARACTER COMENT*16, LEVEL*8, FLDNAM*12
C
      IUINP= 11
      IUOUT= 12
      WRITE(*,*) 'Input horizontal section file name?'
      READ(*,'(A)') INFIL
      WRITE(*,*) 'Output IEEE file name?'
      READ(*,'(A)') OUTFIL
      WRITE(*,*) 'No. of records?'
      READ(*,*) NRECD
C
C     Open input file
      OPEN(IUINP, FILE=INFIL, FORM='UNFORMATTED', STATUS='OLD',
     1            ACCESS='DIRECT', RECL=48+(4+NLON*NLAT)*LENWRD)
C
C     Open output file
      OPEN(IUOUT, FILE=OUTFIL, FORM='UNFORMATTED', STATUS='NEW',
     1            ACCESS='DIRECT', RECL=(NLON*NLAT)*LENWRD)
C
      DO 190 IRECD=1,NRECD
C       Read input record
        READ(IUINP,REC=IRECD,ERR=199)  HEADER, FMEAN, FK1, FK2, FDUMMY,
     1                                ((FIELD(I,J),I=1,NLON),J=1,NLAT)
C       Check data resolution
        IF ((NINT(FK1) .NE. NLON) .OR. (NINT(FK2) .NE. NLAT)) THEN
          WRITE(*,*) 'Incorrect data resolution specified'
          GOTO 199
        ENDIF
C
C       Add mean value to all deviations to obtain true values
C       on the gaussian grid
        DO 110 J=1,NLAT
          DO 110 I=1,NLON
  110       FIELD(I,J)= FIELD(I,J) + FMEAN
C
C       Write output record in IEEE format, with actual
C       data values (rather than deviations from the mean)
        WRITE(IUOUT,REC=IRECD) ((FIELD(I,J),I=1,NLON),J=1,NLAT)
C
C       Read each component of the header separately
        READ(HEADER,'(A16,I8,I4,A8,A12)')
     1       COMENT, IDATE, ITIME, LEVEL, FLDNAM
C
C       Display header information
        WRITE(*,'(A,A16,A,I8,A,I4,A,A8,A,A12)') 'comment=', COMENT,
     1          ', date (yyyymmdd)= ', IDATE, ', time (hhmm): ', ITIME,
     2          ', level: ', LEVEL,           ', field: ', FLDNAM
  190   CONTINUE
C
      WRITE(*,*) NRECD, ' records successfully converted'
C     Close files
      CLOSE(IUINP)
      CLOSE(IUOUT)
      STOP
C
C     Error in input
  199 WRITE(*,*) 'Error in reading record ', IRECD
      CLOSE(IUINP)
      CLOSE(IUOUT)
      STOP
      END
